#include <LOLA/discrete.h>
#include <XBitmaps/Italia_Bd.xbm>
#include <XBitmaps/Italia_Int.xbm>
#include <XBitmaps/German_Bd.xbm>
#include <XBitmaps/German_Int.xbm>

static window* Wp;
static discrete* Du;
static discrete* Dp;
static bool WI; //flag for showing the demands
static bool WF; //flag for showing the fixcosts

static void redraw_func() { Du->draw_loladiscrete(*Wp, *Dp); }

discrete::discrete(matrix Cost) {  
	this->COST = Cost;
}

discrete::discrete(list<location> EX, matrix Cost) {
	this->DEMAND=EX;
	this->SUPPLY=EX;
	this->same=true;
	this->COST = Cost;
}

discrete::discrete(list<location> dem, list<location> sup, matrix Cost) {
	this->DEMAND=dem;
	this->SUPPLY=sup;
	this->same=false;
	this->COST = Cost;
}

discrete::~discrete() {}

void discrete::del_sol() {
  this->solution.clear();
  belongto = COST;
  for (int i=0;i<COST.dim1();i++) 
	  for (int j=0;j<COST.dim2();j++)
		  belongto[i][j]= 0;
}

void discrete::DiscView(double objval)
/********************************************************************
* Purpose    : Show optimum and objective value with actual norm graphically
* Returns    : %
* Influences : %
* Date       : July 1998
* Author     : Holger Hennes
* Last Change: Jan 1999 (Holger Hennes)
*********************************************************************/
{
	window W(600,650,"LOLA VERSION 2.0    View Graphics");
	W.set_node_width(4);
	W.set_show_coordinates(true);
	W.set_redraw(redraw_func);

	string file_name = "results", tmp;
	point p,q;
	int i=0;
	location loctmp,loctemp;
	list<location> AlgSol = this->solution;
	list<location> supply, demand = this->DEMAND;
	if (!same) supply = this->SUPPLY;
	else supply = demand;
	list<int> belong;
	for (i=0;i<this->COST.dim1();i++) 
		for (int j=0; j<this->COST.dim2();j++) 
			if (this->belongto[i][j]) belong.append(j);

	W.buttons_per_line(3);

	W.button("Save Results",1);
	W.button("View Results",3);
	W.button("View Cost Matrix",4);
	W.button("Show/Hide Demand",5);
	W.button("Show/Hide Fix Costs",6);
	W.button("Refresh",8);
	W.button("Close",9);
	W.display(650,200);

	double winmax = L_MAX(this->max_direction_value(0),this->max_direction_value(1));
	double diff_loc = L_MAX(this->diff_value(0),this->diff_value(1));
	double winminx = this->min_direction_value(0) - (0.2 * fabs(diff_loc));
	double winminy = this->min_direction_value(1) - (0.2 * fabs(diff_loc));
	winmax += (0.25 * fabs(diff_loc));
	W.init(winminx, winmax, winminy);
	winmax += winmax-L_MIN(W.xmax(),W.ymax());
	W.init(winminx, winmax, winminy);

	Wp = &W;
	list<bool> W_inf; W_inf.append(false); W_inf.append(false);
	Dp = this;
	Wp = &W; 
	redraw_func();
	WF=false;

	for(;;)  {
		int but = W.read_mouse(p);
		if (but == -1) { // mouse points into the drawing area
			// check, if the mouse points at a Opt-node, only for N-Median
			redraw_func();
			int pos=0;
			double eps = (W.xmax()-W.xmin())/100.0; // 1/100th of window width is close enough
			list_item it=supply.first();
			while (it) { // Auf welchen Knoten zeigt die Maus????
				loctmp=supply.contents(it);
				q = loctmp.loc2point();
				if ((fabs(p.xcoord()-q.xcoord())<=eps) &&
					(fabs(p.ycoord()-q.ycoord())<=eps)) break;
				it = supply.succ(it);
				++pos; // entspricht dem Knoten
			}

			int Arank=0;
			forall(i,belong) {
				if (Arank==pos) {
					it=demand.first();
					int no_belong,listpos=0;
					forall(no_belong,belong) {
						loctmp = demand.contents(it);
						if(no_belong==Arank && listpos!=pos) { 
							// Knoten gehoert zu pos, pos-Selber soll blau bleiben
							p = loctmp.loc2point();
							W.draw_filled_node(p,red);
						}   // if no_belong==Arank
						it = demand.succ(it);
						listpos++;
					} // forall
				} // if
				Arank++;
			} //forall
		} // if (button == -1)

		if (but == 1)  {
			panel PSave("Save results");
			PSave.string_item("file: ",file_name);
			PSave.text_item(" ");
			PSave.text_item(" ");
			PSave.buttons_per_line(2);
			int bScancel = PSave.button("Cancel"), bSok =PSave.button("Ok");
			int butS = PSave.open(780,200);
			if (butS == bScancel) {
				file_name = "results"; continue;
			} else if (but == bSok) {
				ofstream F(file_name,ios::out);
				if (!F) {
					tmp = string("Can't open file \"%s\"!", file_name);
					W.message(tmp);
				} else {
					this->SaveDisc(F);
					F << "begin {results}" << endl;
					forall(loctmp,AlgSol) F<<loctmp<<endl;
					F << "end {results}"<<endl;
					F << "OBJECTIVE VALUE: \n";
					F << objval << endl;
				}
			} //else if (but == bSok)
		} //if (but == 1)

		if (but == 3) {
			i=0;
			panel Presults("Results of computation");
			Presults.text_item(string("%-21s        %-13s","OPTIMUM:","OBJECTIVE VALUE:"));
			Presults.text_item("");
			loctmp = AlgSol.head();
			tmp = string("(%8.2lf, %8.2lf)           %13.2lf ", loctmp[0], loctmp[1], objval);

			Presults.text_item(tmp);
			Presults.text_item("");

			if (AlgSol.size() > 1) {
				list_item A_it;
				for(A_it = AlgSol.succ(AlgSol.first()); A_it; A_it = AlgSol.succ(A_it)) {
					loctmp = AlgSol.contents(A_it);
					tmp = string("(%8.2lf,  %8.2lf) ", loctmp[0], loctmp[1]);
					Presults.text_item(tmp);
					Presults.text_item("");
				} //for
			} //if
			Presults.button("Ok",10);
			Presults.display(0,0);
			int butP=0;
			butP = Presults.read_mouse();
			if (butP == 10) Presults.close();
		} //if (but == 3)

		if (but == 4) {
			panel Pcost("Cost matrix");
			for(i=0;i<this->COST.dim1();i++) {
				tmp = string("");
				for(int j=0;j<this->COST.dim2();j++)
					tmp+=string("%8.2lf ",this->COST(i,j));
				Pcost.text_item(tmp);
				Pcost.text_item("");
			}
			Pcost.button("Ok",11);
			Pcost.display(window::max,window::min);
			int butP=0;
			butP = Pcost.read_mouse();
			if (butP == 11) Pcost.close();
		} //if (but == 4)
		
		if (but == 5) {
			if (WI) WI = false;
			else { WI = true; WF = false; }
			redraw_func();
		}
		if (but == 6) {
			if (WF) WF = false;
			else { WF = true; WI = false; }
			redraw_func();
		}
		if (but == 8) redraw_func();
		if (but == 9) break;
	} // for
} // DiscView


void discrete::SaveDisc(ofstream& file)
{
	matrix C=this->COST;
	location loctmp;
	string txt;
	list_item pointer_to_txt = this->Loctxt.first();

	if (!file) lola_error("Can't open file !", 3103,1);
	else {
		file << "begin {discrete}"<<endl;
		file << "begin {demand}"<<endl;
		forall(loctmp,this->DEMAND) {
			txt=this->Loctxt.contents(pointer_to_txt);
			pointer_to_txt=this->Loctxt.succ(pointer_to_txt);
			file << loctmp << " [" << txt << "]"<<endl;
		}
		file << "end {demand}"<<endl;
		if (!same) {
			file << "begin {supply}"<<endl;
			forall(loctmp,this->SUPPLY) {
				txt=this->Loctxt.contents(pointer_to_txt);
				pointer_to_txt=this->Loctxt.succ(pointer_to_txt);
				file << loctmp << " [" << txt << "]"<<endl;
			}
			file << "end {supply}"<<endl;
		}
		file << "begin {costmatrix}"<<endl;
		for (int i=0;i<C.dim1();i++) {
			for (int j=0;j<C.dim2();j++)
				file << C(i,j)<<" ";
			file << endl;
		}
		file << "end {costmatrix}"<<endl;
		file << "end {discrete}"<<endl;
	}
} //SaveDisc

void discrete::draw_loladiscrete(window& W, discrete& Dis)
/********************************************************************
* Purpose    : Show optimum and objective value with actual norm graphically
* Returns    : %
* Date       : Sep 1998
* Author     : Holger Hennes
* Last Change: Jan 1999 (Holger Hennes)
*********************************************************************/
{
	location loctmp;
	list<node> NG, NLG;
	GRAPH<location,double> LG;
	list<location> demand = Dis.DEMAND;
	list<location> supply = Dis.SUPPLY;
	bool italia=false,german=false;
	char *xbm1, *xbm2;
	color c=black,xbm_col(255,255,50);

	double winmax = L_MAX(Dis.max_direction_value(0),Dis.max_direction_value(1));
	double diff_loc = L_MAX(Dis.diff_value(0),Dis.diff_value(1));
	double winminx = Dis.min_direction_value(0) - (0.2 * fabs(diff_loc));
	double winminy = Dis.min_direction_value(1) - (0.2 * fabs(diff_loc));
	winmax += (0.25 * fabs(diff_loc));
	W.init(winminx, winmax, winminy);
	winmax += winmax-L_MIN(W.xmax(),W.ymax());
	W.init(winminx, winmax, winminy);

	if (!Dis.Loctxt.empty()) {
		if (Dis.Loctxt.head()=="Milano") {
			italia=true;
			W.init(0,100,0);
			xbm1 = W.create_bitmap(500,500,Italia_Int_bits);
			W.put_bitmap(9,4,xbm1,xbm_col);
			xbm2 = W.create_bitmap(500,500,Italia_Bd_bits);
			W.put_bitmap(9,4,xbm2,black);
		}
		if (Dis.Loctxt.head()=="Kiel" || Dis.Loctxt.head()=="Hamburg") {
			german=true;
			W.init(0,100,0);
			xbm1 = W.create_bitmap(400,500,German_Int_bits);
			W.put_bitmap(16,5,xbm1,xbm_col);
			xbm2 = W.create_bitmap(400,500,German_Bd_bits);
			W.put_bitmap(16,5,xbm2,black);
		}
	}

	list_item facs_it;
	list_item txt_it=Dis.Loctxt.first();
	for (facs_it=demand.first(); facs_it; facs_it=demand.succ(facs_it)) {
		loctmp = demand.contents(facs_it);
		point p = loctmp.loc2point();
		W.draw_filled_node(p,c);
		W.draw_text(p,Dis.Loctxt.contents(txt_it),black);
		if (WI) {
			string txt = string("%8.2lf",loctmp(0));
			double qx = p.xcoord()-diff_loc/50.0;
			double qy = p.ycoord()+diff_loc/50.0;
			point q = point(qx,qy);
			W.draw_ctext(q,txt,black);
		}
		txt_it = Dis.Loctxt.succ(txt_it);
	}
	txt_it=Dis.Loctxt_sup.first();

	if(WF) {
		list_item fixcost_it;
		for (fixcost_it=supply.first(); fixcost_it; fixcost_it=supply.succ(fixcost_it)) {
			loctmp = supply.contents(fixcost_it);
			point p = loctmp.loc2point();
			string txt = string("%8.2lf",loctmp(0));
			double qx = p.xcoord()-diff_loc/50.0;
			double qy = p.ycoord()+diff_loc/50.0;
			//		  point q = point(qx,qy);
			W.draw_ctext(point(qx,qy),txt,black);
		}
	}

	if (!Dis.same) {
		for (facs_it=supply.first(); facs_it; facs_it=supply.succ(facs_it)) {	  
			loctmp = supply.contents(facs_it);
			point p = loctmp.loc2point();
			W.draw_filled_node(p,c);
			W.draw_text(p,Dis.Loctxt_sup.contents(txt_it),green);
			if (WI) {
				string txt = string("%8.2lf",loctmp(0));
				double qx = p.xcoord()-diff_loc/50.0;
				double qy = p.ycoord()+diff_loc/50.0;
				point q = point(qx,qy);
				W.draw_ctext(q,txt,black);
			}
			txt_it = Dis.Loctxt_sup.succ(txt_it);
		}
	}
	list<double> belong;
	for (int i=0;i<Dis.COST.dim1();i++) {
		for (int j=0; j<Dis.COST.dim2();j++) {
			if (Dis.belongto[i][j]) {
				point p = demand.inf(demand[i]).loc2point();
				point q = supply.inf(supply[j]).loc2point();
				belong.append(j);
				W.draw_edge(p,q,c);
			}
		}
	}
	int optnr = 1;
	int S;
	belong.sort();
	belong.unique();
	forall(S,belong) {
		string txt;
		loctmp =  supply.inf(supply[S]);
		point p =loctmp.loc2point();
		txt = Dis.Loctxt_sup.contents(Dis.Loctxt_sup.get_item(S));
		string s;
		if (txt.length()>0) s =  txt + "=Opt" + (string ("%1d",optnr));
		else s = "Opt" + (string ("%1d",optnr));
		++optnr;
		color col(0,0,255);
		W.draw_filled_node(p,col);
		W.draw_text(p,s,col);
	}
	if((italia) || (german)) { W.del_bitmap(xbm1); W.del_bitmap(xbm2); }
} //draw_loladiscrete

double discrete::max_direction_value(int i)
{
	double    maximum;
	location  tmp;
	list_item pointer_to_location;

	tmp=this->DEMAND.head();
	maximum=tmp[i];
	for(pointer_to_location=this->DEMAND.first();pointer_to_location;pointer_to_location=this->DEMAND.succ(pointer_to_location))
	{
		tmp=this->DEMAND.contents(pointer_to_location);
		if (tmp[i]>maximum) maximum=tmp[i];
	}
	if (!same) {
		for(pointer_to_location=this->SUPPLY.first();pointer_to_location;pointer_to_location=this->SUPPLY.succ(pointer_to_location))
		{
			tmp=this->SUPPLY.contents(pointer_to_location);
			if (tmp[i]>maximum) maximum=tmp[i];	  
		}
	}
	return maximum;
}//max_direction_value

double discrete::min_direction_value(int i)
{
	double    minimum;
	location  tmp;
	list_item pointer_to_location;

	tmp=this->DEMAND.head();
	minimum=tmp[i];
	for(pointer_to_location=this->DEMAND.first();pointer_to_location;pointer_to_location=this->DEMAND.succ(pointer_to_location))
	{
		tmp=this->DEMAND.contents(pointer_to_location);
		if (tmp[i]<minimum) minimum=tmp[i];
	}
	if (!same) {
		for(pointer_to_location=this->SUPPLY.first();pointer_to_location;pointer_to_location=this->SUPPLY.succ(pointer_to_location))
		{
			tmp=this->SUPPLY.contents(pointer_to_location);
			if (tmp[i]<minimum) minimum=tmp[i];	  
		}
	}
	return minimum;
} //min_direction_value

double discrete::diff_value(int i)
{
	return max_direction_value(i)-min_direction_value(i);
}

int discrete::disc_read(string& locfile)
{
	if (locfile=="") {
		panel Pload("Load Example");
		Pload.string_item("file:",locfile);
		Pload.text_item("");
		Pload.buttons_per_line(2);
		int bLcancel = Pload.button("cancel");
		int bLok = Pload.button("ok");
		int butL = Pload.open(780,200);
		if (butL == bLcancel) {
			return 0;
		} 
		else if (butL == bLok)  {
			ifstream file (locfile,ios::in);
			if (!file) {
				lola_error("Can't open file",3104,1);
				return 1;
			}
			else {
				this->ReadDiscrete(file);
			}
		}
	}
	else {
		ifstream file (locfile,ios::in);
		if (!file) {
			lola_error("Can't open file",3105,1);
			return 1;
		}
		else {
			this->ReadDiscrete(file);
		}
	}
	same = 0;
	if (SUPPLY.empty()) {
		SUPPLY=DEMAND;
		Loctxt_sup=Loctxt;
		same=1;
	}
  return 0;
}
