#include <LOLA/discrete.h>


double discrete::UFLP_greedy()
/********************************************************************
* Purpose    : Compute opt. sol. for UFLP with Greedy-heuristic
* Returns    : Objectiv value
* Influences : %
* Date       : July 1998
* Author     : Joerg Kalcsics
* Last Change: Jan 1999 (Holger Hennes)
*********************************************************************/
{
	int m,n,k=0,i=0,j=0,min_int,max_int,count=1;
	double res,obj_val,min=L_INF,rho_j,max=0,min_cik;
	location loctemp,demtemp;
	list<location> sol;

// J={1,...,m} supply, I={1,...,n} demand, S=Set of already found locations,
// S_comp=J\S set of yet free locations
	matrix C=this->COST;
	n=C.dim1(); m=C.dim2(); 
	array<double> F(m),minS(n);
	set<int> S_comp,S;
	forall(loctemp,this->SUPPLY) { 
		F[k]=loctemp(0); 
/*		i=0;
		forall (demtemp,this->DEMAND) {
			C[i][k]*=demtemp(0);
			i++;
		}*/
		k++; 
	}
	for(k=0;k<m;k++) S_comp.insert(k);
	for(i=0;i<n;i++) minS[i] = L_INF;
	// Initialisation step, choose first location
	for(j=0;j<m;j++) {
		res=0;
		for(i=0;i<n;i++) res += C[i][j];
		rho_j=res+F[j];
		if(rho_j<min) { min=rho_j; min_int=j; }
	}
	max_int=min_int; obj_val=min;
// first location found, now choose further nodes
	do {
		S.insert(max_int); S_comp.del(max_int); obj_val-=max;

		for(i=0;i<n;i++) {          // compute min{c_ik : k in S} for all i
			if(C[i][max_int]<minS[i]) minS[i]=C[i][max_int];
		}

		max=-1;
		forall(j,S_comp) {          // compute rho_j for j in S_comp
			res=0;
			for(i=0;i<n;i++) {
				if(0 < (minS[i]-C[i][j])) res += minS[i]-C[i][j];
			}
			rho_j=res-F[j];
			if(rho_j<0) S_comp.del(j);
			else {
				if(max<rho_j) { max=rho_j; max_int=j; }
			}
		}
		// Es wurde nun das opt. j in S_comp bestimmt, fuer das rho_j maximal wird
		// Falls max>0, fuege max_int in S ein und loesche es aus S_comp
	}while((max>0) && (!S_comp.empty()));
	// compute sol and belongto
	for(i=0;i<n;i++) {
		min_cik=L_INF; min_int=-1;
		forall(j,S) {             // compute min{c_ij : j in S}, derive belongto
			if(C[i][j]<min_cik) {
				min_cik=C[i][j]; min_int=j;
			}
		}
		this->belongto[i][min_int]=1;
	}
	forall(j,S) sol.append(this->SUPPLY.inf(this->SUPPLY.get_item(j)));
	this->solution=sol;
	return obj_val;
}


double discrete::UFLP_stingy() 
/********************************************************************
* Purpose   : Compute opt. sol. for UFLP with Stingy-heuristic
* Returns   : Objectiv value
* Influences: %
* Date      : July 1998
* Author    : Joerg Kalcsics
*********************************************************************/
{
	int m,n,k=0,i=0,j=0,min_int,count=1;
	double res=0,obj_val=0,min_cik,sigma_j,min=0,epsilon=0.001;
	location loctemp,demtemp;
	list<location> sol;

// J={1,...,m}, I={1,...,n}, S=Set of already found locations,
// S_comp=J\S set of yet free locations
	matrix C=this->COST;
	n=C.dim1(); m=C.dim2(); 
	array<double> F(m),minS(n);
	set<int> S_j,S;
	forall(loctemp,this->SUPPLY) { 
		F[k]=loctemp(0); 
/*		i=0;
		forall (demtemp,this->DEMAND) {
			C[i][k]*=demtemp(0);
			i++;
		}*/
		k++; 
	}
	for(k=0;k<m;k++) {
		S.insert(k); S_j.insert(k);
	}
	if (!same) {
		for (i=n;i--; ) {
			minS[i]=L_INF;
			forall(k,S) {
				if(C[i][k]<minS[i]) minS[i]=C[i][k];
			}
			obj_val+=minS[i];
		}
	}
	for(j=0;j<m;j++) obj_val+=F[j];

	do {

		obj_val+=min; min=1;
		for(i=0;i<n;i++) {          // compute min{c_ik : k in S}
			minS[i]=L_INF;
			forall(k,S) {
				if(C[i][k]<minS[i]) minS[i]=C[i][k];
			}
		}

		forall(j,S) {               // compute sigma_j for j in S
			res=0; S_j.del(j);
			for(i=0;i<n;i++) {
				if(C[i][j]>minS[i]+epsilon)
					min_cik=minS[i];
				else {
					min_cik=L_INF;
					forall(k,S_j) {       // compute min{c_ik : k in S\{j}}
						if(C[i][k]<min_cik) min_cik=C[i][k];
					}
				}
				if(0 < (min_cik-C[i][j])) res += min_cik-C[i][j];
			}
			sigma_j=res-F[j];
			if(min>sigma_j) { min=sigma_j; min_int=j; }
			S_j.insert(j);
		}
		// Es wurde nun das opt. j in S bestimmt, fuer das sigma_j minimal wird
		// Falls min<0, loesche min_int aus S
		S.del(min_int); S_j.del(min_int);
	}while((min<0) && (!S.empty()));
	// compute sol und belongto
	for(i=0;i<n;i++) {
		min_cik=L_INF; min_int=-1;
		forall(j,S) {               // compute min{c_ij : j in S} derive belongto
			if(C[i][j]<min_cik) {
				min_cik=C[i][j]; min_int=j;
			}
		}
		this->belongto[i][min_int]=1;
	}
	forall(j,S) sol.append(this->SUPPLY.inf(this->SUPPLY.get_item(j)));
	this->solution=sol;
	return obj_val;
}



double discrete::UFLP_Interchange()
/********************************************************************
* Purpose   : Compute opt. sol. for UFLP with Interchange-heuristic
* Returns   : Objectiv value
* Influences: %
* Date      : July 1998
* Author    : Joerg Kalcsics
*********************************************************************/
{
	int i,j,k=0,l,m,n,count=1,int_k,step,min_int;
	double change,res,tau_jk,greedy_res,stingy_res,obj_val;
	double min,min_cik;
	set<int> S_comp,S_greedy,S_j,S;
	location loctemp,loc_search;
	list<location> sol;

	matrix C=this->COST;
	n=C.dim1(); m=C.dim2();
	array<double> minS(n),minS_j(n),F(m);
	forall(loctemp,this->SUPPLY) { 
		F[k]=loctemp(0); 
/*		i=0;
		forall (loc_search,this->DEMAND) {
			C[i][k]*=loc_search(0);
			i++;
		}*/
		k++; 
	}

	for(step=0;step<2;step++) {
		count=0;
		if(step==0) {                 // Greedy
			greedy_res=this->UFLP_greedy();
			obj_val=greedy_res;
		}//end greedy

		else {                        // Stingy
			S_greedy=S; S.clear(); greedy_res=obj_val;
			stingy_res=this->UFLP_stingy();
			obj_val=stingy_res;
		} // end stingy

    // compute initial solution set S from greedy resp. stingy algorithm
		loc_search=this->solution.pop();
		forall(loctemp,this->SUPPLY) {
			if(loc_search==loctemp) {
				S.insert(count);
				if(!this->solution.empty()) loc_search=this->solution.pop();
				else break;
			}
			count++;
		}
		this->solution.clear();

		if(S.size()==m) continue;

		S_comp.clear();
		for(l=0;l<m;l++) {              // calculate S_comp=J\S
			if(!S.member(l)) S_comp.insert(l);
		}

		do {                // exchange k and j as long as the obj. value improves
			for(i=0;i<n;i++) {            // compute min{c_il : l in S} for all i
				min=L_INF;
				forall(l,S) {
					if(C[i][l] < min) min=C[i][l];
				}
				minS[i]=min;
			}

			forall(j,S) {                 // delete j from S
				change=-0.1;
				for(i=0;i<n;i++) {          // calculate min{c_il : l in S\{j}} for all i
					min=L_INF;
					forall(l,S) {
						if((C[i][l] < min) && (l!=j)) min=C[i][l];
					}
					minS_j[i]=min;
				}

				forall(k,S_comp) {          // add k to S\{j} and calculate tau_jk(S)
					res=0;
					for(i=0;i<n;i++) {
						if((minS[i]-C[i][k]) > (minS[i]-minS_j[i]))
							res+=minS[i]-C[i][k];
						else
							res+=minS[i]-minS_j[i];
					}
					tau_jk=res+F[j]-F[k];

					if(tau_jk>change) {
						change=tau_jk; int_k=k;
					}
				} // end forall(k,S_comp)

				// if change>0, then S\{j}+{k} is a better sol. than S; S:=S\{j}+{k}.
				if(change>0) {            // forall(j,S)-loop terminated; back to 'do'-loop
					S.del(j); S.insert(int_k); S_comp.del(int_k); S_comp.insert(j);
					obj_val-=change; break;
				}
			} // end forall(j,S)
		}while(change>0);

    // no further improvment of opt. sol. through exchange of nodes possible
    // optional: greedy- or stingy-step

	} // end for(step=0;step<2;step++)

	stingy_res=obj_val;
	if(greedy_res<stingy_res) {   // compare sol.s and choose the better one
		S.clear(); S=S_greedy; obj_val=greedy_res;
	}
	else obj_val=stingy_res;

	// compute sol und belongto
	for(i=0;i<n;i++) {
		for(j=0;j<m;j++)
			this->belongto[i][j]=0;
		min_cik=L_INF; min_int=-1;
		forall(j,S) {               // compute min{c_ij : j in S} derive belongto
			if(C[i][j]<min_cik) {
				min_cik=C[i][j]; min_int=j;
			}
		}
		this->belongto[i][min_int]=1;
	}
	forall(j,S) sol.append(this->SUPPLY.inf(this->SUPPLY.get_item(j)));
	this->solution=sol;

	return obj_val;
}


double discrete::UFLP_dualoc()
/********************************************************************
* Purpose   :  returns a guaranteed optimal and feasible solution for the UFLP
* Returns   : double (=objval)
* Influences: discrete-object(belongto) 
* Date      : July 98
* Author    : Karen Schulze
*********************************************************************/
{
	double primal_res,dual_res;
	matrix C=this->COST;
	int n=C.dim1(),m=C.dim2(),j=0;
	location loctemp;
/*	for(int k=0;k<m;k++) {		
		j=0;
		forall (loctemp,this->DEMAND) {
			C[j][k]*=loctemp(0);
			j++;
		}
	}*/
	matrix X(n,m),Mu(n,m);
	array<double> Y(m),lambda(n);
	// zulassige duale Anfangslsg lambda(i)=0
	dual_res=this->UFLP_dualascent(X,Y,lambda,Mu,primal_res,C);
	matrix X_rem=X,Mu_rem=Mu;
	array<double> Y_rem=Y,lambda_rem=lambda;
	double primal_rem=primal_res;
	bool is_opt1=this->UFLP_branchbound(X,Y,lambda,Mu,primal_res,C,1); // schliessen
	bool is_opt= this->UFLP_branchbound(X_rem,Y_rem,lambda_rem,Mu_rem,primal_rem,C,0); //oeffnen
	if (!is_opt && !is_opt1) lola_error("No feasible solution found!",3101,1);
	if( !is_opt1 || (is_opt && is_opt1 && primal_rem<primal_res)) {Y=Y_rem;X=X_rem;primal_res=primal_rem;Mu=Mu_rem;}
	this->belongto = X;
	for(j=0;j<m;j++)
		if (Y[j]==1)
			this->solution.append(SUPPLY[SUPPLY[j]]);
	return primal_res;
}

int is_violated(array<double> Y,matrix Mu)
/********************************************************************
* Purpose   : tests if matrix Mu is violated for any new facility 
* Returns   : int (=violated facility) 
* Influences: %       
* Date      : July 98
* Author    : Karen Schulze
*********************************************************************/
{
	for(int j=0;j<Mu.dim2();j++) { 
		if(Y[j]==1) {
			for(int i=0;i<Mu.dim1();i++)
				if(Mu(i,j)!=0) {
					return j;
				}
		} // if
	} // for
	return -1;
}



bool discrete::UFLP_branchbound(matrix& X_rem,array<double>& Y_rem,array<double>& lambda_rem,matrix& Mu_rem,double& upperbound_rem, matrix& Kosten, bool close)
/********************************************************************
* Purpose   : returns a feasible solution for the UFLP, recursiv!!!
* Returns   : bool 
* Influences: % 
* Date      : July 98
* Author    : Karen Schulze 
*********************************************************************/
{
	double upperbound=upperbound_rem;
	discrete D=*this;
//	matrix Kosten=this->COST;
	int n=Kosten.dim1();
	double dual_res,new_upbd;
	// current solution not feasible?
	int violated=is_violated(Y_rem,Mu_rem);
	if (violated==-1) return 1;  // solution is feasible!
	//close or open facility j=violated
	int i,j=0;
	location loctemp;
	double f_j;
	if(close ) // schliessen
		D.SUPPLY[D.SUPPLY[violated]](0)=L_INF;  // Objekt D aendern
	if(!close ) {// oeffnen
		f_j=D.SUPPLY[D.SUPPLY[violated]](0);
		D.SUPPLY[D.SUPPLY[violated]](0)=0;    // Objekt D aendern
		for(i=0;i<n;i++) // zulaessige Dualloesung erzeugen
			if(lambda_rem[i]>Kosten[i][violated])
				lambda_rem[i]=Kosten[i][violated];        
	} //if
	dual_res=D.UFLP_dualascent(X_rem,Y_rem,lambda_rem,Mu_rem,upperbound_rem,Kosten);
	if (!close) {dual_res+=f_j;upperbound_rem+=f_j;} 
	if (dual_res>upperbound) return 0; // branch is fathomed
	// further branching
	upperbound=upperbound_rem;
	matrix X=X_rem,Mu=Mu_rem;
	array<double> Y=Y_rem,lambda=lambda_rem;
	new_upbd=upperbound_rem;
	bool is_opt0=D.UFLP_branchbound(X,Y,lambda,Mu,new_upbd,Kosten,1); // schliessen-Zweig
	bool is_opt=D.UFLP_branchbound(X_rem,Y_rem,lambda_rem,Mu_rem,upperbound_rem,Kosten,0); //oeffnen-Zweig
	if ((is_opt0 && !is_opt) ||(is_opt0 && is_opt && new_upbd<upperbound_rem)) {X_rem=X;Mu_rem=Mu;Y_rem=Y;lambda_rem=lambda;upperbound_rem=new_upbd;is_opt=is_opt0;}
	if (!close && upperbound_rem<upperbound) {upperbound_rem+=f_j;} 
	return is_opt;
} 


double discrete::UFLP_dualascent(matrix& X,array<double>& Y,array<double>& lambda,matrix& Mu,double& primal_res, matrix& C)
/********************************************************************
* Purpose   : returns a solution for the UFLP, no feasibility guaranteed 
* Returns   : double 
* Influences: %
* Date      : July 98 
* Author    : Joerg Kalcsics 
*********************************************************************/
{
	bool violated=false,unbound_lambda_i;
	register int i=0,j=0,k=0;
	int n,m,no_improv=0,min_j,int_violated,decr_i=0;
	double res,obj_val=0,min_cij,min=0,lambda_i;
	double epsilon=0.0001,min_change,dual_res=0,fc_change,fc_sum=0;
	double last_dual_res=0;
	set<double> SLam; 
	location loctemp;  
	list<location> sol;
	
//	matrix C=this->COST;
	n=C.dim1(); m=C.dim2(); // J={1,...,m} supply, I={1,...,n} demand
	array<double> F(m),lambda_last(n),Fc(m),old_lambda(n);
	forall(loctemp,SUPPLY) { F[k]=loctemp(0); k++;if(k==m) break; }

	// update Fc[i] for the new lambda[decr_i]
	for(j=0;j<m;j++) {
		res=0;
		for(i=0;i<n;i++) {
			if(lambda[i]-C[i][j]>0) res+=lambda[i]-C[i][j];
		}
		Fc[j]=res-F[j];
	}

	i=0;

	do {
		do {
			if(i==n) i=0;
			violated=false; unbound_lambda_i=false;

			// compute min{c_ij : c_ij > lambda_i}
			min_cij=L_INF;
			for(j=0;j<m;j++) {
				if((C[i][j]>lambda[i]+epsilon) && (C[i][j]<min_cij)) min_cij=C[i][j];
				// NB F_c erfuellt?
			}
			lambda_i=min_cij; fc_change=0; min_change=L_INF;
			if(lambda_i==L_INF) unbound_lambda_i=true;

			for(j=0;j<m;j++) {                  // compute impact of lambda_i on Fc[i]
				fc_change=0;
				if(lambda[i]-C[i][j]>epsilon)
					fc_change=lambda_i-lambda[i];
				else {
					if(lambda_i-C[i][j]>epsilon) fc_change=lambda_i-C[i][j];
				}

				if(Fc[j]+fc_change>epsilon) {     // (F^c) is violated for j for new lambda_i
					violated=true;
					if(-Fc[j]<min_change) {
						min_change=-Fc[j]; int_violated=j;
						// make min_change and therefore lambda_i as large as possible
					}
				}
			}

			if(unbound_lambda_i && !violated) {
				i++; continue;
			}

			//      if(fabs(min_change)>epsilon && fc_sum>0) {    // lambda[i] can be increased
			if(fabs(min_change)>epsilon) {    // lambda[i] can be increased
				no_improv=0; lambda_last[i]=lambda[i];
				if(violated) {                // compute the new lambda[i] from min_change
					if(lambda[i]-C[i][int_violated]>epsilon)
						lambda[i]+=min_change;
					else
						lambda[i]=C[i][int_violated]+min_change;
				} // if violated
				else lambda[i]=lambda_i;

				for(j=0;j<m;j++) {              // update Fc[] for the new lambda[i]
					if(lambda_last[i]-C[i][j]>epsilon)
						Fc[j]+=lambda[i]-lambda_last[i];
					else {
						if(lambda_i-C[i][j]>epsilon) Fc[j]+=lambda[i]-C[i][j];
					} //if
				} //for
			} else no_improv++;               // lambda[i] can not be increased

			i++;
		}while(no_improv<n);

		// main loop finished, now compare results
		last_dual_res=dual_res; dual_res=0; no_improv=0;
		for(i=0;i<n;i++) dual_res+=lambda[i];

		if(dual_res>last_dual_res) {        // we have found a better dual solution
			primal_res=0;decr_i=0; SLam.clear();
			for(i=0;i<n;i++) old_lambda[i]=lambda[i];
			for(j=0;j<m;j++) {                // compute S(lambda)
				if(fabs(Fc[j])<epsilon) {
					SLam.insert(j); Y[j]=1;
				}
				else Y[j]=0;
			}
			for(i=0;i<n;i++) {                // compute the primal solution
				for(j=0;j<m;j++) X[i][j]=0;
				min_cij=L_INF;
				forall(j,SLam) {
					if(C[i][j]<min_cij) {
						min_cij=C[i][j]; min_j=j;
					}
				}
				X[i][min_j]=1;
			}
			for(j=0;j<m;j++) {
				for(i=0;i<n;i++) {
					primal_res+=C[i][j]*X[i][j];
					if(lambda[i]-C[i][j]>0)
						Mu[i][j]=(Y[j]-X[i][j])*(lambda[i]-C[i][j]);
					else Mu[i][j]=0;
				}
				primal_res+=F[j]*Y[j];
			}
		}
		else {                    // no better dual found, decrease next lambda_i
			decr_i++;
		}

		if((fabs(primal_res-dual_res)<epsilon) || (decr_i==n))
			break;    // optimal sol. found or no further improvment possible

		// decrease a lambda[i] to the next smaller C_ij
		for(i=0;i<n;i++) lambda[i]=old_lambda[i];
		min_cij=0;
		for(j=0;j<m;j++) {
			if((C[decr_i][j]<lambda[decr_i]-epsilon) && (C[decr_i][j]>min_cij))
				min_cij=C[decr_i][j];
		}
		lambda[decr_i]=min_cij;

		// update Fc[i] for the new lambda[decr_i]
		for(j=0;j<m;j++) {
			res=0;
			for(i=0;i<n;i++) {
				if(lambda[i]-C[i][j]>0) res+=lambda[i]-C[i][j];
			}
			Fc[j]=res-F[j];
		}
		i=decr_i+1;
	}while(decr_i<n);

	return dual_res;
}



