#include "lola.h"
#include <LEDA/stream.h>
#include <string.h>
#include <LOLA/lgraphalg.h>
#include <LOLA/facs_util.h>
#include <LOLA/graph_util.h>
#include <LOLA/planealg.h>
#include <LOLA/gaugealg.h>
extern "C" {
# include <stdlib.h>
}


int _Int_all_Restr (char *restrfile) 
/*******************************************************************
 * Purpose   : Returns true if restriction are not disjunct
 * Returns   :
 * Influences: %
 * Date      : Feb 96
 * Author    : Lothar Schnell
 *********************************************************************/
{
	int result=0;
	int restrid;
	point p;
	list<point> intscpnts,pltmp;
	restriction *restr_temp1,*restr_temp2;
	restrictions Restr;
	ifstream filerestr (restrfile,ios::in);
	Restr=(restrictions)ReadRestr(filerestr);
	for(list_item it_1 = Restr.first(); it_1; it_1 = Restr.succ(it_1))
		for(list_item it_2 = Restr.succ(it_1); it_2; it_2 = Restr.succ(it_2)) {
			restr_temp1=Restr.contents(it_1);
			restr_temp2=Restr.contents(it_2);
			restrid=restr_temp2->subtype();
			switch(restrid)
			{
			case restr_p: { 
				restr_poly *restrict = NULL;
				restrict=(restr_poly*)(restr_temp2);
				polygon Ptmp(restrict->vertices());
				pltmp=restr_temp1->intersection(Ptmp);
				forall(p,pltmp) intscpnts.append(p);
				break;
						  }
			case restr_r: { 
				restr_rect *restrict = NULL;
				restrict=(restr_rect*)(restr_temp2);
				polygon Ptmp(restrict->vertices());
				pltmp=restr_temp1->intersection(Ptmp);
				forall(p,pltmp) intscpnts.append(p);
				break;
						  }
			case restr_c: { 
				restr_circle *restrict = NULL;
				restrict=(restr_circle*)(restr_temp2);
				circle Ctmp(restrict->center(),restrict->radius());
				pltmp=restr_temp1->intersection(Ctmp);
				forall(p,pltmp) intscpnts.append(p);
				break;
						  }
			}
		}
	if (intscpnts.size()>0) result=1;
	return result;
}

void _algoch(int choice,const char *locfile,const char *extrafile,const char *gaugefile, const char *outfile, bool win, bool net, bool gauge)
/********************************************************************
* Purpose   : Execute an algorithm from LOLA library
* Returns   : 
* Influences: %
* Date      : Sep 95
* Author    : Lothar Schnell
*********************************************************************/
{ // _algoch
	int p, maxit;
	double delta,epsi;
	double* objval;
	list<vector> objvec;
	bool dir = true;
	string normact;
	matrix w;

	restriction* restrtmp;
	facs_util EX;
	facilities Ex;
	list<string> Loctxt;
	planealg A;
	lgraphalg GrA;
	gaugealg GaA;
	restrictions Restr;
	barrier Barr;
	graph_restriction EL;
	list<edge_segment> Qsol;
	list<polygauge> LG;
	list<int> Ln;
	graph_util tempgraph;
	list<int> belongto;	// nur fuer N-Mediane auf Graphen
	ifstream file (locfile,ios::in);

	
	// the following declaration aerobic avoids ugly dereference operators in
	// the rest of the function and keeps all variables automatic
	ostream *outp;
	ofstream outf;
	
	if (outfile) {
		if (strlen(outfile) == 0) {
			outp = &cout; 
		} else {
			outf.open(outfile);
			if (!outf) {
				lola_error(string("Cannot open %s for output",outfile),1001,1,win);
			} else { outp = &outf; }
		}
	} else {
		outp = &cout; // just to satisfy the definition of out 
	}

	ostream &out = *outp;      
	
	if (!net && !gauge) {
		EX.ReadLoc(file);
	}
	else if (gauge) {
		EX.ReadLoc(file,Ln);
		location loctmp;
		forall(loctmp,EX) Ex.append(loctmp);
	}
	else if (net) {	tempgraph.ReadGraph(file,Ex,Loctxt); }
	if (EX.size()==1) { lola_message("There is only one location given, so this location is optimal",1,win); }

	int Q;
	if (!net) {
		Q=EX.head().wht_dim();
	}
	else
    {
		if (!EX.empty()) Q=Ex.head().wht_dim();
		else Q=tempgraph.inf(tempgraph.first_node()).size();
    }
	objval=new double[Q];

	switch(choice)
/********************************************************************
*  median problems
*********************************************************************/
    { // switch (choice)
    case 0: { 
			objval[0]=A.l1_sum(EX);
			normact="1|P|.|l1|sum";
			break;
            }
    case 1: {
            #ifdef UNIX
			ifstream file ("/tmp/lpinfo",ios::in);
            #else
			string tmp = string(getenv("tmp"));
			tmp += string("/lpinfo"); 
			ifstream file (tmp,ios::in);
            #endif
			if (file) {
				file >> p >> epsi >> delta >> maxit;
			}
			else epsi=0.001;
			objval[0]=A.l2_sum(EX,epsi);
			normact=string ("1|P|.|l2 with eps: %6.3E|sum",epsi);
			break;
            }
    case 2: { objval[0]=A.l2sqr_sum(EX);
			normact="1|P|.|l2sqr|sum";
			break;
            }
    case 3: {
            #ifdef UNIX
			ifstream file ("/tmp/lpinfo",ios::in);
            #else
			string tmp = string(getenv("tmp"));
			tmp += string("/lpinfo"); 
			ifstream file (tmp,ios::in);
            #endif
			file >> p >> epsi >> delta >> maxit;
			objval[0]=A.lp_sum(EX,p,epsi,delta,maxit);
			normact=string ("1|P|.|l%d with eps: %6.3E|sum",p,epsi);
			break;
            }
    case 4: { objval[0]=A.linf_sum(EX);
			normact="1|P|.|linf|sum";
			break;
            }
/********************************************************************
*  center problems
*********************************************************************/
    case 5: { objval[0]=A.l1_max(EX);
			normact="1|P|.|l1|max";
			break;
            }
    case 6: { objval[0]=A.elzhearn(EX);
			normact="1|P|.|l2|max";
			break;
            }
    case 7: { objval[0]=A.linf_max(EX);
			normact="1|P|.|linf|max";
			break;
            } 
    case 8: { objval[0]=A.l1_v1_max(EX);
			normact="1|P|v=1|l1|max";
			break;
            } 
    case 9: { objval[0]=A.linf_v1_max(EX);
			normact="1|P|v=1|l1|max";
			break;
            } 
/********************************************************************
*  q-criterielle median problems
*********************************************************************/
    case 10: {	objval[0]=A.l2sqr_qsum(EX);
				normact="1|P|.|l2sqr|qsum";
				break;
			 } 
    case 11: {	objval[0]=A.l1_2sum(EX);
				normact="1|P|.|l1|2sum";
				break;
			 }
    case 12: {  objval[0]=A.linf_2sum(EX);
                normact="1|P|.|linf|2sum";
                break;
			 }
/********************************************************************
*  N location problems - median
*********************************************************************/
    case 20: {ifstream file (extrafile,ios::in); 
              w=EX.ReadMat(file);
              objval[0]=A.N_l1_sum(EX,Q,w);
			  normact=string("%d|P|.|l1|sum",Q);
              break;
            } 
    case 21: {ifstream file (extrafile,ios::in); 
              w=EX.ReadMat(file);
              objval[0]=A.N_l2sqr_sum(EX,Q,w);
              normact=string("%d|P|.|l2sqr|sum",Q);
              break;
            } 
    case 22: {ifstream file (extrafile,ios::in); 
              w=EX.ReadMat(file);
              objval=A.N_lp_sum_v1(EX,Q,2,w);
              normact=string("%d|P|.|l2|sum",Q);
              break;
            } 
    case 23: {ifstream file (extrafile,ios::in); 
              w=EX.ReadMat(file);
              objval[0]=A.N_linf_sum(EX,Q,w);
              normact=string("%d|P|.|linf|sum",Q);
              break;
            } 
    case 24: {ifstream filemat (extrafile,ios::in); 
              w=EX.ReadMat(filemat);
              #ifdef UNIX
                ifstream file ("/tmp/lpinfo",ios::in);
              #else
                string tmp = string(getenv("tmp"));
                tmp += string("/lpinfo"); 
                ifstream file (tmp,ios::in);
              #endif
              file >> p;
              objval=A.N_lp_sum_v1(EX,Q,p,w);
              normact=string ("%d|P|.|l%d|sum",Q,p);
              break;
            } 
/********************************************************************
*  N location problems - center
*********************************************************************/
    case 40: {ifstream filemat (extrafile,ios::in); 
              w=EX.ReadMat(filemat);
              objval[0]=A.N_l1_max(EX,Q,w);
              normact=string("%d|P|.|l1|max",Q);
              break;
            } 
    case 41: {ifstream filemat (extrafile,ios::in); 
              w=EX.ReadMat(filemat);
              objval[0]=A.N_linf_max(EX,Q,w);
              normact=string("%d|P|.|linf|sum",Q);
              break;
            } 
/********************************************************************
*  restrictive problems - median
*********************************************************************/
    case 50: {ifstream filerestr (extrafile,ios::in);
              Restr=ReadRestr(filerestr);
              objval[0]=A.l2sqr_sum(EX,Restr);
              normact="1|P|restr|l2sqr|sum";
              break;
            }
    case 51: {ifstream filerestr (extrafile,ios::in);
              Restr=ReadRestr(filerestr);
              objval[0]=A.l1_sum(EX,Restr);
              normact="1|P|restr|l1|sum";
              break;
            }
    case 52: {ifstream filerestr (extrafile,ios::in);
              Restr=ReadRestr(filerestr);
              objval[0]=A.linf_sum(EX,Restr);
              normact="1|P|restr|linf|sum";
              break;
            } 
    case 53: {ifstream filerestr (extrafile,ios::in);
              Restr=ReadRestr(filerestr);
              #ifdef UNIX
                ifstream file ("/tmp/lpinfo",ios::in);
              #else
                string tmp = string(getenv("tmp"));
                tmp += string("/lpinfo"); 
                ifstream file (tmp,ios::in);
              #endif
              file >> p >> epsi >> delta >> maxit;
              objval[0]=A.lp_sum(EX,Restr,p,epsi,delta,maxit);
              normact=string ("1|P|restr|l%d with eps: %6.3E|sum",p,epsi);
              break;
            }
/********************************************************************
*  restrictive problems - median (restriction outside)
*********************************************************************/
    case 55: {ifstream filerestr (extrafile,ios::in);
              Restr=ReadRestr(filerestr);
              forall(restrtmp,Restr) restrtmp->out_forbid();
              objval[0]=A.l2sqr_sum(EX,Restr);
              normact="1|P|restr(out)|l2sqr|sum";
              break;
            }
    case 56: {ifstream filerestr (extrafile,ios::in);
              Restr=ReadRestr(filerestr);
              forall(restrtmp,Restr) restrtmp->out_forbid();
              objval[0]=A.l1_sum(EX,Restr);
              normact="1|P|restr(out)|l1|sum";
              break;
            }
    case 57: {ifstream filerestr (extrafile,ios::in);
              Restr=ReadRestr(filerestr);
              forall(restrtmp,Restr) restrtmp->out_forbid();
              objval[0]=A.linf_sum(EX,Restr);
              normact="1|P|restr(out)|linf|sum";
              break;
            }
/********************************************************************
*  restrictive problems - center
*********************************************************************/
    case 60: {ifstream filerestr (extrafile,ios::in);
              Restr=ReadRestr(filerestr);
              objval[0]=A.l2_max(EX,Restr);
              normact="1|P|restr|l2|max";
              break;
            }
    case 61: {ifstream filerestr (extrafile,ios::in);
              Restr=ReadRestr(filerestr);
              objval[0]=A.linf_max(EX,Restr);
              normact="1|P|restr|linf|max";
              break;
            }
    case 64: {ifstream filerestr (extrafile,ios::in);
              Restr=ReadRestr(filerestr);
              objval[0]=A.l1_max(EX,Restr);
              normact="1|P|restr|l1|max";
              break;
            }
/********************************************************************
*  restrictive problems - center  (restriction outside)
*********************************************************************/
    /*case 62: {ifstream filerestr (extrafile,ios::in);
              Restr=ReadRestr(filerestr);
              forall(restrtmp,Restr) restrtmp->out_forbid();
              objval[0]=A.l2_max(EX,Restr);
              normact="1|P|restr(out)|l2|max";
              break;
            }
    case 63: {ifstream filerestr (extrafile,ios::in);
              Restr=ReadRestr(filerestr);
              forall(restrtmp,Restr) restrtmp->out_forbid();
              objval[0]=A.linf_max(EX,Restr);
              normact="linf";
              break;
            }
    case 65: {ifstream filerestr (extrafile,ios::in);
              Restr=ReadRestr(filerestr);
              forall(restrtmp,Restr) restrtmp->out_forbid();
              objval[0]=A.l1_max(EX,Restr);
              normact="l1";
              break;
            }*/
/********************************************************************
*  restrictive problems - barrier 
*********************************************************************/
  case 70: { ifstream filerestr (extrafile,ios::in);
			 Restr=ReadRestr(filerestr);
             Barr.append(Restr.head());
             objval[0]=A.l2_sum(EX,Barr);
             normact="1|P|barr|l2|sum";
             break;
           }
/********************************************************************
*  network problems - median 
*********************************************************************/
    case 100: { loladirected G=tempgraph;
				objval[0]=GrA.median(G);
				normact="1|G|.|d(V,V)|sum";
				break;
            }
    case 102: { dir = false;
				lolaundirected U(tempgraph);
                objval[0]=GrA.median(U);
				normact="1|G|.|d(V,V)|sum";
                break;
            }
    case 103: { dir = false;
				lolaundirected U=tempgraph;
                objval[0]=GrA.abs_median(U);
				normact="1|G|.|d(V,G)|sum";
                break;
            }
    case 104: { dir = false;
				lolaundirected U=tempgraph;
				lolatree T=U;
                objval[0]=GrA.loc_tree_median(T);
				normact="1|Tree|.|d(V,V)|sum";
                break;
            }
    case 105: { dir = false;
				lolaundirected U=tempgraph;
				lolatree T=U;
                objval[0]=GrA.abs_tree_median(T);
				normact="1|Tree|.|d(V,T)|sum";
                break;
            }
/********************************************************************
*  network problems - center
*********************************************************************/
    case 110: { loladirected G=tempgraph;
				objval[0]=GrA.center(G);
				normact="1|G|.|d(V,V)|max";
                break;
            }
    case 111: { dir = false;
				lolaundirected U=tempgraph;
                objval[0]=GrA.abs_center(U);
				normact="1|G|.|d(V,G)|max";
                break;
            }
    case 112: { dir = false;
				lolaundirected U=tempgraph;
				lolatree T=U;
                objval[0]=GrA.loc_tree_center(T);
				normact="1|Tree|.|d(V,V)|max";
                break;
            }
    case 113: { dir = false;
				lolaundirected U=tempgraph;
				lolatree T=U;
                objval[0]=GrA.abs_tree_center(T);
				normact="1|Tree|.|d(V,T)|max";
                break;
            }
    case 114: { dir = false;
				lolaundirected U=tempgraph;
				lolatree T=U;
                objval[0]=GrA.tree_two_center(T);
				normact="2|Tree|.|d(V,V)|max";
                break;
            }
    case 115: { 
                loladirected G=tempgraph;
                objval[0]=GrA.abs_center(G);
				normact="1|G|.|d(V,G)|max";
                break;
            }

    case 116: { lolaundirected U=tempgraph;
				dir=false;
				objval[0]=GrA.center(U);
				normact="1|G|.|d(V,V)|max";
                break;
            }
/********************************************************************
*  network problems - N-facility-problems 
*********************************************************************/
    case 120: { lolaundirected U=tempgraph;
				dir=false;
				int n=atoi(extrafile);	
				objval[0]=GrA.N_median_cplex(U,n,belongto);
				normact=string("%d|G|.|d(V,V)|sum: exact",n);
                break;
            }
    case 121: { lolaundirected U=tempgraph;
                dir=false;
                int n=atoi(extrafile);
                objval[0]=GrA.N_median_partitioning(U,n,belongto);
				normact=string ("%d|G|.|d(V,V)|sum: Partitioning",n);
                break;
            }
    case 122: { lolaundirected U=tempgraph;
                dir=false;
                int n=atoi(extrafile);
                objval[0]=GrA.N_median_exchange(U,n,belongto);
				normact=string ("%d|G|.|d(V,V)|sum: Exchange",n);
                break;
            }
    case 123: { lolaundirected U=tempgraph;
                dir=false;
                int n=atoi(extrafile);
                objval[0]=GrA.N_median_greedy(U,n,belongto);
				normact=string ("%d|G|.|d(V,V)|sum: Greedy",n);
                break;
            }
    case 131: { lolaundirected U=tempgraph;
                dir=false;
                int n=atoi(extrafile);
                objval[0]=GrA.N_center_partitioning(U,n,belongto);
				normact=string ("%d|G|.|d(V,V)|max: Partitioning",n);
                break;
            }
    case 133: { lolaundirected U=tempgraph;
                dir=false;
                int n=atoi(extrafile);
                objval[0]=GrA.N_center_greedy(U,n,belongto);
				normact=string ("%d|G|.|d(V,V)|max: Greedy",n);
                break;
            }
/********************************************************************
*  network problems -  multicriterial - Pareto 
*********************************************************************/
    case 140: {lolaundirected U=tempgraph;
		dir=false;
		int nofweights;
		nofweights=U.inf(U.first_node()).length();
		if (nofweights==2) {
			objvec=GrA.medPareto_bi(U,Qsol);
			normact=string ("1|G|.|d(V,G)|2-sum-par");
			}else {
			objvec=GrA.medPareto_Q(U,Qsol);
			normact=string ("1|G|.|d(V,G)|Q-sum-par");
			}
		break;
	   }
     case 141: {lolaundirected U=tempgraph;
		dir=false;
		objvec=GrA.loc_medPareto(U,Qsol);
		normact=string ("1|G|.|d(V,V)|Q-sum-par");
		break;
	   }
     case 142: {lolaundirected U=tempgraph;
		dir=false;
		objvec=GrA.loc_cenPareto(U,Qsol);
		normact=string ("1|G|.|d(V,V)|Q-max-par");
		break;}
     case 145: {loladirected G=tempgraph;
		objvec=GrA.medPareto(G,Qsol);
		normact=string ("1|G_D|.|d(V,G)|Q-sum-par");
		break;
	   }
     case 146: {loladirected G=tempgraph;
		objvec=GrA.loc_medPareto(G,Qsol);
		normact=string ("1|G_D|.|d(V,V)|Q-sum-par");
		break;
	   }
     case 147: {loladirected G=tempgraph;
		objvec=GrA.loc_cenPareto(G,Qsol);
		normact=string ("1|G_D|.|d(V,V)|Q-max-par");
		break;
	   }
     case 148: {loladirected G=tempgraph;
		objvec=GrA.cenPareto(G,Qsol);
		normact=string ("1|G_D|.|d(V,G)|Q-max-par");
		break;
	   }
/********************************************************************
*  network problems -  multicriterial - Lexikographical 
*********************************************************************/
    case 150: {lolaundirected U=tempgraph;
                dir=false;
                objvec=GrA.medLexi(U,Qsol);
                normact=string ("1|G|.|d(V,G)|Q-sum-lex");
                break;
           }
     case 151: {lolaundirected U=tempgraph;
                dir=false;
                objvec=GrA.loc_medLexi(U,Qsol);
                normact=string ("1|G|.|d(V,V)|Q-sum-lex");
                break;
           }
     case 152: {lolaundirected U=tempgraph;
                dir=false;
                objvec=GrA.loc_cenLexi(U,Qsol);
                normact=string ("1|G|.|d(V,V)|Q-max-lex");
                break;}
     case 155: {loladirected G=tempgraph;
                objvec=GrA.medLexi(G,Qsol);
                normact=string ("1|G_D|.|d(V,G)|Q-sum-lex");
                break;
           }
     case 156: {loladirected G=tempgraph;
                objvec=GrA.loc_medLexi(G,Qsol);
                normact=string ("1|G_D|.|d(V,V)|Q-sum-lex");
                break;
           }
     case 157: {loladirected G=tempgraph;
                objvec=GrA.loc_cenLexi(G,Qsol);
                normact=string ("1|G_D|.|d(V,V)|Q-max-lex");
                break;
           }
/********************************************************************
*  network problems - median - restrictive
*********************************************************************/
/*    case 160: { ifstream filerestr (extrafile,ios::in);
                EL.ReadGRestr(filerestr);
				loladirected G=tempgraph;
                objval[0]=GrA.median(G,EL);
                break;
            }
    case 161: { ifstream filerestr (extrafile,ios::in);
                EL.ReadGRestr(filerestr);
				dir = false;
				lolaundirected G=tempgraph;
                objval[0]=GrA.median(G,EL);
                break;
            }
    case 162: { ifstream filerestr (extrafile,ios::in);
                EL.ReadGRestr(filerestr);
                dir = false;
				lolaundirected U=tempgraph;
                objval[0]=GrA.abs_median(U,EL);
                break;
            }*/
/********************************************************************
*  median problems with gauge
*********************************************************************/
    case 200: {ifstream filegauge (gaugefile,ios::in);
              LG=ReadLPGauge(filegauge);
              objval[0]=GaA.sum(LG,Ln,Ex);
              normact="1|P|.|gauges|sum";
              break;
            }
    case 201: {ifstream filegauge (gaugefile,ios::in);
              LG=ReadLPGauge(filegauge);
              objval[0]=GaA.bi_crit_sum(LG,Ln,Ex);
			  normact="1|P|.|gauges|qsum";
              break;
              }
/********************************************************************
*  center problems with gauge
*********************************************************************/
    case 210: {ifstream filegauge (gaugefile,ios::in);
				LG=ReadLPGauge(filegauge);
				objval[0]=GaA.max(LG,Ln,Ex);
				normact="1|P|.|gauges|max";
				break;
              }
	case 211: {ifstream filegauge (gaugefile,ios::in);
				LG=ReadLPGauge(filegauge);
				ifstream filerestr (extrafile,ios::in);
				Restr=ReadRestr(filerestr);
				objval[0]=GaA.max(LG,Ln,Ex,Restr);
				normact="1|P|R|gauges|max";
				break;
              }
/********************************************************************
* One line median problems
*********************************************************************/
    case 300: {
      objval[0]=A.L_l2_sum_M3(EX);
      normact="1L|P|.|l2|sum: M3";
      break;
    }
    case 301: {
      objval[0]=A.L_l1_sum_M3(EX);
      normact="1L|P|.|l1|sum";
      break;
    }
    case 302: {
      objval[0]=A.L_linf_sum_M3(EX);
      normact="1L|P|.|linf|sum";
      break;
    }
    case 303: {
#ifdef UNIX
      ifstream file ("/tmp/lpinfo",ios::in);
#else
      string tmp = string(getenv("tmp"));
      tmp += string("/lpinfo"); 
      ifstream file (tmp,ios::in);
#endif
      file >> p >> epsi >> delta >> maxit;
      objval[0]=A.L_lp_sum_M3(EX,p);
      normact=string ("1L|P|.|l%d|sum",p);
      break;
    }
    case 304: {
      objval[0]=A.L_l2_sum_M2logM(EX);
      normact="1L|P|.|l2|sum: M2logM";
      break;
    }
    case 305: {
      objval[0]=A.L_l2_sum_M2(EX);
      normact="1L|P|v=1|l2|sum";
      break;
    }
    case 306: {
      ifstream filegauge (gaugefile,ios::in);
      LG=ReadLPGauge(filegauge);
      objval[0]=GaA.L_block_sum_M3(LG,Ln,Ex);
      normact="1L|P|.|block|sum";
		  break;
    }
    case 307: {
      ifstream filerestr (extrafile,ios::in);
      Restr=ReadRestr(filerestr);
      objval[0]=A.L_RkonvPoly_l2_sum(EX,Restr);
      normact="1L|P|restr|l2|sum";
      break;
    }
    case 308: {
#ifdef UNIX
      ifstream file ("/tmp/lpinfo",ios::in);
#else
      string tmp = string(getenv("tmp"));
      tmp += string("/lpinfo"); 
      ifstream file (tmp,ios::in);
#endif
      file >> p >> epsi >> delta >> maxit;
      ifstream filerestr (extrafile,ios::in);
      Restr=ReadRestr(filerestr);
      objval[0]=A.L_RkonvPoly_lp_sum(EX,Restr,p);
      normact=string ("1L|P|restr|l%d|sum",p);
      break;
    }
    case 309: {
      ifstream filerestr (extrafile,ios::in);
      Restr=ReadRestr(filerestr);
      objval[0]=A.L_RkonvPoly_l1_sum(EX,Restr);
      normact="1L|P|restr|l1|sum";
      break;
    }
    case 310: {
      ifstream filerestr (extrafile,ios::in);
      Restr=ReadRestr(filerestr);
      objval[0]=A.L_RkonvPoly_linf_sum(EX,Restr);
      normact="1L|P|restr|linf|sum";
      break;
    }
    case 311: {
	  ifstream filerestr (extrafile,ios::in);
      Restr=ReadRestr(filerestr);
      ifstream filegauge (gaugefile,ios::in);
      LG=ReadLPGauge(filegauge);
      objval[0]=GaA.L_RkonvPoly_block_sum(LG,Ln,EX,Restr);
      normact="1L|P|restr|block|sum";
      break;
    }
/********************************************************************
* One line center problems
*********************************************************************/

    case 350: {
      objval[0]=A.L_l2_max_M4(EX);
      normact="1L|P|.|l2|max: M4";
      break;
    }
    case 351: {
      objval[0]=A.L_l1_max_M4(EX);
      normact="1L|P|.|l1|max";
      break;
    }
    case 352: {
      objval[0]=A.L_linf_max_M4(EX);
      normact="1L|P|.|linf|max";
      break;
    }
    case 353: {
      objval[0]=A.L_l2_max_MlogM(EX);
      normact="1L|P|v=1|l2|max";
      break;
    }
    case 354: {
      objval[0]=A.L_l2_max_M2logM(EX);
      normact="1L|P|.|l2|max: M2logM";
      break;
    }
    case 355: {
#ifdef UNIX
      ifstream file ("/tmp/lpinfo",ios::in);
#else
      string tmp = string(getenv("tmp"));
      tmp += string("/lpinfo"); 
      ifstream file (tmp,ios::in);
#endif
      file >> p >> epsi >> delta >> maxit;
      objval[0]=A.L_lp_max_M4(EX,p);
      normact=string ("1L|P|.|l%d|sum",p);
      break;
    }
    case 356: {
      ifstream filegauge (gaugefile,ios::in);
      LG=ReadLPGauge(filegauge);
      objval[0]=GaA.L_block_max_M4(LG,Ln,Ex);
      normact="1L|P|.|block|sum";
      break;
    }
    }
  // provide graphical solution display
  if (win) {
    cout<<"Open Window"<<endl<<flush;
    if (net) { 
		if(choice>=160 || choice <140) {
	    list<sol_typ> ASol = GrA.alg_solution();
			tempgraph.LGraphView(objval[0],normact,ASol,Ex,Loctxt,EL,dir,belongto); 
		}
	  else
		tempgraph.QGraphView(objvec,normact,Ex,Loctxt,Qsol,dir);
	} else {
	    list<location> AlgSol = A.alg_solution();
		if (gauge) AlgSol = GaA.alg_solution(); 
		if (choice < 300) EX.View(objval[0],normact,AlgSol,Restr,LG,Ln);
		else EX.View(objval[0],normact,AlgSol,Restr,LG,Ln,choice);
    }
  }
  // provide textual solution
  if (outfile) {
   out << "begin {result}" << endl;
   out << "begin {classification}" << endl << normact << endl;
   out << "end {classification}" << endl;
   if (net && (choice >=150 || choice <140)) {
     out << "begin {objective value} " << endl;
     out << string ("%13.3lf",objval[0]) << endl << "end {objective value}" << endl;
     GrA.WriteOpt(out);
   } else if (net && choice<150 && choice >=140) {
     out << " " ; // noch zu tun 
   } else if (gauge) {
     out << "begin {situations}" << endl << objval[0] << endl << "end {situations}" << endl;
     GaA.WriteOpt(out);
   } else {
     out << "begin {objective value} " << endl;
     out << string ("%13.3lf",objval[0]) << endl << "end {objective value}" << endl;
     A.WriteOpt(out);
   }  
   out << "end {result}" << endl;
  } // if (outfile)
} // _algoch  

void press_ret(void)
{
  cout<<"[return]"<<flush;
  getchar();
}

void usage() {
/********************************************************************
* Purpose   : print out command-line help
* Returns   : never (exit(0))
* Influences: main
* Date      : Jul 97
* Author    : Martin C. Mueller (Lothar Schnell)
* TODO      : pager
*********************************************************************/
  string help("\n\
  lola -a <algorithm> [-l|-r|-g|-d|-m <file>] [-p <n>] [-n <n>] [--output=<arguments>]\n\
\n\
Options:\n\
\n\
   -a : the algorithm, which is to be performed on the data (see below)\n\
   -l : <file> contains the existing facilities\n\
   -r : <file> contains restrictions\n\
   -g : <file> contains a (directed or undirected) graph\n\
   -d : <file> contains polygonal gauge definitions\n\
   -m : <file> contains a cost matrix for n-facility problems\n\
   -p : <n> is the exponent for an lp-norm\n\
   -n : <n> is the number of new facilities for problems on graphs\n\
   --output:\n\
        this options takes a comma-seperated list of arguments:\n\
        [no]windowed : [dont] present the solution graphically\n\
        [file=]<file>: write solution as text into <file>\n\
        only the rightmost argument of each type takes effect\n\
\n\
The following table lists all feasible algorithm/data combinations:\n\
\n\
\n\
\n\
\n\
");
cout << help;
press_ret();
help=string("\n\
-a <algorithm>                       | -l| -m| -r| -p| -g| -d| -n|\n\
-------------------------------------------------------------------\n\
l1_max, l2_max, linf_max,             | X |   |   |   |   |   |   |\n\
l1_v1_max, linf_v1_max, l2sqr_qsum    | X |   |   |   |   |   |   |\n\
l1_sum, l2_sum, l2sqr_sum, linf_sum   | X |   |   |   |   |   |   |\n\
l1_2sum, linf_2sum                    | X |   |   |   |   |   |   |\n\
-------------------------------------------------------------------\n\
lp_sum                                | X |   | O | X |   |   |   |\n\
-------------------------------------------------------------------\n\
in_l1_sum, out_l1_sum, in_l2sqr_sum,  | X |   | X |   |   |   |   |\n\
out_l2sqr_sum, in_linf_sum,           | X |   | X |   |   |   |   |\n\
out_linf_sum, in_l2_max,              | X |   | X |   |   |   |   |\n\
in_convex_linf_max, in_convex_l1_max  | X |   | X |   |   |   |   |\n\
barr_l2_sum                           | X |   | X |   |   |   |   |\n\
-------------------------------------------------------------------\n\
N_l1_sum_v1, N_l2sqr_sum, N_l2_sum_v1,| X | X |   |   |   |   |   |\n\
N_linf_sum_v1, N_l1_max, N_linf_max   | X | X |   |   |   |   |   |\n\
-------------------------------------------------------------------\n\
dir_median, undir_median,             |   |   |   |   | X |   |   |\n\
abs_undir_median                      |   |   |   |   | X |   |   |\n\
abs_tree_median, abs_undir_center,    |   |   |   |   | X |   |   |\n\
abs_dir_center,loc_tree_center,       |   |   |   |   | X |   |   |\n\
abs_tree_center,loc_tree_median       |   |   |   |   | X |   |   |\n\
loc_dir_center,loc_undir_center       |   |   |   |   | X |   |   |\n\
");
cout << help;
press_ret();
help=string("\n\
-------------------------------------------------------------------\n\
N_median_cplex, N_median_partitioning,|   |   |   |   | X |   | X |\n\
N_median_exchange, N_median_greedy,   |   |   |   |   | X |   | X |\n\
N_center_partitioning, N_center_greedy|   |   |   |   | X |   | X |\n\
-------------------------------------------------------------------\n\
medPareto,loc_medPareto,loc_cenPareto,|   |   |   |   | X |   |   |\n\
dir_medPareto, dir_loc_medPareto,     |   |   |   |   | X |   |   |\n\
dir_loc_cenPareto, dir_cenPareto      |   |   |   |   | X |   |   |\n\
-------------------------------------------------------------------\n\
medLexi,loc_medLexi,loc_cenLexi,      |   |   |   |   | X |   |   |\n\
dir_medLexi, dir_loc_medLexi,         |   |   |   |   | X |   |   |\n\
dir_loc_cenLexi,                      |   |   |   |   | X |   |   |\n\
-------------------------------------------------------------------\n\
gauge_median, bi_crit_gauge_median    | X |   |   |   |   | X |   |\n\
gauge_center                          | X |   |   |   |   | X |   |\n\
-------------------------------------------------------------------\n\
restr_gauge_center                    | X |   | X |   |   | X |   |\n\
-------------------------------------------------------------------\n\
L_l1_sum_M3, L_linf_sum_M3,           | X |   |   |   |   |   |   |\n\
L_l2_sum_M3, L_l2_sum_M2logM,         | X |   |   |   |   |   |   |\n\
L_l2_sum_M2,                          | X |   |   |   |   |   |   |\n\
L_lp_sum_M3,                          | X |   |   | X |   |   |   |\n\
L_block_sum_M3                        | X |   |   |   |   | X |   |\n\
L_l1_max_M4, L_linf_max_M4,           | X |   |   |   |   |   |   |\n\
");
cout << help;
press_ret();
help=string("\n\
L_l2_max_M4, L_l2_max_MlogM,          | X |   |   |   |   |   |   |\n\
L_l2_max_M2logM,                      | X |   |   |   |   |   |   |\n\
L_lp_max_M4,                          | X |   |   | X |   |   |   |\n\
L_block_max_M3                        | X |   |   |   |   | X |   |\n\
L_RkonvPoly_l2_sum, L_RkonvPoly_l1_sum| X |   | X |   |   |   |   |\n\
L_RkonvPoly_lp_sum                    | X |   | X | X |   |   |   |\n\
L_RkonvPoly_linf_sum                  | X |   | X |   |   |   |   |\n\
L_RkonvPoly_block_sum                 | X |   | X |   |   | X |   |\n\
-------------------------------------------------------------------\n\
\n\
X = mandatory , O = optional\n\
\n\
\n\
Example:  ./lola.exe -a l1_sum -l mydata.loc\n\
\n\
");

#ifdef WITHTCLTK
  help = help + "\n\
       lola.exe\n\
without options calls the graphical frontend using Tcl/Tk.\n\
";
#endif
  help = help + "\n\
	For solving the UFL you can call discrete.exe in the bin \n\
	directory without parameters. \n\
";

  cout << help;
  exit(0);
} // void usage()

int main(int argc, char **argv)
/********************************************************************
* Purpose   : MAIN - Calls lolatk that evaluates the TK-Scripts
*             or _algoch for text-based operation
* Returns   : 
* Influences: %
* Date      : Sep 95
* Author    : Lothar Schnell
*********************************************************************/
{
	int i;
	bool net=false,gauge=false;
	int maxit=50;
	double eps=0.0001, delta=0.0001;
	bool help=false, loc=false, extra=false, mat=false, algo=false, pflag=false;
	bool nflag=false;
#ifdef WITHTCLTK
	bool win=true;
	LOLA_ERROR::windowed = true;
#else
	bool win=false;
	LOLA_ERROR::windowed = false;
#endif
	const char* locc;
	const char* extrac;
	const char* gaugec;
	const char* ptxtc;
	const char* outc = ""; // C-string version von outfile
	string locfile, extrafile, gaugefile, algotxt, ptxt, ntxt;
	string outfile; // name of output file, empty means cout
	string argstring, arghead;
	string separators = ":="; // permissible separators for options (space is also allowed but handled otherwise)
	string has_argument = "algrmdpn"; // one-letter options which take an argument
	if (argc == 1) {
#ifdef WITHTCLTK
		_lolatk(argc,argv);
#else
		usage();
#endif
		return 0;
	} else { // process options
		for (i = 1; i < argc; ++i) {
			argstring = string(argv[i]);
			arghead = argstring.head(2);
			// allow for seperators after single-letter option
			if ((has_argument.pos(argstring(1,1)) > -1) &&
				(argstring.length() > 2) &&
				(separators.pos(argstring(2,2)) > -1)) {
				argstring = argstring.del(2,2);
			}
			// allow for space after single-letter option
			if ((argstring.length() == 2) && (argstring[0] == '-') && 
				(has_argument.pos(string(argstring[1])) > -1) &&
				(i < argc - 1)) {
				if (*(argv[i+1]) == '-') { // another option following?
					lola_error("Missing value for option "+argstring,1002,1,win);
				}
				// OK, just append the next argument
				argstring = argstring + string(argv[++i]);
			}
			if (argstring == "-help" || argstring == "--help" || argstring == "-?" || argstring == "-h") {
				usage();
			} else if (arghead == "--") { // double-hyphen options
				int pos; // scratch space for positions in string
				string argstringsave = argstring;
				
				if (argstring.pos("output") == 2) { // output-option
					argstring = argstring.del(0,7); // delete starting "--output"
					if (argstring.length() == 0) { // maybe the next argv
						if ((i >= argc - 1) || 
							((argstring = string(argv[++i]))[0] == '-')) {
							lola_error("Missing value for option --output.",1003,1,win);
							exit(1);
						}
					} else { // option-argument follows directly
						if (separators.pos(argstring[0]) > -1) {
							argstring = argstring.del(0,0);
						} else {
							lola_error("Option "+argstringsave+" is not a valid --output option.",1004,1,win);	   
						}
					}
					// now eat the options one by one
					while (argstring.length() > 0) {
						string arg;
						
						if ((pos = argstring.pos(",")) > -1) {
							if (pos == 0) { // ignore empty option
								argstring = argstring.del(0,pos);
								continue;
							}
							arg = argstring(0,pos - 1);
							argstring = argstring.del(0,pos);
						} else {
							arg = argstring;
							argstring = "";
						}
						if (arg == "windowed") {
							LOLA_ERROR::windowed = true;
							win = true;
							continue;
						}
						if (arg == "nowindowed") {
							LOLA_ERROR::windowed = false;
							win = false;
							continue;
						}
						if (arg.pos("file=") == 0) {
							outfile=arg.del(0,4);
							continue;
						}
						outfile = arg;
					}
				} // processing option "output"
				if (outfile.length() > 0) {
					outc = outfile;
				} else {
					outc = ""; // output to cout
				}
			} // processing double hyphen options
			else if (arghead == "-l" && !loc) { // location file
				locfile = argstring(2,argstring.length()-1);
				locc=locfile;
				loc = true;
			} else if (arghead == "-g" && !loc) { // graph file
				locfile = argstring(2,argstring.length()-1);
				locc=locfile;
				loc = true;
				net = true;
			} else if (arghead == "-r" && !extra) { // restriction file
				extrafile = argstring(2,argstring.length()-1);
				extrac=extrafile;
				extra = true;
			} else if (arghead == "-m" && !mat) { // matrix file
				extrafile = argstring(2,argstring.length()-1);
				extrac=extrafile;
				extra = true;
				mat = true;
			} else if (arghead == "-d" && !gauge) { // polygauge file
				gaugefile = argstring(2,argstring.length()-1);
				gaugec=gaugefile;
				gauge = true;
			} else if (arghead == "-p" && !pflag) { // lp-distance exponent
				ptxt = argstring(2,argstring.length()-1);
				ptxtc=ptxt;
				int p = atoi(ptxtc);
#ifdef UNIX
				ofstream file ("/tmp/lpinfo",ios::out);
#else
				string tmp = string(getenv("tmp"));
				tmp += string("/lpinfo"); 
				ofstream file (tmp,ios::out);
#endif
				file << p << " " << eps << " " << delta << " " << maxit;
				pflag = true;
			} else if (arghead == "-n" && !nflag) { // number of new facilities
				ntxt = argstring(2,argstring.length()-1);
				extrac = ntxt;
				nflag = true;
			} else if (arghead == "-a" && !algo) { // algorithm
				algotxt = argstring(2,argstring.length()-1);
				algo = true;
			} else {
				lola_error("Unrecognized option "+argstring,1005,1,win);
			}
		}
		int choice=999;
		if (!algo || !loc) {
			lola_error("You have to specify an algorithm and a location file.",1006,1,win);
		} else if (!extra && !gauge) {
			if (strcmp(algotxt, "l1_sum") == 0) choice=0;
			else if (strcmp(algotxt, "l2_sum") == 0) choice=1;
			else if (strcmp(algotxt, "l2sqr_sum") == 0) choice=2;
			else if (strcmp(algotxt, "lp_sum") == 0) choice=3;
			else if (strcmp(algotxt, "linf_sum") == 0) choice=4;
			else if (strcmp(algotxt, "l1_max") == 0) choice=5;
			else if (strcmp(algotxt, "l2_max") == 0) choice=6;
			else if (strcmp(algotxt, "linf_max") == 0) choice=7;
			else if (strcmp(algotxt, "l1_v1_max") == 0) choice=8;
			else if (strcmp(algotxt, "linf_v1_max") == 0) choice=9;
			else if (strcmp(algotxt, "l2sqr_qsum") == 0) choice=10;
			else if (strcmp(algotxt, "l1_2sum") == 0) choice=11;
			else if (strcmp(algotxt, "linf_2sum") == 0) choice=12;
			else if (strcmp(algotxt, "dir_median") == 0) choice=100;
			else if (strcmp(algotxt, "undir_median") == 0) choice=102;
			else if (strcmp(algotxt, "abs_undir_median") == 0) choice=103;
			else if (strcmp(algotxt, "loc_tree_median") == 0) choice=104;
			else if (strcmp(algotxt, "abs_tree_median") == 0) choice=105;
			else if (strcmp(algotxt, "loc_dir_center") == 0) choice=110;
			else if (strcmp(algotxt, "abs_undir_center") == 0) choice=111;
			else if (strcmp(algotxt, "loc_tree_center") == 0) choice=112;
			else if (strcmp(algotxt, "abs_tree_center") == 0) choice=113;
//			else if (strcmp(algotxt, "2_tree_center") == 0) choice=114;
			else if (strcmp(algotxt, "abs_dir_center") == 0) choice=115;
			else if (strcmp(algotxt, "loc_undir_center") == 0) choice=116;
			else if (strcmp(algotxt, "N_median_cplex") == 0) choice=120;
			else if (strcmp(algotxt, "N_median_partitioning") == 0) choice=121;
			else if (strcmp(algotxt, "N_median_exchange") == 0) choice=122;
			else if (strcmp(algotxt, "N_median_greedy") == 0) choice=123;
			else if (strcmp(algotxt, "N_center_partitioning") == 0) choice=131;
			else if (strcmp(algotxt, "N_center_greedy") == 0) choice=133;
			else if (strcmp(algotxt, "medPareto") == 0) choice=140;
			else if (strcmp(algotxt, "loc_medPareto") == 0) choice=141;
			else if (strcmp(algotxt, "loc_cenPareto") == 0) choice=142;
			else if (strcmp(algotxt, "dir_medPareto") == 0) choice=145;
			else if (strcmp(algotxt, "dir_loc_medPareto") == 0) choice=146;
			else if (strcmp(algotxt, "dir_loc_cenPareto") == 0) choice=147;
			else if (strcmp(algotxt, "dir_cenPareto") == 0) choice=148;
			else if (strcmp(algotxt, "medLexi") == 0) choice=150;	  
			else if (strcmp(algotxt, "loc_medLexi") == 0) choice=151;
			else if (strcmp(algotxt, "loc_cenLexi") == 0) choice=152;
			else if (strcmp(algotxt, "dir_medLexi") == 0) choice=155;
			else if (strcmp(algotxt, "dir_loc_medLexi") == 0) choice=156;
			else if (strcmp(algotxt, "dir_loc_cenLexi") == 0) choice=157;
			else if (strcmp(algotxt, "L_l2_sum_M3") == 0) choice=300;
			else if (strcmp(algotxt, "L_l1_sum_M3") == 0) choice=301;
			else if (strcmp(algotxt, "L_linf_sum_M3") == 0) choice=302;
			else if (strcmp(algotxt, "L_lp_sum_M3") == 0) choice=303;
			else if (strcmp(algotxt, "L_l2_sum_M2logM") == 0) choice=304;
			else if (strcmp(algotxt, "L_l2_sum_M2") == 0) choice=305;
			else if (strcmp(algotxt, "L_l2_max_M4") == 0) choice=350;
			else if (strcmp(algotxt, "L_l1_max_M4") == 0) choice=351;
			else if (strcmp(algotxt, "L_linf_max_M4") == 0) choice=352;
			else if (strcmp(algotxt, "L_l2_max_MlogM") == 0) choice=353;
			else if (strcmp(algotxt, "L_l2_max_M2logM") == 0) choice=354;
			else if (strcmp(algotxt, "L_lp_max_M4") == 0) choice=355;
			else lola_error("Illegal algorithm or missing data.",1007,1,win);
		} else if (extra && mat && !gauge) {
			if (strcmp(algotxt, "N_l1_sum_v1") == 0) choice=20;
			else if (strcmp(algotxt, "N_l2sqr_sum") == 0) choice=21;
			else if (strcmp(algotxt, "N_l2_sum_v1") == 0) choice=22;
			else if (strcmp(algotxt, "N_linf_sum_v1") == 0) choice=23;
			else if (strcmp(algotxt, "N_lp_sum_v1") == 0) choice=24;
			else if (strcmp(algotxt, "N_l1_max") == 0) choice=40;
			else if (strcmp(algotxt, "N_linf_max") == 0) choice=41;
			else lola_error("Illegal algorithm or missing data.",1008,1,win);
		} else if (extra && !mat && !gauge) {
			if (strcmp(algotxt, "in_l1_sum") == 0) choice=51;
			else if (strcmp(algotxt, "out_l1_sum") == 0) choice=56;
			else if (strcmp(algotxt, "in_linf_sum") == 0) choice=52;
			else if (strcmp(algotxt, "out_linf_sum") == 0) choice=57;
			else if (strcmp(algotxt, "lp_sum") == 0) choice=53;
			else if (strcmp(algotxt, "in_l2sqr_sum") == 0) choice=50;
			else if (strcmp(algotxt, "out_l2sqr_sum") == 0) choice=55;
			else if (strcmp(algotxt, "in_l2_max") == 0) choice=60;
			//     else if (strcmp(algotxt, "out_convPoly_v1_l2_max") == 0) choice=62;
			else if (strcmp(algotxt, "in_convex_linf_max") == 0) choice=61;
			//     else if (strcmp(algotxt, "out_convex_linf_max") == 0) choice=63;
			else if (strcmp(algotxt, "in_convex_l1_max") == 0) choice=64;
			//     else if (strcmp(algotxt, "out_convex_l1_max") == 0) choice=65;
			else if (strcmp(algotxt, "barr_l2_sum") == 0) choice=70;
			else if (strcmp(algotxt, "dir_median")==0) choice=160;
			else if (strcmp(algotxt, "undir_median")==0) choice=161;
			else if (strcmp(algotxt, "abs_undir_median")==0) choice=162;
			else if (strcmp(algotxt, "L_RkonvPoly_l2_sum") == 0) choice=307;
			else if (strcmp(algotxt, "L_RkonvPoly_lp_sum") == 0) choice=308;
			else if (strcmp(algotxt, "L_RkonvPoly_l1_sum") == 0) choice=309;
			else if (strcmp(algotxt, "L_RkonvPoly_linf_sum") == 0) choice=310;
			else lola_error("Illegal algorithm or missing data.",1009,1,win);
		} else if (gauge && !extra && ! mat) {
			if (strcmp(algotxt, "gauge_median")==0) choice=200;
			else if (strcmp(algotxt, "bi_crit_gauge_median")==0) choice=201;
			else if (strcmp(algotxt, "gauge_center")==0) choice=210;
			else if (strcmp(algotxt, "L_block_sum_M3") == 0) choice=306; 
			else if (strcmp(algotxt, "L_block_max_M4") == 0) choice=356; 
			else lola_error("Illegal algorithm or missing data.",1010,1,win);
		} else if (gauge && extra && !mat) {
			if (strcmp(algotxt, "restr_gauge_center") == 0) choice=211;
			if (strcmp(algotxt, "L_RkonvPoly_block_sum") == 0) choice=311;
		} 
		if ((choice==3 || choice==24 || choice== 53 || choice==303 || choice==308 || choice==355) && !pflag) lola_error("Missing p-value for lp-algorithm.",1011,1,win);
		if ((choice==120 || choice==121 || choice==122 || choice==123) && !nflag) lola_error("Missing number of new facilities for algorithms on graphs.",1012,1,win);
		if (choice == 999) { usage(); }
		else { _algoch(choice,locc,extrac,gaugec,outc,win,net,gauge); }
	} // of processing options
	return 0;
}

