// --
// Declaration of class circle_barrier
// --

#ifndef CLASS_CIRCLE_BARRIER_H
#define CLASS_CIRCLE_BARRIER_H

#include <LOLA/restriction.h>
#include <LOLA/halfspace.h>

/*{\Manpage {circle_barrier} {} {Circle as a barrier} }*/

class circle_barrier:public restriction {
/*{\Mdefinition
An instance $C$ of the data type $circle\_barrier$ is a circle in the 
two-dimensional plane, i.e.~the set of points having a certain distance 
$r$ from a given point $p$. $r$ is called the radius and $p$ is called the 
center of $C$.  The $circle\_barrier$ with center $(0,0)$ and radius $0$ is
called the empty $circle\_barrier$.}*/
protected: 
  point                          _center;
  double                         _radius;
public:    
/*{\Mcreation C }*/
  circle_barrier();
/*{\Mcreate
introduces a variable $C$ of type $circle\_barrier$. $C$ is initialized to the
empty $circle\_barrier$.}*/
  circle_barrier(const point& p,double r);
/*{\Mcreate introduces a variable $C$ of type $circle\_barrier$. $C$ is initialized to the 
circle with center $p$ and radius $r$.}*/
  circle_barrier(int i, const point& p,double r) {circle_barrier(p,r);}
  circle_barrier(double x,double y,double r);
/*{\Mcreate introduces a variable $C$ of type $circle\_barrier$. $C$ is initialized to the 
circle with center $(x,y)$ and radius $r$.}*/
  circle_barrier(const point& a,const point& b,const point& c);
/*{\Mcreate introduces a variable $C$ of type $circle\_barrier$. $C$ is initialized to the 
circle through points $a$, $b$, and $c$. \precond $a$, $b$, and $c$ are not collinear. }*/
  circle_barrier(const circle_barrier& c);
  ~circle_barrier();

/*{\Moperations 2.5 4.6}*/

  circle_barrier& operator=(const circle_barrier &c);
  int type() {return t_barrier;}
  int subtype() {return t_circle;};
  point center() {return _center;} 
/*{\Mop  returns the center of $C$.}*/
  double radius() {return _radius;}
/*{\Mop  returns the radius of $C$.}*/

  bool inout() {return false;}
  void in_forbid() {return;}
  void out_forbid() {return;}

  bool inside(const point& p);
  bool inside(const location& l);
  bool inside(const segment& l) { return false; } 
  bool on_edge(const point& p);
  bool on_edge(const location& l);
  list<point> intersect(point&,point&) { list<point> temp; return temp; }
  list<point> intersect(ray &r, point &p) { list<point> temp; return temp; }
  list<point> intersection(const line& l);
  list<point> intersection(const segment& s);
 list<point> intersection(const polygon&) { list<point> temp; return temp; }
  list<point> intersection(const circle&) { list<point> temp; return temp; }
bool visible(const point& p1,const point& p2);
/*{\Mop	   returns true if $p2$ is visible from $p1$ with respect to $P$. }*/
  point r_tang_point(const point p);
  point l_tang_point(const point p);
  list<point> t_points(const point& p);
  double LSP2(const point& p1,const point& p2);
  double l_2(const location& L,const point& p);
  double objective(const facilities& EX,const point& p);
  friend ostream& operator << (ostream& os,const circle_barrier& B);
  double left_path(const point&,const point&);
  double right_path(const point&,const point&);  

  list<location> proj_l2sqr(const location &opt) {list<location> L; return L;}
private:
  bool spccmp(const point& p1,const point& p2);
  double myabs(double d);
};
#endif

