/***************************************************************************
* class lolagraph for LOLA (Library of Location Algorithms)
* Date   : May 1997
***************************************************************************/

#ifndef _LOLAGRAPH_H_
#define _LOLAGRAPH_H_

#include <LEDA/graph_alg.h>
#include <LOLA/facs_util.h>
#include <LOLA/graphsolution.h>
/*{\Manpage {classlolagraph} {} {Lolagraph}}*/

class lolagraph: public GRAPH<node_weight,double>

/*{\Mdefinition An instance of the data type $lolagraph$ consists of a LEDA-Graph
graph$<$node$\_$weight, double$>$.}*/ 
{

 protected:
 list<sol_typ> solution;

/*{\Mcreation G }*/
/*{\Mtext {\it lolagraph} G; \hspace{1cm}
creates a LEDA-Graph graph$<$node$\_$weight,double$>$ $G$}*/

/*{\Moperations 3.3 4.9 }*/

//  lolagraph();
 
  public:

 // HILFSMETHODEN

/*{\Mtext
\medskip
{\bf 3.1 Helpfunctions}
\medskip }*/

  matrix shortest_path();

/*{\Mop
returns the distance matrix for the graph $G$. }*/

  list<sol_typ> med_nodes(double& objval);

  list<segment> dist_node(double l1, double l2, double weight, double length);

  list<point> dist_edge(int node_num, array<segment> d1, array<segment> d2);

 edge inz_edge(node& v, node& w);

/*{\Mop
returns the edge between node $v$ and node $w$ in graph $G$.}*/
 
// Algorithmen

/*{\Mtext
\medskip
{\bf 3.2 Algorithms}
\medskip }*/

 list<sol_typ> abs_cent(double& objval);

/*{\Mop
returns the solution of the 1-center problem of a whole graph, directed
 and undirected.}*/

 list<sol_typ> center(double&);

/*{\Mop
returns the solution of the 1-center problem on the nodes of a graph.}*/

 list<sol_typ> median(const char& wert, double& objval);

/*{\Mopl
returns the solution of the 1-median problem of the nodes for a directed or undirected graph.}*/


list<sol_typ> N_median(int ,double&,list<int>& belongto);
/*{\Mopl
returns the solution of the N-median problem of the nodes for a directed or undirected graph.}*/

list<sol_typ> N_partitioning(const int N,double& objval,list<int>& belongto,bool typ);
/*{\Mopl
returns the solution of the N-median(typ=1) or N-Center(typ=0)  problem of the nodes for a directed or undirected graph, using the "node-partitioning-heuristic" .}*/

list<sol_typ> Nmed_exchange(const int N,double& objval,list<int>& belongto);
/*{\Mopl
returns the solution of the N-median problem of the nodes for a directed or undirected graph, using the "exchange-heuristic" .}*/

list<sol_typ> N_greedy(const int N,double& objval,list<int>& belongto,bool typ);
/*{\Mopl
returns the solution of the N-median (typ=1) or N-center (typ=0)  problem of the nodes for an undirected graph, using the "greedy-heuristic" .}*/


   friend ostream& operator << (ostream&,const lolagraph&);
   friend istream& operator >> (istream&, lolagraph&);

/* Karens Diplomarbeit: */
//parloc.cpp (siehe weitere header in lolaundirected.h)
list<edge_segment> parlocation(list<vector>& objval,bool dir, bool abs, bool center);
matrix F_median(matrix& D,bool directed);
matrix F_center(matrix& D,bool directed);
vector x_center(matrix& D,int v_i, int v_j,double laenge, double t_value);
bool par_dominates(vector f1,vector f2);
//lexloc.cpp
vector x_median(matrix& D,int v_i, int v_j,double length);
vector compute_Min(matrix& F,list<int>& J_list);
list<int> cancel_list(list<int>& A, list<int>& B);
list<edge_segment> lexlocation(list<vector>& objval,bool dir, bool abs,bool center );
};
/* Global (Karen's Diplomarbeit): */
//parloc.cpp
int compare (const edge_segment& a, const edge_segment& b);
int ptcomp_g (const rat_point& a, const rat_point& b);
int compare (const rat_segment& a, const rat_segment& b);
int compare (const rat_line& a, const rat_line& b);
list<rat_segment> mergestep(list<rat_segment> one,list<rat_segment> two);
list<rat_segment> Div_and_Conq(list<rat_segment> envelope);
list<rat_segment> Div_and_Conq2(list< list<rat_segment> > envelope);
list<rat_segment> hershi(list<rat_segment> Seggis);

#endif

