/*********************************************************************
* LoLA (Library of Location Algorithms)                              *
*                                                                    *
* CLASS DISCRETE                                                     *
**********************************************************************/
#ifndef LOLA_DISCRETE_H
#define LOLA_DISCRETE_H

#include <LOLA/graph_util.h>

/*{\Manpage {discrete} {} {Discrete Locations}}*/
class discrete  
/*{\Mdefinition An instance of the data type $discrete$ two list of location giving the
positions for the demand and the supply points, and a matrix with travelling costs from
a supply point to a demand point. }*/


{
private:
	matrix		    COST;
	list<location>  SUPPLY;
	list<location>  DEMAND;
protected:
	list<location>  solution;
	list<string>    Loctxt;
	list<string>    Loctxt_sup;
	matrix          belongto;
	bool			same;

public:
/*{\Mcreation D}*/

	discrete() {};
	discrete(matrix Cost);
/*{\Mcreate
introduces a variable $D$ of type $discrete$. $D$ is initialized with the
cost matrix M.}*/
	discrete(list<location>, matrix);
	discrete(list<location>, list<location>, matrix);
	~discrete();

/*{\Moperations 1.5 5.1 }*/
/*{\Mtext
\medskip
{\bf 3.1 Input and Output}
\medskip }*/

 	void del_sol();
	matrix Ret_Cost() {return this->COST;}
	list<location> Ret_Supply() {return this->SUPPLY;}
	list<location> Ret_Demand() {return this->DEMAND;}
	list<string> Ret_demtxt() {return this->Loctxt;}
	list<string> Ret_suptxt() {return this->Loctxt_sup;}
	matrix Ret_belongto() {return this->belongto;}
	list<location> alg_solution() {return this->solution;}

//Inputfunctions
	int disc_read(string&);
	void ReadDiscrete(ifstream& file);
/*{\Mop reads all data for $D$ from $file$.}*/

	//Outputfunctions
	void DiscView(double objval);
/*{\Mopl shows the results and locations of the current problem.}*/
 	void draw_loladisc(window& W);
	void SaveDisc(ofstream& file);
/*{\Mop saves all data of $D$ into $file$.}*/
	void draw_loladiscrete(window& W,discrete& D);
	double max_direction_value(int i);
	double min_direction_value(int i);
	double diff_value(int i);




//Algorithms
/*{\Mtext
\medskip
{\bf 3.1 Algorithms}
\medskip }*/

	double UFLP_greedy();
/*{\Mopl
returns the solution of the UFL problem using the "greedy-heuristic" .}*/

	double UFLP_stingy();
/*{\Mopl
returns the solution of the UFL problem using the "stingy-heuristic" .}*/

	double UFLP_Interchange();
/*{\Mopl
returns the solution of the UFL problem using the "interchange-heuristic" .}*/

	double UFLP_dualoc();
/*{\Mopl
returns the solution of the UFL problem using the dualoc as an exact algorithm .}*/


    double UFLP_dualascent(matrix& ,array<double>& ,array<double>&,matrix&, double&, matrix&);
    bool UFLP_branchbound(matrix& ,array<double>& ,array<double>&,matrix&, double&, matrix& ,bool);

	
};

#endif
