/*********************************************************************
* LoLA (Library of Location Algorithms)                              *
*                                                                    *
* CLASS FACILITIES                                                   *
**********************************************************************/

#ifndef LOLA_FACS_UTIL_H
#define LOLA_FACS_UTIL_H

//#include <LEDA/panel.h>
//#include <LEDA/array.h>
#include <LOLA/restr_util.h>
#include <LOLA/gauge_util.h>

/*{\Manpage {facs_util} {} {Facilities-utilities}}*/


class facs_util : virtual public facilities {
/*{\Mdefinition
An instance $EX$ of the data type \name\  supports the data type $facilities$.
It provides some useful routines to handle the class $facilities$.}*/

/*{\Mcreation EX}*/
/*{\Mtext {\it facs\_util EX }; \hspace{1cm} creates a list of location $EX$ 
of type $facs\_util$.}*/
/*{\Moperations 1.5 5.1 }*/
/*{\Mtext
\medskip
{\bf 3.1 Input and Output}
\medskip }*/

public: 
  void            ReadLoc(ifstream& file);
/*{\Mop reads all locations from $file$.}*/
  void            ReadLoc_Res(ifstream& file);
/*{\Mop reads all locations with results from $file$.}*/
  void            ReadLoc(ifstream& file, list<int>& Dist);
/*{\Mopl reads all locations from $file$ and returns a list to associate
distance functions.}*/
  void            ReadRestr(ifstream& file);
/*{\Mop reads all restrictions from $file$.}*/
  void            SaveLoc(ostream& file);
/*{\Mop saves all locations into $file$.}*/
  void            SaveLoc_Res(ostream& file,double objval,string normact);
/*{\Mopl saves all locations and the results of the current problem into $file$.}*/
  matrix          ReadMat(ifstream& file);
/*{\Mop reads a matrix from $file$ for a $multi-facilities-problem$.}*/
  void            View(double objval,string normact,list<location>& AlgSol,restrictions& Restr);
/*{\Mopl shows the results and locations of the current problem.}*/
  void            View(double objval,string normact,list<location>& AlgSol,restrictions& Restr,list<polygauge>& LP, list<int>& LI);
/*{\Mopl shows the results and locations of the current problem.}*/
  void            View(double objval,string normact,list<location>& AlgSol,restrictions& Restr, list<polygon>& LPoly);
/*{\Mopl shows the results and locations of the current problem.}*/
  void			  View(double objval, string normact, list<location>& AlgSol, restrictions& Restr, list<polygauge>& LP, list<int>& LI, list<polygon>& LPoly);
/*{\Mopl shows the results and locations of the current problem.}*/
  void            View(double objval, string normact, list<location>& AlgSol, restrictions& Restr, list<polygauge>& LP, list<int>& LI, int choice);

};

#endif




