/***************************************************************************
* Utilities for class gauge
* Purpose: 
* Author : Lothar Schnell
* Date   : Dez-11-1995
***************************************************************************/
#ifndef LOLA_GAUGEUT_H
#define LOLA_GAUGEUT_H

#include <LEDA/string.h>
#include <LOLA/polygauge.h>
#include <LOLA/mixgauge.h>
#include <fstream.h>

/*{\Manpage {gauge_util} {} {Gauge-utilities}}*/

/*{\Mtext Gauge-utilities provides useful routines to handle gauges.}*/
/*{\Mtext 
\begin{tabular}{llp{5cm}}
{\it list$<$polygauge$>$} & ReadLPGauge(ifstream file) &
returns the list of polygauges. \\
{\it list$<$mixgauge$>$} & ReadLMGauge(ifstream file) &
returns the list of mixgauges. \\
{\it list$<$int$>$} & ReadGNumber(ifstream file) &
returns the list of numbers to connect the locations with their gauges. \\
& & Precondition: size of facilities is equal to size of numbers \\
{\it void} & SaveGauge(ofstream file, polygauge G) &
saves the list of extremalpoints from polygauge $G$ in {\it file}. \\
{\it void} & SaveGauge(ofstream file, mixgauge M) &
saves the list of extremalpoints and the conetyp from mixgauge $M$ in {\it file}. \\
{\it void} & DrawGauge(window W, polygauge G) &
draws a polygauge $G$ in a window $W$.\\
{\it void} & DrawGauge(window W, mixgauge M) &
draws a mixgauge $M$ in a window $W$.\\
{\it void} & ViewGauge(polygauge G) &
opens a window and a panel where you can choose Load, Save or Create a polygauge. \\
{\it void} & ViewGauge(mixgauge M) &
opens a window and a panel where you can choose Load, Save or Create a mixgauge. 
\end{tabular}

%*******
\begin{tabbing}
{\it list$<$polygauge$>$}  \qquad \= ReadLPGauge(ifstream file) \\
\> \qquad \quad returns the list of polygauges. \\ [0.8ex]
{\it list$<$mixgauge$>$}  \qquad \= ReadLMGauge(ifstream file) \\
\> \qquad \quad returns the list of mixgauges. \\ [0.8ex]
{\it list$<$int$>$}  \qquad \= ReadGNumber(ifstream file) \\
\> \qquad \quad returns the list of numbers to connect the locations with their gauges. \\
\> \qquad \quad Precondition: size of facilities is equal to size of numbers \\
{\it void}  \qquad \= SaveGauge(ofstream file, polygauge G) \\
\> \qquad \quad saves the list of extremalpoints from polygauge $G$ in {\it file}. \\ [0.8ex]
{\it void}  \qquad \= SaveGauge(ofstream file, mixgauge M) \\
\> \qquad \quad saves the list of extremalpoints and the conetyp from mixgauge $M$ in {\it file}. \\ [0.8ex]
{\it void}  \qquad \= DrawGauge(window W, polygauge G) \\
\> \qquad \quad draws a polygauge $G$ in a window $W$.\\ [0.8ex]
{\it void}  \qquad \= DrawGauge(window W, mixgauge M) \\
\> \qquad \quad draws a mixgauge $M$ in a window $W$.\\ [0.8ex]
{\it void}  \qquad \= ViewGauge(polygauge G) \\
\> \qquad \quad opens a window and a panel where you can choose Load, Save or Create a polygauge. \\ [0.8ex]
{\it void}  \qquad \= ViewGauge(mixgauge M) \\
\> \qquad \quad opens a window and a panel where you can choose Load, Save or Create a mixgauge. 
\end{tabbing}
}*/
/*{\Mtext
\newpage
\medskip
{\bf Input-format for the files }
\medskip }*/
/*{\Mtext 
\begin{tabular}{ll}
{\it ReadGauge(file)} & begin \{polygauge\} \\
 & $x_1 \quad y_1$ \\
 & \vdots \\
 & $x_n \quad y_n$ \\
 & end \{polygauge\} \\
{\it ReadGauge(file,tl)} & begin \{mixgauge\} \\
 & $x_1 \quad y_1 \quad typ_1$ \\
 & \vdots \\
 & $x_n \quad y_n \quad typ_n$ \\
 & end \{mixgauge\} \\
{\it ReadLPGauge(file)} & begin \{polygaugelist\} \\
 & begin \{polygauge\} \\
 & $x_1 \quad y_1$ \\
 & \vdots \\
 & $x_n \quad y_n$ \\
 & end \{polygauge\} \\
 & \vdots \\
 & begin \{polygauge\} \\
 & $x_1 \quad y_1$ \\
 & \vdots \\
 & $x_m \quad y_m$ \\
 & end \{polygauge\} \\
 & end \{polygaugelist\} \\
{\it ReadLMGauge(file)} & begin \{mixgaugelist\} \\
 & begin \{mixgauge\} \\
 & $x_1 \quad y_1 \quad typ_1$ \\
 & \vdots \\
 & $x_n \quad y_n \quad typ_n$ \\
 & end \{mixgauge\} \\
 & \vdots \\
 & begin \{mixgauge\} \\
 & $x_1 \quad y_1 \quad typ_1$ \\
 & \vdots \\
 & $x_m \quad y_m \quad typ_m$ \\
 & end \{mixgauge\} \\
 & end \{mixgaugelist\} \\
{\it ReadGNumber(file)} & begin \{gaugenumber\} \\
 & $num_1$ \\
 & \vdots \\
 & $num_n$ \\
 & end \{gaugenumber\} \\
\end{tabular}
}*/

list<point> ReadGauge(ifstream& file);  
list<point> ReadGauge(ifstream& file, list<bool>& tl);  
list<polygauge> ReadLPGauge(ifstream& file);  
list<mixgauge> ReadLMGauge(ifstream& file);  
list<int> ReadGNumber(ifstream& file);  
void SaveGauge(ofstream& file,polygauge& G);  
void SaveGauge(ofstream& file,mixgauge& G);  
void DrawGauge(window& W,polygauge& G);
void DrawGauge(window& W,mixgauge& G);
void ViewGauge(polygauge& G);
void ViewGauge(mixgauge& G);
void CreatePolyGauge();
void DrawCreate(window& W, list<point>& G_PKT);

#endif
