/*********************************************************************
* LoLA (Library of Location Algorithms)                              *
*                                                                    *
* CLASS GAUGEALG
**********************************************************************/

#ifndef LOLA_GAUGEALG_H
#define LOLA_GAUGEALG_H

//#include <LOLA/lines.h>
#include <LOLA/restrictions.h>


/*{\Manpage {gaugealg} {} {Algorithm}}*/

class gaugealg
/*{\Mdefinition An instance of the data type $gaugealg$ consists of the algorithm to solve planar location problems with gauges as the distance functions.}*/
{
 protected:
 list<location> solution;

 public: 
/*{\Mcreation GA }*/
/*{\Mtext {\it gaugealg} GA; \hspace{1cm}
creates a variable $GA$ of type $gaugealg$.}*/
/*{\Moperations 2.3 4.7 }*/

  list<location>  alg_solution() {return solution;}

/*{\Mop returns the solution for the current problem.}*/

  void            WriteOpt();
  void            WriteOpt(ostream& out);
/*{\Mop writes the solution for the current problem.}*/


/*{\Mtext
\medskip
{\bf Algorithms}
\medskip }*/

  double          sum(list<polygauge>& LG, list<int>& Ln, facilities& EX);
/*{\Mopl computes the optimum for problem class $1/P/./\gamma/\sum$ and returns the objective value.}*/

  double          bi_crit_sum(list<polygauge>& LG, list<int>& Ln, facilities& EX);
/*{\Mopl computes the optimum for problem class $1/P/./\gamma/2-\sum_{par}$ and returns the number of different situations of the solution.}*/

double            L_block_sum_M3(list<polygauge> &LG, list<int> &Ln, facilities &Ex);
/*{\Mopl $1L/P/./\gamma_B/\sum$ with $ O(M^3)$.}*/


  double          L_block_max_M4(list<polygauge> &LG, list<int> &Ln, facilities &Ex);
/*{\Mopl $1L/P/./\gamma_B/\max$ with$ O(M^4)$.}*/

  double          L_RkonvPoly_block_sum(list<polygauge> &LG, list<int> &Ln, facilities EX, restrictions &Restr);
/*{\Mopl $1L/P/./\gamma_B/\max$ with $O(M^4)$.}*/

double    max(list<polygauge>& LG, list<int>& Ln, facilities& EX);
double    max(list<polygauge> &LG, list<int> &Ln, facilities EX, restrictions &Restr);


};

#endif

