/*********************************************************************
* LoLA (Library of Location Algorithms)                              *
*                                                                    *
* CLASS graph_util                                                   *
**********************************************************************/

#ifndef graph_util_H
#define graph_util_H

//#include <LOLA/classlolagraph.h>
#include <LOLA/graph_restriction.h>
/*{\Manpage {graph_util} {} {Graph-utilities}}*/

class graph_util : public lolagraph {

/*{\Mdefinition
An instance of the data type $graph\_util$ supports the data type $lolagraph$. It
 is derived from the
data type $lolagraph$. The class $graph\_util$ 
provides input and output routines to handle the class $lolagraph$.}*/

/*{\Mcreation G}*/
/*{\Mtext {\it graph\_util G }; \hspace{1cm} creates a lolagraph as 
graph$\_$util $G$.}*/
/*{\Moperations 3.5 4.1 }*/
/*{\Mtext 
{\bf  Input and Output}
\medskip }*/

public:

void draw_lolagraph(window& W,const lolagraph& Gtmp, const list<sol_typ>& A, facilities& EX, list<string>& Loctxt, list<bool> W_inf, graph_restriction& EL, bool dir);


void QGraphView(list<vector> objvec, string normact, facilities& EX, list<string>& Loctxt, list<edge_segment>& EL, bool dir);
 
void LGraphView(double objval, list<sol_typ>& AlgSol, facilities& EX, list<string>& Loctxt); 

void LGraphView(double objval, string normact, list<sol_typ>& AlgSol, facilities& EX, list<string>& Loctxt, list<int> belongto, bool dir = false);

void LGraphView(double objval, string normact, list<sol_typ>& AlgSol, facilities& EX, list<string>& Loctxt, graph_restriction& EL, bool dir,list<int> belongto); 

/*{\Mopl
draws a lolagraph. }*/


lolagraph CreateGraph(const string locfile, const string adjfile, facilities& EX)
;

/*{\Mopl
creates a lolagraph file and returns a parameterized graph and the facilities $EX
$. }*/


void SaveGraph(ofstream& file, facilities& EX, list<string>& Loctxt);

/*{\Mopl
saves a lolagraph in a file. }*/

void SaveAdjlist(ofstream& file );

/*{\Mopl
saves the adjacent list of a lolagraph in a file. }*/


graph_util& ReadGraph(ifstream& file, facilities& EX, list<string>& Loctxt);

/*{\Mopl
reads a lolagraph file and returns a parameterized graph and the facilities $EX$.
 }*/

graph_util ReadAdjlist(ifstream& file, list<string>& Loctxt);
};

#endif

