#ifndef _GRAPHSOLUTION_H_
#define _GRAPHSOLUTION_H_

/*{\Manpage {sol_typ, edge_segment, node_weight in graphsolution.h} {} {Basic Classes for Graphs}}*/

/*{\Mdefinition An instance of the data type $sol\_typ$ consists of two integers
and one double to describe the solution of non restrictive problems. $sol\_typ$ is used to denote a location along the edge, where
 the integers describe the adjacent nodes and the double is a parameter t $\in$
[0..1] to give the position along the edge.}*/ 

/*{\Mtext {\it sol\_typ} S; \hspace{1cm}
creates an instance of $sol\_typ$.}*/
/*{\Mtext  S.source:=i; \hspace{1cm}
defines the startnode $i$ of edge $e$.}*/
/*{\Mtext  S.target:=j; \hspace{1cm}
defines the endnode $j$ of edge $e$.}*/
/*{\Mtext  S.alpha\_s:=t; \hspace{1cm}
defines the point $t$ on edge $e$.}*/

class  sol_typ{
   public:

   int source;
   int target;
   double alpha_s;
   
   sol_typ() {source=0; target=0;alpha_s=1.0;}
   sol_typ(int i, int j, double alpha) {source=i; target=j; alpha_s=alpha;}
   ~sol_typ() {};

   friend ostream& operator << (ostream&,const sol_typ&);
   friend istream& operator >> (istream&, sol_typ&);
};

/*{\Mtext \medskip {\bf 2. Definition} \\ 
\medskip An instance of the data type $edge\_segment$ is derived from the
data type $sol\_typ$ with an additional double to describe the solution of restrictive problems. $edge\_segment$ is used to 
denote a part of an edge.}*/
/*{\Mtext {\it edge\_segment} ES; \hspace{1cm}
creates an instance of $edge\_segment$.}*/
/*{\Mtext  ES.source:=i; \hspace{1cm}
defines the startnode $i$ of edge $e$.}*/
/*{\Mtext  ES.target:=j; \hspace{1cm}
defines the endnode $j$ of edge $e$.}*/
/*{\Mtext  ES.alpha\_s:=$t_1$; \hspace{1cm}
defines the starting point $t_1$ on edge $e$.}*/
/*{\Mtext  ES.alpha\_t:=$t_2$; \hspace{1cm}
defines the endpoint $t_2$ on edge $e$.}*/
 
class  edge_segment: public sol_typ{
  public:

   double alpha_t;
   friend ostream& operator << (ostream&,const edge_segment&);
   friend istream& operator >> (istream&, edge_segment&);
};

/*{\Mtext \medskip {\bf 3. Definition} \\ 
\medskip  An instance of the data type $node\_weight$ is a list of doubles. 
A $lolagraph$ uses this data-type as node-type.}*/
 
class  node_weight: public list<double>{
   public:

   list<double> weights;

   friend ostream& operator << (ostream&,const node_weight&);
   friend istream& operator >> (istream&, node_weight&);
};

class bottle_elem {
public:
double t_value;
vector F_column;
node_weight weight;
friend ostream& operator<<(ostream&, const rat_segment& );
friend istream& operator>>(ostream&, rat_segment& );
friend ostream& operator<<(ostream&, const bottle_elem& );
friend istream& operator>>(istream& , bottle_elem &);
};
#endif




