// --
// Declaration of class halfspace
// --

#ifndef CLASS_HALFSPACE_H
#define CLASS_HALFSPACE_H

#include <math.h>
#include <LEDA/point.h>
#include <LEDA/bool.h>
#include <LEDA/string.h>

class halfspace {
/****************************************************
* Purpose   : Class representing a halfspace in plane
* Date      : Aug 1997
* Author    : Michael Ochs
*****************************************************/ 
protected:
  point _supp1; // These are the supporters
  point _supp2; // for the hyperplane
  int _halfspace; // halfspace descriptor
  double _xfact; // x-factor and
  double _yfact; // y-factor and 
  double _bvalu; // b-value of   xf*x+yf*y=bv.
  double _m; // First derivative of hyperplane.
  bool _xpara; // Bool for hyperplane parallel to x-axis
  bool _ypara; // Bool for hyperplane parallel to y-axis
  int  _dir_on_para; // Direction of hyperplane in case of
		     // x- or y-axis parallelity
private:
  // Service function to compute the values describing a
  // hyperplane for halfspace separation.
  // Used by point-point and point-point-int constructor.
  void   compute(const point&,const point&);
public:
/* Constructors */
  // empty constructor.
  halfspace();
  // point-point constructor (defaults to right(1) halfspace).
  halfspace(const point&,const point&);
  // point-point-int constructor (needs halfspace descriptor (int))
  // 1:right, -1:left.
  halfspace(const point&,const point&,int);
  // copy constructor.
  halfspace(const halfspace&);

/* Destructor */
  ~halfspace();

/* Public methods */
  // Check whether point is in defined halfspace.
  bool check_point(const point&);
  // Check in which halfspace a point is.
  int in_which(const point&);   
  // Check in which halfspace a point is and give verbose output.
  string say_which(const point&);  
  friend ostream& operator << (ostream&,const halfspace&);
};
#endif

