/*********************************************************************
* LoLA (Library of Location Algorithms)                              *
*                                                                    *
* CLASS LINES                                                        *
**********************************************************************/

#ifndef LOLA_LINES_H
#define LOLA_LINES_H


#include <LOLA/blocknorm.h>


/*{\Manpage {lines} {} {1-line-algorithms}}*/

class lines:virtual public facilities {
/*{\Mdefinition An instance of the data type $lines$ consists of 1-line 
algorithms.}*/

  public:

/*{\Mtext
\medskip
{\bf 2. Median-problems}
\medskip }*/

double L_l2_sum_M3(facilities& facs);
/*{\Mopl computes the optimum for problem class $1L/P/./l_2/\sum$ 
and returns the objective value.}*/
	
double L_l1_sum_M3(facilities& facs);
/*{\Mopl computes the optimum for problem class $1L/P/./l_1/\sum$
 and returns the objective value.}*/

double L_linf_sum_M3(facilities& facs);
/*{\Mopl computes the optimum for problem class $1L/P/./l_\infty/\sum$
and returns the objective value.}*/
	
double L_lp_sum_M3(facilities& facs, int p);
/*{\Mopl computes the optimum for problem class $1L/P/./l_p/\sum$
 and returns the objective value.}*/
	
double L_l2_sum_M2logM(facilities& facs);
/*{\Mopl computes the optimum for problem class $1L/P/./l_2/\sum$
 and returns the objective value.}*/
	
double L_l2_sum_M2(facilities& facs);
/*{\Mopl computes the optimum for problem class $1L/P/v_i=1/l_2/\sum$
 and returns the objective value.}*/
	
double L_block_sum_M3(list<polygauge> &LG,list<int> &Ln, facilities& facs);
/*{\Mopl computes the optimum for problem class $1L/P/./l_{block}/\sum$
and returns the objective value.}*/

/*{\Mtext
\medskip
{\bf 3. Center-problems}
\medskip }*/

double L_l2_max_M4(facilities& facs);
/*{\Mopl computes the optimum for problem class $1L/P/./l_2/max$
 and returns the objective value.}*/

double L_l1_max_M4(facilities& facs);
/*{\Mopl computes the optimum for problem class $1L/P/./l_1/max$ 
and returns the objective value.}*/

double L_linf_max_M4(facilities& facs);
/*{\Mopl computes the optimum for problem class $1L/P/./l_\infty/max$
 and returns the objective value.}*/

double L_l2_max_MlogM(facilities& facs);
/*{\Mopl computes the optimum for problem class $1L/P/v_i=1/l_2/max$
 and returns the objective value.}*/


double L_l2_max_M2logM(facilities& facs);
/*{\Mopl computes the optimum for problem class $1L/P/./l_2/max$
and returns the objective value.}*/


double L_lp_max_M4(facilities& facs, int p);
/*{\Mopl computes the optimum for problem class $1L/P/./l_p/max$
 and returns the objective value.}*/


double L_block_max_M4(list<polygauge> &LG,list<int> &Ln, facilities& facs);
/*{\Mopl computes the optimum for problem class $1L/P/./l_{block}/max$
 and returns the objective value.}*/

/*{\Mtext
\medskip
{\bf 4. Useful functions}
\medskip }*/
	
double lp_distance_2_line(location &l1, location &l2, location &L, int p);
/*{\Mopl calculates the $l_p$-distance of $L$ to the line defined by $l1$ and $l2$.}*/
		
double objective_function_l2(location *locs, int start, int N, location &l1, location &l2);
/*{\Mopl calculates the value of the objective function with metric $l_2$ for the line defined by $l1$ and $l2$ and for the locations with $start$ as the beginning index (first element has to be 1) and $N$ as the index for the final element.}*/

void max_line_l2(location &l1, location &l2, location &l3, location &r1, location &r2);
/*{\Mopl calculates the optimal $l_2$-line defined by $r1$ and $r2$ for $l1$, $l2$ and $l3$.}*/

double objective_function_linf(location *locs, int start, int N, location &result1, location &result2);
/*{\Mopl calculates the value of the objective function with metric $l_\infty$ for the line defined by $l1$ and $l2$ and for the locations with $start$ as the beginning index (first element has to be 1) and $N$ as the index for the final element.}*/

double objective_function_l1(location *locs, int start, int N, location &result1, location &result2);
/*{\Mopl calculates the value of the objective function with metric $l_1$ for the line defined by $l1$ and $l2$ and for the locations with $start$ as the beginning index (first element has to be 1) and $N$ as the index for the final element.}*/

void max_line_l1(location &l1, location &l2, location &l3, location &r1, location &r2);
/*{\Mopl calculates the optimal $l_1$-line defined by $r1$ and $r2$ for $l1$, $l2$ and $l3$.}*/

double objective_function_lp(location *locs, int start, int N, location &result1, location &result2, int p);
/*{\Mopl calculates the value of the objective function with metric $l_p$ for the line defined by $l1$ and $l2$ and for the locations with $start$ as the beginning index (first element has to be 1) and $N$ as the index for the final element.}*/

void max_line_lp(location &l1, location &l2, location &l3, location &r1, location &r2, int p);
/*{\Mopl calculates the optimal $l_p$-line defined by $r1$ and $r2$ for $l1$, $l2$ and $l3$.}*/

double objective_function_block(location *locs, int start, int N, location &l1, location &l2, Blocknorm &B);
/*{\Mopl calculates the value of the objective function with block-metric for the line defined by $l1$ and $l2$ and for the locations with $start$ as the beginning index (first element has to be 1) and $N$ as the index for the final element.}*/

void max_line_block(location &l1, location &l2, location &l3, location &r1, location &r2, Blocknorm &B);
/*{\Mopl calculates the optimal $block$-line defined by $r1$ and $r2$ for $l1$, $l2$ and $l3$.}*/

int geometric_duality(location *locs, location &pivot, int M);
/*{\Mopl sorts all $M$ locations $locs$ with respect to $pivot$ using geometric duality.}*/

void y_quicksort(location *tosort, int l, int r, location *old);
/*{\Mopl sorts the locations $tosort$ from $l$ (usual 0) to $r$ (usual number of locations minus 1) dependent on the $y$-coordinate (all locations have to be 2-dimensional) using a recursive quicksort-algorithm ($old$ should be the last location).}*/

/*{\Mtext
\medskip
{\bf 5. Implementation} \\ [3mm]
\medskip
 The time needed by the algorithms is given in the name of each, e.g. 
algorithm $L \_ l2 \_ max M2 \log M $ takes $o(M^2 \log M)$ time to run.}*/

};

#endif

