// --
// Declaration of class polygon_barrier
// --

#ifndef CLASS_POLYGON_BARRIER_H
#define CLASS_POLYGON_BARRIER_H

/*#include <LEDA/list.h>
#include <LEDA/point.h>
#include <LEDA/segment.h>
#include <LEDA/line.h>
#include <LEDA/polygon.h>*/
#include <LEDA/dictionary.h>
//#include <LEDA/plane_alg.h>
#include <LOLA/restriction.h>
#include <LOLA/halfspace.h>

/*{\Manpage {polygon_barrier} {} {Polygon as a barrier} }*/

class polygon_barrier:public restriction {
/*{\Mdefinition
An instance $P$ of the data type $polygon\_barrier$ is a simple polygon
in the two-dimensional plane defined by the sequence of its vertices. The
number of vertices is called the size of $P$. A $polygon\_barrier$ with
empty vertex sequence is called empty.}*/

protected:
  list<point> _pt_list;
  list<point> _mid_list;
  list<segment> _ed_list;

public:
/*{\Mcreation P}*/
  polygon_barrier();
/*{\Mcreate
introduces a variable $P$ of type $polygon\_barrier$. $P$ is initialized to the
empty $polygon\_barrier$.}*/
  polygon_barrier(const list<point> &Pt);
/*{\Mcreate
introduces a variable $P$ of type $polygon\_barrier$. $P$ is initialized to the 
polygon with vertex sequence $Pt$.\\
\prcond The vertices in $Pt$ define a simple polygon.}*/
  polygon_barrier(int i,const list<point>& L) { }
  polygon_barrier(const polygon_barrier&);
  ~polygon_barrier();

  polygon_barrier& operator=(const polygon_barrier &);

/*{\Moperations 2.5 4.6}*/

  list<point> vertices() { return _pt_list; }
/*{\Mop    returns the sequence of vertices of $P$ in counterclockwise 
           ordering.  }*/

  int type() {return t_barrier;}
  int subtype() {return t_polygon;}
  point center(); 
  bool inout() {return false;}
  void in_forbid() {return;}
  void out_forbid() {return;}

  double maxdiam();
  bool inside(const point&);
  bool inside(const location&);
  bool inside(const segment&) { return false; }
  bool on_edge(const point&);
  bool on_edge(const location&);
  list<point> intersect(point&,point&) { list<point> temp; return temp; }
  list<point> intersect(ray& r,point& p) { list<point> temp; return temp; }
  list<point> intersection(const line&);
  list<point> intersection(const segment&);
  list<point> intersection(const polygon&) { list<point> temp; return temp; }
  list<point> intersection(const circle&) { list<point> temp; return temp; }
  bool visible(const point &p1 ,const point &p2);
/*{\Mop	   returns true if $p2$ is visible from $p1$ with respect to $P$. }*/
  point r_tang_point(const point&);
  point l_tang_point(const point&);
  double* path_length(const point&,const point&);
  double right_path(const point&,const point&);
  double left_path(const point&,const point&);
  list<point> t_points(const point&);
  double LSP2(const point&,const point&);
  double l_2(const location&,const point&);
  double objective(const facilities&,const point&);
  list<location> proj_l2sqr(const location &opt) {list<location> L; return L;}
  friend ostream& operator << (ostream&,const polygon_barrier&);
};
#endif  
