#ifndef _RESTR_CIRCLE_H
#define _RESTR_CIRCLE_H

#include <LOLA/restriction.h>


/*{\Manpage {restr_circle} {} {Circle as a restriction} }*/

class restr_circle:public restriction {
/*{\Mdefinition
An instance $C$ of the data type $restr\_circle$ is a circle in the 
two-dimensional plane, i.e.~the set of points having a certain distance 
$r$ from a given point $p$. $r$ is called the radius and $p$ is called the 
center of $C$.  The $restr\_circle$ with center $(0,0)$ and radius $0$ is
called the empty $restr\_circle$.}*/

 protected:
   point centre;
   double _radius;
   bool ioflag;
   
 public:

/*{\Mcreation C }*/

   restr_circle();
/*{\Mcreate
introduces a variable $C$ of type $restr\_circle$. $C$ is initialized to the
empty $restr\_circle$.}*/
   restr_circle(const point &c, double r);
/*{\Mcreate introduces a variable $C$ of type $restr\_circle$. $C$ is initialized to the circle with center $c$ and radius $r$.}*/
   restr_circle(double x, double y, double r);
/*{\Mcreate introduces a variable $C$ of type $restr\_circle$. $C$ is initialized to the circle with center $(x,y)$ and radius $r$.}*/
   restr_circle(const point &a, const point &b, const point &c);
/*{\Mcreate introduces a variable $C$ of type $restr\_circle$. $C$ is initialized to the circle through points $a$, $b$, and $c$. \precond $a$, $b$, and $c$ are not collinear. }*/
   restr_circle(const restr_circle& c);
   ~restr_circle();


/*{\Moperations 2.5 4.6}*/

   restr_circle& operator=(const restr_circle &C);

   int subtype() { return 2; }
   int type() { return t_forbidden_region; }
   void in_forbid() { ioflag = true; }
   void out_forbid() { ioflag = false; }
   bool inout() { return ioflag; }
   point center() const { return centre; } 
/*{\Mop  returns the center of $C$.}*/
   double radius() const { return _radius; }
/*{\Mop  returns the radius of $C$.}*/
   double distance(const point& p);
/*{\Mop   returns the distance between $C$ and $p$
          (negative if $p$ inside $C$).}*/

   bool inside(const point &p);
   bool inside(const location &l);
   bool inside(const segment &s);
   list<point> intersect(point &Q, point &P);
   list<point> intersect(ray &r, point &r2);
   list<point> intersection(const line &l);
   list<point> intersection(const segment &s);
   list<point> intersection(const polygon &P);
   list<point> intersection(const circle &C);

   double objective(const facilities& EX,const point& p) {return 0;}

   list<location> proj_l2sqr(const location &opt);

   double gauge_max(list<location> &, facilities &, double, list<polygauge> &, list<int> &);
   void megiddo(list<double>& ,facilities& , list<int>& , list<polygauge>&);
   int opt3(double& , facilities& , list<int>&, list<polygauge>&, list<location>&);

   bool operator == (const restr_circle& D);
/*{\Mbinop    tests for equality of $C$ and $D$.}*/

friend ostream& operator<<(ostream& out, const restr_circle& c);
friend istream& operator>>(istream& in, restr_circle& c);  
};

#endif


