#ifndef _RESTR_POLY_H
#define _RESTR_POLY_H

#include <LOLA/restriction.h>


/*{\Manpage {restr_poly} {} {Polygons as a restriction} }*/

class restr_poly:public restriction {
/*{\Mdefinition
An instance $P$ of the data type $restr\_poly$ is a simple polygon
in the two-dimensional plane defined by the sequence of its vertices.
The number of vertices is called the size
of $P$. A $restr\_poly$ with empty vertex sequence is called empty.}*/

 protected:
   list<segment> seg_list;
   bool ioflag;
   
 public:

/*{\Mcreation P }*/

   restr_poly();
/*{\Mcreate
introduces a variable $P$ of type $restr\_poly$. $P$ is initialized to the
empty $restr\_poly$.}*/
   restr_poly(const list<point> &p);
/*{\Mcreate
introduces a variable $P$ of type $restr\_poly$. $P$ is initialized to the
$restr\_poly$ with vertex sequence $p$.\\
\precond The vertices in $p$ define a simple polygon.}*/
//   restr_poly(const restr_poly &P, double bar);
   restr_poly(const restr_poly &P);
   ~restr_poly();

   restr_poly& operator = (const restr_poly &P);

/*{\Moperations 2.5 4.6}*/
   
   int subtype() { return 0; }
   int type() { return t_forbidden_region; }
   void in_forbid() { ioflag = true; }
   void out_forbid() { ioflag = false; }
   bool inout() { return ioflag; }
   list<point> vertices();  
/*{\Mop     returns the sequence of vertices of $P$ in
             counterclockwise ordering.  }*/

   list<segment> segments() { return seg_list; }
/*{\Mop     returns the sequence of bounding segments
            of $P$ in counterclockwise ordering. }*/

   bool inside(const point &p);  
   bool inside(const location &l);
   bool inside(const segment &s);
   list<point> intersect(ray &r, point &r2);
   list<point> intersect(point &p, point &q);
   list<point> intersection(const line &l);
   list<point> intersection(const segment &s);
   list<point> intersection(const polygon &P);
   list<point> intersection(const circle &C);

   double objective(const facilities& EX,const point& p) {return 0; }

   list<location> proj_l2sqr(const location &opt);

   double proj_v1_l2_max(location &opt, facilities &facs, double z);

   double gauge_max(list<location> &, facilities &, double, list<polygauge> &, list<int> &);
   void megiddo(list<double>& ,facilities& , list<int>& , list<polygauge>&);
   int opt2(double& , facilities& , list<int>&, list<polygauge>&, list<location>&);
	   
   bool convex();
/*{\Mop     returns true if $P$ is convex, false otherwise. }*/
   int size() { return seg_list.size(); }
/*{\Mop     returns the size of $P$. }*/
   bool empty() { return seg_list.empty(); }
/*{\Mop     returns true if $P$ is empty, false otherwise. }*/
   
   bool operator == (const restr_poly& P1);
/*{\Mop   test for equality of $P$ and $P1$. }*/

   list<point> reflex();
/*{\Mop		returns the reflex corner points of $P$. }*/

friend ostream& operator << (ostream& out, restr_poly &p);
friend istream& operator >> (istream& in,  restr_poly &p);
};

#endif 

