#ifndef LOLA_RECTANGLE_H
#define LOLA_RECTANGLE_H

#include <LOLA/restr_poly.h>

/*{\Manpage {restr_rect} {} {Rectangle as a restriction} }*/


class restr_rect:public restriction {
/*{\Mdefinition
An instance $R$ of the data type $restr\_rect$ is a simple rectangle
in the two-dimensional plane defined by the sequence of its vertices.
The number of vertices is called the size
of $R$. A $restr\_rect$ with empty vertex sequence is called empty.}*/

 protected:
  point *pointer_to_rect;
  double *length_of_edges;
  bool ioflag;
    
 public:

/*{\Mcreation R }*/

   restr_rect();
   restr_rect(const restr_rect&);
   restr_rect(point p,double x,double y,double rad);
/*{\Mcreate
introduces a variable $R$ of type $restr\_rect$. $R$ is initialized to the
$restr\_rect$ with corner $p$ and sides of length $x$ and $y$ and rotated
by $rad$. }*/
   restr_rect(point p,double x,double y);
/*{\Mcreate
introduces a variable $R$ of type $restr\_rect$. $R$ is initialized to the
$restr\_rect$ with corner $p$ and sides of length $x$ and $y$. }*/
   restr_rect(point p1,point p2,point p3,point p4);
/*{\Mcreate
introduces a variable $R$ of type $restr\_rect$. $R$ is initialized to the
$restr\_rect$ with vertices $p1$, $p2$, $p3$, $p4$. }*/
   restr_rect(point p1,point p2);
/*{\Mcreate
introduces a variable $R$ of type $restr\_rect$. $R$ is initialized to the
$restr\_rect$ with opposite vertices $p1$ and $p2$. }*/
   ~restr_rect();

/*{\Moperations 2.5 4.6}*/

   int subtype() { return 1; }
   int type() { return t_forbidden_region; }
   void in_forbid() { ioflag = true; }
   void out_forbid() { ioflag = false; }
   bool inout() { return ioflag; }
   double area();
/*{\Mop     returns the area of $R$.}*/

   bool inside(const location& l);
   bool inside(const point& p);
   bool inside(const segment &s);
   list<point> vertices();
   list<segment> segments();
   list<point> intersect(point &p, point &q);
   list<point> intersect(ray &r, point &r2);
   list<point> intersection(const line &l);
   list<point> intersection(const segment &s);
   list<point> intersection(const polygon &P);
   list<point> intersection(const circle &C);

   double objective(const facilities& EX,const point& p) {return 0; }

   list<location> proj_l2sqr(const location &opt);
   double proj_v1_l2_max(location &opt, facilities &facs, double z_opt_tmp);

   double gauge_max(list<location> &, facilities &, double, list<polygauge> &, list<int> &);
   
   point& operator [] (int i);
   double& operator () (int j);
   int operator == (const restr_rect& r1);
/*{\Mop   tests for equality of $R$ and $r1$. }*/
   restr_rect& operator = (const restr_rect& r1);
   
friend ostream& operator<<(ostream& Out, const restr_rect& r);
};

#endif
