/***************************************************************************
* Utilities for restrictions 
* Purpose: 
* Author : Lothar Schnell
* Date   : Dez-11-1995
***************************************************************************/
#ifndef _RESTR_UTIL_H
#define _RESTR_UTIL_H

//#include <LEDA/plane.h>
#include <LEDA/window.h>
#include <LOLA/restrictions.h>
#include <LOLA/restr_rect.h>
#include <LOLA/restr_circle.h>

/*{\Manpage {restr_util} {} {Restriction-utilities}}*/

/*{\Mtext Restriction-utilities provides useful routines to handle restrictions.}*/
/*{\Mtext
\begin{tabular}{llp{5cm}}
{\it double} & Restr\_max\_dir(int i, restrictions Restr) &
returns the maximal coordinate in direction $i$ ($i=0$ $x$-coordinate, $i=1$ 
$y$-coordinate). \\
{\it double} & Restr\_min\_dir(int i, restrictions Restr) &
returns the minimal coordinate in direction $i$ ($i=0$ $x$-coordinate, $i=1$ 
$y$-coordinate). \\
{\it double} & Restr\_diff\_value(int i, restrictions Restr) &
returns the maximal difference between the coordinates in direction $i$ ($i=0$ $x$-coordinate, $i=1$ $y$-coordinate). \\
{\it void} & DrawRestr(window W, restrictions Restr) &
draws the restrictions in a window $W$. \\
{\it restrictions} & ReadRestr(ifstream file) &
returns the restrictions given in the correct format in {\it file}. \\
{\it restr\_poly} & ReadPoly(ifstream file) &
returns a polygon as a restriction given in the correct format in {\it file}. \\
{\it restr\_circle} & ReadCirc(ifstream file) &
returns a circle as a restriction given in the correct format in {\it file}. \\
{\it restr\_rect} & ReadRect(ifstream file) &
returns a rectangle as a restriction given in the correct format in {\it file}. 
\end{tabular}

\begin{tabbing}
{\it double} \qquad \= Restr\_max\_dir(int i, restrictions Restr) \\
\> \qquad \quad returns the maximal coordinate in direction $i$ ($i=0$ $x$-coordinate, $i=1$ 
$y$-coordinate). \\ [0.8ex]
{\it double} \qquad \=  Restr\_min\_dir(int i, restrictions Restr) \\
\> \qquad \quad returns the minimal coordinate in direction $i$ ($i=0$ $x$-coordinate, $i=1$ 
$y$-coordinate). \\ [0.8ex]
{\it double} \qquad \= Restr\_diff\_value(int i, restrictions Restr) \\
\> \qquad \quad returns the maximal difference between the coordinates in direction $i$ ($i=0$ $x$-coordinate, $i=1$ $y$-coordinate). \\ [0.8ex]
{\it void} \qquad \= DrawRestr(window W, restrictions Restr) \\
\> \qquad \quad draws the restrictions in a window $W$. \\ [0.8ex]
{\it restrictions} \qquad \=  ReadRestr(ifstream file) \\
\> \qquad \quad returns the restrictions given in the correct format in {\it file}. \\ [0.8ex]
{\it restr\_poly} \qquad \=  ReadPoly(ifstream file) \\
\> \qquad \quad returns a polygon as a restriction given in the correct format in {\it file}. \\ [0.8ex]
{\it restr\_circle} \qquad \=  ReadCirc(ifstream file) \\
\> \qquad \quad returns a circle as a restriction given in the correct format in {\it file}. \\ [0.8ex]
{\it restr\_rect} \qquad \=  ReadRect(ifstream file) \\
\> \qquad \quad returns a rectangle as a restriction given in the correct format in {\it file}. 


\end{tabbing}



}*/


double Restr_max_dir(int i,restrictions& Restr);  
double Restr_min_dir(int i,restrictions& Restr);  
double Restr_diff_value(int i,restrictions& Restr);  
void DrawRestr(window& W,restrictions& Restr);  
restrictions ReadRestr(ifstream& file);  
restr_poly ReadPoly(ifstream& file);  
restr_circle ReadCirc(ifstream& file);  
//restr_ell ReadElli(ifstream& file);  
restr_rect ReadRect(ifstream& file);  

#endif
