#ifndef LOLA_RESTRALG_H
#define LOLA_RESTRALG_H

#include <LOLA/sbo.h>
//#include <LOLA/blocknorm.h>
#include <LOLA/lines.h>

/*{\Manpage {restrictions} {} {Algorithm}}*/
 
class restrictions : virtual public list<restriction*>
/*{\Mdefinition An instance of the data type $restrictions$ consists of the algorithm to solve planar location problems with restrictions.}*/
{
 protected:
   list<location> solution;

 public:
/*{\Mcreation R }*/
/*{\Mtext {\it restrictions} R; \hspace{1cm}
creates a variable $R$ of type $restrictions$.}*/
/*{\Moperations 2.3 4.7 }*/

/*
   restrictions();
   ~restrictions() { solution.clear(); }
*/
   
   list<location> alg_solution() {return solution;}

/*{\Mopl returns the solution for the current problem.}*/

   void WriteOpt();
   void WriteOpt(ostream& out);

   double l1_sum(facilities& EX,bool erase);
/*{\Mopl computes the optimum for problem class $1/P/R/l_1/\sum$
and returns the objective value.}*/

   double linf_sum(facilities& EX,bool erase);
/*{\Mopl computes the optimum for problem class $1/P/R/l_\infty/\sum$
and returns the objective value.}*/

   double l2sqr_sum(facilities& EX,bool erase);
/*{\Mopl computes the optimum for problem class $1/P/R/l_2^2/\sum$
and returns the objective value.}*/

//   double l2_sum(facilities& EX,bool erase,int choose=1,double percent=0.1,double accel=1.0,double init_ss=0.01,double epsilon=0.001);
/*{\Mopl computes the optimum for problem class $1/P/(R,\infty)/l_2/\sum$
and returns the objective value.}*/

   double lp_sum(facilities& EX,int p,double epsilon,double delta,int iter_max,bool erase);
/*{\Mopl computes the optimum for problem class $1/P/R/l_p/\sum$
and returns the objective value.}*/

   double l2_v1_max(facilities& EX,bool erase);
/*{\Mopl computes the optimum for problem class 
$1/P/R=conv Poly,w_m=1/l_1/\sum$ and returns the objective value.}*/

   double linf_max(facilities& EX,bool erase);
/*{\Mopl computes the optimum for problem class 
$1/P/R=conv Restr/l_\infty/\sum$ and returns the objective value.}*/

   double l1_max(facilities &EX,bool erase);
/*{\Mopl computes the optimum for problem class
$1/P/R=conv Restr/l_1/\sum$ and returns the objective value.}*/

   double L_RkonvPoly_l2_sum(facilities& EX, bool erase);
/*{\Mopl computes the optimum for problem class 
$1L/P/R=convex \, polyhedron /l_2/\sum$ and returns the objective value.}*/

   double L_RkonvPoly_lp_sum(facilities& EX, bool erase, int norm);
/*{\Mopl computes the optimum for problem class 
$1L/P/R=convex \, polyhedron /l_p/\sum$ and returns the objective value.}*/

	double L_RkonvPoly_l1_sum(facilities& EX, bool erase);
/*{\Mopl computes the optimum for problem class 
$1L/P/R=convex \, polyhedron/l_1/\sum$ and returns the objective value.}*/

	double L_RkonvPoly_linf_sum(facilities& EX, bool erase);
/*{\Mopl computes the optimum for problem class 
$1L/P/R=convex \, polyhedron/l_\infty/\sum$ and returns the objective value.}*/

	double L_RkonvPoly_block_sum(list<polygauge> &LG, list<int> &Ln, facilities& EX, bool erase);
/*{\Mopl computes the optimum for problem class 
$1L/P/R=convex \, polyhedron/\gamma_B/\sum$ and returns the objective value.}*/
    double gauge_max(facilities& facs, list<polygauge>& LG, list<int>& Ln ,bool erase);

};

// ------------------------- global functions -----------------


void mnbrak(int eck1,int eck2,double *ax, double *bx, double *cx, double *fa, double *fb, double *fc, double (*funkf)(int,int,double,double), double lpMetrik);

double golden(int eck1,int eck2, double ax, double bx, double cx, double (*f)(int,int,double,double), double tol,double *xmin,double lpMetrik);

double GoldenSchnitt(int eck1,int eck2);

double LowBound(int eck1,int eck2,int zaehler);

#endif

