// --
// Declaration of class segment_barrier
// --

#ifndef CLASS_SEGMENT_BARRIER_H
#define CLASS_SEGMENT_BARRIER_H

//#include <math.h>
#include <LEDA/dictionary.h>
#include <LOLA/halfspace.h>
#include <LOLA/restriction.h>

/*{\Manpage {segment_barrier} {} {Segment as a barrier} }*/

class segment_barrier:public restriction {
/*{\Mdefinition
An instance $S$ of the data type $segment\_barrier$ is a segemnt
in the two-dimensional plane defined by its the points including the segment. }*/
protected:
  point          _SP1;
  point          _SP2;
public:
/*{\Mcreation S}*/
  segment_barrier();
/*{\Mcreate
introduces a variable $S$ of type $segment\_barrier$. $S$ is initialized to the
empty $segment\_barrier$.}*/
  segment_barrier(const point& p1, const point& p2);
/*{\Mcreate
introduces a variable $S$ of type $segment\_barrier$. $S$ is initialized to the 
segment $(p1, p2)$. }*/
  segment_barrier(const segment_barrier&);
  ~segment_barrier();
  
/*{\Moperations 2.5 4.6}*/

  int type() {return t_barrier;}
  int subtype() {return t_segment;};
  point point1() const {return _SP1;}
/*{\Mop
    returns the source point of $S$. }*/

  point point2() const {return _SP2;}
/*{\Mop
    returns the target point of $S$. }*/

  point midpoint();
  point center() {return midpoint();}
  bool inout() {return false;}
  void in_forbid() {return;}
  void out_forbid() {return;}

  bool on_edge(const point&);
  bool on_edge(const location&);
  bool inside(const point&);
  bool inside(const location&);
  bool inside(const segment&) { return false; }
  list<point> intersect(point&,point&);
  list<point> intersect(ray &r, point &p) { list<point> temp; return temp; }
  list<point> intersection(const line&);
  list<point> intersection(const segment&);
  list<point> intersection(const polygon&) { list<point> temp; return temp; }
  list<point> intersection(const circle&) { list<point> temp; return temp; }
  bool visible(const point&,const point&);
/*{\Mop	   returns true if $p2$ is visible from $p1$ with respect to $P$. }*/
  point r_tang_point(const point&);
  point l_tang_point(const point&);
  list<point> t_points(const point&);
  double LSP2(const point&,const point&);
  double l_2(const location&,const point&);
  double objective(const facilities&,const point&);
  double left_path(const point&,const point&);
  double right_path(const point&,const point&);
  list<location> proj_l2sqr(const location &opt) {list<location> L; return L;}
private:
  bool spccmp(const point&,const point&);
  double myabs(double);
};
#endif
