#include <LOLA/gauge_util.h>

static window* Wp;
static window* Wc;
static polygauge* Gp;
static mixgauge* Mp;
static list<point> *G_PKT;

static void redraw_func() { DrawGauge(*Wp,*Gp); }
static void redraw_funcm() { DrawGauge(*Wp,*Mp); }
static void refresh_func() { DrawCreate(*Wc, *G_PKT); }

// die zwei folgenden Prozeduren sind nur noch uebergangsweise da, sobald die flex-routinen stabil sind, koennen sie rausfliegen; Achtung: Aufrufe in Viewgraph
#ifdef NO_FLEX
list<point> ReadGauge(ifstream& file)
/********************************************************************
* Purpose   : Reads a polygauge file
* Returns   : List of point as a polygauge
* Influences: %
* Date      : Dec 1995
* Author    : Lothar Schnell
*********************************************************************/
{
	cout << "ungeflextes Readgauge" << endl;
	long act_pos;
	string tmp;
	double x,y;
	list<point> result;

	if (!file) lola_error("Can't open file !",1603,1);
	else {
		while ((tmp!="begin") || file.eof()) file >> tmp;
		file >> tmp;
		if (tmp!="{polygauge}") lola_error("no polygauge file in ReadGauge",1604,1);

		act_pos = file.tellg();
		while ((tmp != "end") && !file.eof() )
		{
			file.seekg (act_pos);
			file >> x >> y;
			act_pos = file.tellg();
			file >> tmp;
			point temp(x,y);
			result.append(temp);
		}
	}
	return result;
}

list<point> ReadGauge(ifstream& file, list<bool>& tl)
/********************************************************************
* Purpose   : Reads a mix file
* Returns   : mixgauge
* Influences: %
* Date      : Dec 1995
* Author    : Lothar Schnell
*********************************************************************/
{
	cout << "ungeflextes Readgauge" << endl;
	long act_pos;
	string tmp;
	double x,y;
	list<point> result;
	bool typ;

	if (!file) lola_error("Can't open file !",1605,1);
	else {
		while ((tmp!="begin") || file.eof()) file >> tmp;
		file >> tmp;
		if (tmp!="{mixgauge}") lola_error("no mixgauge file in ReadMixgauge",1606,1);

		act_pos = file.tellg();
		while ((tmp != "end") && !file.eof() )
		{
			file.seekg (act_pos);
			file >> x >> y >> typ;
			act_pos = file.tellg();
			file >> tmp;
			point temp(x,y);
			result.append(temp);
			tl.append(typ);
		}
	}
	return result;
}
#endif*/
list<int> ReadGNumber(ifstream& file)
/********************************************************************
* Purpose   : Reads the numbers that join the location with a gauge 
* Returns   : List of integer as a numbers
* Influences: %
* Date      : July 1996
* Author    : Lothar Schnell
*********************************************************************/
{
	long act_pos;
	string tmp;
	int i;
	list<int> result;

	if (!file) lola_error("Can't open file !",1601,1);
	else {
		while ((tmp!="begin") || file.eof()) file >> tmp;
		file >> tmp;
		if (tmp!="{gaugenumber}") lola_error("no gaugenumber file in ReadGNumber",1602,1);
		act_pos = file.tellg();
		while ((tmp != "end") && !file.eof() )
		{
			file.seekg (act_pos);
			file >> i;
			act_pos = file.tellg();
			file >> tmp;
			result.append(i);
		}
	}
	return result;
}

void SaveGauge(ofstream& file, polygauge& G)
/********************************************************************
* Purpose    : Saves a polygauge in a file
* Returns    : %
* Influences : %
* Date       : Dec 1995
* Author     : Lothar Schnell
* Last Change: June 1999 (Holger Hennes)
*********************************************************************/
{
	point temp;
	list<point> pl=G.vertices(origin);
	file << "begin {polygauge}" << "\n";
	forall(temp,pl) file << temp.xcoord() << " " << temp.ycoord() << "\n";
	file << "end {polygauge}" << "\n";
}


void SaveGauge(ofstream& file, mixgauge& G)
/********************************************************************
* Purpose    : Saves a mixgauge in a file
* Returns    : %
* Influences : %
* Date       : Dec 1995
* Author     : Lothar Schnell
* Last Change: June 1999 (Holger Hennes)
*********************************************************************/
{
	point temp;
	list<bool> tl;
	list<point> pl=G.vertices(origin);
	for (int i=1;i<=G.size();i++) tl.append(G.conetype(i));
	int j = 0;
	file << "begin {mixgauge}" << "\n";
	forall(temp,pl) {
		file << temp.xcoord() << " " << temp.ycoord() << " " << tl[tl.get_item(j)] << "\n";
		++j;
	}
	file << "end {mixgauge}" << "\n";
}

void DrawGauge(window& W, polygauge& G)
/********************************************************************
* Purpose    : Show a polygauge graphically
* Returns    : %
* Influences : %
* Date       : Dec 1995
* Author     : Lothar Schnell
* Last Change: June 1999 (Holger Hennes)
*********************************************************************/
{
	point tmp;
	double diff_win=0,wmax=0,wmin=L_INF;
	list<point> pl=G.vertices(origin);
  
	wmax = G.maxi(origin,0).xcoord();    
	if (G.maxi(origin,1).ycoord() > wmax) wmax=G.maxi(origin,1).ycoord();

	wmin = G.mini(origin,0).xcoord();    
	if (G.mini(origin,1).ycoord() < wmin) wmin=G.mini(origin,1).ycoord();

	diff_win = G.max_diff(0);
	if (G.max_diff(1) > diff_win) diff_win=G.max_diff(1);
	
	wmax += 0.2*diff_win;
	wmin -= 0.2*diff_win;

	W.init(wmin, wmax, wmin);

	W.draw_polygon(pl,black);
	forall(tmp,pl) {
		W.draw_segment(origin,tmp);
		W.draw_filled_node(tmp,black);
	}
	color col(0,0,255);
	W.draw_filled_node(origin,col);
}    

void DrawGauge(window& W, mixgauge& G)
/********************************************************************
* Purpose    : Show a mixgauge graphically
* Returns    : %
* Influences : %
* Date       : Dec 1995
* Author     : Lothar Schnell
* Last Change: June 1999 (Holger Hennes)
*********************************************************************/
{
	point tmp;
	double diff_win,wmax,wmin;
	list<point> pl=G.vertices(origin);
	list<bool> tl;
	for (int i=1;i<=G.size();i++) tl.append(G.conetype(i));

	wmax = G.maxi(origin,0).xcoord();    
	if (G.maxi(origin,1).ycoord() > wmax) wmax=G.maxi(origin,1).ycoord();

	wmin = G.mini(origin,0).xcoord();    
	if (G.mini(origin,1).ycoord() < wmin) wmin=G.mini(origin,1).ycoord();

	diff_win = G.max_diff(0);
	if (G.max_diff(1) > diff_win) diff_win=G.max_diff(1);

	wmax += 0.2*diff_win;
	wmin -= 0.2*diff_win;

	W.init(wmin, wmax, wmin);

	list_item t_it=tl.first();
	for(list_item it = pl.first(); it; it = pl.succ(it)) {
		tmp=pl.contents(it);
		W.draw_segment(origin,tmp);
		W.draw_filled_node(tmp,black);
#if(__LEDA__ < 360)
		if (tl[t_it]) W.draw_arc(tmp,pl[pl.cyclic_succ(it)],1);
		else W.draw_segment(tmp,pl[pl.cyclic_succ(it)]);
#else
		double erst = (tmp.xcoord()+pl[pl.cyclic_succ(it)].xcoord())/2;
		double zweit = (tmp.ycoord()+pl[pl.cyclic_succ(it)].ycoord())/2;
		point p(erst,zweit);
		erst = p.xcoord()/(sqrt(p.xcoord()*p.xcoord()+p.ycoord()*p.ycoord()));
		zweit = p.ycoord()/(sqrt(p.xcoord()*p.xcoord()+p.ycoord()*p.ycoord()));
		point p1(erst,zweit);
		if (tl[t_it]) W.draw_arc(tmp,pl[pl.cyclic_succ(it)],p1);
		else W.draw_segment(tmp,pl[pl.cyclic_succ(it)]);
#endif
		t_it = tl.succ(t_it);
	}
	color col(0,0,255);
	W.draw_filled_node(origin,col);
}    

void ViewGauge(polygauge& G)
/********************************************************************
* Purpose    : Show a polygauge
* Returns    : %
* Influences : %
* Date       : Dec 1995
* Author     : Lothar Schnell
* Last Change: June 1999 (Holger Hennes)
*********************************************************************/
{
	window W(600,600,"View Gauge");
	W.set_node_width(4);
	W.set_redraw(redraw_func);

	list<point> pl=G.vertices(origin); 
	list<point> pltmp;
	string tmp;

	string file_name="polygauge";

	W.buttons_per_line(3);

	W.button("Save Gauge",1);
	W.button("Load Gauge",2);
	W.button("Create Gauge",3);
	W.button("Refresh",8);
	W.button("Close",9);

	W.display(650,200);
	polygauge Gtmp;

	Wp = &W;
	Gp = &G;

	redraw_func();

	for(;;) 
	{ 
		int but = W.read_mouse();

		if (but == 1) {
			panel PSave("Save Gauge");
			PSave.string_item("file:",file_name);
			PSave.text_item(" ");
			PSave.text_item(" ");
			PSave.buttons_per_line(2);
			int bScancel = PSave.button("cancel");
			int bSok = PSave.button("ok");
			int butS = PSave.open(780,200);
			if (butS == bScancel) {
				file_name = "polygauge";
				continue;
			} else if (butS == bSok) {
				ofstream F(file_name,ios::out);
				if (! F) { 
					tmp = string ("Can't open file \"%s\"!",file_name);
					W.message(tmp);
				} else SaveGauge(F,G);
			}
		}
		if (but == 2) {
			panel PLoad("Load Gauge");
			PLoad.string_item("file:",file_name);
			PLoad.text_item(" ");
			PLoad.text_item(" ");
			PLoad.buttons_per_line(2);
			int bLcancel = PLoad.button("cancel");
			int bLok = PLoad.button("ok");
			int butL = PLoad.open(780,200);
			if (butL == bLcancel) {
				file_name = "polygauge";
				continue;
			} else if (butL == bLok) {
				ifstream F(file_name,ios::in);
				if (! F) { 
					tmp = string ("Can't open file \"%s\"!",file_name);
					panel PConfirm("Confirm");
					PConfirm.text_item(tmp);
					PConfirm.button("ok");
					PConfirm.open(880,200);
				} else {
					W.clear();
#ifdef NO_FLEX
					Gtmp=polygauge(ReadGauge(F));
#else
					Gtmp=ReadLPGauge(F).head(); //Flexversion
#endif
					Gp = &Gtmp;
					Wp = &W;
					redraw_func();
				}
			}
		}
		if (but == 3) {	// Create Gauge
			CreatePolyGauge();
		}	// Create Gauge

		if (but == 8) redraw_func(); //DrawGauge(W,G);
		if (but == 9) break;
	}
}

void ViewGauge(mixgauge& G)
/********************************************************************
* Purpose    : Show a mixgauge
* Returns    : %
* Influences : %
* Date       : Dec 1995
* Author     : Lothar Schnell
* Last Change: June 1999 (Holger Hennes)
*********************************************************************/
{
	window W(600,600,"View Gauge");
	W.set_node_width(4);
	W.set_redraw(redraw_funcm);

	list<point> pl=G.vertices(origin); 
	list<point> pltmp;
	list<bool> tl;
	for (int i=1;i<=G.size();i++) tl.append(G.conetype(i));
	string tmp;
	mixgauge Gtmp;

	string file_name="mixgauge";
	color col(0,0,255);					//color of the origin

	W.buttons_per_line(3);

	W.button("Save Gauge",1);
	W.button("Load Gauge",2);
	W.button("Create Gauge",3);
	W.button("Refresh",8);
	W.button("Close",9);

	W.display(650,200);

	Wp = &W;
	Mp = &G;
	redraw_funcm();


	for(;;) 
	{ 
		int but = W.read_mouse();

		if (but == 1)
		{
			panel PSave("Save Gauge");
			PSave.string_item("file:",file_name);
			PSave.text_item(" ");
			PSave.text_item(" ");
			PSave.buttons_per_line(2);
			int bScancel = PSave.button("cancel");
			int bSok = PSave.button("ok");
			int butS = PSave.open(780,200);
			if (butS == bScancel)
			{ 
				file_name = "mixgauge";
				continue;
			} else if (butS == bSok)
			{
				ofstream F(file_name,ios::out);
				if (! F) 
				{ 
					tmp = string ("Can't open file \"%s\"!",file_name);
					W.message(tmp);
				} else SaveGauge(F,G);
			}
		}
		if (but == 2)
		{
			panel PLoad("Load Gauge");
			PLoad.string_item("file:",file_name);
			PLoad.text_item(" ");
			PLoad.text_item(" ");
			PLoad.buttons_per_line(2);
			int bLcancel = PLoad.button("cancel");
			int bLok = PLoad.button("ok");
			int butL = PLoad.open(780,200);
			if (butL == bLcancel)
			{ 
				file_name = "mixgauge";
				continue;
			} else if (butL == bLok)
			{
				ifstream F(file_name,ios::in);
				if (! F) 
				{ 
					tmp = string ("Can't open file \"%s\"!",file_name);
					panel PConfirm("Confirm");
					PConfirm.text_item(tmp);
					PConfirm.button("ok");
					PConfirm.open(880,200);
				} else 
				{ 
					W.clear();
					tl.clear();
					long act_pos;
					act_pos = F.tellg();
					while ((tmp!="begin") || F.eof()) F >> tmp;
					F >> tmp;
					F.seekg (act_pos);
					if (tmp!="{polygauge}" && tmp!="{mixgauge}") lola_error("no gauge file in ViewGauge",1611,1);
					else if (tmp=="{polygauge}") {
#ifdef NO_FLEX
						polygauge Gp(ReadGauge(F));
#else 
						polygauge Gp=ReadLPGauge(F).head(); //Flexversion
#endif
						pltmp=Gp.vertices(origin);
						for(int i=1;i<=Gp.size();++i) tl.append(false);
					}
					else if (tmp=="{mixgauge}")
#ifdef NO_FLEX 
						pltmp=ReadGauge(F,tl); // fuer Flexversion auskommentieren
#else
					Gtmp=ReadLMGauge(F).head(); //Flexversion
					else Gtmp=mixgauge(pltmp,tl);//	Flexversion
#endif
					W.clear();
#ifdef NO_FLEX
					Gtmp=mixgauge(pltmp,tl); //fuer Flexversion auskommentieren
#endif
					Wp = &W;
					Mp = &Gtmp;
					redraw_funcm();
				}
			}
		}
		if (but == 3)
		{
			int pressed=MOUSE_BUTTON(1);
			double winmin=-2, winmax=2;
			point p;
			list<point> pl;
			tl.clear();
			pltmp.clear();
			panel PCreate("Create a Gauge");
			PCreate.text_item("Initialize window: ");
			PCreate.text_item(" ");
			PCreate.real_item("winmin:",winmin);
			PCreate.real_item("winmax:",winmax);
			PCreate.text_item(" ");
			PCreate.text_item("---------------------------");
			int bCsym = PCreate.button("symmetrical");
			PCreate.button("non symmetrical");
			int butC = PCreate.open(200,300);
			W.init(winmin, winmax, winmin);
			winmax += winmax-W.ymax();
			W.init(winmin, winmax, winmin);
			W.draw_filled_node(p,col);
			pressed=W.read_mouse(p);
			while (pressed==MOUSE_BUTTON(1)) {
				W.draw_node(p,col);
				pl.append(p);
				tl.append(false);
				if (butC == bCsym) {
					W.draw_node(-p.xcoord(),-p.ycoord(),col);
					pltmp.append(p);
				}
				pressed=W.read_mouse(p);
			}
			if (butC == bCsym)
				forall(p,pltmp) {
					pl.append(point(-p.xcoord(),-p.ycoord()));
					tl.append(false);
				}
			list<point> C = CONVEX_HULL(pl);
			if (C.size() != pl.size()) {
				tmp = string ("Mixgauge is not convex!");
				W.confirm(tmp);
			} else {
				G=mixgauge(pl,tl);
				Mp = &G;
				redraw_funcm();
			}
		}
		if (but == 8) redraw_funcm();
		if (but == 9) break;
	}
}

void CreatePolyGauge()
/********************************************************************
* Purpose    : Create a polygauge or list of polygauges
* Returns    : %
* Influences : %
* Date       : Dec 1995
* Author     : Lothar Schnell
* Last Change: June 1999 (Holger Hennes)
*********************************************************************/
{
	bool gauge_line=false,symmetr=true;
	int pressed=MOUSE_BUTTON(1), with_ball=0;
	double winmin=-2, winmax=2;
	point p(0,0),q;
	string f_name="polygauge",out_tmp;
	list<point> g_pkt,g_pkt_sym,gauge_list;
	polygauge CG;
	color col(0,0,255);				//color for the origin

	// Open a panel for input of winmin, winmax
	panel PCreate("Create a Gauge");
	PCreate.text_item("Initialize window: ");
	PCreate.text_item(" ");
	PCreate.real_item("winmin:",winmin);
	PCreate.real_item("winmax:",winmax);
	PCreate.text_item(" ");
	PCreate.text_item("---------------------------");
	int bC_Ok = PCreate.button("Ok");
	PCreate.open(200,300);

	// Environment is set, open window for graphical input of the gauge
	window WC(600,600,"Create Gauge");
	WC.set_node_width(4);
	WC.set_show_coordinates(true);
	WC.buttons_per_line(3);
	WC.button("Save Gauge",4);
	WC.button("Show/Hide Unit Ball",5);
	WC.button("Clear",6);
	WC.button("Append to Gauge-List",7);
	WC.button("Save Gauge-List",8);
	WC.button("Clear Gauge-List",9);
	WC.button("Symmetr/Unsymmetr",10);
	WC.button("Refresh",11);
	WC.button("Close",12);

    WC.display(250,200);
    WC.init(winmin, winmax, winmin);
    winmax += winmax-WC.ymax();
	WC.init(winmin, winmax, winmin);
	WC.draw_filled_node(p,col);


	for(;;) {
		pressed=WC.read_mouse(p);
		Wc = &WC;
    	G_PKT = &g_pkt;


		if(pressed==MOUSE_BUTTON(1)) {			// Input in drawing area
			WC.draw_filled_node(p,col);
			g_pkt.append(p);
			if (symmetr) {
				q=point(-p.xcoord(),-p.ycoord());
				WC.draw_filled_node(q,col);
				g_pkt.append(q);
			}
		}

		if(pressed==4) {										// Save Gauge
			if(!g_pkt.empty()) {
				list<point> C = CONVEX_HULL(g_pkt);
				if (C.size() != g_pkt.size()) {
					out_tmp = string ("Polygauge is not convex!");
					WC.confirm(out_tmp);
					refresh_func();
				}
				else {
					CG=polygauge(g_pkt);
					Wp = &WC;
					Gp = &CG; 
					redraw_func();
					with_ball=1;

					panel PSaveG("Save Gauge");
					PSaveG.string_item("file:",f_name);
					PSaveG.text_item(" ");
					PSaveG.text_item(" ");
					PSaveG.buttons_per_line(2);
					int b_ok = PSaveG.button("ok");
					int b_cancel = PSaveG.button("cancel");
					int butSG = PSaveG.open(780,200);
					if (butSG == b_cancel) {
						f_name = "polygauge";
						redraw_func();
						continue;
					}
					else {
						if (butSG == b_ok) {
							ofstream FG(f_name,ios::out);
							if (! FG) { 
								out_tmp = string ("Can't open file \"%s\"!",f_name);
								WC.message(out_tmp);
							}
							else SaveGauge(FG,CG);
							f_name = "polygauge";
						}
					}
					redraw_func();
				}
			}
		}	// Save Gauge

		if(pressed==5) {								// Show/Hide Unit Ball
			if(gauge_line) {							// Unit Ball is visible --> Hide
				WC.clear(); 
				WC.draw_filled_node(origin,col);
				forall(q,g_pkt)
					WC.draw_filled_node(q,col);
				gauge_line=false;
			}
			else {										// Unit Ball is not visible --> Show
				CG=polygauge(g_pkt);
				g_pkt=CG.vertices();
				WC.draw_polygon(g_pkt);
				gauge_line=true;
			}
		} // Show/Hide Convex Hull

		if(pressed==6) {								// Clear
			WC.clear(); 
			gauge_line=false;
			with_ball=0;
			WC.draw_filled_node(origin,col);
			g_pkt.clear();
		}

		if(pressed==7) {								// Append to Gauge-List
			if(!g_pkt.empty()) {

				list<point> C = CONVEX_HULL(g_pkt);
				if (C.size() != g_pkt.size()) {
					out_tmp = string ("Polygauge is not convex!");
					WC.confirm(out_tmp);
					refresh_func();
				}
				else {
					CG=polygauge(g_pkt);
					DrawGauge(WC,CG);
					with_ball=1;
					forall(q,g_pkt) gauge_list.append(q);
					gauge_list.append(origin);
				}
			}
		}

		if(pressed==8) {								// Save Gauge-List
			if(!(gauge_list.empty())) {
				f_name="pg_list";
				panel PSaveGL("Save Gauge-List");
				PSaveGL.string_item("file:",f_name);
				PSaveGL.text_item(" ");
				PSaveGL.text_item(" ");
				PSaveGL.buttons_per_line(2);
				int b_ok = PSaveGL.button("ok");
				int b_cancel = PSaveGL.button("cancel");
				int butSG = PSaveGL.open(780,200);
				if (butSG == b_cancel) {
					f_name = "polygauge";
					continue;
				}
				else {
					if (butSG == b_ok) {
						ofstream FGL(f_name,ios::out);
						if (! FGL) { 
							out_tmp = string ("Can't open file \"%s\"!",f_name);
							WC.message(out_tmp);
						}
						else {
							FGL << "begin {polygaugelist}" << "\n";
							list_item l_it=gauge_list.first();
							do {
								list<point> help_list;
								do {
									help_list.append(gauge_list.inf(l_it));
									l_it=gauge_list.succ(l_it);
								}while(gauge_list.inf(l_it) != origin);

								CG = polygauge(help_list);
								SaveGauge(FGL,CG);
							}while((l_it=gauge_list.succ(l_it)) != nil);
							FGL << "end {polygaugelist}" << "\n";
						}
					}
				}
				f_name = "polygauge";
			}
			else {
				WC.confirm("No gauge in list");
				if (with_ball) DrawGauge(WC,CG);
				else refresh_func();
			}
		}

		if(pressed==9) {								// Clear Gauge-List
			gauge_list.clear();
		}

		if(pressed==10) {								// Symmetrical/Unsymmetrical
			if(symmetr)
				symmetr=false;
			else
				symmetr=true;
		}

		if(pressed==11) {								// Refresh
			if (with_ball) redraw_func();
			else refresh_func();
		}

		if(pressed==12) {		// Close
			break;
		}


	} // end for(;;)
	WC.close();
}	// Create Gauge


void DrawCreate(window& W, list<point>& g_pkt)
{
	point q;
	W.clear();
	color col(0,0,255);
	W.draw_filled_node(origin,col);
	forall(q,g_pkt)
		W.draw_filled_node(q,blue);
	return;
}