#include <LOLA/mixgauge.h>

ostream& operator << (ostream& out, const mixgauge &g) {
	g.vertices(point(origin)).print(out);
	return out << endl;
} 

istream& operator >> (istream& in, mixgauge &g) {
	list<point> L; 
	list<bool> T; 
	L.read(in,'\n'); 
	T.read(in,'\n'); 
	g = mixgauge(L,T); 
	return in;
}

mixgauge::mixgauge() {
   list<point> pl;
   list<bool> typ;
   pl.empty();
   typ.empty();
}

mixgauge::mixgauge(const list<point> &pl, const list<bool> &typ){
   bool typold=false;
   point q;
   list<point> pltmp;
   if (pl.size() != typ.size()) 
	   lola_error("mixgauge: must be the same size for points and conetyps",1717,1);
   list_item typ_it = typ.first();
   list_item it;
   for(it = pl.first(); it; it = pl.succ(it)) {
      point q = pl[it];
      if (!(typold && typ[typ_it])) {
        pltmp.append(q);
        seg_list.append(segment(origin,q));
        typ_list.append(typ[typ_it]);
      }
      typold=typ[typ_it];
      typ_it = typ.succ(typ_it);
   }
   if (typold && typ_list.head()) {
     typold = typ_list.pop(); 
     segment st = seg_list.pop(); 
     point p = pltmp.pop();
   }
   segment segf=seg_list.head(),s;
   segment segl=seg_list.tail();
   segment yseg(origin,point(0,1));
   double tmpangle;
   double angletst=-1;
   forall(s, seg_list) {   
      tmpangle=segf.angle(s);
	  if (tmpangle <0) tmpangle += 2*LEDA_PI;
	  if (tmpangle <= angletst) 
          lola_error("mixgauge: must be in counterclockwise order",1718,1);
      alpha_list.append(tmpangle);
      angletst=tmpangle;
   }
   if (segl.angle(yseg)<0)  {
	   lola_error("mixgauge: must be in counterclockwise order",1719,1);
   }
   it=alpha_list.succ(alpha_list.first());
   while (it != alpha_list.first()) {
	   if (fabs(alpha_list[alpha_list.pred(it)]-alpha_list[it])>LEDA_PI) 
		   lola_error("mixgauge: alpha > PI",1720,1);
	   it=alpha_list.cyclic_succ(it);
   }
   list<point> C = CONVEX_HULL(pltmp);
   if (C.size() != size()) lola_error("mixgauge: not convex",1721,1);
}

mixgauge::mixgauge(const mixgauge &G) {
   seg_list = G.seg_list;
   alpha_list = G.alpha_list;
   typ_list = G.typ_list;
}

mixgauge::~mixgauge() {
   list<segment> sempty;
   list<double> aempty;
   list<bool> tempty;
   seg_list = sempty;
   alpha_list = aempty;
   typ_list = tempty;
}

mixgauge& mixgauge::operator = (const mixgauge &G) {
   seg_list = G.seg_list;
   alpha_list = G.alpha_list;
   typ_list = G.typ_list;
   return *this;
}

point mixgauge::operator[](int c) {
   if ((c > size())||(c < 1)) lola_error("mixgauge: Conenumber is not correct",1722,1);
   list_item it=seg_list.first();
   for(int i=1; i<c; ++i) it = seg_list.cyclic_succ(it);
   return (seg_list[it].end());
}

bool mixgauge::conetype(const int& c) {
   if ((c > size())||(c < 1)) lola_error("mixgauge: Conenumber is not correct",1723,1);
   list_item it=typ_list.first();
   for(int i=1; i<c; ++i) it = typ_list.cyclic_succ(it);
   if (typ_list[it]) return true;
   else return false;
}

list<double> mixgauge::alphas() {
   list<double> result;
   double a;
   forall(a, alpha_list) {
      result.append(a);
   }
   return result;
}

list<segment> mixgauge::segments(const point& r) {
   list<segment> result;
   segment s;
   forall(s, seg_list) {
      point q(r.xcoord()+s.xcoord2(),r.ycoord()+s.ycoord2());
      result.append(segment(r,q));
   }
   return result;
}


list<segment> mixgauge::segments(const location& rl) {
   point tmp;
   tmp=rl.loc2point();
   return segments(tmp);
}

list<point> mixgauge::vertices(const point& r) const{
   list<point> result;
   segment s;
   forall(s, seg_list) {
     point tmp(s.xcoord2()+r.xcoord(),s.ycoord2()+r.ycoord());
     result.append(tmp);
   }
   return result;
}


list<point> mixgauge::vertices(const location& rl) const{
   point tmp = rl.loc2point();
   return vertices(tmp);
}

list<point> mixgauge::conepoints(const point& r,int c) {
   if ((c > size())||(c < 1)) lola_error("mixgauge: Conenumber is not correct",1724,1);
   list<point> result;
   list_item it=seg_list.first();
   point tmp;
   for(int i=1; i<c; ++i) it = seg_list.cyclic_succ(it);
   tmp=point(seg_list[it].xcoord2()+r.xcoord(),seg_list[it].ycoord2()+r.ycoord());
   result.append(tmp);
   it = seg_list.cyclic_succ(it);
   tmp=point(seg_list[it].xcoord2()+r.xcoord(),seg_list[it].ycoord2()+r.ycoord());
   result.append(tmp);
   return result;
}

list<point> mixgauge::conepoints(const location& rl,int c) {
   if ((c > size())||(c < 1)) lola_error("mixgauge: Conenumber is not correct",1725,1);
   point tmp = rl.loc2point();
   return conepoints(tmp,c);
}

list<segment> mixgauge::coneseg(const point& r,int c) {
   if ((c > size())||(c < 1)) lola_error("mixgauge: Conenumber is not correct",1726,1);
   list<segment> result;
   list_item it=seg_list.first();
   point tmp;
   for(int i=1; i<c; ++i) it = seg_list.cyclic_succ(it);
   tmp=point(seg_list[it].xcoord2()+r.xcoord(),seg_list[it].ycoord2()+r.ycoord());
   result.append(segment(r,tmp));
   it = seg_list.cyclic_succ(it);
   tmp=point(seg_list[it].xcoord2()+r.xcoord(),seg_list[it].ycoord2()+r.ycoord());
   result.append(segment(r,tmp));
   return result;
}

list<segment> mixgauge::coneseg(const location& rl,int c) {
   if ((c > size())||(c < 1)) lola_error("mixgauge: Conenumber is not correct",1727,1);
   point tmp = rl.loc2point();
   return coneseg(tmp,c);
}


int mixgauge::inCone(const point &r, const point &p, bool& unique) {
   int cone=0;
   int c=0;
   if (r==p) {
     unique=false;
     return 0;
   }
   while (!cone) {
     ++c;
     cone=conetest(r,c,p,unique);
   }
   return c;
}

int mixgauge::inCone(const location &rl, const location &l, bool& unique) {
   point tmp1 = rl.loc2point();
   point tmp2 = l.loc2point();
   return inCone(tmp1,tmp2,unique);
}

int mixgauge::conetest(const point &r, int c, const point &p, bool& unique) {
   if ((c > size())||(c < 1)) lola_error("mixgauge: Conenumber is not correct",1728,1);
   unique=true;
   if (r==p) {
     unique=false;
     return 0;
   }
   segment s=seg_list.head();
   point segfend(r.xcoord()+s.xcoord2(),r.ycoord()+s.ycoord2());
   segment segf(r,segfend);
   segment tmp(r,p);
   list_item it=alpha_list.first();
   double angletst=segf.angle(tmp);
   if (angletst <0) angletst += 2*LEDA_PI;   
   if ((c==size())&&(alpha_list.tail()<angletst)) return 1;
   for(int i=1; i<c; ++i) it = alpha_list.cyclic_succ(it);
   if ((alpha_list[it] == angletst)||(alpha_list[alpha_list.cyclic_succ(it)] == angletst)) unique=false;
   if ((alpha_list[it] <= angletst)&&(alpha_list[alpha_list.cyclic_succ(it)] >= angletst)) return 1;
   else return 0;
}

int mixgauge::conetest(const location &rl, int c, const location &l,bool& unique) {
   if ((c > size())||(c < 1)) lola_error("mixgauge: Conenumber is not correct",1729,1);
   point tmp1 = rl.loc2point();
   point tmp2 = l.loc2point();
   return conetest(tmp1,c,tmp2,unique);
}

list<point> mixgauge::pl_inCone(const point &r, int c, const list<point> &pl) {
   if ((c > size())||(c < 1)) lola_error("mixgauge: Conenumber is not correct",1730,1);
   list<point> result;
   point tmp;
   bool unique;
   result.clear();
   forall(tmp,pl) {
     if (conetest(r,c,tmp,unique)==1) result.append(tmp);
   }
   return result;
}

list<location> mixgauge::ll_inCone(const location &rl, int c, const list<location> &ll) {
   if ((c > size())||(c < 1)) lola_error("mixgauge: Conenumber is not correct",1731,1);

   list<location> result;
   location tmp;
   bool unique;
   result.clear();
   forall(tmp,ll) {
     if (conetest(rl,c,tmp,unique)==1) result.append(tmp);
   }
   return result;
}

double mixgauge::norm(const point &r, const point &p) {
   bool unique;
   if (r==p) return 0;
   int c=inCone(r,p,unique);
   if (!conetype(c)) {
     matrix M(2,2);
     vector b(p.xcoord()-r.xcoord(),p.ycoord()-r.ycoord());
     list<point> ext=conepoints(r,c);
     M(0,0)=ext.head().xcoord()-r.xcoord();
     M(1,0)=ext.head().ycoord()-r.ycoord();
     M(0,1)=ext.tail().xcoord()-r.xcoord();
     M(1,1)=ext.tail().ycoord()-r.ycoord();
     vector x=M.solve(b);
     return (fabs(x[0])+fabs(x[1]));
   } else return r.distance(p);
}


double mixgauge::norm(const location &rl, const location &l) {
   point tmp1 = rl.loc2point();
   point tmp2 = l.loc2point();
   return norm(tmp1,tmp2);
}


list<double> mixgauge::lin_describe(const point& r,int c) {
   list<double> result;
   if (!conetype(c)) {
     list<point> pltmp=conepoints(r,c);
     line l(pltmp.head(),pltmp.tail());   
     if (l.is_vertical()) {
       result.append(L_INF);  
       result.append(l.x_proj(1));
     }
     else {
       result.append(l.slope());  
       result.append(l.y_abs());
     }
   } else {
       result.append(0);
       result.append(0);
   }
   return result;
}


list<double> mixgauge::lin_describe(const location& rl,int c) {
   point tmp=rl.loc2point();
   return lin_describe(tmp,c);
}



point mixgauge::maxi(const point& r,int i) {
   segment s;
   double maximum;
   point result(seg_list.head().end());
   if (i==0) {
     maximum=seg_list.head().xcoord2();
     forall(s, seg_list) {
       if (s.xcoord2()>maximum) {
         maximum=s.xcoord2();
         result=s.end();
       }
     }
   } else if (i==1) {
     maximum=seg_list.head().ycoord2();
     forall(s, seg_list) {
       if (s.ycoord2()>maximum) {
         maximum=s.ycoord2();
         result=s.end();
       }
     }
   } else lola_error("mixgauge:max i must be 0 or 1",1732,1);

   result=point(result.xcoord()+r.xcoord(),result.ycoord()+r.ycoord());
   return result;
}

point mixgauge::mini(const point& r,int i) {
   segment s;
   double minimum;
   point result(seg_list.head().end());
   if (i==0) {
     minimum=seg_list.head().xcoord2();
     forall(s, seg_list) {
       if (s.xcoord2()<minimum) {
         minimum=s.xcoord2();
         result=s.end();
       }
     }
   } else if (i==1) {
     minimum=seg_list.head().ycoord2();
     forall(s, seg_list) {
       if (s.ycoord2()<minimum) {
         minimum=s.ycoord2();
         result=s.end();
       }
     }
   } else lola_error("mixgauge:mini i must be 0 or 1",1733,1);

   result=point(result.xcoord()+r.xcoord(),result.ycoord()+r.ycoord());
   return result;
}

point mixgauge::maxi(const location& rl,int i) {
   point tmp = rl.loc2point();
   return maxi(tmp,i);
}

point mixgauge::mini(const location& rl,int i) {
   point tmp = rl.loc2point();
   return mini(tmp,i);
}

double mixgauge::max_diff(int i) {
   if (i==0)  return maxi(origin,0).xcoord()-mini(origin,0).xcoord();
   else if (i==1) return maxi(origin,1).ycoord()-mini(origin,1).ycoord();
   else lola_error("mixgauge:max_diff i must be 0 or 1",1734,1);
   return 1;
}

mixgauge mixgauge::rotate(double alpha) {
   segment tmp;
   segment s;
   list<point> rot_list;
   list<bool> rot_typlist=typ_list;
   forall(s,seg_list) {
     tmp=s.rotate(alpha);
     rot_list.append(tmp.end());
   }
   mixgauge Gr(rot_list,rot_typlist);
   return Gr;
}

mixgauge mixgauge::dual() {
   segment yseg(origin,point(0,1));
   list<point> ext=vertices(origin);
   list<point> dual_list;
   list<bool> dual_typlist=typ_list;
   matrix M(2,2);
   vector b(1,1);
   list_item it=ext.succ(ext.first());

   while (it != ext.first())
   {
     M(0,0)=ext[ext.pred(it)].xcoord();
     M(0,1)=ext[ext.pred(it)].ycoord();
     M(1,0)=ext[it].xcoord();
     M(1,1)=ext[it].ycoord();
     vector p=M.solve(b);
     dual_list.append(point(p[0],p[1]));
     it = ext.cyclic_succ(it);
   }
   M(0,0)=ext.tail().xcoord();
   M(0,1)=ext.tail().ycoord();
   M(1,0)=ext.head().xcoord();
   M(1,1)=ext.head().ycoord();
   vector p=M.solve(b);
   segment stest(origin,point(p[0],p[1]));
   if (yseg.angle(stest) < 0) dual_list.append(point(p[0],p[1]));
   else dual_list.push(point(p[0],p[1]));
   mixgauge Gd(dual_list,dual_typlist);
   return Gd;
}

bool mixgauge::symmetrical() {
   if ((size()%2) != 0) return false;
   list<point> GP=vertices(origin);
   int i=1;
   list_item it_1=GP.first();
   list_item it_2=GP.first();
   while (i<=(size()/2)) {
     it_2=GP.succ(it_2);
     ++i;
   }
   i=1;
   while (i<=(size()/2)) {
     if ((GP[it_1].xcoord()!=(-1)*GP[it_2].xcoord())||(GP[it_1].ycoord()!=(-1)*GP[it_2].ycoord())) return false;
     else {
       it_1=GP.succ(it_1);
       it_2=GP.succ(it_2);
       ++i;
     }
   }
   return true;
}

mixgauge mixgauge::join(mixgauge& H) {
   double a;
   int C1=1;
   int C2=1;
   point p;
   bool round1=false,round2=false;
   list<bool> typlist;
   list<point> result;
   list<double> Gal;
   list<double> Hal;
   segment yseg(origin,point(0,1));
   list<double> al = alphas();
   segment stmp = coneseg(origin,1).head();
   double alp = yseg.angle(stmp);
   forall(a,al) Gal.append(a+alp);

   al = H.alphas();
   stmp = H.coneseg(origin,1).head();
   alp = yseg.angle(stmp);
   forall(a,al) Hal.append(a+alp);

   list_item G_it=Gal.first();
   list_item H_it=Hal.first();

   do {
     if (Hal[H_it] == Gal[G_it]) {
       if (!round1) p=coneseg(origin,C1).head().end();
       if ((conetype(C1)) || (H.conetype(C2))) round2=true;
       else round2=false;
       G_it = Gal.cyclic_succ(G_it);
       H_it = Hal.cyclic_succ(H_it);
       ++C1;
       ++C2;
     } else if (Gal[G_it] < Hal[H_it])  {
       if (!round1) p=coneseg(origin,C1).head().end();
       if (conetype(C1)) round2=true;
       else round2=false;
       G_it = Gal.cyclic_succ(G_it);
       ++C1;
     } else if (Hal[H_it] < Gal[G_it]) {
       if (!round1) p=H.coneseg(origin,C2).head().end();
       if (H.conetype(C2)) round2=true;
       else round2=false;
       H_it = Hal.cyclic_succ(H_it);
       ++C2;
     }
     if (!round1) {
       stmp = segment(origin,p);
       stmp = segment(origin,stmp.direction(),1.0);
       typlist.append(round2);
       result.append(stmp.end());
     }
     round1=round2;
     if (G_it == Gal.last()) {
       a = Gal.pop();
       Gal.push(a+2*LEDA_PI);
     }
     if (H_it == Hal.last()) {
       a = Hal.pop();
       Hal.push(a+2*LEDA_PI);
     }
   } while ((G_it != Gal.first()) || (H_it != Hal.first()));
   if ((typlist.tail() && typlist.head()) || (typlist.tail() && (Gal.first()!=Hal.first()))) {
     round1 = typlist.pop(); 
     p = result.pop();
   }
   mixgauge Gu(result,typlist);
   return Gu;
}


mixgauge mixgauge::unit() {
   point p;
   segment stmp;
   list<point> Vert=vertices(origin);
   list<point> result;
   forall(p,Vert) {
     stmp = segment(origin,p);
     stmp = segment(origin,stmp.direction(),1.0);
     result.append(stmp.end());
   }
   mixgauge Gu(result,typ_list);
   return Gu;
}

mixgauge mixgauge::scale(double scale) {
   point p;
   segment stmp;
   list<point> Vert=vertices(origin);
   list<point> result;
   if (scale <= 0) lola_error("mixgauge:scale must be greater than 0",1735,1);
   forall(p,Vert) {
     stmp = segment(origin,p);
     stmp = segment(origin,stmp.direction(),stmp.length()*scale);
     result.append(stmp.end());
   }
   mixgauge Gs(result,typ_list);
   return Gs;
}


bool mixgauge::operator == (const mixgauge &G) {
   if (seg_list.size() != G.seg_list.size()) return false;
   list_item trav1 = G.seg_list.first();
   for (list_item trav = seg_list.first(); trav; trav = seg_list.succ(trav)) {
      if (seg_list.contents(trav) != G.seg_list.contents(trav1)) return false;
      trav1 = G.seg_list.succ(trav);
   }
   return true;
}

