#include <LOLA/classlolagraph.h>
#include "graph_lp.h"


matrix lolagraph::shortest_path (){
/********************************************************************
* Purpose   : Returns the distance-matrix for a graph
* Returns   : matrix
* Influences: %
* Date      : Oct 1996
* Author    : Lothar Schnell
*********************************************************************/
	lolagraph  G=*this;
	int i=0,j=0;
	edge_array<double> cost(G);
	node_matrix<double> dist(G);
	matrix M(G.number_of_nodes(),G.number_of_nodes());
	node v,w;
	edge e;

	forall_edges(e,G) cost[e]=G[e];

	if (!ALL_PAIRS_SHORTEST_PATHS(G, cost, dist)) lola_error("Negative cycle in graph",2001,1);

	forall_nodes(v,G) {
		j=0;
		forall_nodes(w,G) {
			M(i,j)=dist(v,w);
			if (M(i,j)> L_INF) M(i,j)=L_INF;
			++j;
		}
		++i;
	}
	return  M;
}

list<sol_typ> lolagraph::med_nodes(double& objval){
/*********************************************************************
* Purpose    : Returns the position of the minimal nodes of a graph
* Returns    : list<sol_typ>
* Influences : objval
* Date       : Oct 1996
* Author     : Holger Hennes
**********************************************************************/
	objval = inf(first_node()).head();
	node v;
	int i=0;
	sol_typ S;
	list<sol_typ> A;
	list<int> pos;
	forall_nodes(v,*this) {
		if (objval < inf(v).head()) {
			i++;
		}
		else 
			if (objval == inf(v).head()) {
				pos.append(i);
				i++;
			}
			else {
				pos.clear();
				pos.append(i);
				objval = inf(v).head();
				i++;
			}
	}
	forall(i,pos) {
		S.source=i;
		S.target=i;
		S.alpha_s=0;
		A.append(S);
	}
	return A;
}

list<sol_typ> lolagraph::median(const char& wert, double& objval)
/*********************************************************************
* Purpose    : Returns solutions of the 1-median problem on nodes
* Returns    : list<sol_typ>
* Influences : %
* Date       : Oct 1996
* Author     : Holger Hennes
* Last Change: Feb 1998 (Holger Hennes)
**********************************************************************/
{
	while (!Is_Connected(*this)) {
		lola_message("Graph is not connected: LOLA could connect it",0,1,true);
		Make_Connected(*this);
	}
	lolagraph PG=*this;
	node v;
	int k = number_of_nodes(),i=0,j=0;
	double s;
	sol_typ S;
	double* weight=new double[k];
	matrix M(k, k);
	list<sol_typ> L_solution;

	objval=L_INF;
	M = PG.shortest_path();
	forall_nodes(v,PG) {
		weight[i++] = PG.inf(v).head();
	}
	forall_nodes(v,PG) {
		s = 0;
		for (i = 0; i<k; i++) s += (weight[i] * M(i,j) + weight[i] * M(j,i));
		if (objval >= s) {
			if (objval == s) {
				S.source=j;
				S.target=j;
				S.alpha_s=0;
				L_solution.append(S);
			}
			else {
				L_solution.clear();
				S.source=j;
				S.target=j;
				S.alpha_s=0;
				L_solution.append(S);
				objval = s;
			}
		}
		j++;
	}
	delete[] weight;
	if (wert=='u') objval *=0.5; 
	return L_solution;
}

list<segment> lolagraph::dist_node(double l1, double l2, double weight, double length)
/*********************************************************************
* Purpose    : returns a list with two segments which define the
*              function d_e(t) for node v, helpfunction for abs_cent
* Returns    : list<segment>
* Influences : %
* Date       : Nov 1996
* Author     : Karen Schulze
**********************************************************************/
{
	list<segment> L;
	point Schnittpunkt;
	point p1(0,l1*weight);
	point p2(1,(length+l1)*weight);
	point p3(0,weight*(length+l2));
	point p4(1,weight*l2);
	segment part1(p1,p2);
	segment part2(p3,p4);
	segment part0(0,0,0,0);
	if (part1.intersection(part2,Schnittpunkt) && Schnittpunkt.xcoord() != 0 && Schnittpunkt.xcoord() !=1) {
		segment part3(p1,Schnittpunkt);
		segment part4(Schnittpunkt,p4);
		L.append(part3);
		L.append(part4);
	} else {
        if (p1.ycoord()>=p3.ycoord())
			L.append(part2);
		else
			L.append(part1);
        L.append(part0); // dummy
	}
	return L;
}

list<point> lolagraph::dist_edge(int node_num, array<segment> d1, array<segment> d2)
/*********************************************************************
* Purpose    : returns a list with all optimal points t for one edge e
*		helpfunction for abs_cent
* Returns    : list<point>
* Influences : %
* Date       : Nov 1996
* Author     : Karen Schulze
**********************************************************************/
{
	int i=0,seggi;
	list<point> opt,may_opt;
	list<int> seg_index;
	double t=0,comp_val=0,comp_valy=0;
	double inf=L_INF;
	list_item L_it;
	point rem_point(0,0);
	for(i=0;i<node_num;i++)
		if (comp_val <= d1[i].ycoord1())
			if (comp_val < d1[i].ycoord1()) {// sofort wechseln
				seggi=i;comp_val=d1[i].ycoord1();comp_valy=d1[i].ycoord2();
			}
			else // unterscheiden sich die y-Koord am Ende der Segmente?
				if (comp_valy < d1[i].ycoord2()) { 
					seggi=i; comp_val=d1[i].ycoord1();comp_valy=d1[i].ycoord2();
				}
				point schnitt(t,comp_val);
				opt.append(schnitt);
				// Looking for next intersectionpoint between maximum segment and other segments, repeat till t=1
				while (t<1) {
					for(i=0;i<node_num;i++)
						if ((i!=seggi) && ( (d1[i].intersection(d2[seggi],schnitt)) || d1[i].intersection(d1[seggi],schnitt)) && (schnitt.xcoord()>t) ) { 
							may_opt.append(schnitt);
							seg_index.append(i);
						}else if ((i!=seggi) && ((d2[i].intersection(d1[seggi],schnitt) || d2[i].intersection(d2[seggi],schnitt))) && (schnitt.xcoord()>t))	{ 
							may_opt.append(schnitt);
							seg_index.append(i);
						}
						inf=1.1;
						for(L_it=may_opt.first();L_it;L_it=may_opt.succ(L_it)) {
							schnitt=may_opt.contents(L_it);
							i=seg_index.pop();
							// nur zu Segment i wechseln, falls Segment i nicht im Schnittpunkt aufhoert!!!
							if(schnitt.xcoord()<inf && d1[i].xcoord2()>schnitt.xcoord()) {
								inf=schnitt.xcoord();
								rem_point=schnitt;
								seggi=i;
							}
						}
						if (rem_point.xcoord()==0) {
							if (1<=d1[seggi].xcoord2())
								t=d1[seggi].ycoord2();
							else t=d2[seggi].ycoord2();
							point end_point(1,t);
							rem_point=end_point;
							t=1;
						} else {
							t=rem_point.xcoord();
						}
						if (!(opt.empty())) {
							schnitt=opt.tail();
							if (schnitt.ycoord()>=rem_point.ycoord() && schnitt.xcoord()!=rem_point.xcoord()) {
								if(schnitt.ycoord()>rem_point.ycoord())
									opt.clear();
								opt.append(rem_point);
							}
						} else {
							opt.append(rem_point);
						}
						rem_point=origin;
						may_opt.clear();
				}			
	return(opt);
}

list<sol_typ> lolagraph::abs_cent(double& objval)
/*********************************************************************
* Purpose    : returns solution of the abs. 1-center of a graph
* Returns    : list<sol_typ>
* Influences : %
* Date       : Nov 1996
* Author     : Karen Schulze
* Last Change: Feb 1999 (Holger Hennes)
**********************************************************************/
{
	while (!Is_Connected(*this)) {
		lola_message("Graph is not strictly connected: LOLA could connect it",0,1,true);
		Make_Connected(*this);
	}
	lolagraph U=*this;
	list<sol_typ> X;
	list<segment> D;
	list<point> t_list;
	sol_typ S;
	int node_num=U.number_of_nodes();
	matrix M(node_num,node_num);
	array<segment> d1_node(node_num);
	array<segment> d2_node(node_num);
	node v;
	edge e;
	double weight,length;
	int k,i=0,j;
	objval=L_INF;
	M = U.shortest_path();
	node_array<int> node2number(U);
	forall_nodes(v,U) node2number[v]=i++;
	forall_edges(e,U){
		i = node2number[U.source(e)];
		j = node2number[U.target(e)];
		length = U.inf(e);
		k=0;
		forall_nodes(v,U) {
			weight = U.inf(v).head();
			D =U.dist_node(M(i,k),M(j,k),weight,length);
			d1_node[k]=D.pop();
			d2_node[k]=D.pop();
			k++;
		}
		t_list=U.dist_edge(node_num,d1_node,d2_node);
		list_item L1_it;
		for(L1_it = t_list.first(); L1_it; L1_it = t_list.succ(L1_it)) {
			if (t_list.contents(L1_it).xcoord() == 0) {
				S.source = i;
				S.target = i;
				S.alpha_s  = 0;
			}
			if (t_list.contents(L1_it).xcoord() == 1) {
				S.source = j;
				S.target = j;
				S.alpha_s  = 0;
			}
			if (t_list.contents(L1_it).xcoord() !=0 && t_list.contents(L1_it).xcoord()!=1)	 { 
				S.source = i;	
				S.target = j;
				S.alpha_s  = t_list.contents(L1_it).xcoord();
			}
			if (objval>=t_list.contents(L1_it).ycoord())
				if (objval>t_list.contents(L1_it).ycoord()) {
					X.clear();
					X.append(S);
					objval=t_list.contents(L1_it).ycoord();
				}
				else if (!(X.tail().source == S.source && X.tail().target == S.target && X.tail().alpha_s==S.alpha_s))
					X.append(S);
		}
	}
	return X;
}

edge lolagraph::inz_edge(node& v, node& w)
/*********************************************************************
* Purpose    : Returns the edge between v and w
* Returns    : edge
* Influences : %
* Date       : Nov 1996
* Author     : Holger Hennes
* Last change: Karen Schulze
**********************************************************************/
{
	edge e;
	forall_adj_edges(e,v) {
		if (target(e)==w && source(e)==v) return e;
		if (target(e)==v && source(e)==w) return e;

	}
	forall_adj_edges(e,w) {
		if (target(e)==w && source(e)==v) return e;
		if (target(e)==v && source(e)==w) return e;
	}
	lola_error("The two nodes are not adjazent",2002,1);
	return e;
} //inz_edge

list<sol_typ> lolagraph::center(double& objval)
/*********************************************************************
* Purpose    : Returns the solution of the locale 1-center problem
* Returns    : list<sol_typ>
* Influences : %
* Date       : Oct 1996
* Author     : Holger Hennes
* Last Change: Feb 1999 (Holger Hennes)
**********************************************************************/
{
	while (!Is_Connected(*this)) {
		lola_message("Graph is not strictly connected: LOLA could connect it",0,1,true);
		Make_Connected(*this);
	}
	int i = 0, j = 0;
	node v;
	int k = number_of_nodes();
	list<double> row;
	node_weight weight;
	lolagraph PG = *this;
	double *w=new double[k];
	matrix M(k, k);
	M = PG.shortest_path();
	forall_nodes(v,PG) {
		w[i++] = PG.inf(v).head();
	}
	forall_nodes(v,PG) {
		for (i = 0; i<k; i++) {
			row.append(M(j,i) * w[i]);
		}
		objval = row.contents(row.max());
		weight.clear();
		weight.append(objval);
		PG.assign(v,weight);
		row.clear();
		j++;
	}
	return PG.med_nodes(objval);
}


list<sol_typ> lolagraph::N_median(int N,double& objval,list<int>& belongto)
/*********************************************************************
* Purpose    : Returns the solution of the local N-median problem,using cplex
* Returns    : list<sol_typ>
* Influences : %
* Date       : Sep 1997
* Author     : Karen Schulze 
* Last Change: Mar 1999 (Holger Hennes)
**********************************************************************/
{
	list<sol_typ> final;
	int nodnum=this->number_of_nodes();
	if (nodnum < N) lola_message("Number of nodes is smaller than N",0);
	belongto.clear();

#ifndef HAVECPLEX
	int i=0,j,cols=nodnum*nodnum;
	matrix D(nodnum);
	D = shortest_path();
	double *w=new double[nodnum];
	node v;
	forall_nodes(v,*this) {
		w[i++] = inf(v).head();
		belongto.append(0);
	}
	double *objfunc=new double[cols+1];
	for(i=0;i<nodnum;i++)
        for(j=0;j<nodnum;j++)
			objfunc[j+1+(i*nodnum)]=w[j]*D(i,j);
	
	int *belong=new int[cols+1], count=0;
	objval=N_median_solve(nodnum,cols,N,objfunc,belong);
	for(i=0;i<nodnum;i++) {
		bool done=false;
		for (j=0;j<nodnum;j++) {
			if (belong[i*nodnum+j]) {
				if (!done) {
					sol_typ S_final;
					S_final.source=S_final.target=i;
					S_final.alpha_s=0;
					final.append(S_final);
					done=true;
				}
				belongto.assign(belongto[j],count);
			}
		}
		if (done) count++;
	}
	delete[] objfunc;
	delete[] belong;
#else
	lolagraph U=*this;
	int row,col,i=0,j;
	matrix D(nodnum);
	D = U.shortest_path();
	array<double> w(nodnum);
	node v;
	forall_nodes(v,U) w[i++] = U.inf(v).head();
	//************* ROWS **********************
	list<char> equ;
	for(i=0;i<nodnum;i++) {
        equ.append('E'); // row2,row 2+nodnum,row 2+2*nodnum, ...
        for(j=0;j<nodnum-1;j++)
			equ.append('G');
	}
	equ.append('E');  // lastrow
	//************* obj ***********************
	vector obj(nodnum*nodnum);
	for(i=0;i<nodnum;i++)
        for(j=0;j<nodnum;j++)
			obj[j+(i*nodnum)]=w[j]*D(i,j);
	//************* Columns *******************
	matrix constraints(1+nodnum*nodnum,nodnum*nodnum);
	i=1;j=1;
	for(col=0;col<(nodnum*nodnum);col++) {
		for(row=0;row<(1+nodnum*nodnum);row++) {
			if (i==j && row==nodnum*nodnum) constraints(row,col)=1;
			if (i==j && (row)%nodnum!=0 && (row)/nodnum==(i-1))
				constraints(row,col)=1;
			if (j>i && (row)%nodnum==j-1 && (row)/nodnum==(i-1))
				constraints(row,col)=-1;
			if (j<i && (row)%nodnum==j && (row)/nodnum==(i-1))
				constraints(row,col)=-1;
			if ((row)%nodnum==0  && row/nodnum==(j-1) ) //i!=j
				constraints(row,col)=1;
		} // row
		if(j==nodnum) {j=1;i++;} // i und j sind indizes fuer x_ij
		else j++;
	} // col
	//************** rhs *********************
	vector rhs(nodnum*nodnum+1);
	rhs[nodnum*nodnum]=N;
	for(i=0;i<nodnum;i++)
		rhs[nodnum*i]=1;
	facilities EX;
	EX.write_mps(obj,constraints,rhs,equ,"N_graph.mps");
	//********* Aufruf *************************
	sol_typ S_final;
	const char* cplexcall   = "cplex < runcplex > N_sol.txt";
	ofstream output("runcplex",ios::out);
	output << "read N_graph.mps" << endl;
	output << "change problem MIP "  << endl;
	for(i=0;i<(nodnum*nodnum);i++) {
		output << "change type " << (i+1)  << " b" << endl;
		output << "change bounds " << (i+1) << " u 1" << endl;
	}
	output << "optimize" << endl;
	output << "display solution -"   << endl;
	output << "quit" << endl;
	output.close();
	system(cplexcall);
	//************* read solution ******************
	fstream input("N_sol.txt",ios::in);
	input.seekg(0,ios::beg);
	const int maxline =110;
	char line[maxline];
	int  col_number;
	j=0;
	while(input.getline(line,maxline)) {
		//************** objvalue **************
		if (line[27]=='O' && line[29]=='j' && line[33]=='i') {
			char col_string[20];
			for(i=0;i<20;i++) col_string[i]=line[38+i];
			objval=atof(col_string);
		}
		//************** x_ii ******************
		if (line[0]=='C' && line[1]=='O' && line[2]=='L') {
			char col_string[15]; // besser: Stellen von n^2
			for(i=0;i<15;i++) col_string[i]=line[3+i];
			col_number=atoi(col_string);
			if (col_number%(nodnum+1)==1) {
				S_final.source=col_number/(nodnum+1);
				S_final.target=S_final.source;
				S_final.alpha_s=0;
				final.append(S_final);
			}
		} // if line=col
	} // while
	//************** cleaning up ***************************
	system("rm N_sol.txt");
	system("rm N_graph.mps");
	system("rm runcplex");
	system("rm cplex.log");
	// welcher Knoten gehoert zu welcher L"osung?
	int* M_sol=new int[nodnum];
	int* sol=new int[N];
	i=0;
	forall(S_final,final)
		sol[i++]=S_final.source;
	for(i=0;i<nodnum;i++)
		M_sol[i]=N-1;
	int k;
	for(k=0;k<nodnum;k++)
		for(i=N-2;i>=0;i--)
			if (D(sol[i],k)<D(sol[M_sol[k]],k)) M_sol[k]=i;
	for( k=0;k<nodnum;k++) {
		belongto.append(M_sol[k]);
	}
#endif
	return final;
}

list<sol_typ> lolagraph::N_partitioning(const int N,double& objval,list<int>& belongto,bool typ)
/*********************************************************************
* Purpose    : Returns the solution of the local N-median(typ=1) or N-center(typ=0)
*              computes the solution by using the "node-partitioning" heuristic
* Returns    : list<sol_typ>
* Influences : %
* Date       : Sep 1997
* Author     : Karen Schulze
**********************************************************************/
{
	sol_typ S;
	list<sol_typ> solution;
	lolagraph G=*this,UG;
	const int nodnum=G.number_of_nodes();
	if (nodnum < N) lola_message("Number of nodes is smaller than N",0);
	matrix D(nodnum,nodnum);
	D=G.shortest_path();
	int i=0,j;
	int* nod_index = new int[N];
	double one_objval;
	double* weight = new double[nodnum];
	node v;
	node* number2node = new node[nodnum];
	edge e;
	forall_nodes(v,G) weight[i++]=G.inf(v).head();
	list<int> rembelongto=belongto; // remember different solutions
	double remobjval=L_INF;
	list<sol_typ> remsolution=solution;
	// Schleife um die Heuristik, mit 3 verschiedenen Anfangslosungen
	for(int scount=0;scount<3;scount++) {
		// Anfangsl"osungen 
		bool m_flag=1,m_break=0;
		for(i=0;i<nodnum;i++) { 
			if (i<N && scount==0) nod_index[i]=i; // Anfangslosung 1
			if (i>=(nodnum-N) && scount==1) nod_index[i-(nodnum-N)]=i; // Anfangslosung 2
			if (scount==2) // Anfangslosung 3 (nur wenn moeglich)
				if (2*N<=nodnum)
					if(m_flag && i<2*N) {nod_index[i/2]=i;m_flag=0;}
					else
						m_flag=1;
					else
						m_break=1;
		}// for i
		if (m_break) continue; // Anfangslosung 3 ex. nicht
		double* mindist= new double[nodnum];
		list<int> used;
		list<int>* M= new list<int>[N];
		bool first=true;
		double f_old;
		objval=L_INF;
		do {
			f_old=objval;
			//Untergraphen und 1-median/center bestimmen
			if (first) first=false; //nicht beim ersten Mal
			else  {
                first=false;
                for(i=0;i<N;i++) {
					UG.clear();UG=G;
					j=0;
					forall_nodes(v,UG)
						number2node[j++]=v;
					for(j=0;j<nodnum;j++)
						if (M[i].search(j)==nil) {
							v=number2node[j];
							forall_adj_edges(e,v)
								UG.del_edge(e);
							UG.del_node(v);
						}
                        // neue Knotenmenge
						if (typ)
							nod_index[i]=M[i].contents(M[i][UG.median('u',one_objval).pop().source]);
						else
							nod_index[i]=M[i].contents(M[i][UG.center(one_objval).pop().source]);
				}
			}
			// Berechne M_n:
			// initialisieren von M[i] und mindist mit 0
			for(j=0;j<nodnum;j++) {
                mindist[j]=L_INF;
                if (j<N) M[j].clear();
			}
			for(i=0;i<N;i++)
                for(j=0;j<nodnum;j++)
					if(mindist[j]>D(nod_index[i],j))
						mindist[j]=D(nod_index[i],j);
					used.clear();
					for(i=0;i<N;i++)
						for(j=0;j<nodnum;j++)
							if(D(nod_index[i],j)==mindist[j] && used.search(j)==nil) {
								used.append(j);
								M[i].append(j);
							}
							//Berechne neuen Funktionswert
							objval=0;
							if (typ)	// Median
								for(i=0;i<N;i++)
									forall(j,M[i])
										objval += weight[j]*D(nod_index[i],j);
							else // Center
								for(i=0;i<N;i++)
									forall(j,M[i])
										if(objval < weight[j]*D(nod_index[i],j))
											objval = weight[j]*D(nod_index[i],j);
		} while (objval<f_old);
		solution.clear();
		for(i=0;i<N;i++) {
			S.source=nod_index[i];
			S.target=S.source;
			S.alpha_s=0;
			solution.append(S);
        }
		// welcher Knoten gehoert zu welcher L"osung?
		belongto.clear();
		for(j=0;j<nodnum;j++)
			belongto.append(j);
		for(i=0;i<N;i++)
			forall(j,M[i]) {
				belongto.assign(belongto.get_item(j),i);
			}
		// Losung kopieren, um mit naechster Anfangslosung zu vergleichen
		if (objval<remobjval) {        // diese Anfanngslosung war besser
			remsolution.clear();
			remsolution=solution;
			remobjval=objval;
			rembelongto.clear();
			rembelongto=belongto;
        }
	}       // end for scount
	objval=remobjval;
	belongto.clear();
	belongto=rembelongto;
	return remsolution;
}

list<sol_typ> lolagraph::Nmed_exchange(const int N,double& objval,list<int>& belongto)
/*********************************************************************
* Purpose    : Returns the solution of the local N-median problem
*              computes the solution by using the "exchange" heuristic
* Returns    : list<sol_typ>
* Influences : %
* Date       : Nov 1997
* Author     : Karen Schulze
**********************************************************************/
{
	sol_typ S;
	list<sol_typ> solution;
	lolagraph G=*this;
	const int nodnum=G.number_of_nodes();
	if (nodnum < N) lola_message("Number of nodes is smaller than N",0);
	matrix D(nodnum,nodnum);
	D=G.shortest_path();
	int i=0,j,k,l; 
	int* sol=new int[N];
	int* newsol=new int[N];
	int* M_sol=new int[nodnum];
	int* M_newsol=new int[nodnum];
	// sol und newsol entsprechen der Menge X und X' (Stefans Script)
	// M_sol und M_newsol, enthalten zu jedem Knoten den Index i der Menge M_i(X)
	double new_objval;
	double* weight=new double[nodnum];
	node v;
	forall_nodes(v,G) weight[i++]=G.inf(v).head();
	list<int> rembelongto=belongto; // remember different solutions
	double remobjval=L_INF;
	list<sol_typ> remsolution=solution;
	// Schleife um die Heuristik, mit 3 verschiedenen Anfangslosungen
	for(int scount=0;scount<3;scount++) {
		// Anfangsl"osungen
		bool m_flag=1,m_break=0;
		for(i=0;i<nodnum;i++) {
			if (i<N && scount==0) {sol[i]=i;newsol[i]=i;} // Anfangslosung 1
			if (i>=(nodnum-N) && scount==1) {sol[i-(nodnum-N)]=i;newsol[i-(nodnum-N)]=i;} // Anfangslosung 2
			if (scount==2) // Anfangslosung 3 (nur wenn moeglich)
				if (2*N<=nodnum)
					if(m_flag && i<2*N) {sol[i/2]=i;newsol[i/2]=i;;m_flag=0;}
					else
						m_flag=1;
				else
					m_break=1;
		}// for i
		if (m_break) continue; // Anfangslosung 3 ex. nicht

		objval=L_INF;
		bool found;
		// ben"otigt wird M(X), erst initialisieren
		for(k=0;k<nodnum;k++)
			M_sol[k]=N-1;
		//jetzt durch Vergleichen der Distanzmatrixwerte bestimmen
		for(k=0;k<nodnum;k++)
			for(l=N-2;l>=0;l--)
				if(D(sol[l],k)<D(sol[M_sol[k]],k)) M_sol[k]=l;
				// Berechne f(X)
		new_objval=0;
		for(j=0;j<nodnum;j++)
			new_objval +=weight[j]*D(sol[M_sol[j]],j);
		// Step 3, Indizes wie im Script
		double delta_i,delta;
		int i_star=0; // nur zur Initialisierung, wird so hoffentlich nicht verwendet
		do {
			objval=new_objval;
			// Austauschpartner f"ur diese L"osung suchen
			for(j=0;j<nodnum;j++) {
				// Test, ob v_j in X
				found = false;
				for(i=0;i<N;i++)
					if(sol[i]==j) {
						found=true;break;
					}
				if (found) continue; // naechstes j
				// Bestimme delta(i*j)
				// ben"otigt wird M(X), erst initialisieren
				for(k=0;k<nodnum;k++)
					M_sol[k]=N-1;
				//jetzt durch Vergleichen der Distanzmatrixwerte bestimmen
				for(k=0;k<nodnum;k++)
					for(l=N-2;l>=0;l--)
						if(D(sol[l],k)<D(sol[M_sol[k]],k)) M_sol[k]=l;
				delta=L_INF;
				for(i=0;i<N;i++) {
					// Bestimme delta(ij)
					if(sol[i]==j) continue; // Versuch j mit j auszutauschen
					newsol[i]=j;
					// ben"otigt wird M(X'), erst initialisieren
					for(k=0;k<nodnum;k++)
						M_newsol[k]=N-1;
						//jetzt durch Vergleichen der Distanzmatrixwerte bestimmen
						for(k=0;k<nodnum;k++)
							for(l=N-2;l>=0;l--)
								if(D(newsol[l],k)<D(newsol[M_newsol[k]],k)) M_newsol[k]=l;
						delta_i=0;
						for(k=0;k<nodnum;k++) {
							// Bestimme delta(ijk)
							if (M_sol[k]==M_newsol[k] && M_sol[k]!=i)
								continue; // delta=0, siehe Script
							if (M_newsol[k]==i) {
								delta_i +=weight[k]*(D(k,j)-D(k,sol[M_sol[k]]));
								continue;
							}
							if (M_sol[k]==i && M_newsol[k]!=i)
								delta_i +=weight[k]*(D(k,M_newsol[k])-D(k,sol[M_sol[k]]));
						} //for(k)
						// Bestimme min(delta_i)
						if(delta_i<delta) {
							delta=delta_i;i_star=i;
						}
						//newsol wieder in Ausgangsl"osung
						newsol[i]=sol[i];
                } // i
				// Bestimme neue L"osungsmenge, tausche i* mit j aus
				if(delta>=0) continue; // keine Verbesserung mit diesem j
				newsol[i_star]=j;
				sol[i_star]=j;
			} // for j
			// neues M(X') berechnen, verwende dazu Variable von M(X)
			for(k=0;k<nodnum;k++)
				M_sol[k]=N-1;
			//jetzt durch Vergleichen der Distanzmatrixwerte bestimmen
			for(k=0;k<nodnum;k++)
				for(l=N-2;l>=0;l--)
					if(D(sol[l],k)<D(sol[M_sol[k]],k)) M_sol[k]=l;
			// Berechne f(X')
			new_objval=0;
			for(j=0;j<nodnum;j++)
				new_objval +=weight[j]*D(sol[M_sol[j]],j);
		} while(new_objval<objval);
		solution.clear();
		for(i=0;i<N;i++) {
			S.source=sol[i];
			S.target=S.source;
			S.alpha_s=0;
			solution.append(S);
        }
		// welcher Knoten gehoert zu welcher L"osung?
		belongto.clear();
		for(k=0;k<nodnum;k++)
			M_sol[k]=N-1;
		for(k=0;k<nodnum;k++)
			for(i=N-2;i>=0;i--)
				if (D(sol[i],k)<D(sol[M_sol[k]],k)) M_sol[k]=i;

		for(k=0;k<nodnum;k++) {
			belongto.append(M_sol[k]);
        }
		// Losung kopieren, um mit naechster Anfangslosung zu vergleichen
		if (objval<remobjval) {        // diese Anfanngslosung war besser
			remsolution.clear();
			remsolution=solution;
			remobjval=objval;
			rembelongto.clear();
			rembelongto=belongto;
        }
	}       // end for scount


	objval=remobjval;
	belongto.clear();
	belongto=rembelongto;
	return remsolution;
}

list<sol_typ> lolagraph::N_greedy(const int N,double& objval,list<int>& belongto,bool typ)
/*********************************************************************
* Purpose    : Returns the solution of the local N-median problem(typ=1)
*	       or local N-center problem (typ=0) 
*              computes the solution by using the "greedy-heuristic"
* Returns    : list<sol_typ>
* Influences : %
* Date       : Nov 1997
* Author     : Karen Schulze
**********************************************************************/
{
	sol_typ S;
	list<sol_typ> solution;
	lolagraph G=*this;
	const int nodnum=G.number_of_nodes();
	if (nodnum < N) lola_message("Number of nodes is smaller than N",0);
	matrix D(nodnum,nodnum);
	D=G.shortest_path();
	int i=0,n,j,k,j_star;
	int* nod_index= new int[N];
	int* M_sol= new int[nodnum];
	// M_sol  enthealt zu jedem Knoten den Index i der Menge M_i(X)
	double* weight= new double[nodnum];
	double min_objval;
	node v;
	bool found;
	forall_nodes(v,G)
		weight[i++]=G.inf(v).head();
	// Anfangsl"osung
	if (typ) nod_index[0]=G.median('u',objval).head().source;
		else nod_index[0]=G.center(objval).head().source;
	for(n=1;n<N;n++) { // n wie im Script
	    min_objval=L_INF;
        for(j=0;j<nodnum;j++) {
			found=false;
			// Test, ob v_j bereits in L"osung ist
			for(i=0;i<n;i++)
				if(nod_index[i]==j) {
					found=true;break;
				}
            if(found) continue; // naechstes j
			// f(X+v_j) berechnen
			nod_index[n]=j;
			// zuerst wieder M(X)
			for(k=0;k<nodnum;k++) {
				M_sol[k]=n;
                // for(k=0;k<nodnum;k++)
				for(i=n-1;i>=0;i--)
					if (D(nod_index[i],k)<D(nod_index[M_sol[k]],k)) M_sol[k]=i;
			}
			// jetzt MedianFunktionswert
			objval=0;
			if (typ) // MedianFunktionswert
				for(k=0;k<nodnum;k++) 
					objval +=weight[k]*D(nod_index[M_sol[k]],k);
			else // jetzt CenterFunktionswert
				for(k=0;k<nodnum;k++)  
					if (objval< weight[k]*D(nod_index[M_sol[k]],k))
						objval =weight[k]*D(nod_index[M_sol[k]],k);
					// mit bisherigem minimalen objval vergleichen...
				if(objval < min_objval) {j_star=j;min_objval=objval;}
		} // for j
        // tatsaechliche neue L"osung
        nod_index[n]=j_star;
	} // for n
	// richtigen Zielfu-wert an Hauptprogramm uebergeben
	objval=min_objval;
	// L"osungsformat
	for(i=0;i<N;i++) {
        S.source=nod_index[i];
        S.target=S.source;
        S.alpha_s=0;
        solution.append(S);
    }
	// welcher Knoten gehoert zu welcher L"osung?
	for(k=0;k<nodnum;k++)
		M_sol[k]=N-1;
	for(k=0;k<nodnum;k++)
		for(i=N-2;i>=0;i--)
			if (D(nod_index[i],k)<D(nod_index[M_sol[k]],k)) M_sol[k]=i;
	for(k=0;k<nodnum;k++) {
		belongto.append(M_sol[k]);
	}
	return solution;
}

