#ifndef HAVECPLEX

#include <stdio.h>
#include "graph_lp.h"
#include <lpkit.h>

double N_median_solve(int nodnum, int cols, int N, double *objfunc, int *belong)
/*********************************************************************
* Purpose    : Computes LP for lp_solve and N|G|.|d(V,V)|sum and let 
*              it be solved
* Returns    : objective (double)
* Influences : %
* Date       : Feb 99
* Author     : Holger Hennes
* Last Change: Mar 99
*********************************************************************/
{
	lprec *lp;
	int i,j,count;
	double objval;

	lp=make_lp(cols+1,cols);
	set_obj_fn(lp, objfunc);
	for(j=1;j<=cols;j++) {
		set_int(lp, j, 1);
		set_upbo(lp, j, 1);
		set_lowbo(lp, j, 0);
		if(j<=nodnum) {
			set_constr_type(lp,j, EQ);
			set_rh(lp,j,1);
			for (i=1;i<=nodnum;i++)
				set_mat(lp,j,nodnum*(i-1)+j,1);
		}
	}
	for (j=1;j<=nodnum;j++) {
		count=0;
		for (i=1;i<=nodnum;i++)
			if(i!=j) {
				count++;
				set_mat(lp,nodnum+(nodnum-1)*(j-1)+count,nodnum*(j-1)+j,-1);
				set_mat(lp,nodnum+(nodnum-1)*(j-1)+count,nodnum*(j-1)+i,1);
			}
	}
	set_rh(lp,cols+1,N);
	for (i=1;i<=nodnum;i++)
		set_mat(lp,cols+1,nodnum*(i-1)+i,1);
	set_constr_type(lp,cols+1, EQ);
	solve(lp);
	objval = (double)lp->best_solution[0];
	for (i=1;i<=lp->columns;i++) {
		belong[i-1]=(int)lp->best_solution[lp->rows+i];
	}
	delete_lp(lp);
	return objval;
}


double* Q_medpar_solve(int Q, double* rhs, double *con1, double *con2, double* lowbo, double* upbo, double *objfunc, int minmax)
/*********************************************************************
* Purpose    : Computes LP for lp_solve and 1|G|.|d(V,G)|Q-sum and let 
*              it be solved
* Returns    : objective (double)
* Influences : %
* Date       : Feb 99
* Author     : Holger Hennes
*********************************************************************/
{
	int i;
	double* objval;

	lprec *lp;

	objval=calloc(3,sizeof(double));
	lp=make_lp(Q,2);
	set_obj_fn(lp, objfunc);	//objective function
	set_rh_vec(lp,rhs);			//right hand side
	for(i=0;i<Q;i++) {			//constraints
		set_mat(lp,i+1,1,con1[i]);
		if (i<=1) {
			set_lowbo(lp,i+1,lowbo[i]);
			set_upbo(lp,i+1,upbo[i]);
		}
	}
	if (minmax==1) set_maxim(lp);
	solve(lp);
	objval[0] = (double)lp->best_solution[lp->rows+1];
	objval[1] = (double)lp->best_solution[lp->rows+2];
	delete_lp(lp);
	return objval;
}

#endif
