/*#include <LOLA/graph_restriction.h>

list<sol_typ> graph_restriction :: abs_median(double& objval,lolaundirected& U)
********************************************************************
* Purpose   : returns solution of the absolute restrictive 1-median of a graph
* Returns   : list<sol_typ>
* Influences: %
* Date      : July 97
* Author    : Karen Schulze
**********************************************************************
{
    list<int> del_list;
    lolaundirected Gt=create_restrG(U,del_list);
    int del, n=Gt.number_of_nodes();
    if (del_list.empty()) return Gt.abs_med(objval);
    else {
        matrix M(n, n);
  	M = Gt.shortest_path();
        forall(del,del_list) M(del,del) = L_INF;
        list<sol_typ> L=median(objval,U);
        return Gt.edge_append(L,M,objval);
}
}
list<sol_typ> graph_restriction::median(double& objval,lolagraph& G)
********************************************************************
* Purpose   : returns solution of the local restrictive 1-median of a graph
* Returns   : list<sol_typ>
* Influences: %
* Date      : 11-13-96
* Author    : Lothar Schnell
**********************************************************************
{
list<int> del_list;
lolagraph Gt=create_restrG(G,del_list);
int del, n=Gt.number_of_nodes();
if (del_list.empty()) {if (Gt.is_undirected()) return Gt.median('u',objval); 
			else return Gt.median('m', objval);}
else {
  array<double> w(n);
  int i=0,j=0;
  double s;
  node_weight S;
  node v;
  matrix M(n, n);
  M = Gt.shortest_path();
  forall(del,del_list) M(del,del) = L_INF;
  forall_nodes(v,Gt) {
    w[i++] = Gt.inf(v).head();
  }
  forall_nodes(v,Gt) {
    s = 0;
    S.clear();
    for (i = 0; i<n; i++) s += (w[i] * M(i,j) + w[i] * M(j,i));
    S.append(s);
    Gt.assign(v,S);
    j++;
  }
  list<sol_typ> solution=Gt.med_nodes(objval);
  if (Gt.is_undirected()) objval=0.5*objval;
  return solution;
}
}

lolagraph graph_restriction::create_restrG(lolagraph& G, list<int>& del_list)
********************************************************************
* Purpose   : returns the new lolagraph with the restrictions
* Returns   : lolagraph&
* Influences: %
* Date      : 11-13-96, July 97
* Author    : Lothar Schnell, Karen Schulze
**********************************************************************
{
graph_restriction EL=*this;
edge e;
bool undir=false;
if (G.is_undirected()) {undir=true;G.make_directed();}
lolagraph Gr=G;
node v, w, vtmp;
double e_weight;
int i,n=Gr.number_of_nodes();
array<int> In_inf(n);
array<int> Out_inf(n);
for(i=0;i<n;i++) {
  In_inf[i]=0;
  Out_inf[i]=0;
}
edge_segment restr;
node_weight nodweights;
i=0;
node* number2node = new node[Gr.number_of_nodes()+1];
forall_nodes(v,Gr) number2node[i++] = v;

forall(restr,EL) {
  v=number2node[restr.source];
  w=number2node[restr.target];
  e=Gr.inz_edge(v,w);
  e_weight=Gr.inf(e);
  if (restr.alpha_s > 0 || restr.alpha_t < 1.0) {
    Gr.del_edge(e);
    if (restr.alpha_s > 0) {
     nodweights.clear();
     nodweights.append(0.0);
     vtmp = Gr.new_node(nodweights);
     Gr.new_edge(v,vtmp,e_weight*restr.alpha_s);
     v = vtmp;
    }
    if (restr.alpha_t < 1.0) {
      nodweights.clear();
     nodweights.append(0.0);
     vtmp = Gr.new_node(nodweights);
     Gr.new_edge(vtmp,w,e_weight*(1.0-restr.alpha_t));
     w = vtmp;
    }
    Gr.new_edge(v,w,e_weight*(restr.alpha_t-restr.alpha_s));
  }
  
  if (restr.alpha_s==0) ++Out_inf[restr.source];
  if (restr.alpha_t==1) ++In_inf[restr.target];
}
i=0;
forall_nodes(v,G) {
  //if ((In_inf[i]==G.indeg(v)) && (Out_inf[i]==G.outdeg(v))) del_list.append(i);
  if ((In_inf[i]+Out_inf[i]==G.indeg(v)+G.outdeg(v)) ) del_list.append(i);
  i++;
}
if (undir) {Gr.make_undirected();G.make_undirected();}
return Gr;
}

void graph_restriction ::ReadGRestr(ifstream& file)
*********************************************************************
* Purpose   : Reads a restriction file for a lolagraph
* Returns   : graph_restriction 
* Influences:
* Date      : 11-13-96
* Author    : Lothar Schnell
*********************************************************************
{
 long act_pos;
 bool app,first=true;
 string tmp;
 int s,t;
 double as,at;
 edge_segment restr,tmp_r;
 graph_restriction EL=*this,tmp_EL;
 EL.clear();

 if (!file) lola_error("Can't open file !",5011,1);
 while ((tmp != "begin") && !file.eof()) file >> tmp;
 file >> tmp;
 if (tmp != "{g_restriction}") lola_error("Is not a graph restriction file!",5012,1);
 act_pos = file.tellg();
 file >> tmp;
 do {
  file.seekg (act_pos);
  file >> s >> t >> as >> at;
  if ((as < 0) || (at > 1.0)) lola_error("Restriction is not correct!",5017,1);
  restr.source=s-1;
  restr.target=t-1;
  restr.alpha_s = as;
  restr.alpha_t = at;
  if (first) EL.append(restr);
  else {
   app = true;
   tmp_EL.clear();
   tmp_EL = EL;
   list_item EL_it=EL.first();
   forall(tmp_r,tmp_EL) {
    if ((tmp_r.source == restr.source)&&(tmp_r.target == restr.target)) {
     if ((as >= tmp_r.alpha_s) && (at <= tmp_r.alpha_t)) app=false;
     else {
      EL.del_item(EL_it);
      if ((tmp_r.alpha_s<as)&&(tmp_r.alpha_t<at)) restr.alpha_s=tmp_r.alpha_s;
      else if ((tmp_r.alpha_s>as)&&(tmp_r.alpha_t>at)) restr.alpha_t=tmp_r.alpha_t;
     }
    }
    EL_it = EL.succ(EL_it);
   } //for
   if (app) EL.append(restr);
  } //if !first
  first = false;
  act_pos = file.tellg();
  file >> tmp;
 } while ((tmp != "end") && !file.eof());
*this=EL;
 return;
}
*/ 
