#include <LOLA/lolaundirected.h>

#ifdef UNIX
int compare (const edge_segment& a, const edge_segment& b) {
	if(a.source > b.source) return(-1);
	if(a.source < b.source) return(1);
	if(a.source == b.source && a.target > b.target) return(-1);
	if(a.source == b.source && a.target < b.target) return(1);
	if(a.alpha_s > b.alpha_s) return(-1);
	if(a.alpha_s < b.alpha_s) return(1);
	if(a.alpha_t > b.alpha_t) return(-1);
	if(a.alpha_t < b.alpha_t) return(1);
	return(0);
}
#endif

vector lolagraph::compute_Min(matrix& F,list<int>& J_list)
/*********************************************************************
* Purpose   : assigns the row-minima of matrix f to vector Min, comparing
* 	      only columns(nodes) mentioned in J_list
*		helpfunction for lexlocation
* Returns   : vector
* Influences: %
* Date      : Mar 1997
* Author    : Karen Schulze
*********************************************************************/
{
int i,j,rown;
rown=F.dim1();
vector Min(rown);
for(j=0;j<rown;j++)	//Min-Vector mit Infinity initialisieren
	Min[j]=L_INF;
forall(i,J_list)	// nur Spalten(Knoten) aus J_list beim Vergleich beruecksichtigen
	for(j=0;j<rown;j++) 
		if (F(j,i)<Min[j]) Min[j]=F(j,i);
return(Min);
}



vector lolagraph::x_median(matrix& D,int v_i, int v_j,double length)
/*********************************************************************
* Purpose   : calculates the objective values for a point x along edge e,t=.5
* Returns   : matrix
* Influences: %
* Date      : Mar 1997
* Author    : Karen Schulze
*********************************************************************/
{
lolagraph U=*this;
int q,rown=U.inf(U.first_node()).length(),k=0;
vector f_x(rown);
node v;
node_weight weight;
forall_nodes(v,U) {
	weight=U.inf(v);
	for(q=0;q<rown;q++)
		(D(k,v_i)<D(k,v_j) ? f_x[q] += weight.pop() *(D(k,v_i) + 0.5*length) : f_x[q] += weight.pop() *(D(k,v_j) + 0.5 * length));
	++k; 
	}
return f_x;
}

list<int> lolagraph::cancel_list(list<int>& A, list<int>& B)
/**********************************************
* Purpose: evaluates list A\B,
*		helpfunction for lexlocation 
**********************************************/
{
list_item cancel;
forall_items(cancel,B) {
        if (A.search(B.contents(cancel)) !=nil) A.del_item(A.search(B.contents(cancel))); // nur zur Sicherheit
	}
return(A);
}

list<edge_segment> lolagraph::lexlocation(list<vector>& objval,bool dir, bool abs, bool center )
/*********************************************************************
* Purpose   : returns the lex-optimal solutions for all permutations of 
*	      a multicriterial graph
* Returns   : list<edge_segment> 
* Influences: list<vector>& objval 
* Date      : Mar 1997
* Author    : Karen Schulze
*********************************************************************/
{
	while (!Is_Connected(*this)) {
		lola_message("Graph is not connected: LOLA is could connect it",0,1,true);
		Make_Connected(*this);
	}
	lolagraph G=*this;
list<edge_segment> solution;
list<int> lexsolution;
edge_segment S;
int coln=G.number_of_nodes(),rown=G.inf(G.first_node()).length(),i,j,k;
matrix D(coln,coln);
node v,w;
edge eddi;
node* Int2Node = new node[coln];
// In Int2Node sind Zeiger auf die Knoten im Graphen zum zugehoerign Integerwert
i=0;
forall_nodes(v,G) Int2Node[i++] = v;
list<sol_typ> Onecenter;
sol_typ ST;
list<edge_segment> Qcenter; // nur fuer absolutes ungerichtetes Center
if (!dir && center && abs) {
		// Spaltenzahl der Matrix F bestimmen:
                double objval;
		node_weight gewichtlist;
                for(i=0;i<rown;i++) {
			lolaundirected U=G;
			forall_nodes(v,U) {
				gewichtlist=U.inf(v); 
				for(j=0;j<i;j++)
					gewichtlist.pop();
				U.assign(v,gewichtlist);
				}
			U.print();
                       	Onecenter=U.abs_cent(objval);
			Onecenter.print();
			forall(ST,Onecenter){	// Eindeutigkeit bewahren!!
				S.source=ST.source;
				S.target=ST.target;
				S.alpha_s=ST.alpha_s;
				S.alpha_t=ST.alpha_s;
				if (Qcenter.search(S)==nil) Qcenter.append(S);
				}
                	}
		coln=Qcenter.length(); // Spaltenzahl
		}
matrix F(rown,coln);
vector F_temp(rown);
if (!dir)	{G.make_undirected();
		D = G.shortest_path();
		if (!center){
			F=G.F_median(D,dir);
			} else {
			if (abs) 	
				for(j=0;j<coln;j++) {	// Matrix F, nicht fuer Knoten
					S=Qcenter.contents(Qcenter.get_item(j));
					v=Int2Node[S.source];
					w=Int2Node[S.target];
			 		if (S.source!=S.target) {
						eddi=G.inz_edge(v,w);
						F_temp=G.x_center(D,S.source,S.target,G.inf(eddi),S.alpha_s);		
						}else
						F_temp=G.x_center(D,S.source,S.target,0,0);	
					for(i=0;i<rown;i++)
						F(i,j)=F_temp[i];
					} // for j
				else
				F=G.F_center(D,dir);	
			} // !center 
	 } else {
		G.make_directed();
		D = G.shortest_path();
		if (!center)
			F=G.F_median(D,dir);
			else
			F=G.F_center(D,dir);
		}
// Step 3: (Spalten in F paarweise verschieden?)
list<int> Notdistinct[2],Lex;
for(j=0;j<coln;j++) 
	for(i=coln-1;i>j;i--) 
		if (F.col(j)==F.col(i)) {Notdistinct[0].append(j);Notdistinct[1].append(i); break;}  
list<int> Strike=Notdistinct[0];
Notdistinct[1].print("  ");
// Liste Lex bilden, ohne doppelte Spalten(stehen in Liste Strike)
for(i=0;i<coln;i++)
	if (!Strike.empty() && i==Strike.head()) Strike.pop();
	   else 	      Lex.append(i);
// Step 4:
Strike.clear();
list<int> *Minlist = new list<int>[coln];
vector Min(rown);
Min=compute_Min(F,Lex);
forall(i,Lex) {
	for(j=0;j<rown;j++)	// Step 4.(a)
		if (F(j,i)==Min[j]) Minlist[i].append(j);
	if (Minlist[i].empty()) Strike.append(i); // Step 4.(b)
	}
Lex=G.cancel_list(Lex,Strike);  // Step 4.(b) endgueltig
// Step 5:
bool ispart=true,skip=true;
list<int> qlist,Q_list,J_list,J_i,lexdelete;	// Listen, die in Step 6 gebraucht werden
for(i=0;i<rown;i++)	// qlist initialisieren mit {1..Q}
	qlist.append(i);
// Step 6:
forall(i,Lex) {
    J_list=Lex;  // Step 6.(a)
    Q_list=qlist; // Step 6.(b)
    J_list=G.cancel_list(J_list,lexdelete);
    do {
	Min=compute_Min(F,J_list); 
	forall(j,J_list) {	// Step 6.(c)
		Minlist[j].clear();
		forall(k,Q_list)  
			if (F(k,j)==Min[k]) Minlist[j].append(k);
		}
	if (Minlist[i].empty()) {skip=false;lexdelete.append(i);
		} // Step 6.(d)
		else {
		J_i.clear();
		forall(j,J_list){	// Step 6.(e) 
			// Minlist[i] in Minlist[j] enthalten??
			ispart=true;
			forall(k,Minlist[i])
				if (Minlist[j].search(k)==nil) {ispart=false; break;} 
			if (ispart) J_i.append(j);
			}
		j=J_i.pop();
		if (j==i && J_i.empty()) {  // Step 6.(f)
			S.source=i;
			S.target=i;
			S.alpha_s=0;
			solution.append(S);
			lexsolution.append(i);
			objval.append(F.col(i));
			objval.append(F.col(i));
			skip=false;
			} else {  // Step 6.(g)
			J_i.push(j);
			J_list=J_i;
			skip=true;
			Q_list=G.cancel_list(Q_list,Minlist[i]);
			}
		} //else sechs.d
    } while (skip);
    }// forall 6.
// Step 7: (gleiche Spalten, die ab Step 3 nicht beruecksichtigt wurden, zur Loesung zufuegen
// falls identische Spalte in der Loesungsmenge ist, ausserdem fuer 1/G/./d(V,G)/Q-Sumlex Kanten anhaengen 
bool adjacent=false;
list_item solelem;
// erst identische Knoten zufuegen, falls die auch zur Loesungsmenge gehoeren 
forall(i,Notdistinct[0]) {
	j=Notdistinct[1].pop();
	solelem=lexsolution.search(j); // ist identischer Knoten in Loesungsliste?
	if (solelem != nil) {
		S.source=i;
		S.target=i;
		S.alpha_s=0;
	 	solution.append(S);
		lexsolution.append(i);
		objval.append(F.col(i));
		objval.append(F.col(i));
		}
	} // forall
list_item elem1,elem2;
// Testen , ob irgendwelche Kanten (bei identischen Knoten-Medianwerten) zugefuegt werden muessen
//if (abs && !dir && !center) cout << endl << "Nur bei absoluten, ungerichteten MedianProblemen: " << endl;
//if (abs && dir && center) cout << endl << "Nur bei absoluten, gerichteten CenterProblemen: " << endl;
if ((abs && !dir && !center) || (abs && dir && center)) // nur bei absoluten, ungerichteten MedianProblemen oder absoluten, gerichteten Centerproblemen
	for(elem1=lexsolution.first();elem1;elem1=lexsolution.succ(elem1)) {
		v=Int2Node[lexsolution.contents(elem1)];
		for(elem2=lexsolution.succ(elem1);elem2;elem2=lexsolution.succ(elem2)) {
			adjacent=false;
			//Sind folgende Knoten adjacent und sind Spalten in F identisch ?
			w=Int2Node[lexsolution.contents(elem2)];
			edge e;
			forall_adj_edges(e,v) if (G.target(e)==w || G.source(e)==w) {adjacent=true;break;} // Test auf Adjazenz
			if (adjacent && F.col(lexsolution.contents(elem1))==F.col(lexsolution.contents(elem2))  && F.col(lexsolution.contents(elem1))==G.x_median(D,lexsolution.contents(elem1),lexsolution.contents(elem2),G.inf(e))) 
				{	
				S.source=lexsolution.contents(elem1);
				S.target=lexsolution.contents(elem2);
				S.alpha_s=0;
				S.alpha_t=1;
				solution.append(S);
				objval.append(F.col(lexsolution.contents(elem1)));
				objval.append(F.col(lexsolution.contents(elem2)));}
		} //for elem1
	} // for elem2
// fuer absolute, ungerichtete Centerprobleme Umwandlung der Losungsmenge noetig:
if (abs && !dir && center) {
	solution.clear();
	forall(i,lexsolution) {
		S=Qcenter.contents(Qcenter.get_item(i));
		solution.append(S);
		} // forall
	} // if abs, undir center
return(solution);
}



