#include <LOLA/lolatree.h>

lolatree::lolatree (lolaundirected U) 
: lolaundirected (U) { if (!is_tree()) lola_error("Graph is  not a tree :-)",2201,1);	}


bool lolatree::is_tree()
/***************************************************************
* Purpose    : Returns true if G is a tree
* Returns    :
* Influences : %
* Date       : Nov 1996
* Author     : Holger Hennes
****************************************************************/
{
	if ((number_of_nodes() == number_of_edges() + 1) && Is_Connected(*this) )
		return true;
	else return false;
}

list<sol_typ> lolatree::tree_node_med(double& objval)
/*********************************************************************
* Purpose    : Returns solutions of the local 1-median problem on trees
* Returns    : list<sol_typ>
* Influences : %
* Date       : Nov 1996
* Author     : Holger Hennes
* Last Update: Oct 1999 (Holger Hennes)
**********************************************************************/
{
	lolatree U=*this;
	edge e;
	node u, v;
	double W = 0, info1, info2;
	double *weights=new double[U.number_of_nodes()];
	lolatree PG = U;
	sol_typ S;
	list<sol_typ> L;
	int i=0 , j;
	node_array<int> node2number(PG);
	forall_nodes(v,PG) {
		weights[i]=PG.inf(v).head();
		W += PG.inf(v).head();
		node2number[v]=i++;
	}
	list<node> cand;
	forall_nodes(v,PG) 
		if (PG.degree(v)==1) cand.append(v);
		for (;;) {
			v=cand.pop();
			if (( PG.number_of_nodes() == 1) || !(PG.inf(v).head()<W/2)) {
				if (PG.inf(v).head()==W/2) {
					u = PG.opposite(v,PG.first_adj_edge(v));
					i = node2number[v];
					j = node2number[u];
					S.source = i;
					S.target = i;
					S.alpha_s = 0;
					L.append(S);
					S.source = j;
					S.target = j;
					S.alpha_s = 0;
					L.append(S);
				}
				else {
					i = node2number[v];
					S.source = i;
					S.target = i;
					S.alpha_s = 0;
					L.append(S);
				}
				objval=0;
				matrix M = U.shortest_path();
				for (int zaehl=M.dim1();zaehl--;) objval+=M[zaehl][i]*weights[zaehl];
				return L; 
			}
			u = PG.opposite(v,PG.first_adj_edge(v));
			info1 = PG.inf(u).head();
			info2 = PG.inf(v).head();
			e = PG.inz_edge(v,u);
			node_weight weight;
			weight.append(info1 + info2);
			PG.assign(u, weight);
			PG.del_edge(e);
			if (PG.degree(u)==1) cand.append(u);
		}
}

list<sol_typ> lolatree::tree_med(double& objval)
/*********************************************************************
* Purpose    : Returns solutions of the absolute 1-median problem on trees
* Returns    : list<sol_typ>
* Influences : %
* Date       : Nov 1996
* Author     : Holger Hennes
*********************************************************************/
{
	list<sol_typ> L;
	sol_typ S, T;
	L = tree_node_med(objval);
	if (L.length() == 2) {
		list_item L1_it = L.first();
		list_item L2_it = L.last();
		S = L.contents(L1_it);
		T = L.contents(L2_it);
		S.target = T.source;
		S.alpha_s = 0;
		L.append(S);}
	return L;
}

list<sol_typ> lolatree::tree_center(double& objval)
/*********************************************************************
* Purpose    : Returns solutions of the absolute 1-center problem on trees
* Returns    : list<sol_typ>
* Influences : %
* Date       : Nov 1996
* Author     : Holger Hennes
* Last Change: June 1999 (Holger Hennes)
**********************************************************************/
{
	lolatree U=*this;    
	list<sol_typ> L;
	sol_typ S;
	node v = this->first_node(),u,w;
	node_array<int> node2number(*this);
	int i=0;
	forall_nodes(u,*this)         {
		node2number[u]=i++;
		if (this->inf(u).head() !=1) lola_error("All weights must be equal to 1",2202,1);
	}
	edge e;
	edge_array<double> cost(*this);
	node_array<double> dist(*this);
	node_array<edge> pred(*this);
	forall_edges(e,*this) cost[e]=this->inf(e);
	DIJKSTRA(*this, v, cost, dist, pred);
   	forall_nodes(u,*this) if (dist[u]>dist[v]) v=u;
	w=v;
	DIJKSTRA(*this, v, cost, dist, pred);
	forall_nodes(u,*this) if (dist[u]>dist[w]) w=u;
	objval = dist[w]/2;
	for(;;) {
		e = pred[w];
		u = this->opposite(w,e);
		dist[u] = dist[w] - this->inf(e);
		if (dist[u]<=objval) {
			S.source = node2number[u];
			S.target = node2number[w];
			S.alpha_s = (objval - dist[u])/this->inf(e);
			L.append(S);
			return L; 
		}
		else w=u; 
	}
	return L;
}

list<sol_typ> lolatree::tree_node_center(double& objval)
/*********************************************************************
* Purpose    : Returns solutions of the locale 1-center problem on trees
* Returns    : list<sol_typ>
* Influences : %
* Date       : Nov 96
* Author     : Holger Hennes
**********************************************************************/
{
	sol_typ S, T;
	list<sol_typ> L;
	L = tree_center(objval);
	S = L.head();
	L.clear();
	if (S.alpha_s == 0.5) { 
		T.source = S.source; T.target = S.source; T.alpha_s = 0;
		S.source = S.target; S.target = S.target; S.alpha_s = 0;
		L.append(T); L.append(S); objval-=0.5; return L;
	} else if (S.alpha_s < 0.5) {
		T.source = S.source; T.target = S.source; T.alpha_s = 0; objval+=S.alpha_s;
		L.append(T); return L;
	} else { 
		T.source = S.target; T.target = S.target; T.alpha_s = 0; objval+=S.alpha_s;
		L.append(T); return L;
    }
}

void lolatree::splitting(node w, node v, GRAPH<node_weight,double> TREE) 
{
/*	node_weight nw=inf(v);
	hide_node(v);
	node v1 = TREE.new_node(nw);
	edge e;
	forall_adj_edges(e,w) {
		v=TREE.new_node(inf(target(e)));
		TREE.new_edge(v1,v,inf(e));
		splitting(v,w,TREE);
	}*/
}
	
list<sol_typ> lolatree::two_tree_center(double& objval)
{
	list<sol_typ> L;
/*	sol_typ S;
	node v,w;
	L = this->tree_center(objval);
	S = L.head();
	L.clear();
	if (S.alpha_s>0) { //solution is not a node
		node* number2node = new node[this->number_of_nodes()+1];
		int i=0;
		forall_nodes(v,*this) number2node[i++] = v;
		v = number2node[S.target];
		w = number2node[S.source];
		GRAPH<node_weight,double> TREE1,TREE2;
		splitting(w,v,TREE1);
		splitting(v,w,TREE2);
		TREE1.print();
		TREE2.print();
	}
/*	S = L.head();
	L.clear();
	array<double> obj(2);
	lolatree PG = U;
	int i=0;
	node* number2node = new node[U.number_of_nodes()+1];
	forall_nodes(v,U) number2node[i++] = v;

	v = number2node[S.target];
	w = number2node[S.source];
	PG.compon(v,w);
	M = PG.tree_center(obj[0]);
	L.conc(M);
	PG = U;
	v = number2node[S.target];
	w = number2node[S.source];
	PG.compon(w,v);
	M = PG.tree_center(obj[1]);
	L.conc(M);
	/*if (obj[0]<obj[1]) objval = obj[1];
	else objval = obj[0];*/
//	(obj[0]<obj[1]) ? objval = obj[1] : objval = obj[0];
	return L;
}



