#include <iomanip.h>

#include <LOLA/lolaundirected.h>
#include "graph_lp.h"

ostream& operator<<(ostream& os, const bottle_elem& output) {
	os << output.t_value << " " << "  " << output.F_column << endl;
	return os;
}

istream& operator>>(istream& is, bottle_elem &input) {
	is >> input.t_value;
	return is;
}

/* ostream& operator<<(ostream& os, const rat_segment& output) {
   os << output.source() << " " << "  " << output.target() << endl;
   return os;
}*/


// Vergleichsfunktionen fuer LEDA's rat_point, rat_line, rat_segment...

int compare (const bottle_elem& a, const bottle_elem& b) {
	if (a.t_value > b.t_value) return(1);
	if (a.t_value < b.t_value) return(-1);
	return(0);
}

int compare (const edge_segment& a, const edge_segment& b) {
	if(a.source > b.source) return(-1);
	if(a.source < b.source) return(1);
	if(a.source == b.source && a.target > b.target) return(-1);
	if(a.source == b.source && a.target < b.target) return(1);
	if(a.alpha_s > b.alpha_s) return(-1);
	if(a.alpha_s < b.alpha_s) return(1);
	if(a.alpha_t > b.alpha_t) return(-1);
	if(a.alpha_t < b.alpha_t) return(1);
	return(0);
}

int ptcomp_g (const rat_point& a, const rat_point& b) {
	if (a.xcoord() < b.xcoord()) return(-1);
        if (a.xcoord() > b.xcoord()) return(1);
        if (a.xcoord() == b.xcoord() && a.ycoord() <  b.ycoord()) return(-1);
        if (a.xcoord() == b.xcoord() && a.ycoord() >  b.ycoord()) return(1);
	return(0);
}

int compare (const rat_segment& a, const rat_segment& b) {
	if (ptcomp_g(a.source(),b.source())==-1 ) return(-1);
	if (ptcomp_g(a.source(),b.source())==1) return(1);
	if (ptcomp_g(a.source(),b.source())==0 && ptcomp_g(a.target(),b.target())==-1) return(-1);
	if (ptcomp_g(a.source(),b.source())==0 && ptcomp_g(a.target(),b.target())==1) return(1);
	return(0);
}
int compare (const rat_line& a, const rat_line& b) {
	if (equal_as_sets(a,b)) return(0);
	if (a.to_line().y_abs() < b.to_line().y_abs()) return(-1);
	if (a.to_line().y_abs() > b.to_line().y_abs()) return(1);
	if (a.to_line().y_abs() == b.to_line().y_abs() && a.to_line().slope()<b.to_line().slope()) return(-1);
	if (a.to_line().y_abs() == b.to_line().y_abs() && a.to_line().slope()>b.to_line().slope()) return(1);
	return(0);
}


matrix lolagraph::F_median(matrix& D,bool directed)
/*********************************************************************
* Purpose   : computes matrix F, where each element(i,j) is the Median-
*             function-value of node j and criteria i
* Returns   : matrix
* Influences: %
* Date      : Mar 1997
* Author    : Karen Schulze
*********************************************************************/
{
	lolagraph U=*this;
	node v;
	node_weight weight;
	int i,k,j=0,coln,rown;
	coln=U.number_of_nodes();  // Spaltenanzahl
	rown=U.inf(U.first_node()).length(); // Zeilenanzahl
	matrix F(rown,coln);
	// jetzt Matrix F berechnen
	for(j=0;j<coln;j++) {
        k=0;
        forall_nodes(v,U) {
			weight.clear();
			weight=U.inf(v);
			for(i=0;i<rown;i++)
				if (!directed)
					F(i,j) += (weight.pop() * D(k,j));
				else
					F(i,j) += (weight.pop() * (D(k,j)+D(j,k)));

			k++;
		} //node v
	} // for j
	return(F);
}

matrix lolagraph::F_center(matrix& D,bool directed)
/*********************************************************************
* Purpose   : computes matrix F, where each element(i,j) is the Center-
*             function-value of node j and criteria i
* Returns   : matrix
* Influences: %
* Date      : Mar 1997
* Author    : Karen Schulze
*********************************************************************/
{
	lolagraph U=*this;
	node v;
	node_weight weight;
	int i,k,j=0,coln,rown;
	coln=U.number_of_nodes();  // Spaltenanzahl
	rown=U.inf(U.first_node()).length(); // Zeilenanzahl
	matrix F(rown,coln);
	double gewicht;
	// jetzt Matrix F berechnen
	for(j=0;j<coln;j++) {
        k=0;
        forall_nodes(v,U) {
			weight.clear();
			weight=U.inf(v);
			for(i=0;i<rown;i++){
				gewicht=weight.pop();
				if (!directed) {
					if (F(i,j)<(gewicht*D(k,j))) F(i,j) = (gewicht * D(k,j));
				} else
					if (F(i,j)<(gewicht*(D(k,j)+D(j,k)))) F(i,j) = (gewicht * (D(k,j)+D(j,k)));
			}
			k++;
		} //node v
	} // for j
	return(F);
}

vector lolagraph::x_center(matrix& D,int v_i, int v_j,double laenge, double t_value)
/*********************************************************************
* Purpose   : calculates the objective values for a point x along edge e
* Returns   : matrix
* Influences: %
* Date      : Mar 1997
* Author    : Karen Schulze
*********************************************************************/
{
	lolagraph U=*this;
	int q,rown=U.inf(U.first_node()).length(),k=0;
	vector f_x(rown);
	node v;
	node_weight weight;
	double objval;
	forall_nodes(v,U) {
        weight=U.inf(v);
        // berechne d(v_m,x)=max ..
        ((D(k,v_i)+t_value*laenge)<(D(k,v_j)+(1-t_value)*laenge) ? objval=D(k,v_i)+t_value*laenge : objval=D(k,v_j)+(1-t_value)*laenge );
        for(q=0;q<rown;q++) {
			// Maximum ?
			if(weight.contents(weight.get_item(q))*objval>f_x[q])  f_x[q] =objval*weight.contents(weight.get_item(q));
		}
        ++k;
	}
	return f_x;
}

bool lolagraph::par_dominates(vector f1, vector f2)
/*********************************************************************
* Purpose   : examines, whether vector f1 pareto-dominates vector f2
* Returns   : bool, where 0 means f1 does not  dominate f2 
* Influences: %
* Date      : Mar 1997
* Author    : Karen Schulze
*********************************************************************/
{
	bool smaller=false;
	int k;
	for(k=0;k<f1.dim();k++)
		if (f1[k]<f2[k]) smaller=true;
		else if(f1[k]>f2[k]) {
			smaller=false;break;
		}
	return(smaller);
}

list<edge_segment> lolagraph::parlocation(list<vector>& objval, bool dir, bool abs, bool center)
/*********************************************************************
* Purpose   : returns the local pareto-optimal solutions for a multicriteria 
*             graph, solves problems 1/G(dir,undir)/./d(V,V)/Q-sumpar, 
*             1/G(dir)/./d(V,G)/Q-sumpar, 1/G(dir,undir)/./d(V,V)/Q-summax,
*	      and 1/G(dir)/./d(V,G)/Q-maxpar
* Returns   : list<edge_segment>
* Influences: %
* Date      : Mar 1997, max-problems: Sep 1998
* Author    : Karen Schulze
*********************************************************************/
{
	// unzulassige Kombination:
	if (!dir && abs) lola_error("This algorihm does not solve this kind of problem, sorry ;-)",2003,0); 
	while (!Is_Connected(*this)) {
		lola_message("Graph is not connected: LOLA is could connect it",0,1,true);
		Make_Connected(*this);
	}
	lolagraph U=*this; 
	list<edge_segment> solution;
	edge_segment S;
	int i,j;
	node v,w;
	// In Int2Node sind Zeiger auf die Knoten im Graphen zum zugehoerign Integerwert
	node* Int2Node = new node[U.number_of_nodes()];
	i=0;
	forall_nodes(v,U) Int2Node[i++] = v;
	// shortest_path matrix D
	matrix D(U.number_of_nodes(),U.number_of_nodes());
	D = U.shortest_path();
	matrix F(U.inf(U.first_node()).length(),U.number_of_nodes());
	if (center)
		F=U.F_center(D,dir);
	else
		F=U.F_median(D,dir);
	list<int> node_list,Par;
	for(i=0;i<U.number_of_nodes();i++)
		node_list.append(i);
	Par=node_list;
	forall(i,node_list)
		forall(j,node_list)
			if (U.par_dominates(F.col(j),F.col(i)))	{
				Par.del_item(Par.search(i));
				break;
			}
	objval.clear();
	forall(i,Par) {
		S.source=i;
		S.target=i;
		S.alpha_s=0;
		S.alpha_t=0;
		solution.append(S);
		objval.append(F.col(i));
		objval.append(F.col(i));
	}
	bool adjacent=false;
	edge e;
	if (dir && abs && center) { // evtl Kanten dazu, Zusatzstep 4'
		forall(i,Par) { 
			v=Int2Node[i];
			forall(j,Par) {
				w=Int2Node[j];
				adjacent=false;	 	 
				forall_out_edges(e,v) 
					if (U.target(e)==w) {adjacent=true;break;}
					forall_out_edges(e,w) 
						if (U.target(e)==v) {adjacent=true;break;}
					if (adjacent && F.col(i)==F.col(j) && F.col(i)==U.x_center(D,i,j,U.inf(e),0.5)) {
						S.source=i;
						S.target=j;
						S.alpha_s=0;
						S.alpha_t=1;
						solution.append(S);
						objval.append(F.col(i));
					} // if		
			} // for j
		} // for i
	} //if
	return(solution); 
}

list<edge_segment> lolaundirected::edge_pareto(matrix& D,matrix& F,int i, int j, double length, list<bottle_elem>& bottlenecks)
/*********************************************************************
* Purpose   : returns the pareto-optimal solution for an edge of a 
*             multicriterial graph
* Returns   : list<edge_segment> 
* Influences: bottlenecks
* Date      : Apr 1997, erweitert von 2 auf Q: Aug 1997
* Author    : Karen Schulze
*********************************************************************/
{
	lolagraph U=*this;
	list<edge_segment> solution;
	edge_segment S;
	int Q=F.dim1();  // Anzahl der Zielfunktionen
	int k=0,l;
	node v;
	double alpha=0,t_1=0,t_2=0;
	list<double> t1,t2;
	vector F_edge(Q),invec(Q);  
	bool   weiter = true; // nur zur einfachen Lesbarkeit...
	list<int> Par;
	// ***************************** Step 1 : ***********************************
	if (!U.par_dominates(F.col(j),F.col(i))) Par.append(i);
	if (!U.par_dominates(F.col(i),F.col(j))) Par.append(j);
	if (Par.length()!=2) {weiter = false;}
	node_weight nodeweight;
	// ***************************** Step 2: ************************************
	if (weiter && F.col(i)==F.col(j)) {
		// berechne f((e,0.5))=F_edge
		double distance_to_k;
		forall_nodes(v,U) {
			nodeweight=U.inf(v);
			(D(k,i)<D(k,j)) ? distance_to_k=D(k,i) : distance_to_k =D(k,j);
			for(l=0;l<Q;l++)
				F_edge[l] +=nodeweight.pop()*(distance_to_k+0.5*length);
			k++;
		}
		//Step 3.a und 3.b
		(F.col(i)==F_edge) ? alpha=1 : alpha=0;
		weiter=false;
	}
	// ***************************** Step 3: ***********************************
	list<int> Q_1,Q_2;
	if (weiter) { 
		for(l=0;l<Q;l++)
			if (F(l,i) >= F(l,j)) Q_1.append(l);
			else Q_2.append(l);
	}
	// ***************************** Step 4: ***********************************
	segment tseg;
	array<node> number2node(U.number_of_nodes());
	k=0;
	forall_nodes(v,U) number2node[k++]=v;
	list<bottle_elem> F_bottlenecks;
	bottle_elem predecessor,onebottle;	
	array<bool> t1_found(Q),t2_found(Q);
	if (weiter) {
		// ******** Liste mit allen bottleneck-Punkten erstellen ***********
		vector w_decrease(Q),w_All(Q);
		for(k=0;k<U.number_of_nodes();k++) {
			nodeweight=U.inf(number2node[k]);
			// existiert ein bottlepoint ?
			if (D(k,i)+length != D(k,j) && D(k,j)+length != D(k,i)) {
				onebottle.t_value=(D(k,j)+length-D(k,i))/2;
				onebottle.weight=nodeweight;
				onebottle.F_column=invec; // zum Initialisieren...
				bottlenecks.append(onebottle);
			}
			//Gesamt V
			for(l=0;l<Q;l++)
				w_All[l] +=nodeweight.pop();
			//w_decrease sammeln
			if (D(k,j)+length == D(k,i)) {
				nodeweight=U.inf(number2node[k]);
				for(l=0;l<Q;l++)
					w_decrease[l] +=nodeweight.pop();
			}
		}
		// Liste der bottleneckpoints nach t_value sortieren:
		bottlenecks.sort();
		// **************** bottleneck-Zielfu-werte berechnen *************** 
		vector F_current(Q); 
		F_current=F.col(i);
		double b_pred=0,b_current;
		forall(onebottle,bottlenecks) {
			b_current=onebottle.t_value;
			nodeweight=onebottle.weight;
			if (b_current!=b_pred) {  
				for(l=0;l<Q;l++) 
					onebottle.F_column[l]=F_current[l]+(w_All[l]-2*w_decrease[l])*(b_current-b_pred);
				F_bottlenecks.append(onebottle);
			}else
				onebottle.F_column=F_current;
			for(l=0;l<Q;l++) 
				w_decrease[l] +=nodeweight.pop();
			F_current=onebottle.F_column;
			b_pred=b_current;
		}
		// ***************** Werte fuer t1 und t2 berechnen ***************
		onebottle.t_value=length;	// Knoten j ans Ende der Liste der Bottleneckpunkte anhaengen
		onebottle.weight=U.inf(number2node[j]);
		onebottle.F_column=F.col(j);
		F_bottlenecks.append(onebottle);
		predecessor.t_value=0;	// Predecessor mit Knotenwerten von i initialisieren
		predecessor.F_column=F.col(i);
		forall(onebottle,F_bottlenecks) {
			for(l=0;l<Q;l++) {
				if (Q_1.search(l)!=nil && !t1_found[l] && onebottle.F_column[l] < F.col(i)[l]) {
					tseg=segment(predecessor.t_value,predecessor.F_column[l],onebottle.t_value,onebottle.F_column[l]);
					t1_found[l]=true;
					t1.append(tseg.x_proj(F.col(i)[l]));
				}
				if (Q_2.search(l) !=nil && !t2_found[l] && onebottle.F_column[l] >= F.col(j)[l]) {
					tseg=segment(predecessor.t_value,predecessor.F_column[l],onebottle.t_value,onebottle.F_column[l]);
					t2_found[l]=true;
					t2.append(tseg.x_proj(F.col(j)[l]));
				}
			}
			predecessor=onebottle;
		}
		forall(l,Q_2)
			if (!t2_found[l]) {// kein bottleneckwert ist echt groesser alsvj
				t2.append(length);
				break;
			}
		// ************* min t1 und max t2 finden ***************************
		t_1=t1[t1.min()];
		t_2=t2[t2.max()];
	}
	// ****************************** Step 5: ***********************************
	// und Loesung in richtiges Format umwandeln
	bottlenecks.clear();
	bool inbetween=false;
	forall(k,Par) {  // sind i und/oder j Loesung?
		S.source=k;
		S.target=k;
		S.alpha_s=0;
		S.alpha_t=0;
		solution.append(S);
	}
	bottle_elem newbottle;
	newbottle.F_column=invec;
	if (t_1 <t_2) { 
		// neue bottleneck-Liste erstellen, die nur Kante [t_1,t_2] enthaelt
		predecessor.t_value=0;
		predecessor.F_column=F.col(i);
		bool t1_found=false,t2_found=false;
		forall(onebottle,F_bottlenecks) {
			inbetween=true;
			if (onebottle.t_value>=t_1 && !t1_found) {
				for(l=0;l<Q;l++) {
					tseg=segment(predecessor.t_value,predecessor.F_column[l],onebottle.t_value,onebottle.F_column[l]);
					newbottle.F_column[l]=tseg.y_proj(t_1);
				} //for
				newbottle.t_value=t_1;
				t1_found=true;
				inbetween=false;
				bottlenecks.append(newbottle);
			} //if       
			if (onebottle.t_value >=t_2 && !t2_found) {     
				for(l=0;l<Q;l++) {
					tseg=segment(predecessor.t_value,predecessor.F_column[l],onebottle.t_value,onebottle.F_column[l]);
					newbottle.F_column[l]=tseg.y_proj(t_2);
				} // for
				newbottle.t_value=t_2;
				t2_found=true;
				inbetween=false;
				bottlenecks.append(newbottle);
			} //if              
			if (t1_found && !t2_found && inbetween) 
				bottlenecks.append(onebottle);
			predecessor=onebottle;
		}  
	} //forall
	if (alpha==1) { // ist die ganze Kante Loesung? (3.a)
		S.source=i;
		S.target=j;
		S.alpha_s=0;
		S.alpha_t=1;
		solution.append(S);
		// in Liste breakpoints 
	}
	return(solution);	 			
}

list<edge_segment> lolaundirected::bi_medpareto(list<vector>& objval)
/*********************************************************************
* Purpose   : returns the pareto-optimal solution for a bi-criterial  
*             graph, solves 1/G(undir)/./d(V,G)/2-sumpar
* Returns   : list<edge_segment>
* Influences: %
* Date      : Apr 1997, finished Aug 1997
* Author    : Karen Schulze
*********************************************************************/
{
	objval.clear();
	while (!Is_Connected(*this)) {
		lola_message("Graph is not connected: LOLA is could connect it",0,1,true);
		Make_Connected(*this);
	}
	lolaundirected U=*this;
	list<edge_segment> solution;
	dictionary<rat_line,list<edge_segment> > F_dic;
	dictionary<rat_line,list<rat_segment> > F_dicrat;
	edge_segment edgi;
	list<bottle_elem> breakpoints;
	list<rat_segment> Fseg,image;
	list<edge_segment> Fedgi;
	int i=0,j;
	edge e;
	double edgelength;
	U.make_undirected();
	node_array<int> node2number(U);
	node knot;
	forall_nodes(knot,U) node2number[knot]=i++;
	// shortest_path matrix D
	matrix D(U.number_of_nodes(),U.number_of_nodes());
	D = U.shortest_path();
	// median-matrix F
	matrix F(U.inf(U.first_node()).length(),U.number_of_nodes());
	F=U.F_median(D,false);
	//************************** STEP 2 *****************************************
	rat_point seg_start,seg_end;
	rat_vector horiz((integer)1,(integer)0);
	rat_segment seggi;
	forall_edges(e,U) {
        breakpoints.clear();
		solution.clear();
		i=node2number[U.source(e)];
		j=node2number[U.target(e)];
		edgelength=U.inf(e);
		solution=U.edge_pareto(D,F,i,j,edgelength,breakpoints);
		// IM(f) erstellen
		// Segmente f"ur die Knoten und evt. ganze Kante bestimmen.....
		forall(edgi,solution) { 
			seg_start=rat_point(rational(F.col(edgi.source)[0]),rational(F.col(edgi.source)[1]));
			seg_end=rat_point(rational(F.col(edgi.target)[0]),rational(F.col(edgi.target)[1]));
			if (seg_start==seg_end) seggi=rat_segment(seg_start,horiz);
			else
				seggi=rat_segment(seg_start,seg_end);	
			if (F_dic.lookup(rat_line(seggi))==nil) {// linie ex. noch nicht
				Fedgi.clear();Fseg.clear();
			}
			else {
				Fedgi=F_dic.inf(F_dic.lookup(rat_line(seggi)));
				Fseg=F_dicrat.inf(F_dicrat.lookup(rat_line(seggi)));
			}
			if (Fedgi.search(edgi)==nil) {
				image.append(rat_segment(seg_start,seg_end));
				Fedgi.append(edgi);Fseg.append(rat_segment(seggi));
				F_dic.insert(rat_line(seggi),Fedgi);
				F_dicrat.insert(rat_line(seggi),Fseg);
			}
		} // forall solution
		// Segmente f"ur Teilkantenabschnitte bestimmen
		if (!breakpoints.empty()) { 
			bottle_elem predecessor=breakpoints.pop(),current;
			forall(current,breakpoints) {
				seg_start=rat_point(rational(predecessor.F_column[0]),rational(predecessor.F_column[1]));
				seg_end=rat_point(rational(current.F_column[0]),rational(current.F_column[1]));
				seggi=rat_segment(seg_start,seg_end);
				if (ptcomp_g(seggi.source(),seggi.target())==1) {
					seggi=seggi.reversal();
					edgi.source=j;
					edgi.target=i;
					edgi.alpha_s=edgelength-current.t_value;
					edgi.alpha_t=edgelength-predecessor.t_value;
				} else {
					edgi.source=i;
					edgi.target=j;
					edgi.alpha_s=predecessor.t_value;
					edgi.alpha_t=current.t_value;
				}
				image.append(seggi);
				if (F_dic.lookup(rat_line(seggi))==nil) // linie ex. noch nicht
				{Fedgi.clear();Fseg.clear();}
				else {
					Fedgi=F_dic.inf(F_dic.lookup(rat_line(seggi)));
					Fseg=F_dicrat.inf(F_dicrat.lookup(rat_line(seggi)));
				}
				Fedgi.append(edgi);Fseg.append(seggi);
				F_dic.insert(rat_line(seggi),Fedgi);
				F_dicrat.insert(rat_line(seggi),Fseg);
				predecessor=current;
			} 
		}
	}
	rat_segment segs,predsegs;
	list<rat_segment> cp_image=image;
	list_item elem,new_cp_elem,cp_elem=image.first();
	// *************************** STEP 1 (nachgeholt) ***********************
	//finde z_1_max
	rational max;
	forall(segs,image) if(segs.target().xcoord() > max) max=segs.target().xcoord();
	max=max+1; // noetig, damit keine trivialen Segmente bei hershi vorkommen..
	// ****************************  Step 3 ***********************************
	//horizontale Segmente zur Liste hinzufuegen
	cp_image=image;cp_elem=image.first();
	forall_items(elem,cp_image) {
        segs=cp_image.contents(elem);
        if (segs.target()==segs.source()) //fuer Knoten Segment nur aendern
			image.assign(cp_elem,rat_segment (segs.target(),rat_point (max,segs.target().ycoord()) ) );
		else 
			image.append(rat_segment (segs.target(),rat_point (max,segs.target().ycoord()) ) );     // neues Segment angehaengt
		predsegs=segs;
        cp_elem=image.succ(cp_elem);
	}
	image.sort();                                      
	bool delflag=false;
	cp_image=image;cp_elem=image.first();
	// identische Segmente entfernen..
	forall_items(elem,cp_image) {
        segs=cp_image.contents(elem);
		if(elem!= cp_image.first() && !delflag) cp_elem=image.succ(cp_elem);
		else delflag=false;	
		if (elem!=cp_image.first() && segs == predsegs) {
			new_cp_elem=image.succ(cp_elem);
			image.del_item(cp_elem);
			cp_elem=new_cp_elem;
			delflag=true;
		}
		predsegs=segs;
	}
	//image.unique();// unique existiert wohl nicht mehr in LEDA ?!?
	// ************************* STEP 4 ***************************************
	// untere Kontur mit Alg. von Hershberger bestimmen
	image=hershi(image);
	// STEP 5 wird gleichzeitig mit Step 6 erledigt
	// ************************** STEP 6 ***************************************
	// jetzt alle weiteren Segmente zurueckwandeln und der Loesung zufuegen
	solution.clear();
	dic_item fit,dit;
	predsegs=rat_segment(0,0,integer(-L_INF),integer(L_INF));
	rat_segment searchi;
	double length,diff;
	bool outflag=true;
	array<node> number2node(U.number_of_nodes());
	i=0;
	node v,w;
	forall_nodes(v,U) number2node[i++]=v;
	vector outvec;	
	forall_items(elem,image) {
		segs=image.contents(elem);
		searchi=segs;
		dit=F_dic.lookup(rat_line(searchi));
		fit=F_dicrat.lookup(rat_line(searchi));
		if(dit !=nil && fit !=nil) {
			Fedgi=F_dic.inf(dit);
			Fseg=F_dicrat.inf(fit);
			forall(seggi,Fseg) {
				outflag=true	;
				edgi=Fedgi.pop();
				//************ zum skalieren f"ur die Ausgabe mit Lola ****************
				v=number2node[edgi.source];
				w=number2node[edgi.target];
           		if (v!=w) {
					e=U.inz_edge(v,w);
					edgelength=U.inf(e);
				}
				else 
					edgelength=1;
				// urspruengliche Segmente suchen
				if (equal_as_sets(seggi,searchi)) { // Segment ist unveraendert
					//********* skalieren **************
					edgi.alpha_s=edgi.alpha_s/edgelength;
					edgi.alpha_t=edgi.alpha_t/edgelength;
					solution.append(edgi);
					objval.append(segs.source().to_vector().to_vector());
					if (edgi.source==edgi.target)
						objval.append(segs.source().to_vector().to_vector());
					else
						objval.append(segs.target().to_vector().to_vector());
				} else {
					// veraendertes Segment berechnen....
					if (edgi.source==edgi.target) 
						if (searchi.source()==rat_point(rational(F.col(edgi.source)[0]),rational(F.col(edgi.source)[1]))) {
							solution.append(edgi);
							objval.append(segs.source().to_vector().to_vector());
							objval.append(segs.source().to_vector().to_vector());
						} else
							outflag=false; // verwerfen
						else { // source!=target
							length=seggi.ycoord1D()-seggi.ycoord2D();
							diff=(edgi.alpha_t-edgi.alpha_s);
							edgi.alpha_s=(seggi.ycoord1D()-segs.source().ycoordD())/length *diff;
							edgi.alpha_t=(seggi.ycoord1D()-segs.target().ycoordD())/length *diff;
							//********** skalieren *************
							edgi.alpha_s=edgi.alpha_s/edgelength;
							edgi.alpha_t=edgi.alpha_t/edgelength;
							solution.append(edgi);
							objval.append(segs.source().to_vector().to_vector());
							objval.append(segs.target().to_vector().to_vector());
						} // if source=target
				} // if equal_as
			} //forall seggi
		} else cout << "error, bitte bei Karen melden ;-)" << endl; // das passiert hoffentlich nicht
	} //forall_items
	// beim ersten Segment eventuell den Startknoten der Loesung zufuegen
	searchi=rat_segment(image.head().source(),rat_vector(integer(1),integer(0)));
	fit=F_dicrat.lookup(rat_line(searchi));
	dit=F_dic.lookup(rat_line(searchi));
	Fedgi=F_dic.inf(dit);
	Fseg=F_dicrat.inf(fit);
	forall(seggi,Fseg) {
		edgi=Fedgi.pop();
		if(equal_as_sets(seggi,searchi) && solution.head().target!=solution.head().source && solution.head().source==edgi.source) {
			solution.push(edgi);
			objval.append(searchi.source().to_vector().to_vector());
			objval.append(searchi.source().to_vector().to_vector()); // zweimal, da zu jedem edgi, zwei Zielfuvek gemerkt werden
			// ********* Ausgabe auf den Bildschirm **************
		} //if   
	}
	edge_segment pre_edge;
	pre_edge=solution.tail();
	i=0;
	// Durch Bottleneckpunkte koennen zwei aufeinanderfolgende Elemente der L"osungsmenge zum gleichen Kantenabschnitt gehoeren -> Ersetzung mit einem Element
	forall(edgi,solution) {
		if(pre_edge.source==edgi.source && pre_edge.target==edgi.target && pre_edge.alpha_t==edgi.alpha_s) {
			edgi.alpha_s=pre_edge.alpha_s;
			solution.assign(solution[i],edgi);
			solution.del(solution[i-1]);
			objval.del(objval[2*i-1]);
			objval.del(objval[2*i-2]);
		} // if
		i++;
		pre_edge=edgi;
	} // for	
	return(solution);
}


list<rat_segment> mergestep(list<rat_segment> uk1,list<rat_segment> uk2)
/*********************************************************************
* Purpose   : merges two lower envelopes into one
* Returns   : list<rat_segment>
* Influences: %
* Date      : July 1997
* Author    : Karen Schulze
*********************************************************************/
{// Voraussetzung..alle Segment sind nicht trivial !!!!!!!!!!!!!!!!!!!! 
// leere Listen? gleich zurueck...
	if (uk1.empty()) return(uk2); 
	if (uk2.empty()) return(uk1);
	list<rat_segment> untereKontur;
	// Liste listr1, enthaelt alle Anfangs- und Endpunkte der Segmente in uk1
	list<rat_point> listr1,listr2;
	rat_segment seg1,seg2,newseg,intseg;
	forall(seg1,uk1) {listr1.append(seg1.source());listr1.append(seg1.target());} 
	// Liste listr2, enthaelt alle Anfangs- und Endpunkte der Segmente in uk2
	forall(seg2,uk2) {listr2.append(seg2.source());listr2.append(seg2.target());}
	// initialisieren vor der Schleife... 
	seg1=uk1.pop();
	seg2=uk2.pop();
	rational line=0,help;
	rat_point r1=listr1.pop(),r2=listr2.pop(),x_INF(integer(L_INF),integer(L_INF)),x_int,new_end;
	x_int=x_INF;
	bool is_r1_src=true,is_r2_src=true,r_equal=false;
	// Startlinie finden
	if (ptcomp_g(r2,r1)==1) { // r2>r1
		// sind x-Koordinaten gleich?
		if (r1.xcoord()==r2.xcoord()) {r2=listr2.pop();is_r2_src=false;}
		newseg=seg1;
		r1=listr1.pop();
		is_r1_src=false;
	} else {  // r2<=r1
		// sind x-Koordinaten gleich?
		if (r1.xcoord()==r2.xcoord()) {r1=listr1.pop();is_r1_src=false;}
		newseg=seg2;
		r2=listr2.pop();
		is_r2_src=false;
	}
	// Schleife
	do {
		// Schnittpunkt der Segmente ggf finden
		if (x_int==x_INF && // SP noch nicht bestimmt? 
			seg1.intersection(seg2,intseg) && // SP existiert? 
			intseg.is_trivial() && // SP ist nur ein Punkt? 
			intseg.source() != r1 && intseg.target() != r2 //  echter SP? 
			&& intseg.source().xcoord()>line) { // rechts von act_line?
			x_int=intseg.source();
		}
		// Minimum aus r1,r2 und x_int bestimmen
        if( r1.xcoord() >= r2.xcoord() ) line=r2.xcoord() ;
		else line=r1.xcoord() ;
        if ( r1.xcoord() >= x_int.xcoord() ) help=x_int.xcoord() ;
		else help=r1.xcoord() ;
        if ( help >= line ) line=line ;
		else  line=help ;
        // untere Kontur erweitern
		new_end=rat_point(line,rational(newseg.to_segment().y_proj(line.to_double())));
		if (!untereKontur.empty() && ptcomp_g(untereKontur.tail().source(),newseg.source())==0) {
			untereKontur.assign(untereKontur.last(),rat_segment(newseg.source(),new_end));}
		else
		if (!untereKontur.empty() && ptcomp_g(untereKontur.tail().target(),newseg.source())==0 && cmp_slopes(untereKontur.tail(),rat_segment(newseg.source(),new_end))==0)  {
			untereKontur.assign(untereKontur.last(),rat_segment(untereKontur.tail().source(),new_end));}
		else {
			untereKontur.append(rat_segment(newseg.source(),new_end));}
		// naechstes Segment bestimmen
		if (r1.xcoord()==r2.xcoord() && line==r1.xcoord()) r_equal=true;
		// sind r1.x==r2.x werden die beiden folg. if's ausgefuehrt, dadurch newseg unklar...
		if (line==r1.xcoord() ) {
			is_r1_src=!is_r1_src;
			if( is_r1_src ) {  // neues Segment seg1 bestimmen, ggf Abbruch
				if (!uk1.empty()){
					seg1=uk1.pop();}
				else { 
					if(r1.xcoord()>seg2.xcoord1())
						seg2=rat_segment(rat_point(r1.xcoord(),seg2.ycoord1()),seg2.target());
					r1=x_INF;
					//continue zu if (line==r2);
				}
			}
			if (r1 !=x_INF) {
				if( ( is_r1_src && r1.xcoord() != listr1.head().xcoord() && ptcomp_g(listr1.head(),r2)==1) // Sprung zwischen den Segmenten in uk1
					|| ptcomp_g(r1,new_end)==1) {// r1 oberhalb
					newseg=seg2;
				}
				else {
					newseg=seg1;
				} // Sprung
				if (r1.xcoord()==listr1.head().xcoord()) {
					// End und Anfangspunkt identisch, gleich weiter
					is_r1_src=!is_r1_src;
					r1=listr1.pop();
				} 
				if ( !listr1.empty() ) r1=listr1.pop();
			} else {
				newseg=seg2;
			}
		} 
		if (line==r2.xcoord() ) {
			is_r2_src=!is_r2_src;
			if( is_r2_src ) {
				if (!uk2.empty()) seg2=uk2.pop();
				else {
					//continue zu if (line==x_int)
					if(r2.xcoord()>seg1.xcoord1())
						seg1=rat_segment(rat_point(r2.xcoord(),seg1.ycoord1()),seg1.target());
					r2=x_INF; 
				}                                                
			}
			if (r2 !=x_INF) {
				if( (is_r2_src && r2.xcoord() != listr2.head().xcoord() && ptcomp_g(listr2.head(),r1)==1) // Sprung zwischen den Segmenten in uk2
					|| ptcomp_g(r2,new_end)==1 ) {
					newseg=seg1;
				}
				else {
					newseg=seg2;
				} // Sprung
				if (r2.xcoord()==listr2.head().xcoord()) {
					// End und Anfangspunkt identisch, gleich weiter
					is_r2_src=!is_r2_src;
					r2=listr2.pop();
				}
				if ( !listr2.empty() ) r2=listr2.pop();
			} else {newseg=seg1;}
		} 
		if (line==x_int.xcoord()) {
			if (ptcomp_g(r2,r1)==1) newseg=rat_segment(x_int,seg1.target());
			else newseg=rat_segment(x_int,seg2.target());
			x_int=x_INF;
		}
		if (r_equal) {   // newseg noch klar machen...
			r_equal=false;
			if (r2.ycoord()>r1.ycoord()) newseg=seg1;
			else newseg=seg2;
		}
	}while (r1!=x_INF || r2!=x_INF);
	return untereKontur;
}

list<rat_segment> Div_and_Conq(list<rat_segment> envelope)
/*********************************************************************
* Purpose   : evaluates the lower envelope of a list of segments, using
                the Divide and Conquer Algorithm
* Returns   : list<segment>
* Influences: %
* Date      : July 1997
* Author    : Karen Schulze
*********************************************************************/
{
	list<rat_segment> list1,list2;
	int leng=envelope.length();
	leng=leng >> 1;
	if(envelope.empty()) return envelope;
	envelope.split(envelope[leng],list1,list2,true);
	if (leng >= 1) {
        list1=Div_and_Conq(list1);
        list2=Div_and_Conq(list2);
	}
	return mergestep(list1,list2);
}

list<rat_segment> Div_and_Conq2(list< list<rat_segment> > envelope)
/*********************************************************************
* Purpose:   : evaluates the lower envelope of a list of a list of segments,
* 		 using a Divide and Conquer Algorithm
* Returns   : list<rat_segment>
* Influences: %
* Date      : July 1997
* Author    : Karen Schulze
*********************************************************************/
{
	list< list<rat_segment> > list1,list2;
	list<rat_segment> l1,l2;
	int leng=envelope.length();
	leng=leng >> 1;
	if(envelope.empty()) return l1;
	envelope.split(envelope[leng],list1,list2,true);
	if (leng >= 1) {
        l1=Div_and_Conq2(list1);
        l2=Div_and_Conq2(list2);
	}
	else {
        if (!list1.empty()) l1=list1.pop();
        if (!list2.empty()) l2=list2.pop();
	}
	return mergestep(l1,l2);
}


list<rat_segment> hershi(list<rat_segment> Seggis)
/*********************************************************************
* Purpose   : evaluates the lower envelope of  list of a list of segments
*                using the algorithm of Hershberger
* Returns   : list<rat_segment>
* Influences: %
* Date      : July 1997
* Author    : Karen Schulze
*********************************************************************/
{
	int i,n=Seggis.length();

//************************* PHASE 1 *****************************************
	int treeheight=0,n_temp=4*n-1;
	// BAUMH"OHE BERECHNEN log_2(4*n-1)
	while(n_temp>=1) {n_temp=n_temp >> 1;treeheight++;}
	//Punkte-Liste aus allen Endpunkten der Segmente erstellen
	list<rat_point> s_dots;
	rat_segment harry;
	forall(harry,Seggis) {
        s_dots.append(harry.source());
        s_dots.append(harry.target());
	}
	// LISTE SORTIEREN
	s_dots.sort();
	// Jeweils die Hoehe des NCA im Binaerbaum zu jedem Segment berechnen und Segment dementsprechend der Menge Tsubset[i] zu ordnen
	int height;
	//const trheight=treeheight;
	list<rat_segment> Tsubset[100];   
	int nodeA,nodeB;
	forall(harry,Seggis) {
		nodeA=s_dots.rank(harry.source());
		s_dots.assign(s_dots.search(harry.source()),rat_point(integer(L_INF),integer(L_INF))); // zur Sicherheit gefundene Punkte veraendern
		nodeB=s_dots.rank(harry.target());
		s_dots.assign(s_dots.search(harry.target()),rat_point(integer(L_INF),integer(L_INF)));
		// NCA bestimmen (beide Knoten sind Blaetter, daher einfache Rechn.!)
		i=0;
		height=nodeA*2-1 ^ nodeB*2-1; // XOR
        while(height>=1) {height=height >> 1; i++;} // log_2(height) bestimmen
        Tsubset[i-1].append(harry);
	}
//*********************  PHASE 2 ******************************************
	for(i=0;i<treeheight;i++) Tsubset[i]=Div_and_Conq(Tsubset[i]);
//*********************  PHASE 3 ******************************************
	list< list<rat_segment> > envs;
	for(i=0;i<treeheight;i++) if (!Tsubset[i].empty()) envs.append(Tsubset[i]);
	list<rat_segment> lowerenv=Div_and_Conq2(envs);
	return lowerenv;
}

#ifdef HAVECPLEX
void parloc_cplex(int Q,dictionary<edge_segment,matrix>& Dsubedge)
/*********************************************************************
* Purpose    : writes the LP for cplex and calls cplex for 
*			   1/G(undir)/./d(V,G)/Q-sumpar
* Returns    : %
* Influences : %
* Date       : Aug 1997
* Author     : Karen Schulze
* Last Change: Feb 1999 (Holger Hennes)
*********************************************************************/
{
	double t_LB,t_UB,s_LB,s_UB, epsilon=0.0001;;
	list<double> result;
	bool LB_equal,UB_equal;
	char*       sol1="t_minmps.txt";
	char*       sol2="t_maxmps.txt";
	const char* cplexcall   = "cplex < runcplex >/dev/null";
	ofstream output("runcplex",ios::out);
	output << "read " << "Qpar.mps" << endl;
	output << "optimize" << endl;
	output << "write " << sol1 << endl;
	output << "change sense obj MAX "  << endl;
	output << "optimize" << endl;
	output << "write " << sol2 << endl;
	output << "quit" << endl;
	output.close();
	dic_item dit,fit,memdit;
	vector obj(1,0); // Zielfunktionsvektor
	matrix constraints(Q,2);
	vector rhs(Q);
	matrix subedge(Q,2), tubedge(Q,2);
	edge_segment edgi, fedgi, hedgi;
	list<char> equ;
	int i,j;
	// constraints sind less or equal
	for(int l=0;l<Q;l++) equ.append('L');
	facilities EX; // die Funktion write_mps ist nur fuer Klasse facilities impl.
	forall_items(dit,Dsubedge) {
        tubedge=Dsubedge.inf(dit);
        edgi=Dsubedge.key(dit);
        forall_items(fit,Dsubedge) {
			subedge=Dsubedge.inf(fit);
			fedgi=Dsubedge.key(fit);
			if (compare(fedgi,edgi)!=0 && (edgi.source!=edgi.target)) {
                // Constraints erzeugen..enthaelt lhs die Diff. der Steigungen
                // rhs die Differenz der y-Achs.abschnitte
                for(l=0;l<Q;l++) {
					constraints(l,0)=-tubedge(l,1);
					constraints(l,1)=subedge(l,1);
					rhs[l]=-subedge(l,0)+tubedge(l,0);
				}
                EX.write_mps(obj,constraints,rhs,equ,"Qparhelp.mps");
                // jetzt noch Bounds erzeugen und an das erzeugte mps file haengen
                const int io_flags=ios::showpoint|ios::uppercase|ios::scientific;
                fstream input("Qparhelp.mps",ios::in);
                fstream output("Qpar.mps",ios::out);
                input.seekg(0,ios::beg);
                string tmp;
                if(!output);
				else {
					while(!input.eof()) {
						tmp.read_line(input);
						if (tmp!="ENDATA" && tmp!="") output << tmp << endl;
					} // while
					output << "BOUNDS" <<endl;
					int blanks,bct;
					double logbase;
					for(j=0;j<2;j++)
                        for(i=0;i<2;i++) {
							if (i!=0) hedgi=fedgi;
							else hedgi=edgi;
							if(hedgi.alpha_t!=0) {
                                if (j==0 ) output << " UP BOUND     COL" << i+1;
								else  output << " LO BOUND     COL" << i+1;
								logbase=log10(i+1);
                                blanks=12-(int)(logbase+1);
                                for(bct=0; bct < blanks; bct++) output << " ";
                                if (j==0 ) output << setiosflags(io_flags) << setprecision(8) << hedgi.alpha_t << endl;
                                else output << setiosflags(io_flags) << setprecision(8) << hedgi.alpha_s << endl;
							} // if hedgi_t!=0
                        } // for
                        output << "ENDATA" << endl;
                        output.close();
				}// !output
                // jetzt mit cplex l"osen und L"osung einlesen
                system(cplexcall);
                string zeile;
				result.clear();
                for(i=0;i<2;i++) {
					j=0; //sicherheitshalber eine 2. Abbruchbedingung fuer while
                    fstream input((i) ? sol2 : sol1,ios::in);
                    if (!input) { ;
					}else {
                        input.seekg(0,ios::beg);
                        double tmp;
                        l=0;
                        //t_min und t_max fuer das file
						zeile.read_line(input);
                        while(l<2 && j<1000) { 
							//  Reading the solutions for the columns  
							if (zeile[10] == 'C' && zeile[11] == 'O' && zeile[12] == 'L') {
								string wert;
								wert=zeile(30,44);
								tmp=atof(wert);
								if(l<2) result.append(tmp);
								l++;
								if(hedgi.alpha_t==0 && l==2) result.Pop();
							} else j++;//if
							zeile.read_line(input);
						} // while
                        (i) ? system("rm t_maxmps.txt"): system("rm t_minmps.txt");
                    } // if input
                } // for
                system("rm Qparhelp.mps");
		system("rm Qpar.mps");
                if (!result.empty()) {
					t_LB=result.pop();
					if(hedgi.alpha_t==0) s_LB=0;
					else s_LB=result.pop();
					t_UB=result.pop();
					if(hedgi.alpha_t==0) s_UB=0;
					else s_UB=result.pop();
					LB_equal=true;
					UB_equal=true;
					for(l=0;l<Q;l++) {
						if ( tubedge(l,0)+tubedge(l,1)*t_LB!=subedge(l,0)+subedge(l,1)*s_LB) LB_equal=false;
						if ( tubedge(l,0)+tubedge(l,1)*t_UB!=subedge(l,0)+subedge(l,1)*s_UB) UB_equal=false;
					} // for
					if (!(LB_equal && UB_equal)) { // Dominanz!!,also aendern
						LB_equal=false;
						memdit=Dsubedge.pred(dit);
						Dsubedge.del_item(dit);
						s_LB=edgi.alpha_t; // zum merken
						if (edgi.alpha_s+epsilon<t_LB) { // Segment vor LB
							edgi.alpha_t=t_LB;
							dit=Dsubedge.insert(edgi,tubedge);
							LB_equal=true; // dit hat wieder einen Wert..
						}
						if (s_LB>t_UB+epsilon) { // Segment nach UB
							edgi.alpha_t=s_LB;
							edgi.alpha_s=t_UB;
							Dsubedge.insert(edgi,tubedge);
							if (!LB_equal) {dit=Dsubedge.lookup(edgi);
							LB_equal=true;}
						}
						if (!LB_equal) {dit=memdit;break;} // keine weiteren Vergleiche mit fit noetig (wiederholung..)
					} // if
				} // result.empty
           } // if fedgi!=edgi
		} //forall fit
	} // forall dit
	return;
}//parloc_cplex

#else

void parloc_lpsolve(int Q,dictionary<edge_segment,matrix>& Dsubedge)
/*********************************************************************
* Purpose   : writes LP for lp_solve and calls solver for
*             1/G(undir)/./d(V,G)/Q-sumpar
* Returns   : %
* Influences: %
* Date      : Feb 1999
* Author    : Holger Hennes
*********************************************************************/
{
	double t_LB,t_UB,s_LB,s_UB, epsilon=0.0001;;
	bool LB_equal,UB_equal;
	dic_item dit,fit,memdit;
	double *constraint1=new double[Q+1],*constraint2=new double[Q+1];
	double *rhs=new double[Q+1];
	double *objfunc=new double[3];
	double *sollow=new double[3];
	double *solup=new double[3];
	objfunc[1]=1;objfunc[2]=0;
	double *lowbo=new double[3], *upbo=new double[3];
	matrix subedge(Q,2), tubedge(Q,2);
	edge_segment edgi, fedgi, hedgi;
	int i,j,l;
	forall_items(dit,Dsubedge) {
        tubedge=Dsubedge.inf(dit);
        edgi=Dsubedge.key(dit);
        forall_items(fit,Dsubedge) {
			subedge=Dsubedge.inf(fit);
			fedgi=Dsubedge.key(fit);
			if (compare(fedgi,edgi)!=0 && (edgi.source!=edgi.target)) {
                // Constraints erzeugen..enthaelt lhs die Diff. der Steigungen
                // rhs die Differenz der y-Achs.abschnitte
                for(l=0;l<Q;l++) {
					constraint1[l]=-tubedge(l,1);
					constraint2[l]=subedge(l,1);
					rhs[l+1]=-subedge(l,0)+tubedge(l,0);
				}
                // jetzt noch Bounds erzeugen und an das erzeugte mps file haengen
				for(j=0;j<2;j++) {
					for(i=0;i<2;i++) {
						if (i!=0) hedgi=fedgi;
						else hedgi=edgi;
						if(hedgi.alpha_t!=0) {
							if (j==0 ) upbo[i]=hedgi.alpha_t;
							else lowbo[i]=hedgi.alpha_s;
						} // if hedgi_t!=0
						else { 							
							if (j==0 ) upbo[i]=1;
							else lowbo[i]=0;
						}
					}
				} // for
                // jetzt mit cplex l"osen und L"osung einlesen
				sollow=Q_medpar_solve(Q,rhs,constraint1,constraint2,lowbo,upbo,objfunc,0);
				solup=Q_medpar_solve(Q,rhs,constraint1,constraint2,lowbo,upbo,objfunc,1);
				t_LB=sollow[0];
				if(hedgi.alpha_t==0) s_LB=0;
				else s_LB=sollow[1];
				t_UB=solup[0];
				if(hedgi.alpha_t==0) s_UB=0;
				else s_UB=solup[1];
				LB_equal=true;
				UB_equal=true;
				for(l=0;l<Q;l++) {
					if ( tubedge(l,0)+tubedge(l,1)*t_LB!=subedge(l,0)+subedge(l,1)*s_LB) LB_equal=false;
					if ( tubedge(l,0)+tubedge(l,1)*t_UB!=subedge(l,0)+subedge(l,1)*s_UB) UB_equal=false;
				} // for
				if (!(LB_equal && UB_equal)) { // Dominanz!!,also aendern
					LB_equal=false;
					memdit=Dsubedge.pred(dit);
					Dsubedge.del_item(dit);
					s_LB=edgi.alpha_t; // zum merken
					if (edgi.alpha_s+epsilon<t_LB) { // Segment vor LB
						edgi.alpha_t=t_LB;
						dit=Dsubedge.insert(edgi,tubedge);
						LB_equal=true; // dit hat wieder einen Wert..
					}
					if (s_LB>t_UB+epsilon) { // Segment nach UB
						edgi.alpha_t=s_LB;
						edgi.alpha_s=t_UB;
						Dsubedge.insert(edgi,tubedge);
						if (!LB_equal) {dit=Dsubedge.lookup(edgi);
						LB_equal=true;}
					}
					if (!LB_equal) {dit=memdit;break;} // keine weiteren Vergleiche mit fit noetig (wiederholung..)
				} // if
			} // if fedgi!=edgi
		} //forall fit
	} // forall dit
	delete[] constraint1;
	delete[] constraint2;
	delete[] rhs;
	delete[] objfunc;
	delete[] sollow;
	delete[] solup;
	delete[] lowbo;
	delete[] upbo;

	return;
}//parloc_lpsolve

#endif

list<edge_segment> lolaundirected::Q_medpareto(list<vector>& objval)
/*********************************************************************
* Purpose   : returns the pareto-optimal solution for a Q-criterial
*             graph, solves 1/G(undir)/./d(V,G)/Q-sumpar
* Returns   : list<edge_segment>
* Influences: %
* Date      : Aug 1997
* Author    : Karen Schulze
*********************************************************************/
{
	list<edge_segment> new_solution;
	while (!Is_Connected(*this)) {
		lola_message("Graph is not connected: LOLA is could connect it",0,1,true);
		Make_Connected(*this);
	}
	lolaundirected U=*this;
	list<edge_segment> solution,edge_sol;
	dictionary<edge_segment,matrix > Dsubedge;
	edge_segment edgi;
	int Q=U.inf(U.first_node()).length();
	matrix subedge(Q,2);
	list<bottle_elem> breakpoints;
	U.make_undirected();
	// shortest_path matrix D
	matrix D(U.number_of_nodes(),U.number_of_nodes());
	D = U.shortest_path();
	// median-matrix F
	matrix F(Q,U.number_of_nodes());
	F=U.F_median(D,false);
	int i=0,j,l;
	// lokale L"osungsmenge bestimmen..
	solution=U.parlocation(objval,0,0,0);
	forall(edgi,solution) {
        // ab ins dictionary
        for(l=0;l<Q;l++) subedge(l,0)=F(l,edgi.source);
        Dsubedge.insert(edgi,subedge);
	}
	// ******************  Kantenl"osungen ************************************
	edge e;
	node_array<int> node2number(U);
	node knot;
	bottle_elem pred,current;
	forall_nodes(knot,U) node2number[knot]=i++;
	forall_edges(e,U) {
        breakpoints.clear();
        edge_sol.clear();
        i=node2number[U.source(e)];
        j=node2number[U.target(e)];
        edge_sol=U.edge_pareto(D,F,i,j,U.inf(e),breakpoints);
        // nach ganzer Kante suchen
        edgi.source=i;edgi.target=j;edgi.alpha_s=0;edgi.alpha_t=1;
        if(edge_sol.length()==3 && solution.search(edge_sol.head())) {
			solution.append(edgi);
			// ab ins dictionary
			for(l=0;l<Q;l++) subedge(l,0)=F(l,edgi.source);
			Dsubedge.insert(edgi,subedge);
		}
        // Teilkanten in Dsubedge mit Steigungen etc. sammeln
        if (!breakpoints.empty()) {
			pred=breakpoints.pop();
			forall(current,breakpoints) {
				edgi.alpha_s=pred.t_value;
				edgi.alpha_t=current.t_value;
				// Steigung und y-Achsenabschnitt zu jeder Zielfunkt.
				for(l=0;l<Q;l++) {
					// Steigung
					subedge(l,1)=(current.F_column[l]-pred.F_column[l])/(edgi.alpha_t-edgi.alpha_s);
					// y-Achsenabschnitt
					subedge(l,0)=pred.F_column[l]-subedge(l,1)*edgi.alpha_s;
				} // for l
				Dsubedge.insert(edgi,subedge);
			} // forall
        } // if
	} // forall_edges
	// Vergleiche 
#ifdef HAVECPLEX
	parloc_cplex(Q,Dsubedge);
	system("rm runcplex");
	system("rm cplex.log");
#else
	parloc_lpsolve(Q,Dsubedge);
#endif
	dic_item dit;
	edge_sol.clear();
	objval.clear();
	vector begobj(Q),endobj(Q);
	forall_items(dit,Dsubedge) {
		subedge=Dsubedge.inf(dit);
       	edge_sol.append(Dsubedge.key(dit));
		// Zielfunktionsvektoren:
		if(Dsubedge.key(dit).source==Dsubedge.key(dit).target) {  // Knoten
			objval.append(F.col(Dsubedge.key(dit).source));
			objval.append(F.col(Dsubedge.key(dit).target));
		} else {
			// Zielfunktionsvektor des Kantenabschnitts-Anfangspunkt(begobj) und Endpunktes(endobj)
			for(i=0;i<Q;i++) {
				begobj[i]=subedge(i,0)+Dsubedge.key(dit).alpha_s*subedge(i,1);	
				endobj[i]=subedge(i,0)+Dsubedge.key(dit).alpha_t*subedge(i,1);	
			} // for
			objval.append(begobj);
			objval.append(endobj);
		} // if
	} // forall
	// skalieren
	node v,w;
	double edgelength;
	array<node> number2node(U.number_of_nodes());
	i=0;
	forall_nodes(v,U) number2node[i++]=v;
	edge_segment ana;
	forall(ana,edge_sol)
        if(ana.source!=ana.target) {
			v=number2node[ana.source];
			w=number2node[ana.target];
			e=U.inz_edge(v,w);
			edgelength=U.inf(e);
			ana.alpha_s=ana.alpha_s/edgelength;
			ana.alpha_t=ana.alpha_t/edgelength;
			new_solution.append(ana);
        } else new_solution.append(ana);
	return new_solution;
}

