#include <LOLA/blocknorm.h>
#include <stdarg.h>

Blocknorm::Blocknorm(int dim) {
   angles = new double[dim];
   points = new location[dim];
   angles_size = dim;
}

Blocknorm::Blocknorm(double dim, double point1x, double point1y,...) {
   va_list parameter;
   va_start (parameter, point1y);
   angles_size = (int)dim;
   points = new location[angles_size];
   angles = new double[dim];
   points[0][0] = point1x;
   points[0][1] = point1y;
   angles[0] = points[0][1]/points[0][0];
   angles[0] = atan(points[0][1]/points[0][0])/LEDA_PI*180;
   for (int i = 1; i < angles_size; i++) {
      points[i][0] = va_arg (parameter, double);
      points[i][1] = va_arg (parameter, double);
      angles[i] = atan(points[i][1]/points[i][0])/LEDA_PI*180;
   }
   va_end(parameter);
}

Blocknorm::Blocknorm(list<point> &pnts) {
	angles_size = (int)pnts.size()/2;
	points = new location[angles_size];
   angles = new double[angles_size];
	point trav = pnts.head();
	int i = 0;
	forall(trav,pnts) {
		if (trav.ycoord() >= 0) {
			if (trav.ycoord() == 0) {
				if (trav.xcoord() > 0) {
					points[i] = point2loc(trav);
                                        angles[i] = atan(points[i][1]/points[i][0])/LEDA_PI*180;
				}
			} else {
				points[i] = point2loc(trav);
                                angles[i] = atan(points[i][1]/points[i][0])/LEDA_PI*180;
			}
			if (++i == angles_size) break;
		}
	}
}

Blocknorm::Blocknorm(list<location> &locs) {
	angles_size = (int)locs.size()/2;
	points = new location[angles_size];
	angles = new double[angles_size];
	location trav = locs.head();
	int i = 0;
	forall(trav,locs) {
		if (trav[1] >= 0) {
			if (trav[1] == 0) {
				if (trav[0] > 0) {
					points[i] = trav;
                                        angles[i] = atan(points[i][1]/points[i][0])/LEDA_PI*180;
				}
			} else {
				points[i] = trav;
                                angles[i] = atan(points[i][1]/points[i][0])/LEDA_PI*180;
			}
			if (++i == angles_size) break;
		}
	}
}
   
Blocknorm::Blocknorm(const Blocknorm &B) {
   angles_size = B.angles_size;
   points = new location[angles_size];
   angles = new double[angles_size];
   for (int i = 0; i < angles_size; i++) {
      points[i] = B.points[i];
      angles[i] = B.angles[i];
   }
}

bool Blocknorm::operator == (Blocknorm &B) {
   if (angles_size != B.angles_size) return false;
   for (int i = 0; i < angles_size; i++)
     if (angles[i] != B.angles[i])
       return false;
   return true;
}

bool Blocknorm::operator != (Blocknorm &B) {
   if (angles_size != B.angles_size) return true;
   for (int i = 0; i < angles_size; i++)
     if (angles[i] != B.angles[i])
       return true;
   return false;
}

Blocknorm& Blocknorm::operator = (const Blocknorm &B) {
   if (angles == B.angles && points == B.points) return *this;
   delete[] angles;
   delete[] points;
   angles_size = B.angles_size;
   angles = new double[angles_size];
   points = new location[angles_size];
   for (int i = 0; i < angles_size; i++)  {
      angles[i] = B.angles[i];
      points[i] = B.points[i];
   }
   return *this;
}

ostream& operator << (ostream &stream, Blocknorm &B) {
   stream << "( ";
   int i;
   for (i = 0; i < B.angles_size; i++)
     stream << B.points[i] << " ";
   stream << "| ";
   for (i = 0; i < B.angles_size; i++)
     stream << B.angles[i] << " ";
   stream << ")";
   return stream;
}
