#include <LOLA/line.h>

Line::Line() {
   M = 0;
   A = 0;
   B = 0;
   C = 0;
}

Line::Line(location Ex_i, location Ex_j) {
   if (Ex_i[0] == Ex_j[0]) {M = L_INF; B = 0;}
   else {
      if (Ex_i[1] == Ex_j[1]) {M = 0; B = 1;}
      else {
	 if (Ex_i[0] < Ex_j[0]) {
	    M = (Ex_j[1]-Ex_i[1])/(Ex_j[0]-Ex_i[0]);
	    B = (-1)*(Ex_j-Ex_i)[0]/(Ex_j-Ex_i)[1];
	 } else {
	    M = (Ex_i[1]-Ex_j[1])/(Ex_i[0]-Ex_j[0]);
	    B = (-1)*(Ex_i-Ex_j)[0]/(Ex_i-Ex_j)[1];
	 }
      }
   }
   l1 = Ex_i;
   l2 = Ex_j;
   if (M == 0) A = 0;
   else A = 1;
   C = (Ex_i[0]*A)+(Ex_i[1]*B);
}

Line::Line(const Line &L) {
   M = L.M;
   l1 = L.l1;
   l2 = L.l2;
   A = L.A;
   B = L.B;
   C = L.C;
}

Line::~Line() {}

bool Line::parallel(Line &L) {
   if (fabs(M-L.M) < L_eps) return true;
   return false;
}

double Line::weighted_distance(location &L) {
   location direction = l1-l2, u = L-l2, norm_direction;
   norm_direction = direction * (1/direction.norm(2));
   double chk = fabs((u*u) - (u*norm_direction)*(u*norm_direction)); 
   if (chk < 1E-10) chk = 0;
   double dist = L(0)*fabs(sqrt(chk));
   return dist;
}

double Line::distance(location &L) {
   location direction = l1-l2, u = L-l2, norm_direction;
   norm_direction = direction * (1/direction.norm(2));
   double chk = fabs((u*u) - (u*norm_direction)*(u*norm_direction)); 
   if (chk < 1E-10) chk = 0;
   double dist = fabs(sqrt(chk));
   return dist;
}

double Line::lp_distance(location &L, int p) {
   /****  not l1 or l_inf  ****/
   double s = 0, b = 0, exponent = 1/(p-1), x = 0, y = 0, dist = 0;
   if (l2[0]-l1[0] == 0) s = L_INF;
   else s = (l2[1]-l1[1])/(l2[0]-l1[0]);
   b = (-s*l1[0])+l1[1];
   if (s < 0) x = (pow(-s,exponent)*(b-L[1])+L[0])/(1-pow(-s,exponent)*s);
   else x = (pow(s,exponent)*(L[1]-b)+L[0])/(1+pow(s,exponent)*s);
   y = s*x+b;
   location linepnt(2.0,1.0,x,y,1.0);
   dist = (linepnt-L).norm(2);
   return dist;
}

location Line::intersection(Line &L) {
   double m1 = (l2[1]-l1[1])/(l2[0]-l1[0]),
		m2 = (L.l2[1]-L.l1[1])/(L.l2[0]-L.l1[0]),
		b1 = (-m1*l1[0])+l1[1], b2 = (-m2*L.l1[0])+L.l1[1];
   location result;
   if (parallel(L) & b1 != b2) {
      result(0) = distance(L.l1);
   } else {
      if (parallel(L) & b1 != b2) {
			result = l1;
			result(0) = L_INF;
      } else {
			result[0] = (b2-b1)/(m1-m2);
			result[1] = m1*result[0]+b1;
      }
   }
   return result;
}

Line& Line::operator = (const Line &L) {
   if (this == &L) return *this;
   M = L.M;
   l1 = L.l1;
   l2 = L.l2;
   A = L.A;
   B = L.B;
   C = L.C;
   return *this;
}

bool Line::operator < (Line &L) {
   if (l1 == L.l1 && l2 == L.l2) return false;
   if (M > L.M) return false;
   if (M == L.M && C == L.C) {
      if (l1 > L.l1) return false;
      if (l1 == L.l1 && l2 > L.l2) return false;
      return true;
   }
   if (M == L.M) {
      if (M > 0 && C > L.C) return false;
      if (M <= 0 && C <= L.C) return false;
   }
   return true;
}

bool Line::operator > (Line &L) {
   if (l1 == L.l1 && l2 == L.l2) return false;
   if (M < L.M) return false;
   if (M == L.M && C == L.C) {
      if (l1 < L.l1) return false;
      if (l1 == L.l1 && l2 < L.l2) return false;
      return true;
   }
   if (M == L.M) {
      if (M > 0 && C < L.C) return false;
      if (M <= 0 && C >= L.C) return false;
   }
   return true;
}

bool Line::operator == (Line &L) {
   if (M == L.M && C == L.C) return true;
   return false;
}

ostream& operator << (ostream& os, Line &output) {
   os << "(" << " " << output.M << " || " << output.l1 << " || " << output.l2
      << " || " << output.C << " )"; 
   return os;
}

