#include <LOLA/lines.h>
#include <LOLA/line.h>

double lines::L_l2_sum_M3(facilities& facs) {
/********************************************************************
* Purpose   : calculates the optimal-median with l2-metric 
* Returns   : function-value
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  solution.clear();
  int border = facs.size();
  double z = L_INF, dist = 0, chk = 0;
  location Ex_i, Ex_j, Ex_k, direction, u, norm_direction;
  list_item tmp_i = facs.first(), tmp_j = facs.first(), tmp_k = facs.first();
  for (int i = 0; i < border; i++) {
    Ex_i = facs.contents(tmp_i);
    tmp_j = facs.succ(tmp_i);
    for (int j = i+1; j < border; j++) {
      Ex_j = facs.contents(tmp_j);
      dist = 0;
      for (int k = 0; k < border; k++) {
	Ex_k = facs.contents(tmp_k);
	direction = Ex_i-Ex_j; u = Ex_k-Ex_j;
	norm_direction = direction * (1/direction.norm(2));
	chk = fabs((u*u) - (u*norm_direction)*(u*norm_direction));
	if (chk < 1E-10) chk = 0;
	dist = dist + Ex_k(0)*fabs(sqrt(chk));
	if (dist > z+1) break;
	tmp_k = facs.succ(tmp_k);
      }
      tmp_k = facs.first();
      if (dist < z) {
	z = dist;
	solution.clear();
	Ex_i(0) = 1; solution.append(Ex_i);
	Ex_j(0) = 1; solution.append(Ex_j);
      }
      tmp_j = facs.succ(tmp_j);
    }
    tmp_i = facs.succ(tmp_i);
  }
  return z;
}

double lines::L_l1_sum_M3(facilities &facs) {
/********************************************************************
* Purpose   : calculates the optimal-median with l1-metric 
* Returns   : function-value
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  int border = facs.size();
  double dist = 0, chk = 0, chk1 = L_INF, chk2 = L_INF, s = 0, z = L_INF;
  location Ex_i, Ex_j, Ex_k, direct_g, isectpnt;
  list_item tmp_i = facs.first(), tmp_j = facs.first(), tmp_k = facs.first();
  for (int i = 0; i < border; i++) {
    Ex_i = facs.contents(tmp_i);
    tmp_j = facs.succ(tmp_i);
    for (int j = i+1; j < border; j++) {
      Ex_j = facs.contents(tmp_j);
      dist = 0; direct_g = Ex_j-Ex_i;
      for (int k = 0; k < border; k++) {
	if (k != i && k != j) {
	  chk1 = L_INF; chk2 = L_INF;
	  Ex_k = facs.contents(tmp_k);
	  if (direct_g[0] != 0) { // not vertical
	    s = (Ex_k[0]-Ex_i[0])/direct_g[0];
	    isectpnt[0] = Ex_k[0];
	    isectpnt[1] = Ex_i[1] + s*direct_g[1];
	    chk1 = Ex_k(0)*(Ex_k-isectpnt).norm(2);
	  }
	  if (direct_g[1] != 0) { // not horizontal
	    s = (Ex_k[1]-Ex_i[1])/direct_g[1];
	    isectpnt[1] = Ex_k[1];
	    isectpnt[0] = Ex_i[0] + s*direct_g[0];
	    chk2 = Ex_k(0)*(Ex_k-isectpnt).norm(2);
	  }
	  if (chk1 < chk2) chk = chk1;
	  else chk = chk2;
	  if (chk < 1E-10) chk = 0;
	  dist += chk;
	  if (dist > z+1) break;
	}
	tmp_k = facs.succ(tmp_k);
      }
      tmp_k = facs.first();
      if (dist < z) {
	z = dist;
	solution.clear();
	Ex_i(0) = 1; solution.append(Ex_i);
	Ex_j(0) = 1; solution.append(Ex_j);
      }
      tmp_j = facs.succ(tmp_j);
    }
    tmp_i = facs.succ(tmp_i);
  }
  return z;
}

double lines::L_linf_sum_M3(facilities &facs) {
/********************************************************************
* Purpose   : calculates the optimal-median with linf-metric 
* Returns   : function-value
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  facilities facs1 = facs;
  trans_all(facs1);
  double z = this->L_l1_sum_M3(facs1);
  list_item trav;
  forall_items(trav,solution)
    solution[trav].retransform();
  return z;
}

double lines::L_lp_sum_M3(facilities &facs, int p) {
/********************************************************************
* Purpose   : calculates the optimal-median with lp-metric 
* Returns   : function-value
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  double dist = 0, chk = 0, z = L_INF;
  int border = facs.size();
  location Ex_i, Ex_j, Ex_k, direction, u, norm_direction;
  list_item tmp_i = facs.first(), tmp_j = facs.first(), tmp_k = facs.first();
  for (int i = 0; i < border; i++) {
    Ex_i = facs.contents(tmp_i);
    tmp_j = facs.succ(tmp_i);
    for (int j = i+1; j < border; j++) {
      Ex_j = facs.contents(tmp_j);
      dist = 0;
      for (int k = 0; k < border; k++) {
	Ex_k = facs.contents(tmp_k);
	chk = this->lp_distance_2_line(Ex_i,Ex_j,Ex_k,p);
	if (chk < 1E-10) chk = 0;
	dist = dist + Ex_k(0)*chk;
	if (dist > z+1) break;
	tmp_k = facs.succ(tmp_k);
      }
      tmp_k = facs.first();
      if (dist < z) {
	z = dist;
	solution.clear();
	Ex_i(0) = 1; solution.append(Ex_i);
	Ex_j(0) = 1; solution.append(Ex_j);
      }
      tmp_j = facs.succ(tmp_j);
    }
    tmp_i = facs.succ(tmp_i);			
  }
  return z;
}

double lines::L_l2_sum_M2(facilities &facs) {
/********************************************************************
* Purpose   : calculates the optimal-median with l2-metric 
* Returns   : function-value
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  int N = facs.size();
  double z = L_INF;
  /**** facs consists of only two locations ****/
  if (N == 2) {
    z = 0;
    solution.append(facs.head()), solution.append(facs.tail());
    return z;
  }
  location Ex_i, Ex_j;
  double W = facs.total_weight()[0], halfwht = (int)W/2;
  int i = 0, j = 0;
  /****  Initializing locations  ****/
  location *locs = new location[N];
  forall(Ex_i,facs) {
    Ex_i(0) = 1;
    locs[i++] = Ex_i;
  }
  /****  Sorting the locations  ****/
  /****  with updating the labels  */
  for (i = N-1; i >= 0; i--)
    for (j = 1; j <= i; j++)
      if (locs[j-1][1] > locs[j][1] || (locs[j-1][1] == locs[j][1] && locs[j-1][0] > locs[j][0])) {
	Ex_i = locs[j-1]; locs[j-1] = locs[j]; locs[j] = Ex_i;
      }
  /****  building the starting lower set  ****/
  double weightedXupper = 0, weightedXlower = 0,
    weightedYupper = 0, weightedYlower = 0,
    lowerwht = 0, upperwht = 0, wht = 0;
  facilities lower, upper, start_points;
  i = 0;
  while (wht < halfwht) {
    Ex_i = locs[i];
    wht += Ex_i(0);
    if (wht > halfwht) break;
    weightedXlower += Ex_i(0)*Ex_i[0];
    weightedYlower += Ex_i(0)*Ex_i[1];
    lowerwht += Ex_i(0);
    lower.append(Ex_i);
    i++;
  }
  location Ex_pivot = locs[i];
  int up_interval_border = 0, k = 0;
  if (wht == halfwht) {Ex_pivot = locs[i-1]; up_interval_border = i;}
  /****  building the starting upper set  ****/
  for (k = i; k < N; k++) {
    Ex_i = locs[k];
    upper.append(Ex_i);
    weightedXupper += Ex_i(0)*Ex_i[0];
    weightedYupper += Ex_i(0)*Ex_i[1];
    upperwht += Ex_i(0);
  }
  /****  Scanning for multiple startingpoints  ****/
  while (i > 0 && locs[i-1][1] == Ex_pivot[1]) {
    Ex_i = locs[--i];
    lower.remove(Ex_i);
    weightedXlower -= Ex_i(0)*Ex_i[0];
    weightedYlower -= Ex_i(0)*Ex_i[1];
    lowerwht -= Ex_i(0);
    upper.push(locs[i]);
    weightedXupper += Ex_i(0)*Ex_i[0];
    weightedYupper += Ex_i(0)*Ex_i[1];
    upperwht += Ex_i(0);
  }
  /****  Building starting points  ****/
  while (i < N && locs[i][1] == Ex_pivot[1]) {
    start_points.append(locs[i++]);
  }
  if (up_interval_border) {
    while (i < N && locs[i][1] == locs[up_interval_border][1])
      start_points.append(locs[i++]);
  }
  /****  Algorithm  ****/
  facilities tmplower, tmpupper, startlower = lower, startupper = upper;
  double dist = L_INF,
    wXupper_start = weightedXupper, wXlower_start = weightedXlower,
    wYupper_start =  weightedYupper, wYlower_start = weightedYlower,
    wlower_start = lowerwht, wupper_start =  upperwht;
  /****  Starting with every starting point  ****/
  for (int start = 0; start < start_points.size(); start++) {
    lower = startlower; upper = startupper;
    weightedXupper = wXupper_start; weightedXlower = wXlower_start;
    weightedYupper = wYupper_start; weightedYlower = wYlower_start;
    lowerwht = wlower_start; upperwht = wupper_start;
    /****  Updating sets in dependency of the current start point ****/
    for (k = 0; k < start; k++) {
      Ex_i = start_points.get_element(k);
      upper.remove(Ex_i);
      weightedXupper -= Ex_i(0)*Ex_i[0];
      weightedYupper -= Ex_i(0)*Ex_i[1];
      upperwht -= Ex_i(0);
      lower.append(Ex_i);
      weightedXlower += Ex_i(0)*Ex_i[0];
      weightedYlower += Ex_i(0)*Ex_i[1];
      lowerwht += Ex_i(0);
    }
    Ex_pivot = start_points.get_element(start);
    upper.remove(Ex_pivot);
    weightedXupper -= Ex_pivot(0)*Ex_pivot[0];
    weightedYupper -= Ex_pivot(0)*Ex_pivot[1];
    upperwht -= Ex_pivot(0);
    int cnt = 0, new_pivotidx = 0, group = 0,
      lowersize = lower.size(), uppersize = upper.size();
    location Ex_comp, new_Ex_pivot, old_Ex_pivot;
    Loc_Vector direct(2.0,1.0,0.0), compdirect(2.0,1.0,0.0), comp;
    double start_a = 0, start_b = 0, start_c = 0,
      a = 1, b = 1, c = 1, m = 1;
    facilities pairs;
    /****  Rotating the line  ****/
    do {
      tmplower.clear(); tmpupper.clear();
      dist = L_INF;
      double s = 0, theta_tmp, theta = 200, test = 0;
      /****  scanning for new pivotpoint in lower  ****/
      for (k = 0; k < lowersize; k++) {
	Ex_comp = lower.get_element(k);
	comp = Ex_comp.pos()-Ex_pivot.pos();
	test = (direct*comp)/(direct.norm(2)*comp.norm(2));
	if (test > 1) test = 1;
	if (test < -1) test = -1;
        theta_tmp = LEDA_PI-acos(test);
	if ((theta_tmp < theta && theta_tmp != 0) ||
	    (theta_tmp == theta && (new_Ex_pivot-Ex_pivot).norm(2) > (Ex_comp-Ex_pivot).norm(2))) {
	  theta = theta_tmp;
	  new_Ex_pivot = Ex_comp;
	  new_pivotidx = k;
	  group = 0;
	}
        if (theta_tmp == 0 || theta_tmp == LEDA_PI) {
	  if (Ex_comp != new_Ex_pivot) {
	    tmplower.append(Ex_comp);
	    weightedXlower -= Ex_comp(0)*Ex_comp[0];
	    weightedYlower -= Ex_comp(0)*Ex_comp[1];
	    lowerwht -= Ex_comp(0);
	    lower.remove(k--);
	    lowersize--;
	  }
	}
      }
      /****  scanning for new pivotpoint in upper  ****/
      for (k = 0; k < uppersize; k++) {
	Ex_comp = upper.get_element(k);
	comp = Ex_comp.pos()-Ex_pivot.pos();
	test = (direct*comp)/(direct.norm(2)*comp.norm(2));
	if (test > 1) test = 1;
	if (test < -1) test = -1;
	theta_tmp = acos(test);
	if ((theta_tmp < theta && theta_tmp != 0) ||
	    (theta_tmp == theta && (new_Ex_pivot-Ex_pivot).norm(2) > (Ex_comp-Ex_pivot).norm(2))) {
	  theta = theta_tmp;
	  new_Ex_pivot = Ex_comp;
	  new_pivotidx = k;
	  group = 1;
	}
        if (theta_tmp == 0 || theta_tmp == LEDA_PI) {
	  if (Ex_comp != new_Ex_pivot) {
	    tmpupper.append(Ex_comp);
	    weightedXupper -= Ex_comp(0)*Ex_comp[0];
	    weightedYupper -= Ex_comp(0)*Ex_comp[1];
	    upperwht -= Ex_comp(0);
	    upper.remove(k--);
	    uppersize--;
	  }
	}
      }
      /******  Calculating the objective functionvalue  *****/
      if (old_Ex_pivot(0) != 0) {
	if (Ex_pivot[0] == old_Ex_pivot[0]) {m = L_INF; b = 0;}
	else {
	  if (Ex_pivot[1] == old_Ex_pivot[1]) {m = 0; b = 1;}
	  else {
	    if (Ex_pivot[0] < old_Ex_pivot[0]) {
	      m = (old_Ex_pivot[1]-Ex_pivot[1])/(old_Ex_pivot[0]-Ex_pivot[0]);
	      b = (-1)*(old_Ex_pivot-Ex_pivot)[0]/(old_Ex_pivot-Ex_pivot)[1];
	    } else {
	      m = (Ex_pivot[1]-old_Ex_pivot[1])/(Ex_pivot[0]-old_Ex_pivot[0]);
	      b = (-1)*(Ex_pivot-old_Ex_pivot)[0]/(Ex_pivot-old_Ex_pivot)[1];
	    }
	  }
	}
	if (m == 0) a = 0;
	else a = 1;
	c = (Ex_pivot[0]*a)+(Ex_pivot[1]*b);
	dist = ((weightedXupper-weightedXlower)*a+
		(weightedYupper-weightedYlower)*b+
		(upperwht-lowerwht)*(-c))/sqrt((a*a)+(b*b));
	dist = fabs(dist);
	/****  comparing the old with the new functionvalue  ****/
	if (dist < z) {
	  z = dist;
	  solution.clear();
	  solution.append(Ex_pivot);
	  solution.append(old_Ex_pivot);
	}
      }
      if (cnt == 0 && (tmpupper.size() != 0 || tmplower.size() != 0)) {
	start_a = 0; start_b = 1; start_c = Ex_pivot[1];
      }
      if (cnt == 1 && start_a == 0 && start_b == 0 && start_c == 0) {
	start_a = a; start_b = b; start_c = c;
      }
      /****  Adding tmplower to upper  ****/
      for (k = 0; k < tmpupper.size(); k++) {
	Ex_comp = tmpupper.get_element(k);
	weightedXlower += Ex_comp(0)*Ex_comp[0];
	weightedYlower += Ex_comp(0)*Ex_comp[1];
	lowerwht += Ex_comp(0);
	lower.append(Ex_comp);
      }
      /****  Adding tmpupper to lower  ****/
      for (k = 0; k < tmplower.size(); k++) {
	Ex_comp = tmplower.get_element(k);
	weightedXupper += Ex_comp(0)*Ex_comp[0];
	weightedYupper += Ex_comp(0)*Ex_comp[1];
	upperwht += Ex_comp(0);
	upper.append(Ex_comp);
      }
      compdirect = Ex_pivot.pos()-old_Ex_pivot.pos();
      if (compdirect[0] != 0) s = direct[0]/compdirect[0];
      else s = direct[1]/compdirect[1];
      /**** Updating the upper and lower set  ****/
      if (group) {
	upper.remove(new_pivotidx);
	weightedXupper -= new_Ex_pivot(0)*new_Ex_pivot[0];
	weightedYupper -= new_Ex_pivot(0)*new_Ex_pivot[1];
	upperwht -= new_Ex_pivot(0);
	if (old_Ex_pivot(0) != 0) {
	  if (s > 0) {
	    upper.append(old_Ex_pivot);
	    weightedXupper += old_Ex_pivot(0)*old_Ex_pivot[0];
	    weightedYupper += old_Ex_pivot(0)*old_Ex_pivot[1];
	    upperwht += old_Ex_pivot(0);
	  } else {
	    lower.append(old_Ex_pivot);
	    weightedXlower += old_Ex_pivot(0)*old_Ex_pivot[0];
	    weightedYlower += old_Ex_pivot(0)*old_Ex_pivot[1];
	    lowerwht += old_Ex_pivot(0);
	  }
	}
	direct = new_Ex_pivot.pos()-Ex_pivot.pos();
      } else {
	lower.remove(new_pivotidx);
	weightedXlower -= new_Ex_pivot(0)*new_Ex_pivot[0];
	weightedYlower -= new_Ex_pivot(0)*new_Ex_pivot[1];
	lowerwht -= new_Ex_pivot(0);
	if (old_Ex_pivot(0) != 0) {
	  if (s > 0) {
	    upper.append(old_Ex_pivot);
	    weightedXupper += old_Ex_pivot(0)*old_Ex_pivot[0];
	    weightedYupper += old_Ex_pivot(0)*old_Ex_pivot[1];
	    upperwht += old_Ex_pivot(0);
	  } else {
	    lower.append(old_Ex_pivot);
	    weightedXlower += old_Ex_pivot(0)*old_Ex_pivot[0];
	    weightedYlower += old_Ex_pivot(0)*old_Ex_pivot[1];
	    lowerwht += old_Ex_pivot(0);
	  }
	}
	direct = Ex_pivot.pos()-new_Ex_pivot.pos();
      }
      if (Ex_pivot == start_points.get_element(start)) {
	if (!pairs.element(new_Ex_pivot)) pairs.append(new_Ex_pivot);
	else cnt = N*N-1;
      }
      old_Ex_pivot = Ex_pivot;
      Ex_pivot = new_Ex_pivot;
      lowersize = lower.size(), uppersize = upper.size();
      /****  Find new pivot index  ****/
      for (int scanidx = 0; scanidx < N; scanidx++)
	if (locs[scanidx] == Ex_pivot) new_pivotidx = scanidx;
      if (cnt > 1 && start_a == a && start_b == b && start_c == c) cnt = N*N-1;
      cnt++;
    }
    while (cnt < N*N);
   }
   delete[] locs;
   return z;
}
double lines::L_l2_sum_M2logM(facilities &facs) {
/********************************************************************
* Purpose   : calculates the optimal-median with l2-metric 
* Returns   : function-value
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  int M = facs.size(), i = 0, border = 0;
  double W = 0, W_2 = 0, ztmp = 0, z = L_INF;
  location *locs = new location[M], *starting_locs = new location[M];
  location Ex_i = facs.head(), pivot;
  /**********  Initializing locations  ********/
  forall(Ex_i,facs) {
    locs[i] = Ex_i;
    starting_locs[i++] = Ex_i;
    W += Ex_i(0);
  }
  W_2 = W/2; z = L_INF;
  /****  The algorithm  ****/
  int cnt_pivots = 0;
  for (cnt_pivots = 0; cnt_pivots < M; cnt_pivots++) {
    pivot = starting_locs[cnt_pivots];
    border = geometric_duality(locs,pivot,M);
    /****  Initializing upper and lower set  ****/
    double weightedXupper = 0, weightedXlower = 0,
      weightedYupper = 0, weightedYlower = 0,
      lowerwht = 0, upperwht = 0;
    facilities lower, upper;
    for (i = 0; i < M-1; i++) {
      Ex_i = locs[i];
      if (i < border) {
	weightedXupper += Ex_i(0)*Ex_i[0];
	weightedYupper += Ex_i(0)*Ex_i[1];
	upperwht += Ex_i(0);
	upper.append(Ex_i);
      } else {
	weightedXlower += Ex_i(0)*Ex_i[0];
	weightedYlower += Ex_i(0)*Ex_i[1];
	lowerwht += Ex_i(0);
	lower.append(Ex_i);
      }
    }
    pivot = locs[M-1];
    location upper_loc, lower_loc; i = 0;
    int j = border, m = 0,
      lower_complete = 0, upper_complete = 0, last_in_upper = 0;
    /****  Which location could be first? ****/
    if (i < border) upper_loc = locs[i];
    else upper_complete = 1;
    if (j < M-1) lower_loc = locs[j];
    else lower_complete = 1;
    /****  Which of the two is nearer?  ****/
    location origin;
    Line first_line(pivot,origin);
    double upper_angle = 0, lower_angle = 0, tmp = 0;
    if (upper_complete) upper_angle = L_INF;
    else {
      tmp = (pivot*-1)*(upper_loc-pivot)/
	((pivot*-1).norm(2)*(upper_loc-pivot).norm(2));
      if (tmp < -1) tmp = -1;
      if (tmp > 1) tmp = 1;
      upper_angle = acos(tmp);
    }
    if (lower_complete) lower_angle = L_INF;
    else {
      tmp = pivot*(lower_loc-pivot)/
	(pivot.norm(2)*(lower_loc-pivot).norm(2));
      if (tmp < -1) tmp = -1;
      if (tmp > 1) tmp = 1;
      lower_angle = acos(tmp);
    }
    if (upper_angle < lower_angle) {
      i++; Ex_i = upper_loc; last_in_upper = 1;
    } else {
      j++; Ex_i = lower_loc; last_in_upper = 0;
    }
    /****  Calculating objective function value  ****/
    for (m = 0; m < M-1; m++) {
      Line line(pivot,Ex_i);
      ztmp = fabs(((weightedXupper-weightedXlower)*line.a()+
		   (weightedYupper-weightedYlower)*line.b()+
		   (upperwht-lowerwht)*(-line.c()))/
		  sqrt((line.a()*line.a())+(line.b()*line.b())));
      /****  Building the result set  ****/
      if (ztmp < z) {
	z = ztmp;
	solution.clear();
	location sol_pivot = pivot, sol_Ex_i = Ex_i;
	sol_pivot(0) = 1; sol_Ex_i(0) = 1;
	solution.append(sol_pivot); solution.append(sol_Ex_i);
      }
      /****  Which location could be next? ****/
      if (i < border) upper_loc = locs[i];
      else upper_complete = 1;
      if (j < M-1) lower_loc = locs[j];
      else lower_complete = 1;
      // ****  Updating the sets  ****
      if (lower.element(Ex_i)) {
	weightedXlower -= Ex_i(0)*Ex_i[0];
	weightedYlower -= Ex_i(0)*Ex_i[1];
	lowerwht -= Ex_i(0);
	lower.remove(Ex_i);
	weightedXupper += Ex_i(0)*Ex_i[0];
	weightedYupper += Ex_i(0)*Ex_i[1];
	upperwht += Ex_i(0);
	upper.append(Ex_i);
      } else {
	weightedXupper -= Ex_i(0)*Ex_i[0];
	weightedYupper -= Ex_i(0)*Ex_i[1];
	upperwht -= Ex_i(0);
	upper.remove(Ex_i);
	weightedXlower += Ex_i(0)*Ex_i[0];
	weightedYlower += Ex_i(0)*Ex_i[1];
	lowerwht += Ex_i(0);
	lower.append(Ex_i);
      }
      /****  Which of the two is nearer?  ****/
      double upper_angle, lower_angle;
      if (upper_complete) upper_angle = L_INF;
      else {
	tmp = ((Ex_i-pivot)*(upper_loc-pivot))/
	  ((Ex_i-pivot).norm(2)*(upper_loc-pivot).norm(2));
	if (tmp < -1) tmp = -1;
	if (tmp > 1) tmp = 1;
	upper_angle = acos(tmp);
        if (last_in_upper == 0) upper_angle = LEDA_PI-upper_angle;
      }
      if (lower_complete) lower_angle = L_INF;
      else {
	tmp = ((Ex_i-pivot)*(lower_loc-pivot))/
	  ((Ex_i-pivot).norm(2)*(lower_loc-pivot).norm(2));
	if (tmp < -1) tmp = -1;
	if (tmp > 1) tmp = 1;
	lower_angle = acos(tmp);
        if (last_in_upper == 1) lower_angle = LEDA_PI-lower_angle;
      }
      if (upper_angle < lower_angle) {
	i++; Ex_i = upper_loc; last_in_upper = 1;
      } else {
	j++; Ex_i = lower_loc; last_in_upper = 0;
      }
    }
  }
  delete[] locs; delete[] starting_locs;
  return z;
}

double lines::L_block_sum_M3(list<polygauge> &LG,list<int> &Ln, facilities& facs) {
/********************************************************************
* Purpose   : calculates the optimal-median with block-metric 
* Returns   : function-value
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  list<point> gaugepnts;
  if (Ln.head() >= 0) gaugepnts = LG.head().vertices();
  Blocknorm B(gaugepnts);
  int border = facs.size(), B_border = B.size(),
    i = 0, j = 0, k = 0, parallel = 0, alpha = 0;
  double dist = 0, chk = 0, s = 0, test_chk = 0, z = L_INF;
  location Ex_i, Ex_j, Ex_k, direct_g, direct_h,
    isectpnt, direction, u, norm_direction, which_direct;
  list_item tmp_i = facs.first(), tmp_j = facs.first(), tmp_k = facs.first();
  for (alpha = 0; alpha < B_border; alpha++) {
    direct_h = B[alpha];
    for (i = 0; i < border; i++) {
      Ex_i = facs.contents(tmp_i);
      tmp_j = facs.succ(tmp_i);
      for (j = i+1; j < border; j++) {
	Ex_j = facs.contents(tmp_j);
	direct_g = Ex_i-Ex_j;
	dist = 0; parallel = 0;
	if ((direct_g[0] == 0 && direct_h[0] == 0) ||
	    (direct_g[1] == 0 && direct_h[1] == 0)) {
	  dist = L_INF;
	} else {
	  if (direct_g[0] != 0 && direct_g[1] != 0 &&
	      direct_h[0] != 0 && direct_h[1] != 0) {
	    if ((direct_g[1]/direct_g[0]) == (direct_h[1]/direct_h[0])) {
	      chk = L_INF; parallel = 1; dist = L_INF;
	    } 
	  }
	  if (!parallel) {
	    norm_direction = direct_g * (1/direct_g.norm(2));
	    for (k = 0; k < border; k++) {
	      Ex_k = facs.contents(tmp_k);
	      u = Ex_k-Ex_j;
	      test_chk = fabs((u*u) -
			      (u*norm_direction)*(u*norm_direction));
	      if (test_chk < 1E-10) chk = 0;
	      else {
		if (direct_g[1] != 0) {
		  s = (Ex_j[0]+(Ex_k[1]-Ex_j[1])/
		       direct_g[1]*direct_g[0]-Ex_k[0]);
		  s = s/(direct_h[0]-(direct_h[1]*direct_g[0])/
			 direct_g[1]);
		  isectpnt = Ex_k + direct_h*s;
		} else {
		  s = (Ex_k[0]+(Ex_j[1]-Ex_k[1])/
		       direct_h[1]*direct_h[0]-Ex_j[0]);
		  s = s/(direct_g[0]-(direct_g[1]*direct_h[0])/
			 direct_h[1]);
		  isectpnt = Ex_j + direct_g*s;
		}
		chk = (isectpnt-Ex_k).norm(2);
	      }
	      if (chk < 1E-10) chk = 0;
	      dist = dist + Ex_k(0)*chk;
	      if (dist > z+1) break;
	      tmp_k = facs.succ(tmp_k);
	    }
	    tmp_k = facs.first();
	  }
	  if (dist < z) {
	    which_direct = direct_h;
	    z = dist;
	    solution.clear();
	    Ex_i(0) = 1; solution.append(Ex_i);
	    Ex_j(0) = 1; solution.append(Ex_j);
	  }
	}
	tmp_j = facs.succ(tmp_j);
      }
      tmp_j = facs.first();
      tmp_i = facs.succ(tmp_i);
    }
    tmp_i = facs.first();
  }
  return z;
}


double lines::L_l2_max_M4(facilities &facs) {
/********************************************************************
* Purpose   : calculates the optimal-max-line with l2-metric 
* Returns   : function-value
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  int N = facs.size(), i = 0;
  double max_1_x = 0, max_1_y = 0, max_2_x = 0, max_2_y = 0;
  location Ex_i, Ex_j, Ex_k, result1, result2, linepnt1, linepnt2,
    direction, u, norm_direction;
  double dist = 0, tmp_dist = 0, z = L_INF;
  /****  Initializing locations  ****/
  location *locs = new location[N];
  forall(Ex_i,facs)
    locs[i++] = Ex_i;
	/****  Algorithm  ****/
  for (i = 0; i < N; i++) {
    Ex_i = locs[i];
    for (int j = i+1; j < N; j++) {
      Ex_j = locs[j];
      for (int k = j+1; k < N; k++) {
	Ex_k = locs[k];
	dist = L_INF;
	
	this->max_line_l2(Ex_i,Ex_j,Ex_k,result1,result2);
	tmp_dist = this->objective_function_l2(locs,0,N,result1,result2);
	if (tmp_dist < dist) {
	  dist = tmp_dist;
	  max_1_x = result1[0]; max_1_y = result1[1];
	  max_2_x = result2[0]; max_2_y = result2[1];
	}
	
	this->max_line_l2(Ex_i,Ex_k,Ex_j,result1,result2);
	tmp_dist = this->objective_function_l2(locs,0,N,result1,result2);
	if (tmp_dist < dist) {
	  dist = tmp_dist;
	  max_1_x = result1[0]; max_1_y = result1[1];
	  max_2_x = result2[0]; max_2_y = result2[1];
	}
	
	this->max_line_l2(Ex_j,Ex_k,Ex_i,result1,result2);
	tmp_dist = this->objective_function_l2(locs,0,N,result1,result2);
	if (tmp_dist < dist) {
	  dist = tmp_dist;
	  max_1_x = result1[0]; max_1_y = result1[1];
	  max_2_x = result2[0]; max_2_y = result2[1];
	}
	
	if (dist < z) {
	  z = dist;
	  solution.clear();
	  linepnt1 = Ex_i; linepnt1(0) = 1; solution.append(linepnt1);
	  linepnt1 = Ex_j; linepnt1(0) = 1; solution.append(linepnt1);
	  linepnt1 = Ex_k; linepnt1(0) = 1; solution.append(linepnt1);
	  linepnt1[0] = max_1_x; linepnt1[1] = max_1_y; linepnt1(0) = 2;
	  solution.append(linepnt1);
	  linepnt2[0] = max_2_x; linepnt2[1] = max_2_y; linepnt2(0) = 2;
	  solution.append(linepnt2);
	}
      }
    }
  }
  delete[] locs;
  return z;
}

double lines::L_l1_max_M4(facilities &facs) {
/********************************************************************
* Purpose   : calculates the optimal-max-line with l1-metric 
* Returns   : function-value
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  int N = facs.size(), i = 0;
  double max_1_x = 0, max_1_y = 0, max_2_x = 0, max_2_y = 0;
  location Ex_i, Ex_j, Ex_k, result1, result2, linepnt1, linepnt2,
    direction, u, norm_direction;
  double dist = 0, tmp_dist = 0, z = L_INF;
  /****  Initializing locations  ****/
  location *locs = new location[N];
  forall(Ex_i,facs)
    locs[i++] = Ex_i;
  /****  The algorithm  ****/
  for (i = 0; i < N; i++) {
    Ex_i = locs[i];
    for (int j = i+1; j < N; j++) {
      Ex_j = locs[j];
      for (int k = j+1; k < N; k++) {
	dist = L_INF;
	Ex_k = locs[k];
	
	this->max_line_l1(Ex_i,Ex_j,Ex_k,result1,result2);
	tmp_dist = this->objective_function_l1(locs,0,N,result1,result2);
	if (tmp_dist < dist) {
	  dist = tmp_dist;
	  max_1_x = result1[0]; max_1_y = result1[1];
	  max_2_x = result2[0]; max_2_y = result2[1];
	}
	
	this->max_line_l1(Ex_i,Ex_k,Ex_j,result1,result2);
	tmp_dist = this->objective_function_l1(locs,0,N,result1,result2);
	if (tmp_dist < dist) {
	  dist = tmp_dist;
	  max_1_x = result1[0]; max_1_y = result1[1];
	  max_2_x = result2[0]; max_2_y = result2[1];
	}
	
	this->max_line_l1(Ex_j,Ex_k,Ex_i,result1,result2);
	tmp_dist = this->objective_function_l1(locs,0,N,result1,result2);
	if (tmp_dist < dist) {
	  dist = tmp_dist;
	  max_1_x = result1[0]; max_1_y = result1[1];
	  max_2_x = result2[0]; max_2_y = result2[1];
	}
	
	if (dist < z) {
	  z = dist;
	  solution.clear();
	  linepnt1 = Ex_i; linepnt1(0) = 1; solution.append(linepnt1);
	  linepnt1 = Ex_j; linepnt1(0) = 1; solution.append(linepnt1);
	  linepnt1 = Ex_k; linepnt1(0) = 1; solution.append(linepnt1);
	  linepnt1[0] = max_1_x; linepnt1[1] = max_1_y; linepnt1(0) = 2;
	  solution.append(linepnt1);
	  linepnt2[0] = max_2_x; linepnt2[1] = max_2_y; linepnt2(0) = 2;
	  solution.append(linepnt2);
	}
      }
    }
  }
  delete[] locs;
  return z;
}

double lines::L_linf_max_M4(facilities &facs) {
/********************************************************************
* Purpose   : calculates the optimal-max-line with linf-metric 
* Returns   : function-value
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  facilities facs1 = facs;
  trans_all(facs1);
  double z = this->L_l1_max_M4(facs1);
  list_item trav;
  forall_items(trav,solution)
    solution[trav].retransform();
  return z;
}

double lines::L_l2_max_MlogM(facilities &facs) {
/********************************************************************
* Purpose   : calculates the optimal-max-line with l2-metric 
* Returns   : function-value
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
   int k = 0, i = 0;
   location Ex_i, Ex_j, Ex_k, Ex_l, Ex_i_tmp, Ex_j_tmp, Ex_k_tmp,
     result1, result2, direction, u, norm_direction;
   double dist = 0, tmp_dist = 0, chk = 0, z = L_INF;
   /****  Calculating the convex hull  ****/
   list<point> hullpnts, facspnts;
   point tmppnt;
   forall(Ex_i,facs) {
     tmppnt = Ex_i.loc2point();
     facspnts.append(tmppnt);
   }
   hullpnts = CONVEX_HULL(facspnts);
   int M = hullpnts.size();
   /****  Initializing locations  ****/
   location *locs = new location[M+1]; i = 1;
   forall(tmppnt,hullpnts) {
     Ex_i = point2loc(tmppnt); Ex_i(0) = 1;
     locs[i++] = Ex_i;
   }
   /****  Building the initial (edge,point) pair ****/
   Ex_i = locs[M]; Ex_j = locs[1];
   for (k = 2; k < M; k++) {
     Ex_k = locs[k];
     direction = Ex_i-Ex_j; u = Ex_k-Ex_j;
     norm_direction = direction * (1/direction.norm(2));
     chk = fabs((u*u) - (u*norm_direction)*(u*norm_direction));
     if (chk < 1E-10) chk = 0;
     tmp_dist = fabs(sqrt(chk))/2;
     if (tmp_dist < dist) {k--; break;}
     if (tmp_dist > dist) {
       solution.clear();
       solution.append(Ex_i); solution.append(Ex_j); solution.append(Ex_k);
       dist = tmp_dist;
       z = dist;
     }
   }
   /****  Scanning for the optimal (edge,point) pair  ****/
   for (i = 1; i < M; i++) {
     Ex_i = locs[i]; Ex_j = locs[i+1];
     dist = 0;
     for (;;) {
       Ex_k = locs[k];
       direction = Ex_i-Ex_j; u = Ex_k-Ex_j;
       norm_direction = direction * (1/direction.norm(2));
       chk = fabs((u*u) - (u*norm_direction)*(u*norm_direction));
       if (chk < 1E-12) chk = 0;
       tmp_dist = fabs(sqrt(chk))/2;
       if (tmp_dist < dist) {
	 if (k == 1) k = M;
	 else k--;
	 Ex_k = locs[k]; break;}
       else {
	 dist = tmp_dist;
	 if (k == M) k = 1;
	 else k++;
       }
     }
     if (dist < z) {
       z = dist;
       solution.clear();
       solution.append(Ex_i); solution.append(Ex_j); solution.append(Ex_k);
     }
   }
   /****  Calculating the line defining points  ****/
   list_item sols = solution.first();
   Ex_i = solution.contents(sols); sols = solution.succ(sols);
   Ex_j = solution.contents(sols); sols = solution.succ(sols);
   Ex_k = solution.contents(sols);
   max_line_l2(Ex_i,Ex_j,Ex_k,result1,result2);
   result1(0) = 2; solution.append(result1);
   result2(0) = 2; solution.append(result2);
   delete[] locs;
   return z;
}


double lines::L_l2_max_M2logM(facilities &facs) {
/********************************************************************
* Purpose   : calculates the optimal-max-line with l2-metric 
* Returns   : function-value
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  int N = facs.size(), S = (N*(N-1)), i = 1;
  location Ex_i, Ex_j, Ex_k, Ex_l, result1, result2,
    direction, u, norm_direction;
  double dist = 0, chk = 0, z = 0;
  /****  Initializing locations  ****/
  location *locs = new location[N+1];
  forall(Ex_i,facs) locs[i++] = Ex_i;
  /****  Building the weighted composite points ****/
  location *composites = new location[S+1];
  location *edges = new location[2*S+1];
  int edgeidx = 1, compidx = 1, j = 1;
  for (i = 1; i < N; i++) {
    Ex_j = locs[i];
    for (j = i+1; j <= N; j++) {
      Ex_k = locs[j];
      composites[compidx][0] = (Ex_j(0)*Ex_k(0))/(Ex_j(0)+Ex_k(0))*(Ex_k[0]-Ex_j[0]);
      composites[compidx][1] = (Ex_j(0)*Ex_k(0))/(Ex_j(0)+Ex_k(0))*(Ex_k[1]-Ex_j[1]);
      composites[compidx](0) = compidx;
      edges[edgeidx++] = Ex_j;
      edges[edgeidx++] = Ex_k;
      composites[++compidx][0] = -composites[compidx-1][0];
      composites[compidx][1] = -composites[compidx-1][1];
      composites[compidx](0) = compidx;
      if (fabs(composites[compidx][0]) == 0) composites[compidx][0] = 0;
      if (fabs(composites[compidx][1]) == 0) composites[compidx][1] = 0;
      edges[edgeidx++] = Ex_k;
      edges[edgeidx] = Ex_j;
      compidx++; edgeidx++;
    }
  }
  /****  Calculating the convex hull  ****/
  list<point> hullpnts, comppnts; point tmppnt;
  for (i = 1; i < S+1; i++) {
    tmppnt = composites[i].loc2point();
    comppnts.append(tmppnt);
  }
  hullpnts = CONVEX_HULL(comppnts);
  int M = hullpnts.size();
  location *hulllocs = new location[M+1];
  i = 1;
  forall(tmppnt,hullpnts) {
    hulllocs[i] = point2loc(tmppnt); i++;
  }
  // ****  Scanning for the maximum (edge,point)-pair  ****
  z = L_INF; int M_2 = (int)M/2;
  Ex_k[0] = 0; Ex_k[1] = 0;
  for (i = 1; i <= M_2; i++) {
    Ex_i = hulllocs[i]; Ex_j = hulllocs[i+1];
    direction = Ex_i-Ex_j; u = Ex_k-Ex_j;
    norm_direction = direction * (1/direction.norm(2));
    chk = fabs((u*u) - (u*norm_direction)*(u*norm_direction));
    if (chk < 1E-10) chk = 0;
    dist = fabs(sqrt(chk));
    if (dist < z) {
      z = dist; result1 = Ex_i; result2 = Ex_j;
    }
  }
  /****  building the line-defining three points  ****/
  location r1, r2;
  /****  choosing the line to result1  ****/
  for (i = 1; i < S+1; i++) {
    if (composites[i].pos() == result1.pos()) r1 = composites[i];
    if (composites[i].pos() == result2.pos()) r2 = composites[i];
  }
  int s = (int)r1(0);
  Ex_j = edges[2*s-1]; Ex_k = edges[2*s];
  /****  choosing the line to result2  ****/
  s = (int)r2(0);
  r1 = edges[2*s-1]; r2 = edges[2*s];
  if (r1 != Ex_j && r1 != Ex_k) Ex_i = r1;
  else Ex_i = r2;
  Ex_i(0) = 1; solution.append(Ex_i);
  /****  Calculating the line-defining points  ****/
  direction = result2-result1;
  result1 = (Ex_j-Ex_k)*(Ex_j(0)/(Ex_j(0)+Ex_k(0)))+Ex_k;
  result2 = result1+direction;
  /****  Calculate the objective function value  ****/
  z = objective_function_l2(locs,1,N+1,result1,result2);
  /****  and building the solution  ****/
  Ex_i(0) = 1; solution.append(Ex_i);
  Ex_j(0) = 1; solution.append(Ex_j);
  Ex_k(0) = 1; solution.append(Ex_k);
  result1(0) = 2; solution.append(result1);
  result2(0) = 2; solution.append(result2);
  delete[] edges; delete[] locs; delete[] composites; delete[] hulllocs;
  return z;
}

double lines::L_lp_max_M4(facilities &facs, int p) {
/********************************************************************
* Purpose   : calculates the optimal-max-line with lp-metric 
* Returns   : function-value
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  int N = facs.size(), i = 0;
  double max_1_x = 0, max_1_y = 0, max_2_x = 0, max_2_y = 0,
    dist = 0, tmp_dist = 0, z = L_INF;
  location Ex_i, Ex_j, Ex_k, result1, result2,
    direction, u, norm_direction, linepnt1, linepnt2;
  /****  Initializing locations  ****/
  location *locs = new location[N];
  forall(Ex_i,facs)
    locs[i++] = Ex_i;
  result1(0) = 1; result2(0) = 1;
  /****  The algorithm  ****/
  for (i = 0; i < N; i++) {
    Ex_i = locs[i];
    for (int j = i+1; j < N; j++) {
      Ex_j = locs[j];
      for (int k = j+1; k < N; k++) {
	dist = L_INF;
	Ex_k = locs[k];
	
	this->max_line_lp(Ex_i,Ex_j,Ex_k,result1,result2,p);
	tmp_dist = this->objective_function_lp(locs,0,N,result1,result2,p);
	if (tmp_dist < dist) {
	  dist = tmp_dist;
	  max_1_x = result1[0]; max_1_y = result1[1];
	  max_2_x = result2[0]; max_2_y = result2[1];
	}
	
	this->max_line_lp(Ex_i,Ex_k,Ex_j,result1,result2,p);
	tmp_dist = this->objective_function_lp(locs,0,N,result1,result2,p);
	if (tmp_dist < dist) {
	  dist = tmp_dist;
	  max_1_x = result1[0]; max_1_y = result1[1];
	  max_2_x = result2[0]; max_2_y = result2[1];
	}
	
	this->max_line_lp(Ex_j,Ex_k,Ex_i,result1,result2,p);
	tmp_dist = this->objective_function_lp(locs,0,N,result1,result2,p);
	if (tmp_dist < dist) {
	  dist = tmp_dist;
	  max_1_x = result1[0]; max_1_y = result1[1];
	  max_2_x = result2[0]; max_2_y = result2[1];
	}
	
	if (dist < z) {
	  z = dist;
	  solution.clear();
	  linepnt1 = Ex_i; linepnt1(0) = 1; solution.append(linepnt1);
	  linepnt1 = Ex_j; linepnt1(0) = 1; solution.append(linepnt1);
	  linepnt1 = Ex_k; linepnt1(0) = 1; solution.append(linepnt1);
	  linepnt1[0] = max_1_x; linepnt1[1] = max_1_y; linepnt1(0) = 2;
	  solution.append(linepnt1);
	  linepnt2[0] = max_2_x; linepnt2[1] = max_2_y; linepnt2(0) = 2;
	  solution.append(linepnt2);
	}
      }
    }
  }
  delete[] locs;
  return z;
}

double lines::L_block_max_M4(list<polygauge> &LG,list<int> &Ln, facilities& facs) {
/********************************************************************
* Purpose   : calculates the optimal-max-line with block-metric 
* Returns   : function-value
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  list<point> gaugepnts;
  if (Ln.head() >= 0) gaugepnts = LG.head().vertices();
  Blocknorm B(gaugepnts);
  int N = facs.size(), i = 0;
  double max_1_x = 0, max_1_y = 0, max_2_x = 0, max_2_y = 0;
  location Ex_i, Ex_j, Ex_k, result1, result2, linepnt1, linepnt2,
    direction, u, norm_direction;
  double dist = 0, tmp_dist = 0, z = L_INF;
  /****  Initializing locations  ****/
  location *locs = new location[N];
  forall(Ex_i,facs)
    locs[i++] = Ex_i;
  /****  Algorithm  ****/
  for (i = 0; i < N; i++) {
    Ex_i = locs[i];
    for (int j = i+1; j < N; j++) {
      Ex_j = locs[j];
      for (int k = j+1; k < N; k++) {
	Ex_k = locs[k];
	dist = L_INF;
	
	this->max_line_block(Ex_i,Ex_j,Ex_k,result1,result2,B);
	tmp_dist = this->objective_function_block(locs,0,N,result1,result2,B);
	if (tmp_dist < dist) {
	  dist = tmp_dist;
	  max_1_x = result1[0]; max_1_y = result1[1];
	  max_2_x = result2[0]; max_2_y = result2[1];
	}
	
	this->max_line_block(Ex_i,Ex_k,Ex_j,result1,result2,B);
	tmp_dist = this->objective_function_block(locs,0,N,result1,result2,B);
	if (tmp_dist < dist) {
	  dist = tmp_dist;
	  max_1_x = result1[0]; max_1_y = result1[1];
	  max_2_x = result2[0]; max_2_y = result2[1];
	}
	
	this->max_line_block(Ex_j,Ex_k,Ex_i,result1,result2,B);
	tmp_dist = this->objective_function_block(locs,0,N,result1,result2,B);
	if (tmp_dist < dist) {
	  dist = tmp_dist;
	  max_1_x = result1[0]; max_1_y = result1[1];
	  max_2_x = result2[0]; max_2_y = result2[1];
	}
	
	if (dist < z) {
	  z = dist;
	  solution.clear();
	  linepnt1 = Ex_i; linepnt1(0) = 1; solution.append(linepnt1);
	  linepnt1 = Ex_j; linepnt1(0) = 1; solution.append(linepnt1);
	  linepnt1 = Ex_k; linepnt1(0) = 1; solution.append(linepnt1);
	  linepnt1[0] = max_1_x; linepnt1[1] = max_1_y; linepnt1(0) = 2;
	  solution.append(linepnt1);
	  linepnt2[0] = max_2_x; linepnt2[1] = max_2_y; linepnt2(0) = 2;
	  solution.append(linepnt2);
	}
      }
    }
  }
  delete[] locs;
  return z;
}


/**************************************/
/******  Some useful functions;  ******/
/******  especially for median   ******/
/**************************************/

double lines::lp_distance_2_line(location &l1, location &l2, location &L, int p) {
/********************************************************************
* Purpose   : calculates the lp_distance of L to the line through l1, l2
* Returns   : see purpose
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  double s = 0, dist = 0, dp = p, expo = 1/(dp-1), x = 0, y = 0;
  if (L == l1 || L == l2) return dist;
  if (l1[0] == l2[0]) s = L_INF; //vertical
  else {
    if (l1[1] == l2[1]) s = 0;
    else {
      if (l1[0] < l2[0]) s = (l2[1]-l1[1])/(l2[0]-l1[0]);
      else
	s = (l1[1]-l2[1])/(l1[0]-l2[0]);
    }
  }
  double b = ((-s)*l1[0])+l1[1]; // line: y = sx+b
  if (L[1] == s*L[0]+b) return dist;
  if (s < 0)
    x=(L[0]+(pow((-s),expo)*b)-(pow((-s),expo)*L[1]))/(1-(pow((-s),expo)*s));
  else x=(L[0]-(pow(s,expo)*b)+(pow(s,expo)*L[1]))/(1+(pow(s,expo)*s));
  y = s*x+b;
  location linepnt(2.0,1.0,x,y,1.0);
  dist = (linepnt-L).norm(2);
  return dist;
}

/**************************************/
/******  Some useful functions;  ******/
/******  especially for center   ******/
/**************************************/

double lines::objective_function_l2(location *locs, int start, int N, location &result1, location &result2) {
/********************************************************************
* Purpose   : calculates the objectve-function-value 
* Returns   : see purpose
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  location Ex_l, norm_direction, direction, u;
  double chk = 0, dist = 0, tmp_dist = 0;
  for (int l = start; l < N; l++) {
    Ex_l = locs[l];
    direction = result1-result2; u = Ex_l-result2;
    norm_direction = direction / direction.norm(2);
    chk = fabs((u*u) - (u*norm_direction)*(u*norm_direction));
    if (chk < 1E-10) chk = 0;
    dist = Ex_l(0)*fabs(sqrt(chk));
    if (dist > tmp_dist) tmp_dist = dist;
  }
  return tmp_dist;
}

void lines::max_line_l2(location &Ex_i, location &Ex_j, location &Ex_k, location &r1, location &r2) {
/********************************************************************
* Purpose   : calculates the optimal-max-line with l2-metric
* Returns   : %
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  r1 = (Ex_k-Ex_i)*(Ex_k(0)/(Ex_i(0)+Ex_k(0)))+Ex_i;
  r2 = (Ex_k-Ex_j)*(Ex_k(0)/(Ex_j(0)+Ex_k(0)))+Ex_j;
}

double lines::objective_function_linf(location *locs, int start, int N, location &Ex_i, location &Ex_j) {
	// Transforming locations
	int i;
	for (i = start; i < N; i++)
		locs[i].transform();
	Ex_i.transform(); Ex_j.transform();
	// Algorithm
	double dist = objective_function_l1(locs,start,N,Ex_i,Ex_j);
	// Retransforming locations
        for ( i = start; i < N; i++)
		locs[i].retransform();
	Ex_i.retransform(); Ex_j.retransform();
   return dist;
}

double lines::objective_function_l1(location *locs, int start, int N, location &Ex_i, location &Ex_j) {
/********************************************************************
* Purpose   : calculates the objectve-function-value 
* Returns   : see purpose
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  double dist = 0, chk = 0, chk1 = L_INF, chk2 = L_INF, s = 0;
  location Ex_k, direct_g, isectpnt;
  for (int k = start; k < N; k++) {
    Ex_k = locs[k];
    chk1 = L_INF; chk2 = L_INF;
    direct_g = Ex_j-Ex_i;
    if (direct_g[0] != 0) { // not vertical
      s = (Ex_k[0]-Ex_i[0])/direct_g[0];
      isectpnt[0] = Ex_k[0];
      isectpnt[1] = Ex_i[1] + s*direct_g[1];
      chk1 = Ex_k(0)*(Ex_k-isectpnt).norm(2);
    }
    if (direct_g[1] != 0) { // not horizontal
      s = (Ex_k[1]-Ex_i[1])/direct_g[1];
      isectpnt[1] = Ex_k[1];
      isectpnt[0] = Ex_i[0] + s*direct_g[0];
      chk2 = Ex_k(0)*(Ex_k-isectpnt).norm(2);
    }
    if (chk1 < chk2) chk = chk1;
    else chk = chk2;
    if (chk < 1E-10) chk = 0;
    if (chk > dist) dist = chk;
  }
  return dist;
}

void lines::max_line_l1(location &l1, location &l2, location &l3, location &r1, location &r2) {
/********************************************************************
* Purpose   : calculates the optimal-max-line with l1-metric
* Returns   : %
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  double slope = 0, linewht = 0, movedist = 0;
  r1 = l1; r2 = l2;
  if (l1[0] == l2[0]) slope = L_INF;
  else {
    if (l1[1] == l2[1]) slope = 0;
      else {
	if (l1[0] < l2[0]) slope = (l2[1]-l1[1])/(l2[0]-l1[0]);
	else
	  slope = (l1[1]-l2[1])/(l1[0]-l2[0]);
      }		
  }
  if (l1(0) > l2(0)) linewht = l1(0);
  else linewht = l2(0);
  if (-1 <= slope && slope <= 1) {
    // moving parallel to y-axis
    if (fabs(l3[1]-l1[1]) > fabs(l3[1]-l2[1]))
      movedist = (l3[1]-l1[1])*(l3(0)/(linewht+l3(0)));
    else
      movedist = (l3[1]-l2[1])*(l3(0)/(linewht+l3(0)));
    r1[1] += movedist; r2[1] += movedist;
  } else {
    // moving parallel to x-axis
    if (fabs(l3[0]-l1[0]) > fabs(l3[0]-l2[0]))
      movedist = (l3[0]-l1[0])*(l3(0)/(linewht+l3(0)));
    else
      movedist = (l3[0]-l2[0])*(l3(0)/(linewht+l3(0)));
		r1[0] += movedist; r2[0] += movedist;
  }
}

double lines::objective_function_lp(location *locs, int start, int N, location &result1, location &result2, int p) {
/********************************************************************
* Purpose   : calculates the objectve-function-value 
* Returns   : see purpose
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  location Ex_l, norm_direction, direction, u;
  double chk = 0, dist = 0, tmp_dist = 0;
  /**** result1 and result2 define the line ****/
  for (int l = start; l < N; l++) {
    Ex_l = locs[l];
    chk = this->lp_distance_2_line(result1,result2,Ex_l,p);
    if (chk < 1E-10) chk = 0;
    tmp_dist = Ex_l(0)*chk;
    if (dist < tmp_dist) dist = tmp_dist;
  }
  return dist;
}

void lines::max_line_lp(location &Ex_i, location &Ex_j, location &Ex_k, location &r1, location &r2, int p) {
/********************************************************************
* Purpose   : calculates the optimal-max-line with lp-metric
* Returns   : %
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  double s = 0, dp = p, expo = 1/(dp-1), x = 0, y = 0;
  if (Ex_i[0] == Ex_j[0]) s = L_INF; //vertical
  else {
    if (Ex_i[1] == Ex_j[1]) s = 0;
    else {
      if (Ex_i[0] < Ex_j[0]) s = (Ex_j[1]-Ex_i[1])/(Ex_j[0]-Ex_i[0]);
      else
	s = (Ex_i[1]-Ex_j[1])/(Ex_i[0]-Ex_j[0]);
    }
  }
  double b = ((-s)*Ex_i[0])+Ex_i[1]; // line: y = sx+b
  if (s < 0)
    x=(Ex_k[0]+(pow((-s),expo)*b)-(pow((-s),expo)*Ex_k[1]))
      /(1-(pow((-s),expo)*s));
  else x=(Ex_k[0]-(pow(s,expo)*b)+(pow(s,expo)*Ex_k[1]))/(1+(pow(s,expo)*s));
  y = s*x+b;
  location linepnt(2.0,1.0,x,y,1.0);
  if (Ex_i(0) > Ex_j(0)) linepnt(0) = Ex_i(0);
  else linepnt(0) = Ex_j(0);
  r1 = (Ex_k-linepnt)*(Ex_k(0)/(linepnt(0)+Ex_k(0)))+linepnt;
  r2[0] = r1[0]+10; r2[1] = s*10+r1[1];
}

double lines::objective_function_block(location *locs, int start, int N, location &result1, location &result2, Blocknorm &B) {
/********************************************************************
* Purpose   : calculates the objectve-function-value 
* Returns   : see purpose
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  int B_border = B.size(), alpha = 0;
  double dist = 0, chk = 0, s = 0, test_chk = 0, z = L_INF;
  location Ex_k, u, isectpnt, direct_g = result1-result2, direct_h,
    norm_direction;
  for (alpha = 0; alpha < B_border; alpha++) {
    direct_h = B[alpha];
    int parallel = 0;
    dist = 0;
    if ((direct_g[0] == 0 && direct_h[0] == 0) ||
	(direct_g[1] == 0 && direct_h[1] == 0)) {
      parallel = 1; dist =  0;
      /* identical lines */
    } else {
      if (direct_g[0] != 0 && direct_g[1] != 0 &&
	  direct_h[0] != 0 && direct_h[1] != 0) {
	/* parallel lines */
	if ((direct_g[1]/direct_g[0]) == (direct_h[1]/direct_h[0])) {
	  chk = L_INF; parallel = 1; dist = L_INF;
	} 
      }
      if (!parallel) {
	norm_direction = direct_g * (1/direct_g.norm(2));
	for (int k = start; k < N; k++) {
	  Ex_k = locs[k];
	  u = Ex_k-result2;
	  test_chk = fabs((u*u) - (u*norm_direction)*(u*norm_direction));
	  if (test_chk < 1E-10) chk = 0;
	  else {
	    if (direct_g[1] != 0) {
	      s = (result2[0]+(Ex_k[1]-result2[1])/
		   direct_g[1]*direct_g[0]-Ex_k[0]);
	      s = s/(direct_h[0]-(direct_h[1]*direct_g[0])/direct_g[1]);
	      isectpnt = Ex_k + direct_h*s;
	    } else {
	      s = (Ex_k[0]+(result2[1]-Ex_k[1])/
		   direct_h[1]*direct_h[0]-result2[0]);
	      s = s/(direct_g[0]-(direct_g[1]*direct_h[0])/direct_h[1]);
	      isectpnt = result2 + direct_g*s;
	    }
	    chk = Ex_k(0)*(isectpnt-Ex_k).norm(2);
	  }
	  if (chk < 1E-10) chk = 0;
	  if (chk > dist) dist = chk;
	}
      }
      if (!parallel && dist < z) {
	z = dist;
      }
    }
  }
  return z;
}


void lines::max_line_block(location &Ex_i, location &Ex_j, location &Ex_k, location &r1, location &r2, Blocknorm &B) {
/********************************************************************
* Purpose   : calculates the optimal max-line for the three locations 
*             using the blocknorm and 
*             puts two line-defining points in r1,r2 
* Returns   : %
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  int B_border = B.size(), alpha = 0;
  double dist = 0, chk = 0, s = 0, test_chk = 0, z = L_INF;
  location linepnt(2,1), isectpnt, nullLoc(2,1),
    u, direct_g = Ex_i-Ex_j, direct_h, norm_direction, which_direct;
  for (alpha = 0; alpha < B_border; alpha++) {
    direct_h = B[alpha];
    int parallel = 0;
    if ((direct_g[0] == 0 && direct_h[0] == 0) ||
	(direct_g[1] == 0 && direct_h[1] == 0)) {
      parallel = 1; dist =  L_INF; isectpnt= nullLoc;
    } else {
      if (direct_g[0] != 0 && direct_g[1] != 0 &&
	  direct_h[0] != 0 && direct_h[1] != 0) {
	if ((direct_g[1]/direct_g[0]) == (direct_h[1]/direct_h[0])) {
	  parallel = 1; dist = L_INF; isectpnt = nullLoc;
	} 
      }
      if (!parallel) {
	norm_direction = direct_g * (1/direct_g.norm(2));
	u = Ex_k-Ex_j;
	test_chk = fabs((u*u) - (u*norm_direction)*(u*norm_direction));
	if (test_chk < 1E-10) chk = 0;
	else {
	  if (direct_g[1] != 0) {
	    s = (Ex_j[0]+(Ex_k[1]-Ex_j[1])/
		 direct_g[1]*direct_g[0]-Ex_k[0]);
	    s = s/(direct_h[0]-(direct_h[1]*direct_g[0])/direct_g[1]);
	    isectpnt = Ex_k + direct_h*s;
	  } else {
	    s = (Ex_k[0]+(Ex_j[1]-Ex_k[1])/
		 direct_h[1]*direct_h[0]-Ex_j[0]);
	    s = s/(direct_g[0]-(direct_g[1]*direct_h[0])/direct_h[1]);
	    isectpnt = Ex_j + direct_g*s;
	  }
	  chk = (isectpnt-Ex_k).norm(2);
	}
	if (chk < 1E-10) chk = 0;
	dist = Ex_k(0)*chk;
      }
    }
    if (!parallel && dist < z) {
      z = dist;
      linepnt = isectpnt;
    }
  }
  location movdirect = Ex_k-linepnt, orth_movdirect;
  movdirect = movdirect*(1/movdirect.norm(2));
  Loc_Vector tmp = movdirect.pos(); tmp = tmp.rotate(LEDA_PI_2); // M_PI_2
  orth_movdirect[0] = tmp[0]; orth_movdirect[1] = tmp[1];
  location scnd_pnt = Ex_k+(orth_movdirect*10);
  Line L(Ex_k,scnd_pnt);
  dist = L.distance(Ex_i);
  r1 = Ex_i+(movdirect*(dist*(Ex_k(0)/(Ex_i(0)+Ex_k(0)))));
  dist = L.distance(Ex_j);
  r2 = Ex_j+(movdirect*(dist*(Ex_k(0)/(Ex_j(0)+Ex_k(0)))));
}

int lines::geometric_duality(location *locs, location &pivot, int M) {
/********************************************************************
* Purpose   : Sorts tosort clockwise w.r.t. to pivot using geometric duality
* Returns   : number of locations above the line origin-pivot
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  /****  Sorting clockwise w.r.t. to pivot using geometric duality  ****/
  facilities one, two, three, four, isect_pnts;
  //Line dual_lines[M]; int m = 0; fuer Windows geaendert
  Line* dual_lines=new Line[M]; int m = 0;
  location tmp1, tmp2, Ex_i, old_pivot = pivot;
  /****  Transformation to get a better resolution  ****/
  pivot = pivot/100;
  tmp1 = pivot/-(pivot[0]*pivot[0]+pivot[1]*pivot[1]);
  tmp2 = tmp1; tmp2[0] += pivot[1]; tmp2[1] -= pivot[0];
  location dual_pivot = tmp1;
  //location old_coord[M]; fuer Windows geaendert:
  location* old_coord= new location[M];
  Line ref_line(tmp1,tmp2);
  for (m = 0; m < M-1; m++) {
    Ex_i = locs[m]; old_coord[m] = Ex_i; Ex_i = Ex_i/100;
    if ((Ex_i-pivot).norm(2) < 1E-12) {
      locs[m--] = locs[M-1]; locs[M-1] = old_pivot;
    } else {
      tmp1 = Ex_i/-(Ex_i[0]*Ex_i[0]+Ex_i[1]*Ex_i[1]);
      tmp2 = tmp1; tmp2[0] += Ex_i[1]; tmp2[1] -= Ex_i[0];
      Line current_line(tmp1,tmp2);
      dual_lines[m] = current_line;
      location isectpnt = ref_line.intersection(current_line);
      if (isectpnt[0] == 0 && isectpnt[1] == 0) {
	if (tmp1[0] > dual_pivot[0]) {
	  isectpnt[0] = -L_INF+isectpnt(0);
	  isectpnt[1] = -L_INF+isectpnt(0);
	} else {
	  isectpnt[0] = -L_INF-isectpnt(0);
	  isectpnt[1] = -L_INF-isectpnt(0);
	}
      }
      isect_pnts.append(isectpnt);
    }
  }
  /****  Sorting the intersectionpoints and their corresponding lines  ****/
  location *i_pnts = new location[isect_pnts.size()];
  for (m = 0; m < isect_pnts.size(); m++)
    i_pnts[m] = isect_pnts.get_element(m);
  this->y_quicksort(i_pnts,0,isect_pnts.size()-1,old_coord);
  /****  Building the four lists  ****/
  location x_axis(2.0,1.0,1.0,0.0,0.0);
  pivot = old_pivot;
  double angle = 0, pivot_angle = pivot.r_angle(x_axis);
  for (m = 0; m < isect_pnts.size(); m++) { 
    Ex_i = old_coord[m];
    angle = Ex_i.r_angle(x_axis);
    if (angle >= pivot_angle && Ex_i.norm(2)*cos(angle-pivot_angle) < pivot.norm(2)) one.append(Ex_i);
    if (angle > pivot_angle &&  Ex_i.norm(2)*cos(angle-pivot_angle) >= pivot.norm(2)) two.append(Ex_i);
    if (angle <= pivot_angle && Ex_i.norm(2)*cos(angle-pivot_angle) > pivot.norm(2)) three.append(Ex_i);
    if (angle < pivot_angle && Ex_i.norm(2)*cos(angle-pivot_angle) <= pivot.norm(2)) four.append(Ex_i);
  }
  /****  Connecting the four lists in order to get the final list  ****/
  int result = one.size()+two.size();
  facilities final = one; final.conc(two);
  final.conc(three); final.conc(four);
  for (m = 0; m < final.size(); m++) locs[m] = final.get_element(m);
  delete[] i_pnts;
  return result;
}

void lines::y_quicksort(location *tosort, int l, int r, location *old) {
/********************************************************************
* Purpose   : sorts tosort with respect to their y-coordinate
* Returns   : %
* Influences: %
* Date      : Dec 1997
* Author    : Ingo Bartling
*********************************************************************/
  int i = 0, j = 0;
  location tmp, ref, tmp_old;
  if (l < r) {
    ref = tosort[r]; i = l-1; j = r;
    for (;;) {
      while (tosort[++i][1] < ref[1]);
      while (j && tosort[--j][1] > ref[1]);
      if (i < j) {
	tmp = tosort[i]; tosort[i] = tosort[j]; tosort[j] = tmp;
	tmp_old = old[i]; old[i] = old[j]; old[j] = tmp_old;
      }
      else break;
    }
    tmp = tosort[i]; tosort[i] = tosort[r]; tosort[r] = tmp;
    tmp_old = old[i]; old[i] = old[r]; old[r] = tmp_old;
    y_quicksort(tosort,l,i-1,old);
    y_quicksort(tosort,i+1,r,old);
  }
}
