#include <stdarg.h>
#include <math.h>
#include <LOLA/loc_vector.h>

Loc_Vector::Loc_Vector (int d) {
	v_size = d;
	data = new double[v_size];
	for (int i = 0; i < d; i++)
		data[i] = 0.0;
}
Loc_Vector::Loc_Vector (double d,...) {
	va_list parameter;
	va_start (parameter, d);
	data = new double[(int)d];
	for (int i = 0; i < d; i++) 
		data[i] = va_arg (parameter, double);
	va_end(parameter);
	v_size = (int)d;
}

Loc_Vector::Loc_Vector(const Loc_Vector &v){
	data = new double[v.v_size];
	for (int i = 0; i < v.v_size; i++) data[i] = v.data[i];
	v_size = v.v_size;
   
}
Loc_Vector Loc_Vector::operator + (const Loc_Vector &v) {
	Loc_Vector result(v.v_size);
	if (v_size != v.v_size) return result;
	for (int i = 0; i < v.v_size; i++)
		result.data[i] = data[i] + v.data[i];
	return result;
}

Loc_Vector Loc_Vector::operator - (const Loc_Vector &v) {
	Loc_Vector result(v.v_size);
	if (v_size != v.v_size) return result;
	for (int i = 0; i < v.v_size; i++)
		result.data[i] = data[i] - v.data[i];
	return result;
}

Loc_Vector& Loc_Vector::operator += (const Loc_Vector &v) {
	if (v_size != v.v_size) return *this;
	for (int i = 0; i < v.v_size; i++) 
		data[i] += v[i]; 
	return *this;
}

Loc_Vector& Loc_Vector::operator -= (const Loc_Vector &v) {
	if (v_size != v.v_size) return *this;
	for (int i = 0; i < v.v_size; i++) 
		data[i] -= v[i]; 
	return *this;
}

Loc_Vector& Loc_Vector::operator *= (double scalar) {
	for (int i = 0; i  < v_size; i++)
		data[i] *= scalar;
	return *this;
}

Loc_Vector& Loc_Vector::operator += (double scalar) {
	for (int i = 0; i  < v_size; i++)
		data[i] += scalar;
	return *this;
}

Loc_Vector& Loc_Vector::operator -= (double scalar) {
	for (int i = 0; i  < v_size; i++)
		data[i] -= scalar;
	return *this;
}

Loc_Vector& Loc_Vector::operator /= (double scalar) {
	for (int i = 0; i  < v_size; i++)
		data[i] /= scalar;
	return *this;
}

Loc_Vector Loc_Vector::operator * (double scalar) {
	Loc_Vector result(v_size);
	for (int i = 0; i  < v_size; i++)
		result.data[i] = data[i] * scalar;
	return result;
}

Loc_Vector Loc_Vector::operator + (double scalar) {
	Loc_Vector result(v_size);
	for (int i = 0; i  < v_size; i++)
		result.data[i] = data[i] + scalar;
	return result;
}

Loc_Vector Loc_Vector::operator - (double scalar) {
	Loc_Vector result(v_size);
	for (int i = 0; i  < v_size; i++)
		result.data[i] = data[i] - scalar;
	return result;
}

Loc_Vector Loc_Vector::operator / (double scalar) {
	Loc_Vector result(v_size);
	for (int i = 0; i  < v_size; i++)
		result.data[i] = data[i] / scalar;
	return result;
}

double Loc_Vector::operator * (const Loc_Vector &v) {
	double result = 0;
	if (v_size != v.v_size) return result;
	for (int i = 0; i < v.v_size; i++)
		result += data[i] * v.data[i];
	return result;
}

bool Loc_Vector::operator < (const Loc_Vector &v) {
	if (v_size != v.v_size) return false;
	for (int i = 0; i < v.v_size; i++)
		if (data[i] >= v.data[i])
			return false;
	return true;
}

bool Loc_Vector::operator <= (const Loc_Vector &v) {
	if (v_size != v.v_size) return false;
	for (int i = 0; i < v.v_size; i++)
		if (data[i] > v.data[i])
			return false;
	return true;
}

bool Loc_Vector::operator > (const Loc_Vector &v) {
	if (v_size != v.v_size) return false;
	for (int i = 0; i < v.v_size; i++)
		if (data[i] <= v.data[i]) 
			return false;
	return true;
}

bool Loc_Vector::operator >= (const Loc_Vector &v) {
	if (v_size != v.v_size) return false;
	for (int i = 0; i < v.v_size; i++)
		if (data[i] < v.data[i])
			return false;
	return true;
}

bool Loc_Vector::operator != (const Loc_Vector &v) {
	if (v_size != v.v_size) return true;
	for (int i = 0; i < v.v_size; i++)
		if (data[i] != v.data[i])
			return true;
	return false;
}

bool Loc_Vector::operator == (const Loc_Vector &v) {
	if (v_size != v.v_size) return false;
	for (int i = 0; i < v.v_size; i++)
		if (data[i] != v.data[i])
			return false;
	return true;
}

Loc_Vector& Loc_Vector::operator = (const Loc_Vector &v) {
	if (data == v.data) return *this;
	delete data;
	v_size = v.v_size;
	data = new double[v_size];
	for (int i = 0; i < v.v_size; i++) 
		data[i] = v.data[i];
	return *this;
}

double Loc_Vector::norm(int p) const {
	double result = 0;
	if (p > 0) {
		for (int i = 0; i < v_size; i++)
			result += pow(fabs(data[i]),p);
		result = pow(result,(1.0/p));
	}
	else if (p == L_inf || p == L_max)
		for(int i = 0; i < v_size; i++)
			if (result < fabs(data[i])) result=fabs(data[i]); 
	return result;
}

void Loc_Vector::abs() {
	for (int i = 0; i < v_size; i++)
		data[i] = fabs(data[i]);
}


bool Loc_Vector::null_chk() {
	if (v_size != 0)
		for (int i = 0; i < v_size; i++)
			if ((*this)[i] != 0) return false;
			return true;
}

Loc_Vector Loc_Vector::rotate(double phi) {
	if (size() != 2) lola_error("Loc_Vector::rotate():To rotate the dimension must be 2.",1101,1);
	Loc_Vector result = *this, result2 = *this;
	result[0] = (result2[0]*cos(phi)) + (result2[1]*sin(phi));
	result[1] = (result2[1]*cos(phi)) - (result2[0]*sin(phi));
	return result;
}
   
ostream& operator << (ostream &stream, const Loc_Vector &v) {
	stream << "( ";
	for (int i = 0; i < v.v_size; i++)
		stream << v.data[i] << " ";
	stream << ")";
	return stream;
}










