#include <stdio.h>
#define mininf -900000000000000.0

#ifndef HAVECPLEX

#include "lpsol.h"
#include <lpkit.h>


double N_l1_sum_solve(int N, int M, int anzrow, double *objfunc, double *rhs1, double *rhs2, double *soltnx, double *soltny)
/*********************************************************************
* Purpose    : Computes LP for lp_solve and N|P|.|l1|sum and let 
*              the LP be solved
* Returns    : objective (double)
* Influences : %
* Date       : 01-14-96
* Author     : Holger Hennes
*********************************************************************/
{
	lprec *lp;
	int i,j,c1,c2;
	double objval;

	lp=make_lp(anzrow,2*anzrow+N);
	set_obj_fn(lp, objfunc);
	
	c1 = 2*N*M+N+1;
	c2 = N*M;

	for (j=1;j<=N;j++) {
		for (i=1;i<=M;i++) {
			int k=i+M*(j-1);
			int l=2*M*(j-1)+N+2*i-1;
			set_mat(lp,k,j,1);
			set_mat(lp,k,l,1);
			set_mat(lp,k,l+1,-1);
		}
		for (i=j+1;i<=N;i++) {
			set_mat(lp,c2+i+j-2,j,1);
			set_mat(lp,c2+i+j-2,i,-1);
			set_mat(lp,c2+i+j-2,c1++,-1);
			set_mat(lp,c2+i+j-2,c1++,1);
		}
	}
	for(j=1;j<=anzrow;j++) set_constr_type(lp,j, EQ);
	for(j=1;j<=N;j++) set_lowbo(lp, j, mininf);
	set_rh_vec(lp,rhs1);
	solve(lp);
    objval = (double)lp->best_solution[0];
	for (i=0;i<N;i++) {
		soltnx[i]=(double)lp->best_solution[lp->rows+i+1];
	}
	set_rh_vec(lp,rhs2);
	solve(lp);
    objval = objval + (double)lp->best_solution[0];
	for (i=0;i<N;i++) {
		soltny[i]=(double)lp->best_solution[lp->rows+i+1];
	}
	delete_lp(lp);
	return objval;
}

double N_linf_max_solve(int N, int M, int anzrow, double *rhs1, double *rhs2, double *costs, double *soltnx, double* soltny)
{
	double objval,hobj, *objfunc;
	int i,j,r_count;
	lprec *lp;

	MALLOC(objfunc,N+2);
	r_count=1;
	lp=make_lp(anzrow,N+1);
	objfunc[0]=0;
	for(i=1;i<=N;i++) objfunc[i]=0;
	objfunc[N+1]=1;
	set_obj_fn(lp,objfunc);
	for (j=1;j<=N;j++) {
		for (i=1;i<=M;i++) {
			set_mat(lp,r_count,j,1);
			set_mat(lp,r_count,N+1,costs[r_count]);
			r_count++;
			set_mat(lp,r_count,j,-1);
			set_mat(lp,r_count,N+1,costs[r_count]);
			r_count++;
		}
	}
	for (j=1;j<=N;j++) {
		for (i=j+1;i<=N;i++) {
			set_mat(lp,r_count,j,1);
			set_mat(lp,r_count,i,-1);
			set_mat(lp,r_count,N+1,costs[r_count]);
			r_count++;
			set_mat(lp,r_count,j,-1);
			set_mat(lp,r_count,i,1);
			set_mat(lp,r_count,N+1,costs[r_count]);
			r_count++;
		}
	}
	set_rh_vec(lp,rhs1);
	for(j=1;j<=anzrow;j++) set_constr_type(lp,j, GE);
	for(j=1;j<=N;j++) set_lowbo(lp, j, mininf);
	solve(lp);
    objval = (double)lp->best_solution[0];
	for (i=0;i<N;i++) {
		soltnx[i]=(double)lp->best_solution[lp->rows+i+1];
	}
	set_rh_vec(lp,rhs2);
	solve(lp);
	hobj=(double)lp->best_solution[0];
	if (hobj>objval) objval=hobj;
	for (i=0;i<N;i++) {
		soltny[i]=(double)lp->best_solution[lp->rows+i+1];
	}
	free(objfunc);
	delete_lp(lp);
	return objval;
}

#endif
