#include <LOLA/planealg.h>

void planealg::WriteOpt() {
  this->WriteOpt(cout);
}

void planealg::WriteOpt(ostream& out) {
  location::WriteAsPolygonList(out, solution);
}

double planealg::l1_sum(facilities& EX){
  double result;
  result = EX.l1_sum(0,true); 
  solution = EX.alg_solution();
  return result;
}

double planealg::l1_sum(facilities& EX, restrictions& R){
  double result;
  result = R.l1_sum(EX,true); 
  solution = R.alg_solution();
  return result;
}

double planealg::l2_sum(facilities& EX, double epsilon){
  double result;
  result = EX.l2_sum(0,epsilon,true); 
  solution = EX.alg_solution();
  return result;
}

double planealg::l2_sum(facilities& EX, barrier& R){
  double result = 0.0;
  result = R.l2_sum(EX,true);
  solution = R.alg_solution();
  return result;
}

double planealg::l2_sum(facilities& EX, barrier& R,bool erase=true,int choose=1,double percent=0.1,double accel=1.0,double init_ss=0.01,double epsilon=0.001){
  double result = 0.0;
  result = R.l2_sum(EX,erase,choose,percent,accel,init_ss,epsilon);
  solution = R.alg_solution();
  return result;
}

double planealg::l2sqr_sum(facilities& EX){
  double result;
  result = EX.l2sqr_sum(0,true); 
  solution = EX.alg_solution();
  return result;
}

double planealg::l2sqr_sum(facilities& EX, restrictions& R){
  double result;
  result = R.l2sqr_sum(EX,true); 
  solution = R.alg_solution();
  return result;
}

double planealg::linf_sum(facilities& EX){
  double result;
  result = EX.linf_sum(0,true); 
  solution = EX.alg_solution();
  return result;
}

double planealg::linf_sum(facilities& EX, restrictions& R){
  double result;
  result = R.linf_sum(EX,true); 
  solution = R.alg_solution();
  return result;
}

double planealg::lp_sum(facilities& EX, int p, double epsilon, double delta, int iter_max){
  double result;
  result = EX.lp_sum(p,epsilon,delta,iter_max,0,true);
  solution = EX.alg_solution();
  return result;
}

double planealg::lp_sum(facilities& EX, restrictions& R, int p, double epsilon, double delta, int iter_max){
  double result;
  result = R.lp_sum(EX,p,epsilon,delta,iter_max,true);
  solution = R.alg_solution();
  return result;
}

double planealg::l2sqr_qsum(facilities& EX){
  double result;
  result = EX.l2sqr_qsum(true); 
  solution = EX.alg_solution();
  return result;
}

double planealg::l1_2sum(facilities& EX){
  double result;
  result = EX.l1_2sum(true);
  solution = EX.alg_solution();
  return result;
}

double planealg::linf_2sum(facilities& EX){
  double result;
  result = EX.linf_2sum(true);
  solution = EX.alg_solution();
  return result;
}

double planealg::N_l1_sum(facilities& EX, int n, matrix& w){
  double result;
  result = EX.N_l1_sum(n,w,true);
  solution = EX.alg_solution();
  return result;
}


double planealg::N_linf_sum(facilities& EX, int n, matrix& w){
  double result;
  result = EX.N_linf_sum(n,w,true);
  solution = EX.alg_solution();
  return result;
}


double planealg::N_l2sqr_sum(facilities& EX, int n, matrix& w){
  double result;
  result = EX.N_l2sqr_sum(n,w,true);
  solution = EX.alg_solution();
  return result;
}

double* planealg::N_lp_sum_v1(facilities& EX, int n, int p, matrix& w){
  double* result;
  result = EX.N_lp_sum_v1(n,p,w,true);
  solution = EX.alg_solution();
  return result;
}

double* planealg::N_lp_sum_v2(facilities& EX, int n, int p, matrix& w, double epsilon){
  double* result;
  result = EX.N_lp_sum_v2(n,p,w,epsilon,true);
  solution = EX.alg_solution();
  return result;
}


/* Center-Problems */

double planealg::l1_max(facilities& EX){
  double result;
  result = EX.l1_max(true);
  solution = EX.alg_solution();
  return result;
}

double planealg::l1_max(facilities& EX, restrictions& R){
  double result;
  result = R.l1_max(EX,true); 
  solution = R.alg_solution();
  return result;
}
double planealg::l1_v1_max(facilities& EX){
  double result;
  result = EX.l1_v1_max(true);
  solution = EX.alg_solution();
  return result;
}

double planealg::l2_max(facilities& EX){
  double result;
  result = EX.elz_hearn(true);
  solution = EX.alg_solution();
  return result;
}

double planealg::l2_max(facilities& EX, restrictions& R){
  double result;
  result = R.l2_v1_max(EX,true); 
  solution = R.alg_solution();
  return result;
}

double planealg::linf_max(facilities& EX){
  double result;
  result = EX.linf_max(true);
  solution = EX.alg_solution();
  return result;
}

double planealg::linf_max(facilities& EX, restrictions& R){
  double result;
  result = R.linf_max(EX,true); 
  solution = R.alg_solution();
  return result;
}

double planealg::linf_v1_max(facilities& EX){
  double result;
  result = EX.linf_v1_max(true);
  solution = EX.alg_solution();
  return result;
}

double planealg::elzhearn(facilities& EX){
  double result;
  result = EX.elz_hearn(true);
  solution = EX.alg_solution();
  return result;
}

double planealg::N_l1_max(facilities& EX, int N, matrix &W){
  double result;
  result = EX.N_l1_max(N,W,true);
  solution = EX.alg_solution();
  return result;
}


double planealg::N_linf_max(facilities& EX, int N, matrix &W){
  double result;
  result = EX.N_linf_max(N,W,true);
  solution = EX.alg_solution();
  return result;
}


/*double planealg::N_l1_max_mps(facilities& EX, int N, matrix &W){
  double result;
  result = EX.N_l1_max_mps(N,W,true);
  solution = EX.alg_solution();
  return result;
}

double planealg::N_linf_max_mps(facilities& EX, int N, matrix &W){
  double result;
  result = EX.N_linf_max_mps(N,W,true);
  solution = EX.alg_solution();
  return result;
}
*/

double planealg::L_l2_sum_M3(facilities& EX){
	double result;
	lines LineAlg;
	result = LineAlg.L_l2_sum_M3(EX);
	solution = LineAlg.alg_solution();
	return result;
}

double planealg::L_l1_sum_M3(facilities& EX){
	double result;
	lines LineAlg;
	result = LineAlg.L_l1_sum_M3(EX);
	solution = LineAlg.alg_solution();
	return result;
}

double planealg::L_linf_sum_M3(facilities& EX){
	double result;
	lines LineAlg;
	result = LineAlg.L_linf_sum_M3(EX);
	solution = LineAlg.alg_solution();
	return result;
}

double planealg::L_lp_sum_M3(facilities& EX, int norm){
	double result;
	lines LineAlg;
	result = LineAlg.L_lp_sum_M3(EX,norm);
	solution = LineAlg.alg_solution();
	return result;
}

double planealg::L_l2_sum_M2logM(facilities& EX){
	double result;
	lines LineAlg;
	result = LineAlg.L_l2_sum_M2logM(EX);
	solution = LineAlg.alg_solution();
	return result;
}

double planealg::L_l2_sum_M2(facilities& EX){
	double result;
	lines LineAlg;
	result = LineAlg.L_l2_sum_M2(EX);
	solution = LineAlg.alg_solution();
	return result;
}
double planealg::L_RkonvPoly_l2_sum(facilities& EX, restrictions& R){
	double result;
	result = R.L_RkonvPoly_l2_sum(EX,true);
	solution = R.alg_solution();
	return result;
}

double planealg::L_RkonvPoly_lp_sum(facilities& EX, restrictions& R, int p){
	double result;
	result = R.L_RkonvPoly_lp_sum(EX,true,p);
	solution = R.alg_solution();
	return result;
}

double planealg::L_RkonvPoly_l1_sum(facilities& EX, restrictions& R){
	double result;
	result = R.L_RkonvPoly_l1_sum(EX,true);
	solution = R.alg_solution();
	return result;
}

double planealg::L_RkonvPoly_linf_sum(facilities& EX, restrictions& R){
	double result;
	result = R.L_RkonvPoly_linf_sum(EX,true);
	solution = R.alg_solution();
	return result;
}

double planealg::L_l2_max_M4(facilities& EX){
	double result;
	lines LineAlg;
	result = LineAlg.L_l2_max_M4(EX);
	solution = LineAlg.alg_solution();
	return result;
}

double planealg::L_l1_max_M4(facilities& EX){
	double result;
	lines LineAlg;
	result = LineAlg.L_l1_max_M4(EX);
	solution = LineAlg.alg_solution();
	return result;
}

double planealg::L_linf_max_M4(facilities& EX){
	double result;
	lines LineAlg;
	result = LineAlg.L_linf_max_M4(EX);
	solution = LineAlg.alg_solution();
	return result;
}

double planealg::L_l2_max_MlogM(facilities& EX){
	double result;
	lines LineAlg;
	result = LineAlg.L_l2_max_MlogM(EX);
	solution = LineAlg.alg_solution();
	return result;
}

double planealg::L_l2_max_M2logM(facilities& EX){
	double result;
	lines LineAlg;
	result = LineAlg.L_l2_max_M2logM(EX);
	solution = LineAlg.alg_solution();
	return result;
}

double planealg::L_lp_max_M4(facilities& EX, int norm){
	double result;
	lines LineAlg;
	result = LineAlg.L_lp_max_M4(EX,norm);
	solution = LineAlg.alg_solution();
	return result;
}
