// $Id: polygon_barrier.cpp,v 1.4 1999/03/15 12:09:14 hennes Exp $

#include <LOLA/polygon_barrier.h>

polygon_barrier::polygon_barrier() {
	_pt_list.clear();
	_mid_list.clear();
	_ed_list.clear();
}

polygon_barrier::polygon_barrier(const list<point>& Pt) {
	_mid_list.clear();
	_pt_list=Pt;
	if (!Is_Simple_Polygon(_pt_list))
		lola_error("polygon_barrier: Polygon is not simple",1413,1);
	polygon P(_pt_list);
	_ed_list=P.edges();
	list_item trav=_pt_list.first();
	for (int i=0; i<_pt_list.length(); i++) {
		point p1=_pt_list.contents(trav);
		point p2=_pt_list.contents(_pt_list.cyclic_succ(trav));
		point p((p1.xcoord()+p2.xcoord())/2,(p1.ycoord()+p2.ycoord())/2);
		_mid_list.append(p);
		trav = _pt_list.cyclic_succ(trav);
	}
}

polygon_barrier::polygon_barrier(const polygon_barrier& P) {
	_pt_list=P._pt_list;
	if (!Is_Simple_Polygon(_pt_list))
		lola_error("polygon_barrier: Polygon is not simple",1414,1);
	_ed_list=P._ed_list;
	_mid_list=P._mid_list;
}

polygon_barrier::~polygon_barrier() {
	_pt_list.clear();
	_mid_list.clear();
	_ed_list.clear();
}

point polygon_barrier::center() {
	point retval(0.0,0.0);
	return retval;
}

double polygon_barrier::maxdiam() {
	list_item trav1;
	list_item trav2;
	double retval=_pt_list.head().distance(_pt_list.tail());
	for(trav1=_pt_list.first();trav1;trav1=_pt_list.succ(trav1))
		for(trav2=_pt_list.first();trav2;trav2=_pt_list.succ(trav2)) {
			double dst=_pt_list.contents(trav1).distance(_pt_list.contents(trav2));
			if (dst > retval) retval = dst;
		}
		return retval;
}

bool polygon_barrier::visible(const point& p1,const point& p2) {
	segment test(p1,p2);
	bool retval = false;
	if (inside(p1) || inside(p2)) retval =  false; 
	if (intersection(test).length()<=1) retval  =true;
	return retval;
}

bool polygon_barrier::inside(const point& p) {
	polygon P(_pt_list);
	bool retval = false;
	retval=P.inside(p);
	return retval;
}

bool polygon_barrier::inside(const location& l) {
	return inside(l.loc2point());
}

bool polygon_barrier::on_edge(const point& p) {
	list_item trav;
	bool retval = false;
	for(trav=_ed_list.first();trav;trav=_ed_list.succ(trav))
		if (_ed_list.contents(trav).contains(p)) {
			retval = true;
			break;
		}
		return retval;
}

bool polygon_barrier::on_edge(const location& l) {
	return on_edge(l.loc2point());
}

list<point> polygon_barrier::intersection(const line& l) {
	list<point> retval;
	retval.clear();
	polygon P(_pt_list);
	retval=P.intersection(l);
	return retval;
}

list<point> polygon_barrier::intersection(const segment& s) {
	list<point> retval;
	retval.clear();  
	polygon P(_pt_list);
	retval=P.intersection(s);
	return retval;
}

point polygon_barrier::r_tang_point(const point& p) {
	point retval = t_points(p).tail();
	return retval;
}

point polygon_barrier::l_tang_point(const point& p) {
	point retval = t_points(p).head();
	return retval;
}

list<point> polygon_barrier::t_points(const point& p) {
	// Preconditions: 1. The points forming the polygon must be given in clockwise or
	//                   counterclockwise order.
	//                2. The polygon must be simple.
	//		       This is already checked in the constructor of this class :-)

    list<point> retval;
	retval.clear();
	list_item trav;
	list_item firstpoint;
	list_item lastpoint;

  // Now we compute the points that are visible from p on the edge of the polygon
  // The first point and the last point of these points are then the two points of
  // tangency from p.
  // Two cases:
  //            1. The first point of the list is visible from p ...

	int rank;

    if (visible(p,_mid_list.head())) { // Search for first point on polygon that is not visible ...
		for(trav=_mid_list.first();trav;trav=_mid_list.cyclic_succ(trav)) {
			if (!visible(p,_mid_list.contents(trav))) {
				rank=_mid_list.rank(_mid_list.contents(trav))-1;
				break;
			}
		}
		firstpoint=_pt_list.get_item(rank);
		for(;trav;trav=_mid_list.cyclic_succ(trav)) {
			if (visible(p,_mid_list.contents(trav))) {
				rank=_mid_list.rank(_mid_list.contents(trav))-1;
				break;
			}
		}
		lastpoint=_pt_list.get_item(rank);		
	} else { 
		//          2. The first point of of the list is NOT visible from p ...
	    // search for first point on polygon that is visible ...
		for(trav=_mid_list.succ(_mid_list.first());trav;trav=_mid_list.cyclic_succ(trav)) {
			if (visible(p,_mid_list.contents(trav))) {
				rank=_mid_list.rank(_mid_list.contents(trav))-1;
				break;
			}
		}
		lastpoint=_pt_list.get_item(rank);
		for(;trav;trav=_mid_list.cyclic_succ(trav)) {
			if (!visible(p,_mid_list.contents(trav))) {
				rank=_mid_list.rank(_mid_list.contents(trav))-1;
				break;
			}
		}
		firstpoint=_pt_list.get_item(rank);
	}
	// points found in a row are now visible from p ... 
	// So we have both points of tangency.
	// Left to do: map these two points to the left and right point of tangency.
	// We do this by constructing a halfspace defined by p and the midpoint of the 
	// segment that connects the left an right point of tangency from p on the 
	// polygon's edge. Then we know which point of tangency lies in which halfspace :-)


	/* Alternative B */
	segment s1(p,_pt_list.contents(firstpoint));
	segment s2(p,_pt_list.contents(lastpoint));
	double alpha12=s1.angle(s2);
	segment halfangle=s1.rotate(0.5*alpha12);
	
	halfspace H(halfangle.source(),halfangle.target());

    retval.clear();
	if (H.in_which(_pt_list.contents(firstpoint)) == -1) {
		retval.append(_pt_list.contents(firstpoint));
		retval.append(_pt_list.contents(lastpoint));
	} else {
		retval.append(_pt_list.contents(firstpoint));
		retval.append(_pt_list.contents(lastpoint));
	}
	// retval.head() is the left one
	// retval.tail() is the right one
	return retval;
}

double polygon_barrier::right_path(const point& p1,const point& p2) {
	return path_length(p1,p2)[0];
}

double polygon_barrier::left_path(const point& p1,const point& p2) {
  	return path_length(p1,p2)[1];
}

double* polygon_barrier::path_length(const point& p1,const point& p2) {
	double* retval;
	double dist1=0;
	double dist2=0;
	list_item trav;
	list<point> tp1=t_points(p1);
	list<point> tp2=t_points(p2);
	// Now we search for the list_items of all 4 points of tangency
	list<list_item> inbetween1;
	inbetween1.clear();
	if (tp1.tail() != tp2.head()) {
		halfspace H1(tp1.tail(),tp2.head(),1);
		for(trav=_pt_list.first();trav;trav=_pt_list.succ(trav)) 
			if(H1.check_point(_pt_list.contents(trav))) 
				inbetween1.append(trav);
	}
	if(inbetween1.length() > 0) {
		if ((_pt_list.contents(inbetween1.head())==tp1.tail()) ||
			(_pt_list.contents(inbetween1.head())==tp2.head())) {
			trav=inbetween1.first();
			inbetween1.del_item(trav);
		}
		if ((_pt_list.contents(inbetween1.tail())==tp1.tail()) ||
			(_pt_list.contents(inbetween1.tail())==tp2.head())) {
			trav=inbetween1.last();
			inbetween1.del_item(trav);
		}
	}
	list<list_item> inbetween2;
	inbetween2.clear();
	if (tp2.tail() != tp1.head()) {
		halfspace H2(tp2.tail(),tp1.head(),1);
		for(trav=_pt_list.first();trav;trav=_pt_list.succ(trav)) 
			if(H2.check_point(_pt_list.contents(trav))) 
				inbetween2.append(trav);
	}
	if(inbetween2.length() > 0) {
		if ((_pt_list.contents(inbetween2.head())==tp2.tail()) ||
			(_pt_list.contents(inbetween2.head())==tp1.head())) {
			trav=inbetween2.first();
			inbetween2.del_item(trav);
		}
		if ((_pt_list.contents(inbetween2.tail())==tp2.tail()) ||
			(_pt_list.contents(inbetween2.tail())==tp1.head())) {
			trav=inbetween2.last();
			inbetween2.del_item(trav);
		}
	}
	dist1 = p1.distance(tp1.tail()) + p2.distance(tp2.head());
	dist2 = p2.distance(tp2.tail()) + p1.distance(tp1.head());
	int count1 = 0;
	int count2 = 0;
	trav=inbetween1.first();  
	while (count1 < inbetween1.length()) {
		dist1 += _pt_list.contents(_pt_list.cyclic_pred(inbetween1.contents(trav))).distance(_pt_list.contents(inbetween1.contents(trav)));
		count1++;
		trav = inbetween1.succ(trav);
	}
	if(inbetween1.length() > 0)
		dist1 += _pt_list.contents(inbetween1.tail()).distance(_pt_list.contents(_pt_list.cyclic_succ(inbetween1.tail())));
	trav=inbetween2.first();  
	while (count2 < inbetween2.length()) {
		dist2 += _pt_list.contents(_pt_list.cyclic_pred(inbetween2.contents(trav))).distance(_pt_list.contents(inbetween2.contents(trav)));
		count2++;
		trav = inbetween2.succ(trav);
	}
	if(inbetween2.length() > 0) 
		dist2 += _pt_list.contents(inbetween2.tail()).distance(_pt_list.contents(_pt_list.cyclic_succ(inbetween2.tail())));
	retval = new double[2];
	retval[0]=dist1;
	retval[1]=dist2;
	return retval;  
}

double polygon_barrier::LSP2(const point& p1,const point& p2) { 
	double retval = 0;
	if (visible(p1,p2)) {
		retval = p1.distance(p2);
		return retval;
	}
	double dist1=0;
	double dist2=0;
	list_item trav;
	list<point> tp1=t_points(p1);
	list<point> tp2=t_points(p2);
	// Now we search for the list_items of all 4 points of tangency
	list<list_item> inbetween1;
	inbetween1.clear();
	halfspace H1(tp1.tail(),tp2.head(),1);
	for(trav=_pt_list.first();trav;trav=_pt_list.succ(trav)) 
		if(H1.check_point(_pt_list.contents(trav))) 
			inbetween1.append(trav);
		if(inbetween1.length() > 0) {
			if ((_pt_list.contents(inbetween1.head())==tp1.tail()) ||
				(_pt_list.contents(inbetween1.head())==tp2.head())) {
				trav=inbetween1.first();
				inbetween1.del_item(trav);
			}
			if ((_pt_list.contents(inbetween1.tail())==tp1.tail()) ||
				(_pt_list.contents(inbetween1.tail())==tp2.head())) {
				trav=inbetween1.last();
				inbetween1.del_item(trav);
    }
  }
  list<list_item> inbetween2;
  inbetween2.clear();
  halfspace H2(tp2.tail(),tp1.head(),1);
  for(trav=_pt_list.first();trav;trav=_pt_list.succ(trav)) 
    if(H2.check_point(_pt_list.contents(trav))) 
      inbetween2.append(trav);
  if(inbetween2.length() > 0) {
    if ((_pt_list.contents(inbetween2.head())==tp2.tail()) ||
	(_pt_list.contents(inbetween2.head())==tp1.head())) {
      trav=inbetween2.first();
      inbetween2.del_item(trav);
    }
    if ((_pt_list.contents(inbetween2.tail())==tp2.tail()) ||
	(_pt_list.contents(inbetween2.tail())==tp1.head())) {
      trav=inbetween2.last();
      inbetween2.del_item(trav);
    }
  }
  dist1 = p1.distance(tp1.tail()) + p2.distance(tp2.head());
  dist2 = p2.distance(tp2.tail()) + p1.distance(tp1.head());
  int count1 = 0;
  int count2 = 0;
  trav=inbetween1.first();  
  while (count1 < inbetween1.length()) {
    dist1 += _pt_list.contents(_pt_list.cyclic_pred(inbetween1.contents(trav))).distance(_pt_list.contents(inbetween1.contents(trav)));
    count1++;
    trav = inbetween1.succ(trav);
  }
  if(inbetween1.length() > 0)
    dist1 += _pt_list.contents(inbetween1.tail()).distance(_pt_list.contents(_pt_list.cyclic_succ(inbetween1.tail())));
  trav=inbetween2.first();  
  while (count2 < inbetween2.length()) {
    dist2 += _pt_list.contents(_pt_list.cyclic_pred(inbetween2.contents(trav))).distance(_pt_list.contents(inbetween2.contents(trav)));
    count2++;
    trav = inbetween2.succ(trav);
  }
  if(inbetween2.length() > 0) 
    dist2 += _pt_list.contents(inbetween2.tail()).distance(_pt_list.contents(_pt_list.cyclic_succ(inbetween2.tail())));
  if (dist1 < dist2) retval=dist1;
  else retval=dist2;
  return retval;  
}

double polygon_barrier::l_2(const location& l,const point& p) {
	if (inside(p)) return 100000000.0;
	double* retval;
	retval = path_length(l.loc2point(),p);
	if (retval[0] < retval[1])
		return retval[0];
	return retval[1];
}

double polygon_barrier::objective(const facilities& Ex,const point& p) {
	double retval = 0;
	list_item trav;
	for(trav=Ex.first();trav;trav=Ex.succ(trav))
		retval += Ex.contents(trav)(0) * l_2(Ex.contents(trav),p);
	return retval;
}

ostream& operator << (ostream& os,const polygon_barrier& P) {
	list_item trav;
	os << "[ ";
	for(trav=P._pt_list.first();trav;trav=P._pt_list.succ(trav)) 
		os << P._pt_list.contents(trav) << " ";
	os << "]";
	return os;
}
