#include <LOLA/restr_circle.h>

restr_circle::restr_circle() {
   _radius = 0;
}

restr_circle::restr_circle(const point &c, double r) {
   if (r >= 0) {
      _radius = r;
      centre = c;
   }
   else restr_circle();
}

restr_circle::restr_circle(double x, double y, double r) {
   if (x >= 0 && y >= 0 && r >= 0) {
      centre = point(x,y);
      _radius = r;
   }
   else restr_circle();
}

restr_circle::restr_circle(const point &a, const point &b, const point &c) {
   if (collinear(a,b,c)) lola_error("Points are colinear.",1401,1);
   line l1 = p_bisector(a,b);
   line l2 = p_bisector(b,c);
   l1.intersection(l2, centre);
   _radius = centre.distance(a);
}

restr_circle::restr_circle(const restr_circle &c) {
      _radius = c._radius;
      centre = c.centre;
}

restr_circle::~restr_circle() {
   point empty_pnt;
   centre = empty_pnt;
   _radius = 0;
}

restr_circle& restr_circle::operator = (const restr_circle &c) {
   if (this == &c) return *this;
   centre = c.centre;
   _radius = c._radius;
   return *this;
}

double restr_circle::distance(const point &p) {
   double d = p.distance(centre);
   return (d - _radius);
}
   
bool restr_circle::inside(const point &p) {
   if (distance(p) < 0) return true;
   return false;
}

bool restr_circle::inside(const location &l) {
   point tmp = l.loc2point();
   return inside(tmp);
}

bool restr_circle::inside(const segment &s) {
   if (inside(s.source()) && inside(s.target())) return true;
   return false;
}

list<point> restr_circle::intersect(point &P, point &Q) {
   Loc_Vector m(2.0,centre.xcoord(),centre.ycoord()), p(2.0,P.xcoord(),P.ycoord()), u(2.0,Q.xcoord()-p[0],Q.ycoord()-p[1]), d(p-m);
   list<point> results;
   double a = u*u, b = u*d*2, c = d*d-(_radius*_radius), radicant = (b*b)-(4*a*c), s1, s2;
   if (radicant < 0) return results;
   if (radicant == 0) {
      s1 = ((-1)*b) / (2*a);
      Loc_Vector x1 = u*s1+p;
      point result(x1[0],x1[1]);
      results.append(result);
      return results;
   }
   s1 = ((-1)*b + sqrt(radicant)) / (2*a); s2 = ((-1)*b - sqrt(radicant)) / (2*a);
   Loc_Vector x1 = u*s1+p, x2 = u*s2+p;
   point result1(x1[0],x1[1]), result2(x2[0],x2[1]);
   results.append(result1), results.append(result2);
   return results;
}

list<point> restr_circle::intersect(ray &r,point& r2) {
   point r1 = r.source();
   list<point> result = intersect(r1,r2);
   point totest = result.head();
   return result;
}

list<point> restr_circle::intersection(const line &l) {
    circle C(centre,_radius);
    return C.intersection(l);
}

/*
list<point> restr_circle::intersection(const line &l) {
   list<point> result; line l1;
   if (l.vertical()) l1 = line(centre, point(centre.xcoord()+1,centre.ycoord()));
   else if (l.horizontal()) l1 = line(centre, point(centre.xcoord(),centre.ycoord()+1));
   else {
      double direction = l.slope()+M_PI_2;
      l1 = line(centre, direction);
   }
   point r; l.intersection(l1,r);
   segment s(centre,r); double d = s.length(); point F = s.end();
   if (d == _radius) result.append(F);
   if (d < _radius) {
      double alpha = l.angle();
      double x = sqrt(L_sqr(_radius) - L_sqr(d));
      point  p = F.translate(alpha,x), q = F.translate(alpha,-x);
      result.append(q); result.append(p);
   }  
   return result;
}
*/

list<point> restr_circle::intersection(const segment &s) {
   list<point> result,L;
   point r1 = s.source(), r2 = s.target();
   L = intersect(r1,r2);
   point p;
   double d = s.length();
   forall(p,L) {
      double d1 = s.source().distance(p);
      double d2 = s.target().distance(p);
      if (d1 <= d && d2 <= d) result.append(p);
   }
   return result;
}

list<point> restr_circle::intersection(const circle &C) {
  list<point> result;
  segment s(centre, C.center());
  double d  = s.length();
  double r1 = _radius;
  double r2 = C.radius();

  if (d > (r1+r2) || (d+r2) < r1 || (d+r1) < r2) return result;

  double x = (d*d + r1*r1 - r2*r2)/(2*d);
  double alpha = acos(x/r1);
  double beta  = s.angle() + alpha;
  double gamma = s.angle() - alpha;

  result.append(centre.translate(beta,r1));
  if (alpha!=0) result.append(centre.translate(gamma,r1));

  return result;
}


list<point> restr_circle::intersection(const polygon &P) {
   list<point> result;
   segment t;
   list<point> pl;
   point p;
   for(list_item it = P.segments().first(); it; it = P.segments().succ(it))
   {
     t = P.segments().contents(it);
     pl=intersection(t);
     if (pl.size()>0)
     forall(p, pl)
       if (result.empty()) result.append(p);
       else if (p != result.tail() && p != result.head()) result.append(p);
   }
   return result;
}

list<location> restr_circle::proj_l2sqr(const location &opt) {
   list<location> result;
   point p=opt.loc2point();
   location loc;
   if (p==centre) {
     point p2(p.xcoord(),p.ycoord() + _radius);
     loc = point2loc(p2);
     result.append(loc);
     lola_message("restr_circle::proj_l2sqr(): The whole boundary of the forbidden region is optimal. The marked point is one optimal point",1); 
     return result;
   }
   else {
   line l(p,centre);
   list<point> Int = intersection(l);
   if (p.distance(Int.head()) < p.distance(Int.tail()))
     loc = point2loc(Int.head());
   else loc = point2loc(Int.tail());
   result.append(loc);
   return result;
   }
}


bool restr_circle::operator == (const restr_circle &c) {
   return centre == c.centre && _radius == c.radius();
}

ostream& operator<<(ostream& out, const restr_circle &c) {
   out << c.center() << " "<< c.radius(); 
   return out;
} 

istream& operator>>(istream& in,  restr_circle &c) {
   point cent;
   double rad;
   if (in) in >> cent;
   if (in) in >> rad;
   c = restr_circle(cent, rad);
   return in;
}


double test3 (double& ztest, facilities& EX, list<int>& Ln, list<polygauge>& LG, restr_circle& RC)  
{
	point tmp;
  
	int gnumi;
	list<point> L;
	double test=0;
	location tmpex=EX.head();
	gnumi=Ln.contents(Ln.get_item(0));
	polygauge gaugei=LG.contents(LG.get_item(gnumi));
	list<point> L3=gaugei.vertices(origin);
	forall (tmp, L3)
    {
		double xE=(ztest-L_eps)/tmpex(0)*tmp.xcoord()+tmpex[0];
		double yE=(ztest-L_eps)/tmpex(0)*tmp.ycoord()+tmpex[1];
		point p(xE,yE);
		L.push(p);               
    }
	polygon P(L);
	for (int i=0;i<EX.length();i++)
    {
		L.clear();
		tmpex=EX.contents(EX.get_item(i));
		gnumi=Ln.contents(Ln.get_item(i));
		gaugei=LG.contents(LG.get_item(gnumi));
		L3=gaugei.vertices(origin);
		forall (tmp, L3)                  /*alle Ecken des Polygons*/
		{
			double xE=(ztest-L_eps)/tmpex(0)*tmp.xcoord()+tmpex[0];
			double yE=(ztest-L_eps)/tmpex(0)*tmp.ycoord()+tmpex[1];	  
			point p(xE,yE);
			L.push(p);
		}
		polygon Q(L);
		polygon S=poly_intersection(P,Q);      
		P=S;
    }     
	if (P.empty()!=true)
    {
		int anzahl=0;
		int randzahl=0;
		L=P.vertices();
		forall (tmp, L) 
		{
			if(RC.center().distance(tmp)==RC.radius())  randzahl=randzahl+1;	    
			if(RC.inside(tmp)==true)  anzahl=anzahl+1;
			if((anzahl-randzahl)==P.size())
			{
				int test=0;
			}
			else
			{
				int test=1;
			}
		}
    }
	return test;
}/*test3*/



int restr_circle::opt3(double& z, facilities& EX, list<int>& Ln, list<polygauge>& LG, list<location>& OPTIMUM)
{
	list<point> OPTLIST;
	point tmp;
	polygon P=EX.opt11 (z, Ln, LG);
	list<point> L=P.vertices();
	list<point> optimal;
	int hilf3=0;
	forall (tmp, L) 
    {
		if(this->center().distance(tmp)>=this->radius()-L_eps)  optimal.push(tmp);
    }  
	OPTLIST=optimal;
	if(OPTLIST.length()==0) 
    {
		hilf3=1;
    }
	else
    {
		EX.doppel(OPTLIST);
		OPTIMUM.clear();
		OPTIMUM.append(EX.alg_solution().head());
    }
	return hilf3;
}

void restr_circle::megiddo(list<double>& niveauliste,facilities& EX, list<int>& Ln, list<polygauge>& LG)
{
	double ztest,tempe;
	int test;
	while (niveauliste.length()>1)
    {
		list<double> hilfliste;
		ztest=gauge_median(niveauliste);
		test=(int) test3 (ztest, EX, Ln, LG, *this);            //Entscheidungskriterium

		if (test==0)                                //ztest ist unzulssig
		{
			forall(tempe, niveauliste)
			{
				if (tempe>ztest)  hilfliste.append(tempe);
			}
			hilfliste.push(ztest);
		}
		else
		{
			forall(tempe, niveauliste)
			{
				if (tempe<ztest)  hilfliste.append(tempe);
			} 
			hilfliste.append(ztest);
		}
		
		if (hilfliste.length()==2)  //Bei Listenlnge 2 kann es passieren,
		{                        //da man in einer Endlosschleife landet
			if(hilfliste.head()!=ztest)     
			{  
				double kgb=hilfliste.head();
				test=(int) test3(kgb, EX, Ln, LG, *this); 
				if (test==0)  hilfliste.Pop();
				else   hilfliste.pop();
			}
			else
			{
				double kgb=hilfliste.tail();
				test=(int) test3(kgb, EX, Ln, LG, *this);
				if (test==0)  hilfliste.pop();  
				else   hilfliste.Pop();
			}	
			niveauliste=hilfliste;
		}
		else  niveauliste=hilfliste;
    }//while
}


double restr_circle::gauge_max(list<location> &opt, facilities &EX, double zmax, list<polygauge> &LG, list<int> &Ln)
/*****************************************************************************
* Purpose    : Compute the optimum for problem class 1|P|R=ConvexPolyeder|gauge|max
* Returns    : %
* Influences : %
* Date       : May 99
* Author     : Ralf Leipe (included by Holger Hennes)
*****************************************************************************/
{
	location tmpex;
	double A,B,a,b,d,e,g;

	double zoben=1.0e8;
	int schnell;            /*beschrnkt die Niveaumenge zunchst auf 500 Werte*/
	schnell=500;            /*bei kleinen Problemen lohnt sich das nicht*/
	if (EX.length()<31)   schnell=5000;
	list<double> niveauliste;
	list<double> niveauhilfe;
	int jnum,lnum,gnumi,gnumk,i;
	double z=0;
	for (i=0;i<EX.length();i++)
    {
		tmpex=EX.contents(EX.get_item(i));           /*Standort i, Ecke j*/
		gnumi=Ln.contents(Ln.get_item(i));
		polygauge gaugei=LG.contents(LG.get_item(gnumi));
		list<point> Listei=gaugei.vertices(origin);
		for (jnum=0;jnum<Listei.length();jnum++)
		{
			point j=Listei.contents(Listei.get_item(jnum));
			double c1=j.xcoord()/tmpex(0);
			double c2=j.ycoord()/tmpex(0);
			double d1=tmpex[0];
			double d2=tmpex[1];
			A=c1*(d1-this->center().xcoord())+c2*(d2-this->center().ycoord());
			B=c1*c1+c2*c2;
			double radikand=((A/B)*(A/B)-((d1-this->center().xcoord())*(d1-this->center().xcoord())+(d2-this->center().ycoord())*(d2-this->center().ycoord())-this->radius()*this->radius())/B);
			if(radikand>=0)
			{
				z=-(A/B)+sqrt(radikand);         /*Ecke = Kreis setzen*/
				if(z>=zmax)
				{         
					if(niveauliste.length()<schnell)  niveauliste.push(z);		  
					else  niveauhilfe.push(z);
				}
			}
		}
    }
	/*kritische Situationen vom Typ b'*/
	for (i=0;i<EX.length()-1;i++)
    {
		tmpex=EX.contents(EX.get_item(i));       /*Kugel i*/
		gnumi=Ln.contents(Ln.get_item(i));
		polygauge gaugei=LG.contents(LG.get_item(gnumi));
		list<point> Listei=gaugei.vertices(origin);
		for (jnum=0;jnum<Listei.length();jnum++)
		{
			point j=Listei.contents(Listei.get_item(jnum));
			for (int k=i+1;k<EX.length();k++)                  /*Kugel k fr Facette*/
			{
				location tmpex1=EX.contents(EX.get_item(k));
				gnumk=Ln.contents(Ln.get_item(k));
				polygauge gaugek=LG.contents(LG.get_item(gnumk));
				list<point> Listek=gaugek.vertices(origin);
				for(lnum=0;lnum<Listek.length();lnum++)  /*Facette von l und l+1*/
				{
					point l=Listek.contents(Listek.get_item(lnum)),m,n3;
					if (j==Listei.tail())  m=Listei.head();
					else  m=Listei.contents(Listei.get_item(jnum+1));
					if (l==Listek.tail())  n3=Listek.head();
					else  n3=Listek.contents(Listek.get_item(lnum+1));
					if (j.xcoord()==m.xcoord())
					{
						if (l.xcoord()!=n3.xcoord())
						{
							a=(l.ycoord()-n3.ycoord())/(l.xcoord()-n3.xcoord());
							b=tmpex1[1]-a*tmpex1[0];
							d=(l.ycoord()-a*l.xcoord())/tmpex1(0);
							double c1=j.xcoord()/tmpex(0);
							double c2=a*c1+d; 
							double d1=tmpex[0];
							double d2=a*d1+b;	
							A=c1*(d1-this->center().xcoord())+c2*(d2-this->center().ycoord());   
							B=c1*c1+c2*c2;
							double radikand=((A/B)*(A/B)-((d1-this->center().xcoord())*(d1-this->center().xcoord())+(d2-this->center().ycoord())*(d2-this->center().ycoord())-this->radius()*this->radius())/B);
							if(radikand>=0)
							{
								z=-(A/B)+sqrt(radikand); /*Ecke = Kreis setzen*/ 
								if(z>=zmax)
								{  
									if(niveauliste.length()<schnell)   niveauliste.push (z);
									else   niveauhilfe.push(z);
								}
							}            /*eine senkrechte Facette fij*/
						}
					} 
					else
					{
						if (l.xcoord()==n3.xcoord())
						{
							a=(j.ycoord()-m.ycoord())/(j.xcoord()-m.xcoord());
							b=tmpex[1]-a*tmpex[0];
							d=(j.ycoord()-a*j.xcoord())/tmpex(0);
							double c1=l.xcoord()/tmpex1(0);
							double c2=a*c1+d;  
							double d1=tmpex1[0];
							double d2=a*d1+b;
							A=c1*(d1-this->center().xcoord())+c2*(d2-this->center().ycoord());
							B=c1*c1+c2*c2;
							double radikand=((A/B)*(A/B)-((d1-this->center().xcoord())*(d1-this->center().xcoord())+(d2-this->center().ycoord())*(d2-this->center().ycoord())-this->radius()*this->radius())/B);
							if(radikand>=0)
							{
								z=-(A/B)+sqrt(radikand); /*Ecke = Kreis setzen*/ 
								if(z>=zmax)
								{          
									if(niveauliste.length()<schnell)   niveauliste.push (z);	    
									else  niveauhilfe.push(z);
								}
							}            /*andere Facette fkl senkrecht*/
						}
						else
						{
							a=(j.ycoord()-m.ycoord())/(j.xcoord()-m.xcoord());
							b=tmpex[1]-a*tmpex[0];
							d=(j.ycoord()-a*j.xcoord())/tmpex(0);
							e=(l.ycoord()-n3.ycoord())/(l.xcoord()-n3.xcoord());
							double f1=tmpex1[1]-e*tmpex1[0];
							g=(l.ycoord()-e*l.xcoord())/tmpex1(0);
							if(a==e)  z=0;
							else
							{
								double c1=(g-d)/(a-e);
								double c2=a*c1+d;
								double d1=(f1-b)/(a-e);
								double d2=a*d1+b;
								A=c1*(d1-this->center().xcoord())+c2*(d2-this->center().ycoord());
								B=c1*c1+c2*c2;
								double radikand=((A/B)*(A/B)-((d1-this->center().xcoord())*(d1-this->center().xcoord())+(d2-this->center().ycoord())*(d2-this->center().ycoord())-this->radius()*this->radius())/B);
								if(radikand>=0)
								{ 
									z=-(A/B)+sqrt(radikand);      /*Ecke = Kreis setzen*/
									if(z>=zmax)
									{               
										if(niveauliste.length()<schnell)   niveauliste.push(z);               
										else  niveauhilfe.push(z);
									}                    /*keine Facette senkrecht*/
								}
							}
						}
					}
				}
			}
		}
    } 
	if (niveauliste.length()==2)  niveauliste.push(niveauliste.head());
                                                      /*Megiddo-Verfahren*/
	this->megiddo(niveauliste, EX, Ln, LG);
	z=niveauliste.head();
	int hilf3=this->opt3 (z, EX, Ln, LG, opt);
	if(schnell<4000)
    {
		if(hilf3==1)
		{
			double zz;
			niveauliste.clear();
			forall(zz, niveauhilfe)
			{
				if(zz>=zmax)
				{
					if(zz<=zoben)   niveauliste.push(zz);
				}
			}
			this->megiddo(niveauliste, EX, Ln, LG);
			z=niveauliste.head();
			this->opt3 (z, EX, Ln, LG, opt);
		}
    }
	return z;   
}
