#include <LOLA/restr_poly.h>

// --------------- some static functions ---------------------

static int check_orientation(const list<segment> &seg_list) {
	double angle = 0;
	list_item it;
	forall_items(it, seg_list) {
		list_item succ = seg_list.cyclic_succ(it);
		segment s1 = seg_list[it];
		segment s2 = seg_list[succ];
		angle += s1.angle(s2);
	}  
	return (angle > 0);
}

static void check_simplicity(const list<segment> &seg_list){  
	if (seg_list.length() < 3)
		lola_error("polygon: must be simple",1402,1);
	point check_start, check_end;
	int size = seg_list.length();
	list_item trav = seg_list.first();
    check_start = seg_list.contents(trav).source();
    check_end = seg_list.contents(trav).target();
    for (int i = 1; i < size; i++) {
		trav = seg_list.succ(trav);
		if (seg_list.contents(trav).source() != check_end) lola_error("polygon: must be simple",1403,1);
		check_end = seg_list.contents(trav).target();
	}
	if (check_end != check_start) lola_error("polygon: must be simple",1404,1);
}

static bool x_proj(location &loc, segment &seg, point &intsec) {
	point startpnt(-L_INF, loc[1]), endpnt(L_INF, loc[1]);
	segment tmp(startpnt, endpnt);
	return seg.intersection(tmp, intsec);
}

static bool y_proj(location &loc, segment &seg, point &intsec) {
	point startpnt(loc[0], -L_INF), endpnt(loc[0], L_INF);
	segment tmp(startpnt, endpnt);
	return seg.intersection(tmp, intsec);
}

static bool x_proj(point &p, segment &seg, point &intsec) {
	point startpnt(-L_INF, p.ycoord()), endpnt(L_INF, p.ycoord());
	segment tmp(startpnt, endpnt);
	return seg.intersection(tmp, intsec);
}

static bool y_proj(point &p, segment &seg, point &intsec) {
	point startpnt(p.xcoord(), -L_INF), endpnt(p.xcoord(), L_INF);
	segment tmp(startpnt, endpnt);
	return seg.intersection(tmp, intsec);
}

// ----------------------------------------------------------------------------------------------

restr_poly::restr_poly() {
	list<segment> empty;
	seg_list = empty;
}

restr_poly::restr_poly(const list<point> &pl){
	for(list_item it = pl.first(); it; it = pl.succ(it)) {
		point p = pl[it];
		point q = pl[pl.cyclic_succ(it)];
		seg_list.append(segment(p,q));
	}
	if (check_orientation(seg_list)) {
		seg_list.clear();
		for(list_item it = pl.last(); it; it = pl.pred(it)) {
			point p = pl[it];
			point q = pl[pl.cyclic_pred(it)];
			seg_list.append(segment(p,q));
		}
	}
	check_simplicity(seg_list);
}

/* restr_poly::restr_poly(const restr_poly &P, double bar) {
   seg_list = P.seg_list;
   _bartyp=bar;
} */

restr_poly::restr_poly(const restr_poly &P) {
	seg_list = P.seg_list;
   //   _bartyp=P._bartyp;
}

restr_poly::~restr_poly() {
	list<segment> empty;
	seg_list = empty;
}

restr_poly& restr_poly::operator = (const restr_poly &P) {
	seg_list = P.seg_list;
	//   _bartyp=P._bartyp;
	return *this;
}

list<point> restr_poly::vertices() {
	list<point> result;
	segment s;
	forall(s, seg_list) result.append(s.start());
	return result;
}

bool restr_poly::inside(const point &p) {
	line l(p, LEDA_PI_2);
	int count = 0;
	segment s;
	forall(s, seg_list) {
		point q;
		if (!l.intersection(s, q)) continue;
		if (q.ycoord() < p.ycoord()) continue;
		if (q == s.source()) continue;
		if (q == s.target()) continue;
		count++;
	}
	list<point> plist = vertices();
	list_item i0 = plist.first();
	while (plist[i0].xcoord() == p.xcoord())  i0 = plist.cyclic_pred(i0);
	list_item i = plist.cyclic_succ(i0);
	for(;;) { 
		while ( i != i0 && (plist[i].xcoord() != p.xcoord() || plist[i].ycoord() < p.ycoord()))
			i = plist.cyclic_succ(i);
		if (i==i0) break;
		point q  = plist[i];
		point q0 = plist[plist.cyclic_pred(i)];
		while ((plist[i].xcoord()==p.xcoord()) && (plist[i].ycoord() >= p.ycoord())) {
			if (plist[i]==p) return 1;
			i = plist.cyclic_succ(i);
		}
		point q1 = plist[i];
		if (q0.xcoord() < p.xcoord() && q1.xcoord() > p.xcoord()) count++;
		if (q0.xcoord() > p.xcoord() && q1.xcoord() < p.xcoord()) count++;
	}
	if (count%2) return true;
	return false;
}

bool restr_poly::inside(const location &p) {
	point tmp = p.loc2point();
	bool B=inside(tmp);
	if (B) return true;
	return false;
}

bool restr_poly::inside(const segment &s) {
	if (inside(s.source()) && inside(s.target())) return true;
	return false;
}

bool restr_poly::convex() {
	list<point> pl=vertices();
	list<point> C = CONVEX_HULL(pl);
	if (C.size() != pl.size()) return false;
	return true;
}

list<point> restr_poly::intersect(point &p, point &q) {
	list<point> result;
	segment s(p,q);
	result = intersection(s);
	return result;   
}

list<point> restr_poly::intersect(ray &r, point &r2) {
	point r1 = r.source();
	if (inside(r2)) r2=r1+10*(r2-r1);
	list<point> intersect_result = intersect(r1,r2) , result;
	point totest;
	if (intersect_result.length() > 0) {
		forall(totest,intersect_result)
			result.append(totest);
	}
	return result;
}

list<point> restr_poly::intersection(const line &l) {
	list<point> result;
	segment t;
	point p;
	forall(t, seg_list)
		if (l.intersection(t, p))
			if (result.empty()) result.append(p);
			else if (p != result.tail() && p != result.head()) result.append(p);
			return result;
}

list<point> restr_poly::intersection(const segment &s) {
	list<point> result;
	segment t;
	point p;
	forall(t, seg_list) 
		if (s.intersection(t, p))
			if (result.empty()) result.append(p);
			else if (p != result.tail() && p != result.head()) result.append(p);
			return result;
}

list<point> restr_poly::intersection(const polygon &P) {
	list<point> result;
	segment t1,t2;
	point p;
	for(list_item it = seg_list.first(); it; it = seg_list.succ(it))
	{
		t1 = seg_list.contents(it);
		forall(t2, P.segments())
			if (t2.intersection(t1, p))
				if (result.empty()) result.append(p);
				else if (p != result.tail() && p != result.head()) result.append(p);
	}
	return result;
}


list<point> restr_poly::intersection(const circle &C) {
	list<point> result;
	segment t;
	list<point> pl;
	point p;
	for(list_item it = seg_list.first(); it; it = seg_list.succ(it))
	{
		t = seg_list.contents(it);
		pl=C.intersection(t);
		if (pl.size()>0)
			forall(p, pl)
			if (result.empty()) result.append(p);
			else if (p != result.tail() && p != result.head()) result.append(p);
	}
	return result;
}

list<location> restr_poly::proj_l2sqr(const location &opt) {
	list<location> result;
	list<point> Vert;
	point p=opt.loc2point(),cut;
	segment t,tmp;
	line l;
	result.clear();
	for(list_item it = seg_list.first(); it; it = seg_list.succ(it))
	{
		t = seg_list.contents(it);
		l = line(t);
		tmp = l.perpendicular(p);
		if (t.intersection(tmp,cut)){
			result.append(point2loc(cut));
		}
	}
	if (!inside(opt)|| !convex()) {
		Vert=vertices();
		forall(p,Vert) result.append(point2loc(p));
	}
	return result;
}

double restr_poly::proj_v1_l2_max(location &opt, facilities &facs, double z_opt_tmp) {
/*****************************************************************************
* Purpose    : Compute the optimum for problem class 1|P|R=Polyeder|l_2|max
* Returns    : %
* Influences : %
* Date       : Dez 95
* Author     : Ingo Bartling
* Last Change: Oct 98 (Holger Hennes)
*****************************************************************************/
	location pnt = opt;
	double epsilon = 1.0E-13;
	restriction *tmp_restrict = this;
	if (tmp_restrict->subtype() != restr_p)
		lola_error("The restriction for this algorithm must be a convex polyeder",1405,1);
	restr_poly *restrict = (restr_poly*)tmp_restrict;
	int conv=1;
	if (!(restrict->convex())) conv=0;
	list<double> z_tmp;
	list<point> possible_sol;
	list<segment> edges = restrict->segments();
	for (list_item loc_tmp = facs.first(); loc_tmp; loc_tmp = facs.succ(loc_tmp)) {
		location loc = facs.contents(loc_tmp);
		for (list_item seg_tmp = edges.first(); seg_tmp; seg_tmp = edges.succ(seg_tmp)) {
			point proj_pnt;
			segment seg = edges.contents(seg_tmp);
			if (x_proj(loc, seg, proj_pnt)) {
				double dist = loc(0) * (loc - point2loc(proj_pnt)).norm(2);
				if (dist > z_opt_tmp) {
					z_tmp.append(dist);
					possible_sol.append(proj_pnt);
				}
			}
			if (y_proj(loc, seg, proj_pnt)) {
				double dist = loc(0) * (loc - point2loc(proj_pnt)).norm(2);
				if (dist > z_opt_tmp) {
					z_tmp.append(dist);
					possible_sol.append(proj_pnt);
				}
			} 
		}
	}
	for (list_item loc_tmp0 = facs.first(); loc_tmp0 != facs.last(); loc_tmp0 = facs.succ(loc_tmp0)) {
		location loc_k = facs.contents(loc_tmp0);
		for (list_item loc_tmp1 = facs.succ(loc_tmp0); loc_tmp1; loc_tmp1 = facs.succ(loc_tmp1)) {
			location loc_l = facs.contents(loc_tmp1);
			if (loc_k(0) == loc_l(0)) {
				line mid_line = mid_perpend(loc_k, loc_l);	
				list<point> intsecpnts = restrict->intersection(mid_line);
				if (intsecpnts.length() <= 2) {
					for (list_item trav = intsecpnts.first(); trav; trav = intsecpnts.succ(trav)) {
						location intsecloc = point2loc(intsecpnts.contents(trav));
						double dist = loc_l(0) * (loc_l-intsecloc).norm(2);
						if (dist > z_opt_tmp) {
							z_tmp.append(dist);
							point sol_pnt = intsecpnts.contents(trav);
							possible_sol.append(sol_pnt);
						}
					}
				}
			}
			else {
				double mu = loc_l(0)/loc_k(0);
				double c1 = (loc_k[0]-loc_l[0]*L_sqr(mu))/(1-L_sqr(mu));
				double c2 = (loc_k[1]-loc_l[1]*L_sqr(mu))/(1-L_sqr(mu));
				double c3 = (mu/fabs(1-L_sqr(mu)))*((loc_k-loc_l).norm(2));
				circle GL1(c1, c2, c3);
				list<point> intsecpnts = restrict->intersection(GL1);
				for (list_item trav = intsecpnts.first(); trav; trav = intsecpnts.succ(trav)) {
					location intsecloc = point2loc(intsecpnts.contents(trav));
					double dist = loc_l(0) * (loc_l-intsecloc).norm(2);
					if (dist > z_opt_tmp) {
						z_tmp.append(dist);
						point sol_pnt = intsecpnts.contents(trav);
						possible_sol.append(sol_pnt);
					}
				}
			}
		}
	}
	if (!conv) {
		list<point> vert = restrict->vertices();
		restr_poly C(CONVEX_HULL(vert));
		for (list_item ver_tmp = vert.first(); ver_tmp; ver_tmp = vert.succ(ver_tmp)) {
			point vertex = vert.contents(ver_tmp);
			if (C.inside(vertex)) {
				double max = 0;
				location Vertex= point2loc(vertex);
				for (list_item loc_tmp = facs.first(); loc_tmp; loc_tmp = facs.succ(loc_tmp)) {
					location loc = facs.contents(loc_tmp);
					double dist = loc(0) * (loc-Vertex).norm(2);
					if (max<dist) {
						max=dist;
					}
				}
				z_tmp.append(max);
				possible_sol.append(vertex);
			}
		}
	}
	double z = L_INF; int chk = 0;
	list_item pnt2sol = possible_sol.first();
	location final_sol = point2loc(possible_sol.head());
	for (list_item trav = z_tmp.first(); trav; trav = z_tmp.succ(trav)) {
		double z_temp = z_tmp.contents(trav);
		point sol = possible_sol.contents(pnt2sol);
		chk = 1;
		for (list_item loc_tmp = facs.first(); loc_tmp; loc_tmp = facs.succ(loc_tmp)) {
			location loc = facs.contents(loc_tmp);
			double dist = loc(0) * (loc - point2loc(sol)).norm(2);
			if (dist - z_temp > epsilon) chk = 0;
		}
		if (chk && z_temp < z) {
			z = z_temp;
			final_sol = point2loc(sol);
		}
		pnt2sol = possible_sol.succ(pnt2sol);
	}
	opt = final_sol;
	return z;
}

/******************* Subroutines for gauge_max **********************/

/*bestimmt die reflexen Ecken eines Polygons R*/

list<point> restr_poly::reflex (void)
{
	list<point> ListeRhilf;
	double det;
	point Ri_1,Ri_2,Ri_3;
	list<point> ListeR=this->vertices();
	for (list_item it=ListeR.first(); it; it=ListeR.succ(it))
	{
		Ri_2=ListeR.contents(it);                      //i te Ecke
		Ri_1=ListeR.contents(ListeR.cyclic_pred(it));//i-1 te Ecke
		Ri_3=ListeR.contents(ListeR.cyclic_succ(it));//i+1 te Ecke
		det=Ri_1.xcoord()*Ri_2.ycoord()+Ri_2.xcoord()*Ri_3.ycoord()+Ri_3.xcoord()*Ri_1.ycoord()-Ri_2.xcoord()*Ri_1.ycoord()-Ri_3.xcoord()*Ri_2.ycoord()-Ri_1.xcoord()*Ri_3.ycoord();      // Determinante ausrechnen
		if(det>0)   ListeRhilf.append(Ri_2); //positive Determinante heit reflexe Ecke
	}
	return ListeRhilf;
}/*reflex*/


double test2 (double& ztest, facilities& EX, list<int>&Ln, list<polygauge>& LG, restr_poly& R)
{
	polygon Poly;
	point tmp;

	list<point> LP;
	int in,gnumi;
	double test;
	list<point> L;
	polygon RPOL=R.vertices();
	location tmpex=EX.head();
	gnumi=Ln.contents(Ln.get_item(0));
	polygauge gaugei=LG.contents(LG.get_item(gnumi));
	list<point> L3=gaugei.vertices(origin);
	forall (tmp, L3)
	{
		double xE=(ztest-L_eps)/tmpex(0)*tmp.xcoord()+tmpex[0];
		double yE=(ztest-L_eps)/tmpex(0)*tmp.ycoord()+tmpex[1];
		point p(xE,yE);
		L.push(p);
	}
	polygon P(L);
	for (int i=1;i<EX.length();i++)
	{
		L.clear();
		tmpex=EX.contents(EX.get_item(i));
		gnumi=Ln.contents(Ln.get_item(i));
		gaugei=LG.contents(LG.get_item(gnumi));
		L3=gaugei.vertices(origin);
		forall (tmp, L3)          /*alle Ecken des Polygons*/
		{
			double xE=(ztest-L_eps)/tmpex(0)*tmp.xcoord()+tmpex[0];
			double yE=(ztest-L_eps)/tmpex(0)*tmp.ycoord()+tmpex[1];
			
			point p(xE,yE);
			L.push(p);     
		}
		polygon Q(L);
		polygon S=poly_intersection(P,Q);
		P=S;
	}
	if (!P.empty())          /*Niveaumenge nicht leer*/
	{
		polygon S=poly_intersection(P,RPOL);   /*Schnitt Niveaumenge, verbotenes Polygon*/
		if(!S.empty())
			Poly=S;        /*testen, ob Schnitt von Niveaumenge P und*/
		int tau=0;                       /*verbotenem Gebiet gleich der Niveaumenge*/
		point EP,EE;                 /*ist, wenn ja, ist die ganze Niveaumenge*/  
		forall (EP, Poly.vertices()) /*verboten und ztest unzulssig*/
		{
			for (in=0;in<P.size();in++)
			{
				LP=P.vertices();
				EE=LP.contents(LP.get_item(in));
				if (EP==EE)  tau=tau+1;
			}
		}
		if(tau==P.size())
		{
			test=0;
		}
		else
		{
			test=1;
		}
	}
	return test;
}


void restr_poly::megiddo(list<double>& niveauliste,facilities& EX, list<int>& Ln, list<polygauge>& LG)
{
	double ztest,tempe;
	int test;
	while (niveauliste.length()>1)
    {
		list<double> hilfliste;
		ztest=gauge_median(niveauliste);
		test=(int) test2 (ztest, EX, Ln, LG, *this);            //Entscheidungskriterium

		if (test==0)                                //ztest ist unzulssig
		{
			forall(tempe, niveauliste)
			{
				if (tempe>ztest)  hilfliste.append(tempe);
			}
			hilfliste.push(ztest);
		}
		else
		{
			forall(tempe, niveauliste)
			{
				if (tempe<ztest)  hilfliste.append(tempe);
			} 
			hilfliste.append(ztest);
		}
		
		if (hilfliste.length()==2)  //Bei Listenlnge 2 kann es passieren,
		{                        //da man in einer Endlosschleife landet
			if(hilfliste.head()!=ztest)     
			{  
				double kgb=hilfliste.head();
				test=(int) test2(kgb, EX, Ln, LG, *this); 
				if (test==0)  hilfliste.Pop();
				else   hilfliste.pop();
			}
			else
			{
				double kgb=hilfliste.tail();
				test=(int) test2(kgb, EX, Ln, LG, *this);
				if (test==0)  hilfliste.pop();  
				else   hilfliste.Pop();
			}	
			niveauliste=hilfliste;
		}
		else  niveauliste=hilfliste;
    }//while
}

int restr_poly::opt2(double& z, facilities& EX, list<int>& Ln, list<polygauge>& LG, list<location>& OPTIMUM)
{
	list<point> OPTLIST;
	point PP;
	location loc;
	int hilf3=0;
	list<point> Effi1;
	list<segment> RSEG,PSEG;
	segment PS;
	polygon RPOL=this->vertices();
	polygon P=EX.opt11 (z, Ln, LG);
	RSEG=RPOL.edges();
	forall (PP, P.vertices())
    {
		int optnum=0;
		for(int in=0;in<RPOL.size();in++)
		{
			PS=RSEG.contents(RSEG.get_item(in));
			segment RSEG1(PS.source(), PP);
			segment RSEG2(PP, PS.target());
			if ((PS.length()+L_eps)>=((RSEG1.length()+RSEG2.length()-L_eps)))  optnum=optnum+1;
		}
		if (optnum>0)  Effi1.append(PP);
    }
	PSEG=P.edges();
	list<point> ListeRhilf=this->reflex();
	forall (PP, ListeRhilf)              /*bei nicht konvexem R kann eine*/
    {                                  /*(reflexe) Ecke von R auf einer*/
		int optnum=0;                    /*Kante der Niveaumenge liegen*/  
		for(int in=0;in<P.size();in++)           /*und optimal sein*/
		{
			PS=PSEG.contents(PSEG.get_item(in));
			segment PSEG1(PS.source(), PP);
			segment PSEG2(PP, PS.target());
			if (PS.length()+L_eps>=(PSEG1.length()+PSEG2.length()-L_eps))  optnum=optnum+1;
		}
		if (optnum>0)  Effi1.append(PP);
    }
	OPTLIST=Effi1;
	if(OPTLIST.length()==0) 
	{
		hilf3=1;
	}
	else
    {
		EX.doppel(OPTLIST);
		OPTIMUM=EX.alg_solution();
		int zaehle=0;
		forall(loc, OPTIMUM)
		{
			zaehle=zaehle+1;
			loc(0)=zaehle;
		}       
    }
	return hilf3;
}/*opt2*/


double restr_poly::gauge_max(list<location> &opt, facilities &EX, double zmax, list<polygauge> &LG, list<int> &Ln)
/*****************************************************************************
* Purpose    : Compute the optimum for problem class 1|P|R=ConvexPolyeder|gauge|max
* Returns    : %
* Influences : %
* Date       : May 99
* Author     : Ralf Leipe (included by Holger Hennes)
*****************************************************************************/
{
	location tmpex;
	double a,b,d,e,g; 
	
	int schnell;           //beschrnkt die Niveauliste zunchst auf 500 Werte
	schnell=500;           //bei kleinen Problemenlohnt sich das nicht
	if (EX.length()<11)   schnell=5000;
	if (EX.length()>199)  schnell=1000;
	double zz;
	list<double> niveauliste;
	list<double> niveauhilfe;
	int i;
	int jnum,onum,tmpex1num,gnumi,gnumk,lnum;
	double z=0;
	for (i=0;i<EX.length();i++)                       //kritische Situation a'
	{
		tmpex=EX.contents(EX.get_item(i));
		gnumi=Ln.contents(Ln.get_item(i));
		polygauge gaugei=LG.contents(LG.get_item(gnumi));
		list<point> Listei=gaugei.vertices(origin);                  //Standort i, Ecke j
		for (jnum=0;jnum<Listei.length();jnum++)
		{
			point j=Listei.contents(Listei.get_item(jnum));
			list<point> ListeR=this->vertices();
			for (onum=0;onum<ListeR.length();onum++)   //Facette von o, m
			{                                      //des verbototenen Gebietes R
				point o=ListeR.contents(ListeR.get_item(onum));
				point m;
				if (o==ListeR.tail())  m=ListeR.head();
				else  m=ListeR.contents(ListeR.get_item(onum+1));
				if (o.xcoord()==m.xcoord())          //senkrechte Facette
				{
					if(j.xcoord()==0)  z=0;
					else  z=((o.xcoord()-tmpex[0])*tmpex(0)/j.xcoord());
				}                                //Ecke = Facette setzen
				else                                //sonstige Facette
				{ 
					double c1=j.xcoord()/tmpex(0);
					double c2=j.ycoord()/tmpex(0);
					double  d1=tmpex[0];
					double d2=tmpex[1];
					double Ra=(o.ycoord()-m.ycoord())/(o.xcoord()-m.xcoord());
					double Rb=o.ycoord()-Ra*o.xcoord();
					if(c2-Ra*c1==0)  z=0;
					else   z=(Ra*d1+Rb-d2)/(c2-Ra*c1);   //Ecke = Facette setzen
				}
				if(z>0)
				{
					if(z>=zmax)
					{
						if(niveauliste.length()<schnell)  niveauliste.push(z);
						else  niveauhilfe.push(z);
					}
				}
			}
		}
    }                
//kritische Situationen vom Typ b'
	for (i=0;i<EX.length()-1;i++)
    {                                                
		tmpex=EX.contents(EX.get_item(i));
		gnumi=Ln.contents(Ln.get_item(i));
		polygauge gaugei=LG.contents(LG.get_item(gnumi));
		list<point> Listei=gaugei.vertices(origin);
		for (jnum=0;jnum<Listei.length();jnum++)
		{
			point j=Listei.contents(Listei.get_item(jnum));
			for(tmpex1num=i+1;tmpex1num<EX.length();tmpex1num++)
			{
				location tmpex1=EX.contents(EX.get_item(tmpex1num)); //Kugel k fr Facette
				gnumk=Ln.contents(Ln.get_item(tmpex1num));
				polygauge gaugek=LG.contents(LG.get_item(gnumk));
				list<point> Listek=gaugek.vertices(origin);
				for(lnum=0;lnum<Listek.length();lnum++)
				{
					point l=Listek.contents(Listek.get_item(lnum));//Facette von l und l+1
					list<point> ListeR=this->vertices();
					for(onum=0;onum<ListeR.length();onum++)   //Facette von R
					{
						point o=ListeR.contents(ListeR.get_item(onum));
						point m,n3,o3;
						if (j==Listei.tail())  m=Listei.head();
						else  m=Listei.contents(Listei.get_item(jnum+1));
						if (l==Listek.tail())  n3=Listek.head();
						else  n3=Listek.contents(Listek.get_item(lnum+1));
						if (o==ListeR.tail())  o3=ListeR.head();
						else  o3=ListeR.contents(ListeR.get_item(onum+1));
						if (j.xcoord()==m.xcoord())
						{
							if (l.xcoord()!=n3.xcoord())
							{
								if (o.xcoord()==o3.xcoord()) 
								{
									z=0;
								}
								else
								{
									a=(l.ycoord()-n3.ycoord())/(l.xcoord()-n3.xcoord());
									b=tmpex1[1]-a*tmpex1[0];
									d=(l.ycoord()-a*l.xcoord())/tmpex1(0);
									double c1=j.xcoord()/tmpex(0);
									double c2=a*c1+d;  
									double d1=tmpex[0];
									double d2=a*tmpex[0]+b;
									double Ra=(o.ycoord()-o3.ycoord())/(o.xcoord()-o3.xcoord());
									double Rb=o.ycoord()-Ra*o.xcoord();		   
									if(c2-Ra*c1==0)   z=0;
									else   z=(Ra*d1+Rb-d2)/(c2-Ra*c1);  //eine senkrechte Facette fij 
								}
							}
						}
						else
						{
							if (l.xcoord()==n3.xcoord())
							{
								if (o.xcoord()==o3.xcoord())  z=0;
								else
								{
									a=(j.ycoord()-m.ycoord())/(j.xcoord()-m.xcoord());
									b=tmpex[1]-a*tmpex[0];
									d=(j.ycoord()-a*j.xcoord())/tmpex(0);
									double c1=l.xcoord()/tmpex1(0);
									double c2=a*c1+d;
									double d1=tmpex1[0];
									double d2=a*tmpex1[0]+b;
									double Ra=(o.ycoord()-o3.ycoord())/(o.xcoord()-o3.xcoord());
									double Rb=o.ycoord()-Ra*o.xcoord();
									if(c2-Ra*c1==0)  z=0;
									else  z=(Ra*d1+Rb-d2)/(c2-Ra*c1);    //andere Facette fkl senkrecht
								}
							}
							else
							{
								a=(j.ycoord()-m.ycoord())/(j.xcoord()-m.xcoord());
								b=tmpex[1]-a*tmpex[0];
								d=(j.ycoord()-a*j.xcoord())/tmpex(0);
								e=(l.ycoord()-n3.ycoord())/(l.xcoord()-n3.xcoord());
								double f1=tmpex1[1]-e*tmpex1[0];
								g=(l.ycoord()-e*l.xcoord())/tmpex1(0);
								if(a==e)  z=0;
								else
								{
									double c1=(g-d)/(a-e);
									double c2=a*c1+d;
									double d1=(f1-b)/(a-e);
									double d2=a*d1+b;
									if (o.xcoord()==o3.xcoord())
									{
										if (c1==0)  z=0;
										else  z=(o.xcoord()-d1)/c1;
									}
									else
									{
										double Ra=(o.ycoord()-o3.ycoord())/(o.xcoord()-o3.xcoord());
										double Rb=o.ycoord()-Ra*o.xcoord();			      
										if(c2-Ra*c1==0)  z=0;				   
										else  z=(Ra*d1+Rb-d2)/(c2-Ra*c1);   //keine Facette senkrecht
									}
								}
							}
						}
						if(z>0)
						{
							if(z>=zmax)
							{
								if(niveauliste.length()<schnell)  niveauliste.push(z);
								
								else  niveauhilfe.push(z);
							}
						}
					}
				}
			}
		}
    }
	//kritische Situationen CSc nur fr nicht konvexe verbotene Polygone
	list<point> ListeRhilf=this->reflex();
	for (i=0;i<EX.length();i++)
	{
		tmpex=EX.contents(EX.get_item(i));
		polygauge gaugei=LG.contents(LG.get_item(Ln.contents(Ln.get_item(i))));
		list<point> Listei=gaugei.vertices(origin);
		for (jnum=0;jnum<Listei.length();jnum++)
		{
			point j=Listei.contents(Listei.get_item(jnum));
			point l,m;
			forall(l, ListeRhilf)          //Ecke des verbotenen Gebietes
			{
				if (j==Listei.tail())  m=Listei.head();
				else  m=Listei.contents(Listei.get_item(jnum+1));
				if (j.xcoord()==m.xcoord())  z=(l.xcoord()-tmpex[0])/j.xcoord()*tmpex(0);
				else
				{
					a=(j.ycoord()-m.ycoord())/(j.xcoord()-m.xcoord());
					b=tmpex[1]-a*tmpex[0];
					d=(j.ycoord()-a*j.xcoord())/tmpex(0);
					z=(l.ycoord()-a*l.xcoord()-b)/d;		  
				}
				if(z>0)
				{
					if(z>=zmax)
					{
						if(niveauliste.length()<schnell)  niveauliste.push(z);
						else  niveauhilfe.push(z);
					}
				}
			}
		}
    }
	if (niveauliste.length()==2)  niveauliste.push(niveauliste.head());
	//Megiddo-Verfahren
	this->megiddo(niveauliste, EX, Ln, LG);
	z=niveauliste.head();
	int hilf3=this->opt2 (z, EX, Ln, LG, opt);
	if (schnell<4000)
    {
		if (hilf3==1)
		{
			niveauliste.clear();
			forall(zz, niveauhilfe)
			{
				if(zz>=zmax)
				{
					if(zz<=1.0e8)  niveauliste.push(zz);
				}
			}    
			if (niveauliste.length()==2)  niveauliste.push(niveauliste.head());
			//Megiddo-Verfahren
			megiddo(niveauliste,EX,Ln,LG);
			z=niveauliste.head(); 
			this->opt2 (z, EX, Ln, LG, opt);
		}    
	}
	return z;
}

bool restr_poly::operator == (const restr_poly &P1) {
	if (seg_list.size() != P1.seg_list.size()) return false;
	list_item trav1 = P1.seg_list.first();
	for (list_item trav = seg_list.first(); trav; trav = seg_list.succ(trav)) {
		if (seg_list.contents(trav) != P1.seg_list.contents(trav1)) return false;      
		trav1 = P1.seg_list.succ(trav);
	}
	return true;
}

ostream& operator << (ostream& out, restr_poly &p) {
	p.vertices().print(out);
	return out << endl;
} 

istream& operator >> (istream& in, restr_poly &p) {
	list<point> L; 
	L.read(in,'\n'); 
	p = restr_poly(L); 
	return in;
}
