#include <LOLA/restriction.h>


static restriction* restr_null = NULL;

bool inside_all(restriction* &restrict, list<location> &opt) {
   if (restrict == NULL) lola_error("Restriction is empty in inside_all",1412,1);
   location tmp_loc1(2,1);
   forall(tmp_loc1,opt)
        if (!restrict->inside(tmp_loc1)) return false;
   return true;
}


restriction* inside_which(list<restriction*> &konv, location &Loc) {
   restriction *test_restr = NULL;
   for (list_item trav = konv.first(); trav; trav = konv.succ(trav)) {
      test_restr = konv.contents(trav);
      if (test_restr->inside(Loc)) return test_restr;
   }
   return restr_null;
}

restriction* inside_which(list<restriction*> &konv, segment &opt_seg) {
   list<location> tmp_sol;
   location l1 = point2loc(opt_seg.source());
   location l2 = point2loc(opt_seg.target());
   tmp_sol.append(l1);
   tmp_sol.append(l2);
   restriction *test_restr = NULL;
   int cut = 0, restr_inside = 0;
   if (tmp_sol.length() == 2) {
      list<point> intsecpnt;
      for (list_item trav = konv.first(); trav; trav = konv.succ(trav)) {
	     test_restr = konv.contents(trav);
	     intsecpnt = test_restr->intersection(opt_seg);
	     if (test_restr->inside(opt_seg)) return test_restr;
	     if (intsecpnt.length() > 0) return test_restr;
      }
      if (!cut && !restr_inside) return restr_null;
   }
   return restr_null;
}

restriction* inside_which(list<restriction*> &konv, list<location> &Loc) {
   restriction *test_restr = NULL;
   for (list_item loc_trav = Loc.first(); loc_trav; loc_trav = Loc.succ(loc_trav)) {
      location test = Loc.contents(loc_trav);
      for (list_item trav = konv.first(); trav; trav = konv.succ(trav)) {
	     test_restr = konv.contents(trav);
	     if (test_restr->inside(test)) return test_restr;
      }
   }
   return restr_null;
}

double restriction::proj_v1_l2_max(location &opt, facilities &facs, double z_opt_tmp)
{
  location dummy_loc = opt; dummy_loc = facs.head()[0];
  lola_error("Not implemented for this kind of restriction.",1411,1);
  return z_opt_tmp;
}

double restriction::gauge_max(list<location> &opt, facilities &facs, double z_opt_tmp, list<polygauge>& LG, list<int> &Ln)
{
  location dummy_loc = opt.head(); dummy_loc = facs.head()[0];
  lola_error("Not implemented for this kind of restriction.",1411,1);
  return z_opt_tmp;
}


double restriction::proj_conv_linf_max(list<location> &solution, facilities &facs, double z_tmp) {
/*****************************************************************************
* Purpose   : Compute the optimum for problem class 1|P|R=convex|linf|max
* Returns   : %
* Influences: %
* Date      : Feb 96
* Author    : Ingo Bartling
******************************************************************************/
  int i;
  list<location> tmp_sol;
  tmp_sol.append(solution.head());
  tmp_sol.append(solution.tail());
  restriction *test_restr = this;
  int cut = 0, restr_inside = 0;
  if (tmp_sol.length() == 2) {
    location l1 = solution.head(), l2 = solution.tail();
    segment opt_seg(l1[0],l1[1],l2[0],l2[1]);
    list<point> intsecpnt;
    intsecpnt = test_restr->intersection(opt_seg);
    // *********  Opt_seg lies complete in test_restr?  ***********
    if (test_restr->inside(opt_seg)) restr_inside = 1;
    // *********  Opt_seg cuts test_restr?  ***********
    if (intsecpnt.length()) cut = 1;
    // *********  No cuts and lies outside of test_restr  ***********
    if (!cut && !restr_inside) return z_tmp;
    solution.clear();
    // ******  Opt_seg cuts test_restr with al least one point outside *******
    if (cut && !restr_inside) {
      if (intsecpnt.length() == 1) {
	solution.append(point2loc(intsecpnt.head()));
	if (test_restr->inside(opt_seg.source())) 
          solution.append(point2loc(opt_seg.target()));
	else solution.append(point2loc(opt_seg.source()));
	return z_tmp;
      }
      if (intsecpnt.length() == 2) {
	point2loc(opt_seg.source())(0) = 0;
	solution.append(point2loc(opt_seg.source()));
        if (opt_seg.source().distance(intsecpnt.tail()) > 
                                  opt_seg.source().distance(intsecpnt.head())) {
	  point2loc(intsecpnt.head())(0) = 0;
	  solution.append(point2loc(intsecpnt.head()));
	  point2loc(intsecpnt.tail())(0) = 1;
	  solution.append(point2loc(intsecpnt.tail()));
	  } else {
	    point2loc(intsecpnt.tail())(0) = 0;
	    solution.append(point2loc(intsecpnt.tail()));
	    point2loc(intsecpnt.head())(0) = 1;
	    solution.append(point2loc(intsecpnt.head()));
	  }
	  point2loc(opt_seg.target())(0) = 1;
	  solution.append(point2loc(opt_seg.target()));
	  return z_tmp;
        } 
      }
    }
    if (tmp_sol.length() == 1) {
      if (test_restr->inside(solution.head())) cut = 1;
      if (cut == 0) return z_tmp;
      tmp_sol.append(tmp_sol.head());
    }
    // *********  The projection-algorithm  ***********
    z_tmp = (int)(z_tmp+5);
    location tmp_P3 = facs.A_plus(z_tmp);
    location tmp_P1 = facs.A_minus(z_tmp);
    location up_opt = tmp_sol.tail(), low_opt = tmp_sol.head();
    if (up_opt.pos() < low_opt.pos()) { 
      low_opt = tmp_sol.tail(); up_opt = tmp_sol.head(); 
    }
    point upper_source = up_opt.loc2point();
    point lower_source = low_opt.loc2point();
    point P1 = tmp_P1.loc2point();
    point P3 = tmp_P3.loc2point();
    point P2(P1.xcoord(), P3.ycoord()), P4(P3.xcoord(), P1.ycoord());
    ray r1(lower_source,P1), r2(upper_source,P2), r3(upper_source,P3), 
        r4(lower_source,P4);
    if (low_opt[1] == up_opt[1]) { 
      r2 = ray(lower_source,P2); r4 = ray(upper_source,P4); 
    }
    point test_results[4]; point test_result; list<point> tmp_result;
    // *********  Calculating the intersectionpoints  ***********
    tmp_result = test_restr->intersect(r1,P1); 
    test_results[0] = tmp_result.head(); tmp_result.clear();
    tmp_result = test_restr->intersect(r2,P2); 
    test_results[1] = tmp_result.head(); tmp_result.clear();
    tmp_result = test_restr->intersect(r3,P3); 
    test_results[2] = tmp_result.head(); tmp_result.clear();
    tmp_result = test_restr->intersect(r4,P4); 
    test_results[3] = tmp_result.head(); tmp_result.clear();
    location tmp(2,1); location poss_results[4];
    for (i = 0; i < 4; i++) {
      tmp = point2loc(test_results[i]);
      poss_results[i] = tmp;
    }
    // *********  Calculating the objective functionvalues  ***********
    Loc_Vector tmp_z(2), tmp_z1(2), tmp_z2(2);
    for(list_item trav = facs.first(); trav ;trav = facs.succ(trav) ) {
      tmp = facs.contents(trav);
      tmp_z = (tmp.pos() - poss_results[0].pos()) * tmp(0); // A_minus
      if ( (tmp_z[0] >= tmp_z[1]) && (tmp_z[0] > poss_results[0](0)) )
	poss_results[0](0) = tmp_z[0];
      if ( (tmp_z[0] < tmp_z[1]) && (tmp_z[1] > poss_results[0](0)) )
	poss_results[0](0) = tmp_z[1];
      //****************************************************************
      tmp_z1 = (poss_results[1].pos() - tmp.pos()) * tmp(0); // A_plus
      tmp_z2 = (tmp.pos() - poss_results[1].pos()) * tmp(0); // A_minus
      tmp_z[0] = tmp_z2[0];  tmp_z[1] = tmp_z1[1];
      if ( (tmp_z[0] >= tmp_z[1]) && (tmp_z[0] > poss_results[1](0)) )
	poss_results[1](0) = tmp_z[0];
      if ( (tmp_z[0] < tmp_z[1]) && (tmp_z[1] > poss_results[1](0)) )
	poss_results[1](0) = tmp_z[1];
      //****************************************************************
      tmp_z = (poss_results[2].pos() - tmp.pos()) * tmp(0); // A_plus
      if ( (tmp_z[0] >= tmp_z[1]) && (tmp_z[0] > poss_results[2](0)) )
	poss_results[2](0) = tmp_z[0];
      if ( (tmp_z[0] < tmp_z[1]) && (tmp_z[1] > poss_results[2](0)) )
	poss_results[2](0) = tmp_z[1];
      //****************************************************************
      tmp_z1 = (poss_results[3].pos() - tmp.pos()) * tmp(0); // A_plus
      tmp_z2 = (tmp.pos() - poss_results[3].pos()) * tmp(0); // A_minus
      tmp_z[0] = tmp_z1[0];  tmp_z[1] = tmp_z2[1];
      if ( (tmp_z[0] >= tmp_z[1]) && (tmp_z[0] > poss_results[3](0)) )
	poss_results[3](0) = tmp_z[0];
      if ( (tmp_z[0] < tmp_z[1]) && (tmp_z[1] > poss_results[3](0)) )
	poss_results[3](0) = tmp_z[1];
    }
    // *********  Choosing the lowest functionvalue  ***********
    double min_z = poss_results[0](0);
    poss_results[0](0) = 0; solution.append(poss_results[0]);
    // *********  Building the solution  ***********
    for (i = 1; i < 4; i++) {
      if (fabs(poss_results[i](0) - min_z) < L_eps) {
	poss_results[i](0) = i; solution.append(poss_results[i]);
      }
      else {
	if ((min_z - poss_results[i](0)) > L_eps) {
	  min_z = poss_results[i](0); poss_results[i](0) = i;
	  solution.clear(); solution.append(poss_results[i]);
        }
        else poss_results[i](0) = i; 
      }
  }   
  return min_z;
}

list<point> restriction::cutGauge(polygauge &G) {
	list<point> gaugepnts = G.vertices(), result;
	point origin = G.origin(), currpnt;
	forall (currpnt,gaugepnts) {
		ray gaugeray(origin,currpnt);
		list<point> cutpnts = this->intersect(gaugeray,currpnt);
		if (cutpnts.length() == 2) {
			location cutLoc1 = point2loc(cutpnts.head()),
				cutLoc2 = point2loc(cutpnts.tail()),
				originLoc = point2loc(origin);
			if ((cutLoc1-originLoc).norm(2) > (cutLoc2-originLoc).norm(2))
			  cutpnts.pop();
			else cutpnts.Pop();
		}
		result.conc(cutpnts);
	}
	return result;
}
