#include <LOLA/restr_poly.h>
#include <LOLA/restr_rect.h>
#include <LOLA/restr_circle.h>
#include <LOLA/restrictions.h>
//#include <LOLA/lines.h>

void restrictions::WriteOpt() {
   this->WriteOpt(cout);
}

void restrictions::WriteOpt(ostream& out) {
  location::WriteAsPolygonList(out, solution);
}

double restrictions::l1_sum(facilities &EX,bool erase)
/********************************************************************
* Purpose   : Compute the optimum for problem class 1|P|R|l1|sum
*		(Konstruktionslinienalgorithmus)
* Returns   : objective values(double)
* Influences: list<location> solution
* Date      : May 96
* Author    : Lothar Schnell
*********************************************************************/
{
   int objind=0;
   bool todo=false;
   restrictions   R(*this);
   double z = EX.l1_sum(0,true);
   double znew;
   point p;
   bool flag=true;
   list_item Kon_it;
   list<point> Konstr,Konstr_tmp;
   list<location> Kandi;
   location  tmp_loc1(2,1),tmp_loc2(2,1);
   list<location> tmp_sol = EX.alg_solution();
   solution = tmp_sol;
   restriction *chk = inside_which(R, tmp_sol);
   restriction *iochk = R.head();
   if (chk != NULL) {
     if (iochk->inout() || (!inside_all(chk, tmp_sol) && !iochk->inout()))
       todo = true;
   }
   if (todo || ((chk == NULL) && !iochk->inout())) {
     z = L_INF;
     Kandi.clear();
     forall(tmp_loc1,EX) {
        line l1(tmp_loc1.loc2point(),0);

        if (chk == NULL) {
          Konstr_tmp.clear();
          for(list_item r_it=R.first(); r_it; r_it=R.succ(r_it)) {
            iochk=R.contents(r_it);
            Konstr_tmp = iochk->intersection(l1);
            for(list_item it=Konstr_tmp.first(); it; it=Konstr_tmp.succ(it)) {
              p=Konstr_tmp.contents(it);
              Konstr.append(p);
            }
          }
        } else Konstr=chk->intersection(l1);
        for(Kon_it=Konstr.first();Kon_it;Kon_it=Konstr.succ(Kon_it)) {
          p = Konstr.contents(Kon_it);
          Kandi.append(point2loc(p));
        }
        line l2(tmp_loc1.loc2point(),LEDA_PI/2);

        if (chk == NULL) {
          Konstr_tmp.clear();
          for(list_item r_it=R.first(); r_it; r_it=R.succ(r_it)) {
            iochk=R.contents(r_it);
            Konstr_tmp = iochk->intersection(l2);
            for(list_item it=Konstr_tmp.first(); it; it=Konstr_tmp.succ(it)) {
              p=Konstr_tmp.contents(it);
              Konstr.append(p);
            }
          }
        } else Konstr=chk->intersection(l2);
        for(Kon_it=Konstr.first();Kon_it;Kon_it=Konstr.succ(Kon_it)) {
          p = Konstr.contents(Kon_it);
          Kandi.append(point2loc(p));
        }
     } //end forall
     if ((chk != NULL) && (chk->subtype() == restr_p)) {
        restr_poly *polytemp = NULL;
        polytemp=(restr_poly*)(chk);
        if (!polytemp->convex()) {
          list<point> Polyvert=polytemp->vertices();
          for(list_item it=Polyvert.first();it;it=Polyvert.succ(it)) {
            p = Polyvert.contents(it);
            tmp_loc2 = point2loc(p);
            Kandi.append(tmp_loc2);
          }
        }
     } else if (chk == NULL) {
       list<point> Vert;
       for(list_item r_it=R.first(); r_it; r_it=R.succ(r_it)) {
         iochk=R.contents(r_it);
         if (iochk->subtype() == restr_p){
           restr_poly *polytemp = NULL;
           polytemp=(restr_poly*)(iochk);
           Vert=polytemp->vertices();
         } else if (iochk->subtype() == restr_r){
           restr_rect *recttemp = NULL;
           recttemp=(restr_rect*)(iochk);
           Vert=recttemp->vertices();
         } else Vert.clear();
         for(list_item it=Vert.first();it;it=Vert.succ(it)) {
           p = Vert.contents(it);
           tmp_loc2 = point2loc(p);
           Kandi.append(tmp_loc2);
         }
       }
     }
     for(list_item Kan_it=Kandi.first();Kan_it;Kan_it=Kandi.succ(Kan_it)) {
       tmp_loc2 = Kandi.contents(Kan_it);
       tmp_loc2(0)=objind;
       znew = EX.median_objective_comp(1,tmp_loc2,0);
       if (znew < z) {
         objind = 0;
         z = znew;
         solution.clear();
         tmp_loc2(0)=objind;
         solution.append(tmp_loc2);
       } else if (znew == z) {
         flag=true;
         for(list_item s_it=solution.first();s_it;s_it=solution.succ(s_it)) {
           tmp_loc1=solution.contents(s_it);
           if (tmp_loc1.loc2point()==tmp_loc2.loc2point()) { 
             flag=false;
             break;
           }
         }
         if (flag) { 
           ++objind;
           tmp_loc2(0) = objind;
           solution.append(tmp_loc2);
         }
       }
     }
   }// end if
   return z;
}

double restrictions::linf_sum(facilities &EX,bool erase)
/********************************************************************
* Purpose   : Compute the optimum for problem class 1|P|R|linf|sum
*		(Konstruktionslinienalgorithmus)
* Returns   : objective values(double)
* Influences: list<location> solution
* Date      : May 96
* Author    : Lothar Schnell
*********************************************************************/
{
   int objind=0;
   restrictions   R(*this);
   bool todo = false;
   double z = EX.linf_sum(0,true);
   double znew;
   point p;
   bool flag=true;
   list_item Kon_it;
   list<point> Konstr,Konstr_tmp;
   list<location> Kandi;
   location  tmp_loc1(2,1),tmp_loc2(2,1);
   list<location> tmp_sol = EX.alg_solution();
   solution = tmp_sol;
   restriction *chk = inside_which(R, tmp_sol);
   restriction *iochk = R.head();
   if (chk != NULL) {
     if (iochk->inout() || (!inside_all(chk, tmp_sol) && !iochk->inout()))
       todo = true;
   }
   if (todo || ((chk == NULL) && !iochk->inout())) {
     z = L_INF;
     Kandi.clear();
     forall(tmp_loc1,EX) {
        line l1(tmp_loc1.loc2point(),LEDA_PI/4);

        if (chk == NULL) {
          Konstr_tmp.clear();
          for(list_item r_it=R.first(); r_it; r_it=R.succ(r_it)) {
            iochk=R.contents(r_it);
            Konstr_tmp = iochk->intersection(l1);
            for(list_item it=Konstr_tmp.first(); it; it=Konstr_tmp.succ(it)) {
              p=Konstr_tmp.contents(it);
              Konstr.append(p);
            }
          }
        } else Konstr=chk->intersection(l1);
        for(Kon_it=Konstr.first();Kon_it;Kon_it=Konstr.succ(Kon_it)) {
          p = Konstr.contents(Kon_it);
          Kandi.append(point2loc(p));
        }
        line l2(tmp_loc1.loc2point(),3*LEDA_PI/4);

        if (chk == NULL) {
          Konstr_tmp.clear();
          for(list_item r_it=R.first(); r_it; r_it=R.succ(r_it)) {
            iochk=R.contents(r_it);
            Konstr_tmp = iochk->intersection(l2);
            for(list_item it=Konstr_tmp.first(); it; it=Konstr_tmp.succ(it)) {
              p=Konstr_tmp.contents(it);
              Konstr.append(p);
            }
          }
        } else Konstr=chk->intersection(l2);
        for(Kon_it=Konstr.first();Kon_it;Kon_it=Konstr.succ(Kon_it)) {
          p = Konstr.contents(Kon_it);
          Kandi.append(point2loc(p));
        }
     }
     if ((chk != NULL) && (chk->subtype() == restr_p)) {
        restr_poly *polytemp = NULL;
        polytemp=(restr_poly*)(chk);
        if (!polytemp->convex()) {
          list<point> Polyvert=polytemp->vertices();
          for(list_item it=Polyvert.first();it;it=Polyvert.succ(it)) {
            p = Polyvert.contents(it);
            tmp_loc2 = point2loc(p);
            Kandi.append(tmp_loc2);
          } //end for
        } //end if
     } else if (chk == NULL) {
       list<point> Vert;
       for(list_item r_it=R.first(); r_it; r_it=R.succ(r_it)) {
         iochk=R.contents(r_it);
         if (iochk->subtype() == restr_p){
           restr_poly *polytemp = NULL;
           polytemp=(restr_poly*)(iochk);
           Vert=polytemp->vertices();
         } else if (iochk->subtype() == restr_r){
           restr_rect *recttemp = NULL;
           recttemp=(restr_rect*)(iochk);
           Vert=recttemp->vertices();
         } else Vert.clear();
         for(list_item it=Vert.first();it;it=Vert.succ(it)) {
           p = Vert.contents(it);
           tmp_loc2 = point2loc(p);
           Kandi.append(tmp_loc2);
         }
       }
     }
     for(list_item Kan_it=Kandi.first();Kan_it;Kan_it=Kandi.succ(Kan_it)) {
       tmp_loc2 = Kandi.contents(Kan_it);
       tmp_loc2(0)=objind;
       znew = EX.median_objective_comp(L_inf,tmp_loc2,0);
       if (znew < z) {
         objind = 0;
         z = znew;
         solution.clear();
         tmp_loc2(0)=objind;
         solution.append(tmp_loc2);
       } else if (znew == z) {
         flag=true;
         for(list_item s_it=solution.first();s_it;s_it=solution.succ(s_it)) {
           tmp_loc1=solution.contents(s_it);
           if (tmp_loc1.loc2point()==tmp_loc2.loc2point()) {
             flag=false;
             break;
           }
         }
         if (flag) {
           ++objind;
           tmp_loc2(0) = objind;
           solution.append(tmp_loc2);
         }
       } //end if
     } //end for Kan_it
   } //end if
   return z;
}

double barrier::l2_sum(facilities& EX,bool erase,int choose, double percent,double accel, double init_ss, double epsilon)
/********************************************************************
* Purpose    : Compute the optimum for problem class 1|P|(R,oo)|l2_sum
* Returns    : objective values(double)
* Influences : list<location> solution
* Date       : July 1998
* Author     : Michael Ochs 
* Last Update: Dec 1998 (Holger Hennes)
*********************************************************************/
{
	barrier R(*this);
	location temp(2,1);
	list_item trav=NULL;
	if (R.length() > 2) 
		lola_message("This algorithm can only handle one barrier.",0);
	if (R.length() == 0) {
		lola_message("No restrictions given. Executing standard algorithm.",0);
	}
	barrier *B=&R;

	restriction *restr=(R.head());
	if (B->type() != t_barrier) lola_error("Restriction is not a barrier.",1801,1);
	bool OK=true;
	for(trav=EX.first();trav;trav=EX.succ(trav)) {
		if (restr->inside(EX.contents(trav)))
			OK=false;
	}
	if (!OK) lola_error("No locations allowed inside of barriers.",1802,1);

	SBO problem(R,&EX);

	switch (choose) {
		case 0 : problem.l2_sum_h1(accel,init_ss,epsilon);
		case 1 : problem.l2_sum_h2(accel,init_ss,epsilon,percent);
	}
	point sol=problem.optimum();
	double obj=problem.objective();
	temp[0]=sol.xcoord();
	temp[1]=sol.ycoord();
	temp(0)=0.0;
	solution.append(temp);
	return obj;
}

/* functions for lp_sum */

const double  EQL=1.0e-4;
const double  TOL=1.0e-5;
const double  h1=0.0000001;
const double  h2=-0.0000001;
const double epsilon=0.0000001;
static int AnzLocation;
static int AnzForbidden;
static int zaehler;
static double lpMetrik;
static point* Region;
static location* Work;
static facilities EX;

static double gerade(int eck1,int eck2,double x)         /*Geradengleichung*/
{
   double m,b,g;
   int zaehlen=0;


   if ( fabs(Region[eck1].ycoord()-Region[eck2].ycoord()) < epsilon) zaehlen=1;
   if ( fabs(Region[eck1].xcoord()-Region[eck2].xcoord()) < epsilon) zaehlen=2;

   switch (zaehlen) {	
   case 1:
	   return Region[eck1].ycoord();                 /*y ist konstant*/
	   break;
   case 2:                                    /*x ist konstant*/
	   return Region[eck1].xcoord();
	   break;
   default:
	   m=(Region[eck1].ycoord()-Region[eck2].ycoord())/(Region[eck1].xcoord()-Region[eck2].xcoord());
	   b=Region[eck1].ycoord()-m*Region[eck1].xcoord();
	   g=m*x+b;
	   return g;
   } /*Ende switch*/

}   /*ENDE gerade(x)*/


static double gabl(int eck1,int eck2)         /*Ableitung von g*/
{
    int zaehlen=0;
    double m;

    if ( fabs(Region[eck1].ycoord()-Region[eck2].ycoord()) < epsilon) zaehlen=1;
    if ( fabs(Region[eck1].xcoord()-Region[eck2].xcoord()) < epsilon) zaehlen=2;

    switch (zaehlen) {
      case 1:
        return 0.00;
        break;
      case 2:
        return 0.00;
        break;
     default: 
          m=(Region[eck1].ycoord()-Region[eck2].ycoord())/(Region[eck1].xcoord()-Region[eck2].xcoord());
          return m;
     } /*Ende switch*/

}   /*ENDE gabl(x)*/



static double funkf(int eck1, int eck2,double x,double lpMetrik)
{
 double f,temp;
 location loctmp(2,1);

 temp = 0;

 if ( fabs(Region[eck1].xcoord()-Region[eck2].xcoord()) < epsilon) {
    for (int i=1;i<= AnzLocation ;i++ ) {
       loctmp=Work[i];
       temp+=(loctmp(0)*(pow((pow(fabs(loctmp[0]-Region[eck1].xcoord()),lpMetrik) + pow(fabs(loctmp[1]-x),lpMetrik)),(1/lpMetrik))));
    }
    f = temp;
 } else {     
    for (int i=1;i<= AnzLocation ;++i ) {
       loctmp=Work[i];
       temp+=(loctmp(0)*(pow((pow(fabs(loctmp[0]-x),lpMetrik) + pow(fabs(loctmp[1]-gerade(eck1,eck2,x)),lpMetrik)),(1/lpMetrik))));
    }
   f = temp;
 }

  return f;
} /*ENDE funkf*/

static double richtabl(int eck1,int eck2, double h,double u1,double u2)     /*Richtungsableitung von f*/
{
   double temp,argument1,argument2;
 
   if (fabs(Region[eck1].xcoord()-Region[eck2].xcoord()) < epsilon) {
      argument1=Region[eck1].ycoord()+h*u2;
      argument2=Region[eck1].ycoord();
   } else {
      argument1=Region[eck1].xcoord()+h*u1;
      argument2=Region[eck1].xcoord();
   }

   temp=(funkf(eck1,eck2,argument1,lpMetrik)-funkf(eck1,eck2,argument2,lpMetrik))/h;

   return temp;
}



/*rechnet Optimum der Kante k aus*/
static double CheckOpt(int eck1,int eck2,int gold,int zaehler)  /*k ist Kante (k,k+1), die zum unbeschr. Opt den*/
{                                                                    /* kuerzesten Abstand hat*/
   
   double Dposk,Dnegj,TempOpt=0,funkTOpt=0,low,u1 ,u2;

   if(zaehler == 1) {
        u1=Region[eck2].xcoord()-Region[eck1].xcoord();  
        u2=Region[eck2].ycoord()-Region[eck1].ycoord();
   } else {
       u1=Region[eck1].xcoord()-Region[eck2].xcoord();  
       u2=Region[eck1].ycoord()-Region[eck2].ycoord();
   }
 
   Dposk = richtabl(eck1,eck2,h1,u1,u2);                /*positive Richtungsableitung in Ecke k*/
   Dnegj = richtabl(eck2,eck1,h2,u1,u2);                 /*negative Richtungsableitung in Ecke j*/


                  /*gold=1 bedeutet Goldene Schnittsuche*/

   if (Dposk >= 0) {  
     TempOpt=Region[eck1].xcoord();                 /*Ecke k ist Optimum*/
     funkTOpt=funkf(eck1,eck2,TempOpt,lpMetrik);
   }

   if (Dnegj <= 0 ) {   
     TempOpt=Region[eck2].xcoord();                   /*Ecke j ist Optimum*/
     funkTOpt=funkf(eck1,eck2,TempOpt,lpMetrik);
   }


   if ((Dposk<0) && (Dnegj>0)) {
     if (gold == 1)
       TempOpt=GoldenSchnitt(eck1,eck2);     /*Optimum wird in der Mitte der Kante gesucht*/
     else {
       low=LowBound(eck1,eck2,zaehler);      /*es wird untere Schranke ausgerechnet*/
       funkTOpt=low;
     }   /*gold=0*/
 
   } /*if*/

if (gold == 1) return TempOpt;
else return funkTOpt; 

} /*ENDE CheckOpt*/


static int umrechnung(int eck)
{
  int p,i;
  if(eck == 1) i=1;
  else {
     p=eck-2;
     i=AnzForbidden-p;
  }
  return i;
} /*Ende umrechnung()*/

static double funkgf(int eck1,int eck2,double x)    /*Ableitung von Zielfunktion f*/
{
 double gfx;
 double temp;
 location loctmp(2,1);

 temp=0;

 if ( fabs(Region[eck1].xcoord()-Region[eck2].xcoord()) < epsilon) {
    for (int i=1;i<= AnzLocation ;++i ) {
        loctmp=Work[i];
        temp+=(loctmp(0)*(((x-loctmp[1])+(loctmp[0]-gerade(eck1,eck2,x))*(-gabl(eck1,eck2))) / (sqrt(pow((loctmp[0]-gerade(eck1,eck2,x)),2.0)+pow((loctmp[1]-x),2.0)))));
    }
 } else {
    for (int i=1;i<= AnzLocation ;++i ) {
        loctmp=Work[i];
       temp+=(loctmp(0)*(((x-loctmp[0])+(loctmp[1]-gerade(eck1,eck2,x))*(-gabl(eck1,eck2))) / (sqrt(pow((loctmp[0]-x),2.0)+pow((loctmp[1]-gerade(eck1,eck2,x)),2.0)))));
    }
 }
 gfx = temp;
 return gfx;
}


static int PolySearch(int eck1,double optx,double opty)   /*eck1 ist Ecke von Ki0 */
{
 int i,n,eck2;
 double gRegEck2x,gEck1ioptx;
 n=0;
 

 i = (int)fmod((eck1 + 2),AnzForbidden);
 eck2 = (int)fmod((eck1 + 1),AnzForbidden);

 if (i == 0) i= AnzForbidden;

 if (eck2 == 0) eck2 = AnzForbidden;

  do
  {
   if (fabs(Region[eck1].xcoord()-Region[i].xcoord()) < 0.0000001)
    {  
     if(!(fabs(optx-Region[i].xcoord())<0.0000001))
     {
      
      if (Region[eck2].xcoord() < Region[i].xcoord())
	 if(optx < Region[i].xcoord()) n=i;
       if(Region[eck2].xcoord() > Region[i].xcoord())
	 if(optx > Region[i].xcoord()) n=i;
     }
    }/*ENDE if*/

   else {
    gRegEck2x = gerade(eck1,i,Region[eck2].xcoord());
    gEck1ioptx = gerade(eck1,i,optx);

    if(!(fabs(opty-gEck1ioptx) < 0.0000001))
     {
       if (Region[eck2].ycoord() > gRegEck2x)
           if (opty > gEck1ioptx) n=i;
       if (Region[eck2].ycoord() < gRegEck2x)
           if (opty < gEck1ioptx) n=i;
     } /*if */
   } /*ENDE else*/
  
 
 i= (int)fmod((i+1),AnzForbidden);
 
 if(i == 0) i = AnzForbidden;
 
 } while(n == 0); /*ENDE do*/
  
   return n;

} /*ENDE */


static int abstand(location& Opt)
{ 
  int i,n,minstrecke;
  int zahl;
  double *laenge;
  laenge = new double[AnzForbidden+1];
  double verbxkl,verbxgr,verbykl,verbygr,sx,laenge1,laenge2;
  double minlaenge,sy,richtvx,richtvy,diffx,diffy,faktor;

  for (i=1;i<= AnzForbidden ;i++ ) {
        if ((i+1) > AnzForbidden) zahl=1;
        else {zahl=i+1;}
        richtvx=(Region[i].xcoord()-Region[zahl].xcoord());
        richtvy=(Region[i].ycoord()-Region[zahl].ycoord());
        diffx=Opt[0]-Region[i].xcoord();
        diffy=Opt[1]-Region[i].ycoord();
        faktor=(richtvx*diffx+richtvy*diffy)/(pow(richtvx,2.0)+pow(richtvy,2.0));
        sx=Region[i].xcoord()+faktor*richtvx;
        sy=Region[i].ycoord()+faktor*richtvy;          

       verbxkl=L_MIN(Region[i].xcoord(),Region[zahl].xcoord()); 
       verbxgr=L_MAX(Region[i].xcoord(),Region[zahl].xcoord());
       verbykl=L_MIN(Region[i].ycoord(),Region[zahl].ycoord());
       verbygr=L_MAX(Region[i].ycoord(),Region[zahl].ycoord());

       if ((sx<verbxkl)||(sx>verbxgr)||(sy<verbykl)||(sy>verbygr)) {
         laenge1=sqrt(pow((Opt[0]-Region[i].xcoord()),2.0)+pow((Opt[1]-Region[i].ycoord()),2.0));
         laenge2=sqrt(pow((Opt[0]-Region[zahl].xcoord()),2.0)+pow((Opt[1]-Region[zahl].ycoord()),2.0));
    
         laenge[i]=L_MIN(laenge1,laenge2);
   
       } else laenge[i]=sqrt(pow((Opt[0]-sx),2.0)+pow((Opt[1]-sy),2.0));

 } /*end for*/


minlaenge=L_MIN(laenge[1],laenge[2]);


for (i=3;i<=AnzForbidden ;i++) minlaenge=L_MIN(minlaenge,laenge[i]);

 n=0;
 minstrecke = 0;
 i=1;
 do {
  if (minlaenge == laenge[i]) {
     n=i;
     minstrecke=1;
  }
  i++;
 }
 while (minstrecke == 0 ); 

 return n;
} /*ENDE abstand*/

double restrictions::lp_sum(facilities &EX,int lp,double eps,double delta,int iter_max,bool erase)
/********************************************************************
* Purpose    : Compute the optimum for problem class 1|P|R|lp|sum
* Returns    : objective values(double)
* Influences : list<location> solution
* Date       : Jun 96
* Author     : Annette Brockmann (Updated: Lothar Schnell)
* Last Update: Nov 98 (Holger Hennes)
*********************************************************************/
{
	location loctmp(2,1);

	double ymini=0,xmini=0,fxmini=0,funkLocOpt=0,funkLocOptx,funkLocOpty,Niveau;
	int Kante,Kante1,eck1=0,eck2,testEckBisPol,EckBisPol,RausEck,g,gplus1,i,q,z,l,p=0,p1=0,p2=0,pz1,pz2;
	int zwischen,I,K;
	double xgol,fxgol;

	restrictions   R(*this);
	double Z = EX.lp_sum(lp,eps,delta,iter_max,0,true);
	lpMetrik = lp;
	solution = EX.alg_solution();
	location tmp_sol = solution.head();
	restriction *chk = inside_which(R, tmp_sol);
	if (chk != NULL) {
		list<point> Vert;
		if (chk->subtype() == restr_p) {
			restr_poly *polytemp = NULL;
			polytemp=(restr_poly*)(chk);
			Vert=polytemp->vertices();
		} else if (chk->subtype() == restr_r){
			restr_rect *recttemp = NULL;
			recttemp=(restr_rect*)(chk);
			Vert=recttemp->vertices();
		}
	AnzLocation = EX.size();
	AnzForbidden = Vert.size();
	int RegDim = AnzForbidden+1;

/************************************/
	Work=new location[AnzLocation+1];
	Region=new point[RegDim];
	point *ptmp;
	ptmp = new point[RegDim];
	int *End, *Eckz1, *Eckz2;
	End = new int[RegDim];
	Eckz1 = new int[RegDim];
	Eckz2 = new int[RegDim];
	int AnfEck[3],EndEck[3];


	i = 1;
	list_item it;
	for (it = EX.first(); it; it = EX.succ(it), ++i)
		Work[i] = EX[it];

	i = 1;
	for (it = Vert.first(); it; it = Vert.succ(it), i++)
		Region[i] = point(Vert[it].xcoord(),Vert[it].ycoord());

	Kante1=abstand(tmp_sol);     /*1 ist erstes Optimum; Kante k hat Ecken k und k+1*/
	Kante=Kante1; 
	for(zaehler=1;zaehler<=2;zaehler++) {
		if (zaehler ==2) {
			for(i=2;i<=AnzForbidden;i++) {
				ptmp[i] = point(Region[i].xcoord(),Region[i].ycoord());
			}
			p=0;
			double regx,regy;
			for(i=2;i<=AnzForbidden;++i) { 
				regx = ptmp[AnzForbidden-p].xcoord();
				regy = ptmp[AnzForbidden-p].ycoord();
				Region[i] = point(regx,regy);
				p++;
			}
			Kante=umrechnung(Kante);
			Kante=(int)fmod((Kante-1+AnzForbidden),AnzForbidden);
			if (Kante==0) Kante=AnzForbidden;
		} /*if zaehler*/
		eck1=Kante; 
		eck2=(int)fmod((Kante + 1),AnzForbidden);
		if (eck2 == 0) eck2 = AnzForbidden;
		if (zaehler == 1) {  
			if ( fabs(Region[eck1].xcoord()-Region[eck2].xcoord()) < epsilon) { 
				funkLocOpty=CheckOpt(eck1,eck2,1,zaehler);          /* mit golden; d.h. Ergebnis ist x-Wert*/
				funkLocOptx=Region[eck1].xcoord();
				funkLocOpt=funkf(eck1,eck2,funkLocOpty,lpMetrik);
			} else {
				funkLocOptx=CheckOpt(eck1,eck2,1,zaehler);          /* mit golden; d.h. Ergebnis ist x-Wert*/
				funkLocOpty=gerade(eck1,eck2,funkLocOptx);
				funkLocOpt=funkf(eck1,eck2,funkLocOptx,lpMetrik);
			}
			xmini=funkLocOptx;
			ymini=funkLocOpty;
			fxmini=funkLocOpt;
		}  //if zaehler
		EckBisPol=PolySearch(eck1,tmp_sol[0],tmp_sol[1]);          /*Das unbeschr. Optimum liegt in Region */
		testEckBisPol = (int)fmod((EckBisPol + 1),AnzForbidden);  
		if (testEckBisPol == 0) testEckBisPol = AnzForbidden;
     /* * * */
		if (testEckBisPol != eck1) {           /*das unbeschr. Opt liegt nicht im ganzen Gebiet */
			do {
				Niveau=CheckOpt(eck1,EckBisPol,0,zaehler);         /*ohne golden; d.h. Ergebnis ist Funktionswert*/
				EckBisPol = (int)fmod((EckBisPol + 1),AnzForbidden);  /*   EckBisPol++;*/
				if (EckBisPol == 0) EckBisPol = AnzForbidden;
			} while (Niveau<funkLocOpt); /* enddo */


			RausEck = (int)fmod((EckBisPol - 2+AnzForbidden),AnzForbidden);        /*EckBisPol-1 ist letzte Ecke des Polyeders,*/
			if(RausEck == 0) RausEck = AnzForbidden;
			if(eck1 == 0) eck1 = AnzForbidden;
			AnfEck[zaehler]=eck1;
			EndEck[zaehler]=RausEck;
         
			if(zaehler==1) {

				if(EndEck[zaehler] > AnfEck[zaehler]) {
					zwischen = EndEck[zaehler] - AnfEck[zaehler] + 1;
					I = AnzForbidden - zwischen;
				}
				if(EndEck[zaehler] < AnfEck[zaehler]) {
					zwischen = EndEck[zaehler] + AnzForbidden;
					I = zwischen - AnfEck[zaehler] +1;
				}
				if(fabs(EndEck[zaehler] - AnfEck[zaehler]) < 0.000000001) I = 0;

			} //Ende if(zaehler==1)
			if(zaehler==2) {
				AnfEck[1]=umrechnung(EndEck[1]);
				AnfEck[1]=(int)fmod((AnfEck[1]-1+AnzForbidden),AnzForbidden);
				if(AnfEck[1]==0) AnfEck[1]=AnzForbidden;
				EndEck[1]=umrechnung(AnfEck[zaehler]);
				EndEck[1]=(int)fmod((EndEck[1]-1+AnzForbidden),AnzForbidden);
				if(EndEck[1]==0) EndEck[1]=AnzForbidden;
			} /*if zaehler*/
			if (AnfEck[zaehler] > EndEck[zaehler]) EndEck[zaehler]=EndEck[zaehler]+AnzForbidden;
   
			q=EndEck[zaehler]-AnfEck[zaehler]+1;    /* q Kanten fuer LineSearch */
			pz1=0;
			pz2=0;
			z=1;
			do {
				if (zaehler==2) {
					Eckz1[z]=AnfEck[1];
					pz1++;
					p1=pz1;
					Eckz2[z]=AnfEck[2];
					pz2++;
					p2=pz2;
				}
				AnfEck[1]=(int)fmod((AnfEck[1]+1),AnzForbidden);
				if (AnfEck[1]==0) AnfEck[1]=AnzForbidden;

				AnfEck[2]=(int)fmod((AnfEck[2]+1),AnzForbidden);
				if (AnfEck[2]==0) AnfEck[2]=AnzForbidden;
				z++;
			} while (z<=q);
			if (zaehler==2) { 
				p=1;
				for(l=1;l<=p1;l++) {
					for (i=1;i<=p2;i++){
						if (Eckz1[l]==Eckz2[i]) {
							if (Eckz2[i] != AnfEck[2]) { 
								End[p]=Eckz2[i];
								p++;
							}
						}
					}
				} 
				if (p != 1) {
					--p;
					K=AnzForbidden-p-1;
	
					for(z=1;z<=p;z++) { 
						g=(int)fmod(End[z],AnzForbidden);
						if (g == 0) g = AnzForbidden;
						gplus1=g+1;
						gplus1=(int)fmod(gplus1,AnzForbidden);
						if (gplus1 == 0) gplus1 = AnzForbidden;

						xgol=CheckOpt(g,gplus1,1,zaehler);
						fxgol=funkf(g,gplus1,xgol,lpMetrik);
						if (fxmini > fxgol) {
							fxmini = fxgol;
							xmini  = xgol;
							eck1 = g;
							ymini = Region[eck1].ycoord();
						}
					}
				} else eck1=Kante1;
			} /* if zaehler==2*/
			} else {                           /*Das Optimum liegt im ganzen Gebiet*/
				AnfEck[zaehler]=eck1;
				EndEck[zaehler]=(int)fmod((eck1-1+AnzForbidden),AnzForbidden);
				if(EndEck[zaehler]==0) EndEck[zaehler]=AnzForbidden;
				if (zaehler == 2) { 
				p=1;
				for(l=1;l<=p1;l++) {
					for (i=1;i<=p2;i++){
						if (Eckz1[l]==Eckz2[i]) {
							if (Eckz2[i] != AnfEck[2]) { 
								End[p]=Eckz2[i];
								p++;
							}
						}
					}
				} 

					if (p != 1) {
						--p;
						for(z=1;z<=p;z++) { 
							g=End[z];
							if (g == 0) g = AnzForbidden;
							gplus1=g+1;
							gplus1=(int)fmod(gplus1,AnzForbidden);
							if (gplus1 == 0) gplus1 = AnzForbidden;

							xgol=CheckOpt(g,gplus1,1,zaehler);
							fxgol=funkf(g,gplus1,xgol,lpMetrik);
							if (fxmini > fxgol) {
								fxmini = fxgol;
								xmini  = xgol;
								eck1 = g;
								ymini = Region[eck1].ycoord();
							}
						} /*for*/
					} else eck1=Kante1;
				} /*if zaehler*/
			}
		} /*for*/
	eck2=eck1+1;
/*	if ( fabs(Region[eck1].xcoord()-Region[eck2].xcoord()) < epsilon) { 
		xmini=gerade(eck1,eck2,ymini);
	} else ymini=gerade(eck1,eck2,xmini);*/

	loctmp[0] = xmini; 
	loctmp[1] = ymini; 
	loctmp(0) = 0; 
	solution.clear();
	solution.append(loctmp);
	Z = fxmini;
	}
	return Z;
}  

double restrictions::l2sqr_sum(facilities &EX,bool erase)
/********************************************************************
* Purpose   : Compute the optimum for problem class 1|P|R|l2sqr|sum
* Returns   : objective values(double)
* Influences: list<location> solution
* Date      : May 96
* Author    : Lothar Schnell
*********************************************************************/
{
   restrictions   R(*this);
   double z = EX.l2sqr_sum(0,true);
   solution = EX.alg_solution();
   location  temp(2,1);
   location tmp_sol = solution.head();
   list<location> Proj,Proj_tmp;
   restriction *chk = inside_which(R, tmp_sol);
   restriction *iochk = R.head();
   if (((chk != NULL) && iochk->inout()) || ((chk == NULL) && !iochk->inout())) {
      Proj.clear();
      if (chk == NULL) {
        Proj_tmp.clear();
        forall(iochk,R) {
          Proj_tmp = iochk->proj_l2sqr(tmp_sol);
          for(list_item it=Proj_tmp.first(); it; it=Proj_tmp.succ(it)) {
            temp=Proj_tmp.contents(it);
            Proj.append(temp);
          }
        }
      } else Proj = chk->proj_l2sqr(tmp_sol);
      location loctmp(2,1);
      z = L_INF;
      forall(loctmp,Proj) {
        double dist = loctmp.loc2point().distance(tmp_sol.loc2point());
        if (dist < z) {
          z = dist;
          solution.clear();
          solution.append(loctmp);
        } else if ((dist == z) && (loctmp != solution.head())) {
          loctmp(0)=solution.tail()(0)+1;
          solution.append(loctmp);
        }
      }
      tmp_sol = solution.head();
      z = 0;
      for(list_item pointer_to_location=EX.first(); pointer_to_location; pointer_to_location=EX.succ(pointer_to_location))
      {
         temp=EX.contents(pointer_to_location);
         z += temp(0) * pow((temp.pos()-tmp_sol.pos()).norm(2),2.0);
      }
   }
   return z;
}

double restrictions::l2_v1_max(facilities &EX,bool erase) {
   restrictions   poly(*this);
   double z = EX.elz_hearn(false);
   solution = EX.alg_solution();
   location tmp_sol = solution.head();
   restriction *chk = inside_which(poly, tmp_sol);
   if (chk != NULL) {
	   z = chk->proj_v1_l2_max(tmp_sol, EX, z);
	   solution.clear();
	   solution.append(tmp_sol);
   }
   return z;
}

double restrictions::linf_max(facilities &EX,bool erase) {
   restrictions   poly(*this);
   double z = EX.linf_max(true);
   list<location> tmp_sol = EX.alg_solution();
   solution = tmp_sol;
   restriction *chk = inside_which(poly, tmp_sol);
   if (chk != NULL) {
      z = chk->proj_conv_linf_max(tmp_sol, EX, z);
      solution.clear();
      solution = tmp_sol;
   }
   return z;
}

double restrictions::l1_max(facilities &EX,bool erase) 
/********************************************************************
* Purpose   :  
* Returns   : % 
* Influences: EX,restrictions 
* Date      : Nov 97 
* Author    : Karen Schulze 
*********************************************************************/
{
restrictions poly(*this); // alte Restriktionen
restrictions neurestr; // neue Restriktionen
// Transformation der Standorte
facilities transEX=EX;
retrans_all(transEX);
// Transformation der Restriktionen
restriction *oneobj;
restriction *restr_temp;
list<point> Eckpunkte,neuEckpkte;
point Ecke;
int restr_id;
forall(oneobj,poly) {
	restr_id=oneobj->subtype();
	switch(restr_id) {
           case restr_p:  { // Restriktion ist ein Polygon
	 	restr_poly *polytemp = NULL;
        	polytemp=(restr_poly*)(oneobj);
		Eckpunkte=polytemp->vertices();
		neuEckpkte.clear();
		forall(Ecke,Eckpunkte) 
			neuEckpkte.append(point(Ecke.xcoord()-Ecke.ycoord(),Ecke.xcoord()+Ecke.ycoord()));
		restr_temp=new restr_poly (neuEckpkte);
		neurestr.append(restr_temp);
		break;
		} // if polygon
	   case restr_r: {// Restriktion ist ein Rechteck
		restr_rect *rectemp =NULL;
		rectemp=(restr_rect*)(oneobj);
		Eckpunkte=rectemp->vertices();
		neuEckpkte.clear();
		forall(Ecke,Eckpunkte) 
			neuEckpkte.append(point(Ecke.xcoord()-Ecke.ycoord(),Ecke.xcoord()+Ecke.ycoord()));
		restr_temp=new restr_rect (neuEckpkte.pop(),neuEckpkte.pop(),neuEckpkte.pop(),neuEckpkte.pop());
		neurestr.append(restr_temp);
		break;
		} // if Rechteck 
	  case restr_c: {// Restriktion ist ein Kreis
		restr_circle *circtemp = NULL;
                circtemp=(restr_circle*)(oneobj);
                Ecke=circtemp->center();
		Ecke=point(Ecke.xcoord()-Ecke.ycoord(),Ecke.xcoord()+Ecke.ycoord());
		double rad=sqrt(2)*(circtemp->radius()); 
		restr_temp=new restr_circle (Ecke,rad);
		neurestr.append(restr_temp);
		break;
		} // if Kreis		
	  } //switch	 
	} // for
// Problem l"osen mit linf_max
double z=0;
if(!erase) z=neurestr.linf_max(transEX,false);
	else z=neurestr.linf_max(transEX,true);
trans_all(neurestr.solution);
solution=neurestr.solution;
return z;
}

double restrictions::L_RkonvPoly_l2_sum(facilities &facs,bool erase) {
/********************************************************************
* Purpose   : Compute the optimum for problem class 1L|P|R=konvPoly|l2|sum
* Returns   : objective values(double)
* Influences: %
* Date      : Jan 98
* Author    : Ingo Bartling
*********************************************************************/
	if (erase) solution.clear();
	restr_poly *R = (restr_poly*)(this->head());
	/**** calling 1L/P/./l2/sum ****/
	lines LineAlg;
	double z = LineAlg.L_l2_sum_M3(facs);
	solution = LineAlg.alg_solution();
	/**** solution contains exact 2 locations ****/
	location r1 = solution.head(), r2 = solution.tail();
	point r1pnt = r1.loc2point(), r2pnt = r2.loc2point();
	/**** line is identical with one segment of restr_poly ****/
	segment seg;
	line resultLine(r1pnt,r2pnt);
   forall(seg, R->segments()) {
		line comp_line(seg);
		if (resultLine == comp_line)
		  return z;
	}
   /**** checking for intersection with restriction R ****/
	list<point> cutPnts = R->intersection(resultLine);
   /**** line touches the restr_poly ****/
	if (cutPnts.size() == 0 || cutPnts.size() == 1) return z;
	/**** line intersects in exact two points the restr_poly ****/
	solution.clear();
	location lineLoc, scndLineLoc, tmpLoc, Ex_i;
	/**** initializing the locs-array ****/
	int N = facs.size(), i = 0;
	location *locs = new location[N];
	forall(Ex_i,facs) locs[i++] = Ex_i;
	/**** algorithm ****/
	z = L_INF; double ztmp = 0;
	point scndLinePnt;
	/**** Initializing the R-vertices-array  ****/
	int verticeNbr = R->vertices().length();
	point *R_vertices = new point[verticeNbr+1]; i = 0;
        list<point> help_list=R->vertices();
        forall(scndLinePnt,help_list)
		R_vertices[i++] = scndLinePnt;
	R_vertices[i] = R_vertices[0];
   /**** Testing lines with one given location ****/
	/**** and one vertice of the restriction    ****/
	forall (lineLoc,facs) {
		if (!(R->inside(lineLoc))) {
			for (i = 0; i < verticeNbr; i++) {
				scndLinePnt = R_vertices[i];
				point linePnt = lineLoc.loc2point();
				line testLine(linePnt,scndLinePnt);
				cutPnts = R->intersection(testLine);
				if (cutPnts.size() == 1) {
					scndLineLoc = point2loc(scndLinePnt);
					ztmp = LineAlg.objective_function_l2(locs,0,N,lineLoc,scndLineLoc);
					if (ztmp < z) {
						z = ztmp; solution.clear();
						tmpLoc = lineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
						tmpLoc = scndLineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
					}
				}
			}
		}
	}
	/**** Testing lines wich contains restrictions-segments ****/
	point tmpPnt1, tmpPnt2; i = 0;
	for (i = 0; i < verticeNbr; i++) {
		tmpPnt1 = R_vertices[i]; tmpPnt2 = R_vertices[i+1];
		lineLoc = point2loc(tmpPnt1); scndLineLoc = point2loc(tmpPnt2);
		ztmp = LineAlg.objective_function_l2(locs,0,N,lineLoc,scndLineLoc);
		if (ztmp < z) {
			z = ztmp; solution.clear();
			tmpLoc = lineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
			tmpLoc = scndLineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
		}
	}
	delete[] locs; delete[] R_vertices;
	return z;
}

double restrictions::L_RkonvPoly_lp_sum(facilities &facs,bool erase,int p) {
/********************************************************************
* Purpose   : Compute the optimum for problem class 1L|P|R=konvPoly|lp|sum
* Returns   : objective values(double)
* Influences: %
* Date      : Jan 98
* Author    : Ingo Bartling
*********************************************************************/
	if (erase) solution.clear();
	restr_poly *R = (restr_poly*)(this->head());
	/**** calling 1L/P/./lp/sum ****/
	lines LineAlg;
	double z = LineAlg.L_lp_sum_M3(facs,p);
	solution = LineAlg.alg_solution();
	/**** solution contains exact 2 locations ****/
	location r1 = solution.head(), r2 = solution.tail();
	point r1pnt = r1.loc2point(), r2pnt = r2.loc2point();
	/**** line is identical with one segment of restr_poly ****/
	segment seg;
	line resultLine(r1pnt,r2pnt);
   forall(seg, R->segments()) {
		line comp_line(seg);
		if (resultLine == comp_line)
		  return z;
	}
   /**** checking for intersection with restriction R ****/
	list<point> cutPnts = R->intersection(resultLine);
   /**** line touches the restr_poly ****/
	if (cutPnts.size() == 0 || cutPnts.size() == 1) return z;
	/**** line intersects in exact two points the restr_poly ****/
	solution.clear();
	location lineLoc, scndLineLoc, tmpLoc, Ex_i;
	/**** initializing the locs-array ****/
	int N = facs.size(), i = 0;
	location *locs = new location[N];
	forall(Ex_i,facs) locs[i++] = Ex_i;
	/**** algorithm ****/
	z = L_INF; double ztmp = 0;
	point scndLinePnt;
	/**** Initializing the R-vertices-array  ****/
	int verticeNbr = R->vertices().length();
	point *R_vertices = new point[verticeNbr+1]; i = 0;
        list<point> help_list=R->vertices();
        forall(scndLinePnt,help_list)
		R_vertices[i++] = scndLinePnt;
	R_vertices[i] = R_vertices[0];
   /**** Testing lines with one given location ****/
	/**** and one vertice of the restriction    ****/
	forall (lineLoc,facs) {
		if (!(R->inside(lineLoc))) {
			for (i = 0; i < verticeNbr; i++) {
				scndLinePnt = R_vertices[i];
				point linePnt = lineLoc.loc2point();
				line testLine(linePnt,scndLinePnt);
				cutPnts = R->intersection(testLine);
				if (cutPnts.size() == 1) {
					scndLineLoc = point2loc(scndLinePnt);
					ztmp = LineAlg.objective_function_lp(locs,0,N,lineLoc,scndLineLoc,p);
					if (ztmp < z) {
						z = ztmp; solution.clear();
						tmpLoc = lineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
						tmpLoc = scndLineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
					}
				}
			}
		}
	}
	/**** Testing lines wich contains restrictions-segments ****/
	point tmpPnt1, tmpPnt2; i = 0;
	for (i = 0; i < verticeNbr; i++) {
		tmpPnt1 = R_vertices[i]; tmpPnt2 = R_vertices[i+1];
		lineLoc = point2loc(tmpPnt1); scndLineLoc = point2loc(tmpPnt2);
		ztmp = LineAlg.objective_function_lp(locs,0,N,lineLoc,scndLineLoc,p);
		if (ztmp < z) {
			z = ztmp; solution.clear();
			tmpLoc = lineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
			tmpLoc = scndLineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
		}
	}
	delete[] locs; delete[] R_vertices;
	return z;
}

double restrictions::L_RkonvPoly_l1_sum(facilities &facs,bool erase) {
/********************************************************************
* Purpose   : Compute the optimum for problem class 1L|P|R=konvPoly|l1|sum
* Returns   : objective values(double)
* Influences: %
* Date      : Jan 98
* Author    : Ingo Bartling
*********************************************************************/
	if (erase) solution.clear();
	restr_poly *R = (restr_poly*)(this->head());
	/**** calling 1L/P/./l1/sum ****/
	lines LineAlg;
	double z = LineAlg.L_l1_sum_M3(facs);
	solution = LineAlg.alg_solution();
	/**** solution contains exact 2 locations ****/
	location r1 = solution.head(), r2 = solution.tail();
	point r1pnt = r1.loc2point(), r2pnt = r2.loc2point();
	/**** line is identical with one segment of restr_poly ****/
	segment seg;
	line resultLine(r1pnt,r2pnt);
   forall(seg, R->segments()) {
		line comp_line(seg);
		if (resultLine == comp_line)
		  return z;
	}
   /**** checking for intersection with restriction R ****/
	list<point> cutPnts = R->intersection(resultLine);
   /**** line touches the restr_poly ****/
	if (cutPnts.size() == 0 || cutPnts.size() == 1) return z;
	/**** line intersects in exact two points the restr_poly ****/
	solution.clear();
	location lineLoc, scndLineLoc, tmpLoc, Ex_i;
	/**** initializing the locs-array ****/
	int N = facs.size(), i = 0;
	location *locs = new location[N];
	forall(Ex_i,facs) locs[i++] = Ex_i;
	/**** algorithm ****/
	z = L_INF; double ztmp = 0;
	point scndLinePnt;
	/**** Initializing the R-vertices-array  ****/
	int verticeNbr = R->vertices().length();
	point *R_vertices = new point[verticeNbr+1]; i = 0;
        list<point> help_list=R->vertices();
        forall(scndLinePnt,help_list)
		R_vertices[i++] = scndLinePnt;
	R_vertices[i] = R_vertices[0];
   /**** Testing lines with one given location ****/
	/**** and one vertice of the restriction    ****/
	forall (lineLoc,facs) {
		if (!(R->inside(lineLoc))) {
			for (i = 0; i < verticeNbr; i++) {
				scndLinePnt = R_vertices[i];
				point linePnt = lineLoc.loc2point();
				line testLine(linePnt,scndLinePnt);
				cutPnts = R->intersection(testLine);
				if (cutPnts.size() == 1) {
					scndLineLoc = point2loc(scndLinePnt);
					ztmp = LineAlg.objective_function_l1(locs,0,N,lineLoc,scndLineLoc);
					if (ztmp < z) {
						z = ztmp; solution.clear();
						tmpLoc = lineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
						tmpLoc = scndLineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
					}
				}
			}
		}
	}
	/**** Testing lines wich contains restrictions-segments ****/
	point tmpPnt1, tmpPnt2; i = 0;
	for (i = 0; i < verticeNbr; i++) {
		tmpPnt1 = R_vertices[i]; tmpPnt2 = R_vertices[i+1];
		lineLoc = point2loc(tmpPnt1); scndLineLoc = point2loc(tmpPnt2);
		ztmp = LineAlg.objective_function_l1(locs,0,N,lineLoc,scndLineLoc);
		if (ztmp < z) {
			z = ztmp; solution.clear();
			tmpLoc = lineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
			tmpLoc = scndLineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
		}
	}
	delete[] locs; delete[] R_vertices;
	return z;
}

double restrictions::L_RkonvPoly_linf_sum(facilities &facs,bool erase) {
/********************************************************************
* Purpose   : Compute the optimum for problem class 1L|P|R=konvPoly|linf|sum
* Returns   : objective values(double)
* Influences: %
* Date      : Jan 98
* Author    : Ingo Bartling
*********************************************************************/
	if (erase) solution.clear();
	restr_poly *R = (restr_poly*)(this->head());
	/**** calling 1L/P/./linf/sum ****/
	lines LineAlg;
	double z = LineAlg.L_linf_sum_M3(facs);
	solution = LineAlg.alg_solution();
	/**** solution contains exact 2 locations ****/
	location r1 = solution.head(), r2 = solution.tail();
	point r1pnt = r1.loc2point(), r2pnt = r2.loc2point();
	/**** line is identical with one segment of restr_poly ****/
	segment seg;
	line resultLine(r1pnt,r2pnt);
   forall(seg, R->segments()) {
		line comp_line(seg);
		if (resultLine == comp_line)
		  return z;
	}
   /**** checking for intersection with restriction R ****/
	list<point> cutPnts = R->intersection(resultLine);
   /**** line touches the restr_poly ****/
	if (cutPnts.size() == 0 || cutPnts.size() == 1) return z;
	/**** line intersects in exact two points the restr_poly ****/
	solution.clear();
	location lineLoc, scndLineLoc, tmpLoc, Ex_i;
	/**** initializing the locs-array ****/
	int N = facs.size(), i = 0;
	location *locs = new location[N];
	forall(Ex_i,facs) locs[i++] = Ex_i;
	/**** algorithm ****/
	z = L_INF; double ztmp = 0;
	point scndLinePnt;
	/**** Initializing the R-vertices-array  ****/
	int verticeNbr = R->vertices().length();
	point *R_vertices = new point[verticeNbr+1]; i = 0;
        list<point> help_list=R->vertices();
        forall(scndLinePnt,help_list)
		R_vertices[i++] = scndLinePnt;
	R_vertices[i] = R_vertices[0];
   /**** Testing lines with one given location ****/
	/**** and one vertice of the restriction    ****/
	forall (lineLoc,facs) {
		if (!(R->inside(lineLoc))) {
			for (i = 0; i < verticeNbr; i++) {
				scndLinePnt = R_vertices[i];
				point linePnt = lineLoc.loc2point();
				line testLine(linePnt,scndLinePnt);
				cutPnts = R->intersection(testLine);
				if (cutPnts.size() == 1) {
					scndLineLoc = point2loc(scndLinePnt);
					ztmp = LineAlg.objective_function_linf(locs,0,N,lineLoc,scndLineLoc);
					if (ztmp < z) {
						z = ztmp; solution.clear();
						tmpLoc = lineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
						tmpLoc = scndLineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
					}
				}
			}
		}
	}
	/**** Testing lines wich contains restrictions-segments ****/
	point tmpPnt1, tmpPnt2; i = 0;
	for (i = 0; i < verticeNbr; i++) {
		tmpPnt1 = R_vertices[i]; tmpPnt2 = R_vertices[i+1];
		lineLoc = point2loc(tmpPnt1); scndLineLoc = point2loc(tmpPnt2);
		ztmp = LineAlg.objective_function_linf(locs,0,N,lineLoc,scndLineLoc);
		if (ztmp < z) {
			z = ztmp; solution.clear();
			tmpLoc = lineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
			tmpLoc = scndLineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
		}
	}
	delete[] locs; delete[] R_vertices;
	return z;
}

double restrictions::L_RkonvPoly_block_sum(list<polygauge> &LG, list<int> &Ln, facilities &facs,bool erase) 
/********************************************************************
* Purpose   : Compute the optimum for problem class 1L|P|R=konvPoly|block|sum
* Returns   : objective values(double)
* Influences: %
* Date      : Jan 98
* Author    : Ingo Bartling
*********************************************************************/
{
	if (erase) solution.clear();
	list<point> gaugepnts;
	if (Ln.head() >= 0) gaugepnts = LG.head().vertices();
	Blocknorm B(gaugepnts);
	restr_poly *R = (restr_poly*)(this->head());
	/**** calling 1L/P/./block/sum ****/
	lines LineAlg;
	double z = 0;
	z = LineAlg.L_block_sum_M3(LG,Ln,facs);
	solution = LineAlg.alg_solution();
	/**** solution contains exact 2 locations ****/
	location r1 = solution.head(), r2 = solution.tail();
	point r1pnt = r1.loc2point(), r2pnt = r2.loc2point();
	/**** line is identical with one segment of restr_poly ****/
	segment seg;
	line resultLine(r1pnt,r2pnt);
	forall(seg, R->segments()) {
		line comp_line(seg);
		if (resultLine == comp_line)
			return z;
	}
   /**** checking for intersection with restriction R ****/
	list<point> cutPnts = R->intersection(resultLine);
   /**** line touches the restr_poly ****/
	if (cutPnts.size() == 0 || cutPnts.size() == 1) return z;
	/**** line intersects in exact two points the restr_poly ****/
	solution.clear();
	location lineLoc, scndLineLoc, tmpLoc, Ex_i;
	/**** initializing the locs-array ****/
	int N = facs.size(), i = 0;
	location *locs = new location[N];
	forall(Ex_i,facs) locs[i++] = Ex_i;
	/**** algorithm ****/
	z = L_INF; double ztmp = 0;
	point scndLinePnt;
	/**** Initializing the R-vertices-array  ****/
	int verticeNbr = R->vertices().length();
	point *R_vertices = new point[verticeNbr+1]; i = 0;
	list<point> help_list=R->vertices();
	forall(scndLinePnt,help_list)
		R_vertices[i++] = scndLinePnt;
	R_vertices[i] = R_vertices[0];
   /**** Testing lines with one given location ****/
	/**** and one vertice of the restriction    ****/
	forall (lineLoc,facs) {
		if (!(R->inside(lineLoc))) {
			for (i = 0; i < verticeNbr; i++) {
				scndLinePnt = R_vertices[i];
				point linePnt = lineLoc.loc2point();
				line testLine(linePnt,scndLinePnt);
				cutPnts = R->intersection(testLine);
				if (cutPnts.size() == 1) {
					scndLineLoc = point2loc(scndLinePnt);
					ztmp = LineAlg.objective_function_block(locs,0,N,lineLoc,scndLineLoc,B);
					if (ztmp < z) {
						z = ztmp; solution.clear();
						tmpLoc = lineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
						tmpLoc = scndLineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
					}
				}
			} // void            View(double objval, string normact, list<location>& AlgSol, restrictions& Restr, list<polygauge>& LP, list<int>& LI, int choice);
		}
	}
	/**** Testing lines wich contains restrictions-segments ****/
	point tmpPnt1, tmpPnt2; i = 0;
	for (i = 0; i < verticeNbr; i++) {
		tmpPnt1 = R_vertices[i]; tmpPnt2 = R_vertices[i+1];
		lineLoc = point2loc(tmpPnt1); scndLineLoc = point2loc(tmpPnt2);
		ztmp = LineAlg.objective_function_block(locs,0,N,lineLoc,scndLineLoc,B);
		if (ztmp < z) {
			z = ztmp; solution.clear();
			tmpLoc = lineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
			tmpLoc = scndLineLoc; tmpLoc(0) = 1; solution.append(tmpLoc);
		}
	}
	delete[] locs; delete[] R_vertices;
	return z;
}


double restrictions::gauge_max(facilities& facs, list<polygauge>& LG, list<int>& Ln ,bool erase)
/********************************************************************
* Purpose   : Compute the optimum for problem class 1|P|R|gauge|max
* Returns   : objective value
* Influences: %
* Date      : May 99
* Author    : Ralf Leipe (updated from Holger Hennes)
*********************************************************************/
{
	if (erase) solution.clear();
	double z = facs.gauge_max(LG,Ln,erase);
	solution = facs.alg_solution();
	location tmp_sol = solution.head();
	restriction *chk = inside_which(*this, tmp_sol);
	if (chk!=NULL) {
		chk->gauge_max(solution,facs,z,LG,Ln);
	}
	return z;
}



// ------------------------- global functions ------



double LowBound(int eck1,int eck2,int zaehler)
{
   double alpha,u1,u2;
   int minPunkt,maxPunkt;

   if (eck1< eck2) {
	   minPunkt=eck1;
	   maxPunkt=eck2;
   }
   else {
	   minPunkt=eck2;
	   maxPunkt=eck1;
   }

/*   if (eck1 > eck2) maxPunkt=eck1;
   else maxPunkt=eck2;*/

   if (zaehler==1) {
       u1=Region[maxPunkt].xcoord()-Region[minPunkt].xcoord();
       u2=Region[maxPunkt].ycoord()-Region[minPunkt].ycoord();
   } else {
       u1=-Region[maxPunkt].xcoord()+Region[minPunkt].xcoord();
       u2=-Region[maxPunkt].ycoord()+Region[minPunkt].ycoord();
   } 

  alpha=(funkf(eck1,eck2,Region[eck1].xcoord(),lpMetrik)-funkf(eck1,eck2,Region[eck2].xcoord(),lpMetrik)+funkgf(eck1,eck2,Region[eck2].xcoord())*u1)/(funkgf(eck1,eck2,Region[eck2].xcoord())*u1-funkgf(eck1,eck2,Region[eck1].xcoord())*u1);

  return (funkf(eck1,eck2,Region[eck1].xcoord(),lpMetrik)+alpha*funkgf(eck1,eck2,Region[eck1].xcoord())*u1);
}


double GoldenSchnitt(int eck1,int eck2)
{
   int i,iflag,j,nmin=0;
   double ax,bx,cx,fa,fb,fc,xmin,gold,amin[21];

   for (i=1;i<=5;i++) {
        ax=i;
        bx=i+1.0;
        mnbrak(eck1,eck2,&ax,&bx,&cx,&fa,&fb,&fc,funkf,lpMetrik);
        gold=golden(eck1,eck2,ax,bx,cx,funkf,TOL,&xmin,lpMetrik);
        if (nmin == 0) {
           amin[1]=xmin;
           nmin=1;
        } else {
           iflag=0;
           for (j=1;j<=nmin;j++) 
              if (fabs(xmin-amin[j]) <= EQL*xmin) iflag=1;
           if (iflag == 0) amin[++nmin]=xmin;
        }
   }
 return xmin;
}

#define GOLD 1.618034
#define GLIMIT 100.0
#define TINY 1.0e-20
#define SHFT(a,b,c,d) (a)=(b);(b)=(c);(c)=(d);

static float maxarg1,maxarg2;
#define FMAX(a,b) (maxarg1=(a),maxarg2=(b),(maxarg1) > (maxarg2) ?\
        (maxarg1) : (maxarg2))
#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))

void mnbrak(int eck1,int eck2,double *ax, double *bx, double *cx, double *fa, double *fb, double *fc, double (*funkf)(int,int,double,double), double lpMetrik)
{
	double ulim,u,r,q,fu,dum;
       
	*fa=(*funkf)(eck1,eck2,*ax,lpMetrik);
	*fb=(*funkf)(eck1,eck2,*bx,lpMetrik);
              if (*fb > *fa) {
		SHFT(dum,*ax,*bx,dum)
		SHFT(dum,*fb,*fa,dum)
	}
	*cx=(*bx)+GOLD*(*bx-*ax);
	*fc=(*funkf)(eck1,eck2,*cx,lpMetrik);
	while (*fb > *fc) {
		r=(*bx-*ax)*(*fb-*fc);
		q=(*bx-*cx)*(*fb-*fa);
		u=(*bx)-((*bx-*cx)*q-(*bx-*ax)*r)/
			(2.0*SIGN(FMAX(fabs(q-r),TINY),q-r));
		ulim=(*bx)+GLIMIT*(*cx-*bx);
		if ((*bx-u)*(u-*cx) > 0.0) {
			fu=(*funkf)(eck1,eck2,u,lpMetrik);
			if (fu < *fc) {
				*ax=(*bx);
				*bx=u;
				*fa=(*fb);
				*fb=fu;
				return;
			} else if (fu > *fb) {
				*cx=u;
				*fc=fu;
				return;
			}
			u=(*cx)+GOLD*(*cx-*bx);
			fu=(*funkf)(eck1,eck2,u,lpMetrik);
		} else if ((*cx-u)*(u-ulim) > 0.0) {
			fu=(*funkf)(eck1,eck2,u,lpMetrik);
			if (fu < *fc) {
				SHFT(*bx,*cx,u,*cx+GOLD*(*cx-*bx))
				SHFT(*fb,*fc,fu,(*funkf)(eck1,eck2,u,lpMetrik))
			}
		} else if ((u-ulim)*(ulim-*cx) >= 0.0) {
			u=ulim;
			fu=(*funkf)(eck1,eck2,u,lpMetrik);
		} else {
			u=(*cx)+GOLD*(*cx-*bx);
			fu=(*funkf)(eck1,eck2,u,lpMetrik);
		}
		SHFT(*ax,*bx,*cx,u)
		SHFT(*fa,*fb,*fc,fu)
	}
}
#undef GOLD
#undef GLIMIT
#undef TINY
#undef SHFT
#undef NRANSI
/* (C) Copr. 1986-92 Numerical Recipes Software "21#+]... */


#define R 0.61803399
#define C (1.0-R)
#define SHFT2(a,b,c) (a)=(b);(b)=(c);
#define SHFT3(a,b,c,d) (a)=(b);(b)=(c);(c)=(d);

double golden(int eck1,int eck2, double ax, double bx, double cx, double (*f)(int,int,double,double), double tol,double *xmin,double lpMetrik)
{
	double f1,f2,x0,x1,x2,x3;
        
	x0=ax;
	x3=cx;
	if (fabs(cx-bx) > fabs(bx-ax)) {
		x1=bx;
		x2=bx+C*(cx-bx);
	} else {
		x2=bx;
		x1=bx-C*(bx-ax);
	}
	f1=(*f)(eck1,eck2,x1,lpMetrik);
	f2=(*f)(eck1,eck2,x2,lpMetrik);

       while (fabs(x3-x0) > tol*(fabs(x1)+fabs(x2))) {  
		if (f2 < f1) {
			SHFT3(x0,x1,x2,R*x1+C*x3)
			SHFT2(f1,f2,(*f)(eck1,eck2,x2,lpMetrik))
		} else {
                        SHFT3(x3,x2,x1,R*x2+C*x0)
                        SHFT2(f2,f1,(*f)(eck1,eck2,x1,lpMetrik))
		}
	}
	if (f1 < f2) {
		*xmin=x1;
		return f1;
	} else {
		*xmin=x2;
		return f2;
	}
}
#undef C
#undef R
#undef SHFT2
#undef SHFT3
/* (C) Copr. 1986-92 Numerical Recipes Software "21#+]... */

