// Definition of class single_barrier_optimizer SBO (algorithm class)

#include <LOLA/sbo.h>

SBO::SBO() {
  _EX = NULL;
  _optimum = point();
  _objective = 0;
  _done = false;
}

SBO::SBO(barrier B,
	 facilities* EX) {
  _B=B;
  _EX=EX;
  _optimum = point();
  _objective = 0;
  _done = false;
}

SBO::~SBO() {
  _EX = NULL;
  _objective = 0;
  _done = false;
}

int sbocmp(const PO& p1,
	   const PO& p2) {
  if (p1.val() < p2.val()) return -1;
  else if (p1.val() == p2.val()) return 0;
  return 1;
}

double SBO::l2_sum_h1(double accel,
		   double initss,
		   double eps) {
	restriction *restr=(_B.head());
	lbound();
	if (!restr->inside(_xlb)) {
		tvec tau_lbound(_B,*_EX,_xlb);
		if (tau_lbound.number() == 0) {
			_done = true;
			_optimum=_xlb;
			_objective=_lb;
			return _objective;
		}
	}
	int l=0;
	double retval=10000000000.0;
	point opt(0,0);
	list<point> startp3=cpt_start_points(cpt_shdw_segs());
	list<point> startp2;
	startp2.clear();
	if (restr->subtype() == t_polygon) {
		bool is_in;
		list<point> p_verts=((polygon_barrier*)restr)->vertices();
		list_item iter1,iter2;
		for(iter1=startp3.first();iter1;iter1=startp3.succ(iter1)) {
			is_in=false;
			for(iter2=p_verts.first();iter2;iter2=p_verts.succ(iter2)) {
				if (startp3.contents(iter1) == p_verts.contents(iter2)) {
					is_in = true;
					break;
				}
			}
			if (!is_in) startp2.append(startp3.contents(iter1));
		}
	} else startp2=startp3; 
	set<integer> visited;
	visited.clear();
	bool doiter=false;
	for(list_item trav=startp2.first();trav;trav=startp2.succ(trav)) {
		doiter=false;
		point test1(startp2.contents(trav).xcoord()+initss*D(1,1),
			startp2.contents(trav).ycoord()+initss*D(1,2));
		point test2(startp2.contents(trav).xcoord()-initss*D(1,1),
			startp2.contents(trav).ycoord()-initss*D(1,2));
		point test3(startp2.contents(trav).xcoord()+initss*D(2,1),
			startp2.contents(trav).ycoord()+initss*D(2,2));
		point test4(startp2.contents(trav).xcoord()-initss*D(2,1),
			startp2.contents(trav).ycoord()-initss*D(2,2));
		tvec tau1(_B,*_EX,test1);
		tvec tau2(_B,*_EX,test2);
		tvec tau3(_B,*_EX,test3);
		tvec tau4(_B,*_EX,test4);
		integer st1=tau1.number();
		integer st2=tau2.number();
		integer st3=tau3.number();
		integer st4=tau4.number();      
		integer append;
		if (restr->objective(*_EX,test1) <
			restr->objective(*_EX,startp2.contents(trav))) {
			if (!visited.member(st1)) {
				doiter=true;
				append=st1;
			}
		} else
			if (restr->objective(*_EX,test2) <
				restr->objective(*_EX,startp2.contents(trav))) {
				if (!visited.member(st2)) {
					doiter=true;
					append=st2;
				}
			} else
				if (restr->objective(*_EX,test3) <
					restr->objective(*_EX,startp2.contents(trav))) {
					if (!visited.member(st3)) {
						doiter=true;
						append=st3;
					}
				} else
					if (restr->objective(*_EX,test4) <
						restr->objective(*_EX,startp2.contents(trav))) {
						if (!visited.member(st4)) {
							doiter=true;
							append=st4;
						}
					}
		point temp;
		bool done;
		double otemp;
		if (doiter) { 
			done=hooke_jeeves_ds(startp2.contents(trav),
				temp,accel,initss,eps,visited,false);
			if (done) {
				otemp=restr->objective(*_EX,temp);
				l++;
				if (otemp < retval) {
					retval = otemp;
					opt = temp;
				}
			} 
		}
	}
	_done = true;
	_optimum=opt;
	_objective=retval;
	return retval;
}

double SBO::l2_sum_h2(double accel, double initss, double eps, double ppp) 
{
	restriction *restr=(_B.head());
	lbound();
	if (!restr->inside(_xlb)) {
		tvec tau_lbound(_B,*_EX,_xlb);
		if (tau_lbound.number() == 0) {
			_done = true;
			_optimum=_xlb;
			_objective=_lb;
			return _objective;
		}
	}
	int l=0;
	double retval=10000000000.0;
	point opt(0,0);
	list<point> startp3=cpt_start_points(cpt_shdw_segs());
	list<point> startp;
	startp.clear();
	if (restr->subtype() == t_polygon) {
		bool is_in;
		list<point> p_verts=((polygon_barrier*)restr)->vertices();
		list_item iter1,iter2;
		for(iter1=startp3.first();iter1;iter1=startp3.succ(iter1)) {
			is_in=false;
			for(iter2=p_verts.first();iter2;iter2=p_verts.succ(iter2)) {
				if (startp3.contents(iter1) == p_verts.contents(iter2)) {
					is_in = true;
					break;
				}
			}
			if (!is_in) startp.append(startp3.contents(iter1));
		}
	} else startp=startp3;   
	list_item trav;
	list<PO> reduce;
	reduce.clear();
	double lb=lbound();
	for(trav=startp.first();trav;trav=startp.succ(trav)) {
		PO tmp(restr->objective(*_EX,startp.contents(trav))-lb,
			startp.contents(trav));
		reduce.append(tmp);
	}  
	reduce.sort(sbocmp);
	int num_of_final_points=(int)(reduce.length() * ppp);
	int count=0;
	list<point> startp2;
	startp2.clear();
	for(list_item pcpy=reduce.first();pcpy;pcpy=reduce.succ(pcpy)) {
		if (count < num_of_final_points) {
			startp2.append(reduce.contents(pcpy).pt());
			count++;
		} else
			break;
	}
	set<integer> visited;
	visited.clear();
	bool doiter=false;
	for(trav=startp2.first();trav;trav=startp2.succ(trav)) {
		doiter=false;
		point test1(startp2.contents(trav).xcoord()+initss*D(1,1),
			startp2.contents(trav).ycoord()+initss*D(1,2));
		point test2(startp2.contents(trav).xcoord()-initss*D(1,1),
			startp2.contents(trav).ycoord()-initss*D(1,2));
		point test3(startp2.contents(trav).xcoord()+initss*D(2,1),
			startp2.contents(trav).ycoord()+initss*D(2,2));
		point test4(startp2.contents(trav).xcoord()-initss*D(2,1),
			startp2.contents(trav).ycoord()-initss*D(2,2));
		tvec tau1(_B,*_EX,test1);
		tvec tau2(_B,*_EX,test2);
		tvec tau3(_B,*_EX,test3);
		tvec tau4(_B,*_EX,test4);
		integer st1=tau1.number();
		integer st2=tau2.number();
		integer st3=tau3.number();
		integer st4=tau4.number();      
		integer append;
		if (restr->objective(*_EX,test1) < restr->objective(*_EX,startp2.contents(trav))) {
			if (!visited.member(st1)) {
				doiter=true;
				append=st1;
			}
		} else
			if (restr->objective(*_EX,test2) < restr->objective(*_EX,startp2.contents(trav))) {
				if (!visited.member(st2)) {
					doiter=true;
					append=st2;
				}
			} else
				if (restr->objective(*_EX,test3) < restr->objective(*_EX,startp2.contents(trav))) {
					if (!visited.member(st3)) {
						doiter=true;
						append=st3;
					}
				} else
					if (restr->objective(*_EX,test4) < restr->objective(*_EX,startp2.contents(trav))) {
						if (!visited.member(st4)) {
							doiter=true;
							append=st4;
						}
					}
					point temp;
					bool done;
					double otemp;
					if (doiter) { 
						done=hooke_jeeves_ds(startp2.contents(trav),
							temp,accel,initss,eps,visited,true);
						if (done) {
							visited.insert(append);
							tvec otau(_B,*_EX,temp);
							visited.insert(otau.number());
							otemp=restr->objective(*_EX,temp);
							l++;
							if (otemp < retval) {
								retval = otemp;
								opt = temp;
							}
						}
					}
	}
	_done = true;
	_optimum=opt;
	_objective=retval;
	return retval;
}

point SBO::optimum() {
	if (_done) return _optimum;
	point p(0,0);
	return p;
}

double SBO::objective() {
	if (_done) return _objective;
	else return -1.0;
}

double SBO::lbound() {
	double retval = _EX->l2_sum(0,0.001,true);
	list<location> Xlb = _EX->alg_solution();
	_xlb=Xlb.head().loc2point();
	_lb=retval;
	return retval;
}

list<segment> SBO::cpt_shdw_segs() {
	restriction *restr=(_B.head());
	list<segment> retval;
	retval.clear();
	list<point> fpoints;
	fpoints.clear();
	list<point> chull;
	chull.clear();



  /* Here is the simple case: If a barrier is fully inside of the convex hull we do not
     need to puzzle around... */

    list_item trav1,trav2;
	for(trav1=_EX->first();trav1;trav1=_EX->succ(trav1)) 
		fpoints.append((_EX->contents(trav1)).loc2point());
	chull=CONVEX_HULL(fpoints);
	double maxdist=0;
	double actdist=0;
	for(trav1=chull.first();trav1;trav1=chull.succ(trav1))
		for(trav2=chull.first();trav2;trav2=chull.succ(trav2)) {
			actdist=chull.contents(trav1).distance(chull.contents(trav2));
			if (actdist > maxdist) maxdist=actdist;
		}

  /* Check if we have the simple case or not... 
     If not we will find one segment of the convex hull intersecting the
     barrier. If that's the case we take the rectangular hull of the existing facilities
     and the barrier points plus some 'space' around the points of the barrier...
     This also takes care for LEDA finding intersection points of segments which might
     be lying on each other in case of the real convex hull of (EX,(R,oo)) where (R,oo)
     is hit. */

    polygon CH(chull);
	bool OK=true;
	list_item p_segm;
	list<segment> p_segms=CH.edges();
	for(p_segm=p_segms.first();p_segm;p_segm=p_segms.succ(p_segm)) 
		if (restr->intersection(p_segms.contents(p_segm)).length() != 0) {
			OK=false;
			break;
		}

 
    if (!OK) {
    /* That's the case when an edge segment of the convex hull of the facilities
       intersects the barrier.
       Here we step back to the convex hull of the existing facilities and the barrier
       including some 'security distance' by using the bounding box of the barrier ... */
		switch (restr->subtype()) {
		case t_circle: {
			circle_barrier *CB=(circle_barrier*)restr;
			double b_ext_fact=2.0;
			double c_rad=CB->radius();
			point c_cent=CB->center();
			point addp1(c_cent.xcoord()+b_ext_fact*c_rad,c_cent.ycoord()+b_ext_fact*c_rad);
			point addp2(c_cent.xcoord()-b_ext_fact*c_rad,c_cent.ycoord()+b_ext_fact*c_rad);
			point addp3(c_cent.xcoord()-b_ext_fact*c_rad,c_cent.ycoord()-b_ext_fact*c_rad);
			point addp4(c_cent.xcoord()+b_ext_fact*c_rad,c_cent.ycoord()-b_ext_fact*c_rad);      
			fpoints.append(addp1);
			fpoints.append(addp2);
			fpoints.append(addp3);
			fpoints.append(addp4);
			chull=CONVEX_HULL(fpoints);
			CH=polygon(chull);
			break;
					   } 
		case t_segment: {
			segment_barrier *SB=(segment_barrier*)restr;
			point sp1(SB->point1());
			point sp2(SB->point2());
			double b_ext_fact=sp1.distance(sp2);
			/* bounding box of segment_barrier */
			double xmin,xmax,ymin,ymax;
			if (sp1.xcoord() < sp2.xcoord()) {
				xmin = sp1.xcoord();
				xmax = sp2.xcoord();
			} else {
				xmin = sp2.xcoord();
				xmax = sp1.xcoord();
			}	
			if (sp1.ycoord() < sp2.ycoord()) {
				ymin = sp1.ycoord();
				ymax = sp2.ycoord();
			} else {
				ymin = sp2.ycoord();
				ymax = sp1.ycoord();
			}	
			point addp1(xmin-b_ext_fact,ymin-b_ext_fact);
			point addp2(xmax+b_ext_fact,ymin-b_ext_fact);
			point addp3(xmin-b_ext_fact,ymax+b_ext_fact);
			point addp4(xmax+b_ext_fact,ymax+b_ext_fact);      
			fpoints.append(addp1);
			fpoints.append(addp2);
			fpoints.append(addp3);
			fpoints.append(addp4);
			chull=CONVEX_HULL(fpoints);
			CH=polygon(chull);      
			break;
						} 
		case t_polygon: {
			polygon_barrier *PB=(polygon_barrier*)restr;
			list<point> p_bar_pts=PB->vertices();
			double xmin=0;
			double xmax=0;
			double ymin=0;
			double ymax=0;      
			list_item vert_ptr;
			/* bounding box */
			xmin=xmax=p_bar_pts.head().xcoord();
			ymin=ymax=p_bar_pts.head().ycoord();
			for(vert_ptr=p_bar_pts.first();vert_ptr;vert_ptr=p_bar_pts.succ(vert_ptr)) {
				if (p_bar_pts.contents(vert_ptr).xcoord() < xmin) 
					xmin=p_bar_pts.contents(vert_ptr).xcoord();
				if (p_bar_pts.contents(vert_ptr).xcoord() > xmax) 
					xmax=p_bar_pts.contents(vert_ptr).xcoord();
				if (p_bar_pts.contents(vert_ptr).ycoord() < ymin) 
					ymin=p_bar_pts.contents(vert_ptr).ycoord();
				if (p_bar_pts.contents(vert_ptr).ycoord() > ymax) 
					ymax=p_bar_pts.contents(vert_ptr).ycoord();
			}
			double b_ext_fact=PB->maxdiam();
			point addp1(xmin-b_ext_fact,ymin-b_ext_fact);
			point addp2(xmax+b_ext_fact,ymin-b_ext_fact);
			point addp3(xmin-b_ext_fact,ymax+b_ext_fact);
			point addp4(xmax+b_ext_fact,ymax+b_ext_fact);      
			fpoints.append(addp1);
			fpoints.append(addp2);
			fpoints.append(addp3);
			fpoints.append(addp4);
			chull=CONVEX_HULL(fpoints);
			CH=polygon(chull);            
			break;
						} 
		}
	}

    fpoints.clear();

	for(trav1=_EX->first();trav1;trav1=_EX->succ(trav1)) 
		fpoints.append((_EX->contents(trav1)).loc2point());
  
    for(trav1=fpoints.first();trav1;trav1=fpoints.succ(trav1)) {
		point LTP;
		point RTP;
		switch (restr->subtype()) {
		case t_circle: { 
				circle_barrier *CB=(circle_barrier*)restr;
				LTP=CB->l_tang_point(fpoints.contents(trav1));
				RTP=CB->r_tang_point(fpoints.contents(trav1));
				break;}
		case t_segment: {
				segment_barrier *SB=(segment_barrier*)restr;
				LTP=SB->l_tang_point(fpoints.contents(trav1));
				RTP=SB->r_tang_point(fpoints.contents(trav1));
				break;}
		case t_polygon:{ 
				polygon_barrier *PB=(polygon_barrier*)restr;
				LTP=PB->l_tang_point(fpoints.contents(trav1));
				RTP=PB->r_tang_point(fpoints.contents(trav1));
				break;}
		}

		vector LV(2);
		vector RV(2);
		LV[0]=fpoints.contents(trav1).xcoord()-LTP.xcoord(); 
		LV[1]=fpoints.contents(trav1).ycoord()-LTP.ycoord();
		RV[0]=fpoints.contents(trav1).xcoord()-RTP.xcoord();
		RV[1]=fpoints.contents(trav1).ycoord()-RTP.ycoord();
		point Lend=LTP+(LV*(2.0*maxdist));
		point Rend=RTP+(RV*(2.0*maxdist));
		segment SL(LTP,Lend);
		segment SR(RTP,Rend);
		list<point> IsectPl=CH.intersection(SL);
		list<point> IsectPr=CH.intersection(SR);

		// cut here (begin)
		if (IsectPl.length() == 0 || IsectPr.length() == 0) {
			lola_error("SBO::cpt_shdw_segs(): Zero intersection found for segment.",1806,1);
		}
		// cut here (end)

		if (IsectPl.length() != 0 && IsectPr.length() != 0) {
			segment ShL(LTP,IsectPl.head());
			segment ShR(RTP,IsectPr.head());
			retval.append(ShL);
			retval.append(ShR);
		}
	}
	return retval;
}

list<point> SBO::cpt_start_points(const list<segment>& SL) {
  list<point> retval;
  retval.clear();
  SEGMENT_INTERSECTION(SL,retval);
  return retval;
}

double SBO::D(int j,int r) {
  if (j==r) return 1.0;
  else return 0.0;
}

bool SBO::hooke_jeeves_ds(const point& start,
			  point& opt,
			  double accel,
			  double initss,
			  double eps,
			  set<integer>& areas,
			  bool heuristic) {
  // This is an implementation of the "Method of Hooke & Jeeves
  // with Discrete Steps". I added some extensions for logging and
  // detecting if an area of F has been visited before.
	restriction *restr=(_B.head());

  /* Initialization Step: */
	int n=2;
  // Choose a scalar epsilon > 0 to be used for terminating
  // the algorithm.
	double epsilon = eps;
  // Furthermore choose an initial stepsize delta >= epsilon,
	double delta = initss;
  // and an acceleration factor alpha > 0
	double alpha = accel;
  // Choose a starting point x1,
  // let y1=x1,
	int anz=n+2;
	point* y;
	y = new point[anz];
	point x[10000];
	x[1]=start;
	y[1]=x[1];
  // let k=j=1, and go to the main step
	int k=1;
	int j=1;
  /* Main Step */
  // control flags for the method of Hooke & Jeeves
	bool gostep1=true;
	bool gostep2=false;
	bool gostep3=false;
	bool gostep4=false;
	bool optimal=false;
  // I added this flag for stopping local search on detecting
  // an area already visited
	bool stopflag=false;
	while (!optimal && !stopflag) {
    //Step1
		if (gostep1) {
			gostep1=false;
			point test1(y[j].xcoord()+delta*D(j,1), y[j].ycoord()+delta*D(j,2));
			point test2(y[j].xcoord()-delta*D(j,1), y[j].ycoord()-delta*D(j,2));
			if(restr->objective(*_EX,test1) < restr->objective(*_EX,y[j])) {
				y[j+1]=test1;
				gostep2=true;
			} else {
				if(restr->objective(*_EX,test2) < restr->objective(*_EX,y[j])) {
					y[j+1]=test2;
					gostep2=true;
				} else {
					y[j+1]=y[j];
					gostep2=true;
				}
			}
			// main modification of the method of Hooke & Jeeves 
			// this "reflects" whether a point is in an area that
			// has once been visited.
			tvec tau(_B,*_EX,y[j+1]);
			integer tauval=tau.number();
			if (areas.member(tauval)) {
				stopflag=true;
				gostep1=false;
				gostep2=false;
				gostep3=false;
				gostep4=false;
				optimal=false;
			}	
		}
		//Step2
		if (gostep2) {
			gostep2=false;
			if (j<n) {
				j++;
				gostep1=true;
			} else if (restr->objective(*_EX,y[n+1]) < restr->objective(*_EX,x[k]))
				gostep3=true;
			else gostep4=true;
		}
		//Step3
		if (gostep3) { 
			gostep3=false;
			x[k+1]=y[n+1];
			point temp(x[k+1].xcoord() + alpha * (x[k+1].xcoord() - x[k].xcoord()),
				x[k+1].ycoord() + alpha *(x[k+1].ycoord() - x[k].ycoord()));
			y[1]=temp;
			k++;
			j=1;
			gostep1=true;
		}
		//Step4
		if (gostep4) {
			gostep4=false;
			if (delta <= epsilon) {
				optimal=true;
			} else {
				delta = delta / 2.0;
				y[1]=x[k];
				x[k+1]=x[k];
				k++;
				j=1;
				gostep1=true;
			}
		}
	}
	if (optimal) {
		opt=x[k];
	}
	// Here the method of Hooke & Jeeves "learns" about the areas
	//visited during the last local search. The more local searches
	// are done the more "intelligent" it becomes. It learns no
	// matter if there was found a local minimum or if it stopped
	// in an area already visited. 
	if (heuristic) {
		for (int i=1;i<=k;i++) {
			tvec sncv(_B,*_EX,x[i]);
			integer sncn=sncv.number();
			if (!areas.member(sncn)) 
				areas.insert(sncn);
		}
	} else {
		tvec snopt(_B,*_EX,x[k]);
		integer snoptn=snopt.number();
		if(!areas.member(snoptn))
			areas.insert(snoptn);
	}
	delete [] y;
	return optimal;
}
