// Definition of class segment_barrier

#include <LOLA/segment_barrier.h>

segment_barrier::segment_barrier() {
  _SP1 = point();
  _SP2 = point();
}

segment_barrier::segment_barrier(const point& p1,const point& p2) {
  _SP1=p1;
  _SP2=p2;
}

segment_barrier::segment_barrier(const segment_barrier& S) {
  _SP1=S._SP1;
  _SP2=S._SP2;
}

segment_barrier::~segment_barrier() {
  _SP1 = point();
  _SP2 = point();
}

point segment_barrier::midpoint() {
  double dx=_SP2.xcoord()-_SP1.xcoord();
  double dy=_SP2.ycoord()-_SP1.ycoord();
  point retval(_SP1.xcoord()+0.5*dx,_SP1.ycoord()+0.5*dy);
  return retval;
}

bool segment_barrier::on_edge(const point& p) {
  return inside(p);
}

bool segment_barrier::on_edge(const location& l) {
  return on_edge(l.loc2point());
}

bool segment_barrier::inside(const point& p) {
  bool retval = false;
  double pp1 = _SP1.distance(p);
  double pp2 = _SP2.distance(p);
  double cmp = _SP1.distance(_SP2);
  if (myabs(pp1+pp2-cmp) < 0.0001)
    retval = true;
  return retval;
}

bool segment_barrier::inside(const location& l) {
  return inside(l.loc2point());
}

list<point> segment_barrier::intersection(const segment& s) {
  segment barr(_SP1,_SP2);
  point   isectP;
  list<point> retval;
  retval.clear();
  if (barr.intersection(s,isectP)) retval.append(isectP);
  return retval;
}

list<point> segment_barrier::intersection(const line& l) {
  segment barr(_SP1,_SP2);
  point   isectP;
  list<point> retval;
  retval.clear();
  if (l.intersection(barr,isectP)) retval.append(isectP);
  return retval;
}

list<point> segment_barrier::intersect(point& P, point& Q) {
  segment to_cut(P,Q);
  return intersection(to_cut);
}

bool segment_barrier::visible(const point& p1,const point& p2) {
  bool retval = false;
  if (inside(p1) || inside(p2))
    retval = true;
  else {
    segment connect(p1,p2);
    segment barr(_SP1,_SP2);
    point   isectP;
    if (!barr.intersection(connect,isectP)) retval = true;
    else if(spccmp(isectP,_SP1) || spccmp(isectP,_SP2)) retval = true;
  }
  return retval;
}

point segment_barrier::r_tang_point(const point& p) { 
  point retval = t_points(p).tail();
  return retval;
}

point segment_barrier::l_tang_point(const point& p) {
  point retval = t_points(p).head();
  return retval;
}

list<point> segment_barrier::t_points(const point& p) {
  list<point> retval;
  retval.clear();
  point M=midpoint();
  halfspace H(p,M);
  if (H.in_which(_SP1) == -1) {
    retval.append(_SP1);
    retval.append(_SP2);
  } else {
    retval.append(_SP2);
    retval.append(_SP1);
  }
  return retval;
}

double segment_barrier::LSP2(const point& p1,const point& p2) {
  double retval = 0;
  if (visible(p1,p2)) return p1.distance(p2);
  else { 
    double LSP2_l = 0;
    double LSP2_r = 0;
    point rt=r_tang_point(p1);
    point lt=l_tang_point(p1);
    LSP2_r = p1.distance(rt) + rt.distance(p2);
    LSP2_l = p1.distance(lt) + lt.distance(p2);
    if (LSP2_r < LSP2_l) retval = LSP2_r;
    else retval = LSP2_l;
    return retval;
  }
}

double segment_barrier::l_2(const location& L,const point& p) {
  return LSP2(L.loc2point(),p);
}

double segment_barrier::objective(const facilities& EX,const point& p) {
  // set up and initialize the return value
  double retval = 0;
  // a pointer for traversing the facilities
  list_item trav;
  // traverse the facilities and add up the weighted Euclidean distance from each facility to
  // the given point p 
  for(trav=EX.first();trav;trav=EX.succ(trav)) 
    retval += EX.contents(trav)(0) * l_2(EX.contents(trav),p);
  // return the objective value of p in R^2 \ circle
  return retval;
}

bool segment_barrier::spccmp(const point& p1,const point& p2) {
  // this is a special method for checking two given points for being equal;
  // they are defined to be equal if there coordinatewise difference is not bigger than
  // a certain value; this method uses "myabs"
  bool retval = false;
  if ( (myabs(p1.xcoord()-p2.xcoord()) < 0.0001) &&
      ((myabs(p1.ycoord()-p2.ycoord()) < 0.0001)) )
    retval = true;
  return retval;
}

double segment_barrier::myabs(double d) {
  // there is no double-valued abs in C++, but I need one; so I define my own
  if (d < 0) return -d;
  return d;
}

double segment_barrier::left_path(const point& p1,const point& p2) {
  return 0;
}

double segment_barrier::right_path(const point& p1,const point& p2) {
  return 0;
}

// ----
// End of definition of class segment_barrier
// ----
// Testing result
//            Faults: undef
//          Failures: undef
// Compiler warnings: none
