#include <LOLA/tvec.h>

tvec::tvec() {
  _data=NULL;
  _length=0;
}

tvec::tvec(barrier B,const facilities& EX,const point& p) {
	restriction *restr=(B.head());

	_length=EX.length();
	_data = new int[_length];
	list_item trav;
	int count = 0;
	for(trav=EX.first();trav;trav=EX.succ(trav)) {
		switch (restr->subtype()) {
		case t_circle: 
			{
				circle_barrier *CB=(circle_barrier*)restr;
				if (CB->visible(EX.contents(trav).loc2point(),p))
					_data[count] = 0;
				else {
					halfspace H(p,CB->center());
					_data[count]=H.in_which(EX.contents(trav).loc2point());
				} 
			}
			break;
		
		case t_segment: 
			{
				segment_barrier *SB=(segment_barrier*)restr;
				if (SB->visible(EX.contents(trav).loc2point(),p))
					_data[count] = 0;
				else {
					double l_path_len = 0;
					double r_path_len = 0;
					point rt = SB->r_tang_point(p);
					point lt = SB->l_tang_point(p);
					l_path_len = p.distance(lt) + lt.distance(EX.contents(trav).loc2point());
					r_path_len = p.distance(rt) +rt.distance(EX.contents(trav).loc2point());
					if (r_path_len < l_path_len) _data[count] = 1;
					else _data[count] = -1;
				}      
			} 
			break;
		case t_polygon: 
			{
				polygon_barrier *PB=(polygon_barrier*)restr;
				if (PB->visible(EX.contents(trav).loc2point(),p)) {
					_data[count] = 0;
				}
				else {
					double l_path_len = PB->left_path(EX.contents(trav).loc2point(),p);
					double r_path_len = PB->right_path(EX.contents(trav).loc2point(),p);
					if (r_path_len < l_path_len) _data[count] = 1;
					else _data[count] = -1;
				}
			} 
			break;
		} /* end of switch */
		count++;
	}
}

tvec::tvec(barrier B,const facilities& EX,const location& l) {
  tvec(B,EX,l.loc2point());
}

tvec::tvec(const tvec& tau) {
  _length = tau._length;
  if (_length > 0) {
    _data   = new int[_length];
    for(int i=0;i<_length;i++)
      _data[i]=tau._data[i];
  } else _data = NULL;
}

tvec::~tvec() {
  _length = 0;
  delete _data;
}

int* tvec::data() const {
  return _data;
}

int tvec::length() const {
  return _length;
}

bool tvec::operator==(const tvec& tau) const {
  bool retval=true;
  if (tau._length != _length)
    retval = false;
  else 
    for(int i=0;i<tau._length;i++) 
      if (tau._data[i] != _data[i]) {
	retval = false;
	break;
      }
  return retval;
}

int tvec::flow_rank(const tvec& tau) {
  int  L=tau.length();
  if (L!=_length) 
    lola_error("tvec::flow_rank(): length of tvecs differs.",1804,1);
  int* T=tau.data();
  int  i=0;
  int retval = 0;
  for(i=0;i<L;i++) 
    if(_data[i]-T[i] == 1 || _data[i]-T[i] == -1)
      retval++;
  return retval;
}

int tvec::crash_rank(const tvec& tau) {
  int  L=tau.length();
  if (L!=_length) 
    lola_error("tvec::crash_rank(): length of tvecs differs.",1805,1);
  int* T=tau.data();
  int  i=0;
  int retval = 0;
  for(i=0;i<L;i++) 
    if(_data[i]-T[i] == 2 || _data[i]-T[i] == -2)
      retval++;
  return retval;
}

integer tvec::number() {
  integer retval(0);
  int     i;
  for(i=_length;i > 0;i--) {
    int bitval;
    if (_data[i-1]==-1) 
      bitval = 2;
    else bitval = _data[i-1];
    integer add(bitval*int(pow(3.0,double(_length-i))));
    retval += add;
  }
  return retval;
}

ostream& operator<<(ostream& os,tvec& tau) {
  int out=0;
  os << "[";
  for(int i=0;i<tau._length;i++) {
    out = tau._data[i];
    switch (out) {
    case -1: os << out << " ";
      break;
    case  1: os << " " << out << " ";
      break;
    case  0: os << " " << out << " ";
      break;
    }
  }
  os << "]" << endl;
  return os;
}

int compare(const tvec& tau1,const tvec& tau2) {
  if (tau1==tau2)
    return 0;
  else return 1;
}
// --
// End of definition of class tvec
// --
// Testing result
//            Faults: none
//          Failures: none
// Compiler warnings: none
