#include <LOLA/facs_util.h> 
#include <XBitmaps/German_Bd.xbm>
#include <XBitmaps/German_Int.xbm>

void ViewGraph(window& W, facs_util &EX, restrictions& Restr);

static window* Wp;
static restrictions* Restrp;
static facs_util* EXp;
static list<location> DrawSol;
static list<point>* Cp; // green polygon to be drawn into window (convex hull)
static list<polygauge>* DrawGp; // list of gauges to be drawn into window
static list<polygon>* LPolyp; // list of blue polygons to be drawn into window
static bool WI; // flag for drawing weights into window
static bool NO_TEXT; // flag for showing number of optimum
static int LineAlg; // label for algorithm

static void redraw_func() { ViewGraph(*Wp, *EXp, *Restrp); }


void facs_util::SaveLoc(ostream& file)
/********************************************************************
* Purpose   : Saves a location file
* Returns   : %
* Influences: %
* Date      : Nov 1995
* Author    : Lothar Schnell
*********************************************************************/
{
	string tmp,txt;
	list_item pointer_to_location;
	list_item pointer_to_txt;
	int dim=head().dim();
	int Q=head().wht_dim();
	location temp(dim,Q);

	if (!file) lola_error("Can't open file !",1512,1);
	else {
		tmp = "begin {location} ["+(string ("%d",dim))+","+(string ("%d",Q))+"]";
		file << tmp << "\n";
		pointer_to_txt=LocTxt.first();
		for(pointer_to_location=first(); pointer_to_location; pointer_to_location=succ(pointer_to_location))
		{
			temp=contents(pointer_to_location);
			txt=LocTxt.contents(pointer_to_txt);
			pointer_to_txt=LocTxt.succ(pointer_to_txt);
			if ((txt.pos("]") > -1) || (txt.pos("[") > -1)) {
				// remove brackets to make it a legal location file
				lola_message("Illegal brackets in location name\""+txt+"\" replaced by parentheses",0);
				txt = txt.replace_all("]","}");
				txt = txt.replace_all("[","{");
			}
			file << temp << " " << "[" << txt << "]"<<"\n";
		}
		file << "end {location}" << "\n";
	}
}


void facs_util::ReadLoc_Res(ifstream& file)
/********************************************************************
* Purpose   : Reads a location file with results at the end 
* Returns   : %
* Influences: %
* Date      : Oct 1995
* Author    : Lothar Schnell
*********************************************************************/
{
	long act_pos;
	double objval;
	string tmp, normtxt, normact;
	string norm="with NORM  ";

	ReadLoc(file);

	int Q=head().wht_dim();
	location temp(2,Q);

	file >> tmp >> tmp;
	act_pos = file.tellg();
	do {
		file.seekg (act_pos);
		file >> temp[0] >> temp[1];
		temp(0)=0.0;
		act_pos = file.tellg();
		file >> tmp;
		solution.append(temp);
	} while ((tmp != "end") && !file.eof() );
	file >> tmp >> tmp >> objval;
	file >> tmp >> tmp;
	file >> normact;
	normtxt = norm+normact;
}




void facs_util::SaveLoc_Res(ostream& file, double objval, string normact)
/********************************************************************
* Purpose    : Saves a location file with results at the end
* Returns    : %
* Influences : %
* Date       : Nov 1995
* Author     : Lothar Schnell
* Last Update: Nov 1998 (Holger Hennes)
*********************************************************************/
{
	string tmp;
	int dim=head().dim();
	int Q=head().wht_dim();
	location temp(dim,Q);

	if (!file) lola_error("Can't open file !",1513,1);
	else {
		SaveLoc(file);
		file << "begin {result}" << endl;
		file << "begin {classification} " << endl << normact<< endl;
    file << "end {classification}" << endl;
		file << "begin {objective value}" << endl;
    file << string ("%13.3lf",objval) << endl << "end {objective value}" << endl;
		this->WriteOpt(file);
		file << "end {result}" << endl;
	}
}


void ViewGraph(window& W, facs_util &EX, restrictions& Restr)
/********************************************************************
* Purpose   : Show optimum and objective value with actual norm graphically
* Returns   : %
* Depends on: file-global variables DrawGp, Cp, LPolyp, MI, DrawSol
* Influences: facs_utils::View, redraw_func
* Date      : Oct 1995
* Author    : Lothar Schnell
*********************************************************************/
{
	int i=0;
	double scale=20; // number of ticks on each coordinate axis
	double xs,ys;
	double winmaxx,winminx,winminy;
	double max_loc,diff_loc;
	double max_all_x, max_all_y;
	double min_all_x, min_all_y;
	double diff_all;
	double eps; // points less than sqrt(eps) apart are considered identical
	double sqrteps; // sqrt(eps)
	list_item pointer_to_location, it; // running variables
	int Q=EX._Q();
	location temp(2,Q);
	point p,q,p_old,r; // running variables
	double qx,qy; // scratch space for coordinates
	string tmp,txt,txtopt;
	segment stmp;
	// remember to put it back at the end of the routine
	// Note: during window::read_mouse it can happen, that this function
	// is called with xor_mode, which yields strange results
	drawing_mode drawmode_save = W.set_mode(src_mode);

	// stop flushing for time of redraw
	W.set_flush(false);

	/***********************************************************************/
	/*  determine window size                                              */
	/***********************************************************************/
	// fit in locations
	max_loc = L_MAX(EX.max_direction_value(0),EX.max_direction_value(1));
	diff_loc = L_MAX(EX.diff_value(0),EX.diff_value(1));
	if (Restr.size()>0 && Restr.head()->subtype()==t_forbidden_region) { // fit in restrictions
		double diff_restr,max_restr;
		max_restr = L_MAX(Restr_max_dir(0,Restr),Restr_max_dir(1,Restr));
		diff_restr = L_MAX(Restr_diff_value(0,Restr),Restr_diff_value(1,Restr));
		diff_all = L_MAX(diff_loc,diff_restr);

		winmaxx = L_MAX(max_loc,max_restr);

		max_all_x=L_MAX(EX.max_direction_value(0),Restr_max_dir(0,Restr));
		max_all_y=L_MAX(EX.max_direction_value(1),Restr_max_dir(1,Restr));
		min_all_x=L_MIN(EX.min_direction_value(0),Restr_min_dir(0,Restr));
		min_all_y=L_MIN(EX.min_direction_value(1),Restr_min_dir(1,Restr));
	}
	else {
		winmaxx=max_loc;
		diff_all=diff_loc;
		max_all_x=EX.max_direction_value(0);
		max_all_y=EX.max_direction_value(1);
		min_all_x=EX.min_direction_value(0);
		min_all_y=EX.min_direction_value(1);
	}
	// fit in solution
	if (DrawSol.size() == 4) {
		double min_solx=L_INF, min_soly=L_INF, max_solx=0, max_soly=0;
		forall(temp,DrawSol) {
			if (temp[0] < min_solx) min_solx=temp[0]; 
			if (temp[1] < min_soly) min_soly=temp[1]; 
		}
		forall(temp,DrawSol) {
			if (temp[0] > max_solx) max_solx=temp[0]; 
			if (temp[1] < max_soly) max_soly=temp[1]; 
		}
		if (min_solx < min_all_x) min_all_x=min_solx;
		if (min_soly < min_all_y) min_all_y=min_soly;
		if (max_solx > max_all_x) max_all_x=max_solx;
		if (max_soly > max_all_y) max_all_y=max_soly;
		diff_all = L_MAX(max_all_x - min_all_x,max_all_y - min_all_y);
	}

	// allow for 20% left and lower margin for axis lables
	winminx = min_all_x - (0.2 * fabs(diff_all));
	winminy = min_all_y - (0.2 * fabs(diff_all));
	winmaxx = L_MAX(max_all_x + 0.2*diff_all, // so all x-coordinates incl. axis arrows. fit in
		(max_all_y - winminy + 0.2*diff_all)*double(W.width())/double(W.height()) + winminx // so all y coordinates fit in
		);
	winmaxx += 0.1*(winmaxx-winminx); // 10% right and upper margin

	W.init(winminx, winmaxx, winminy); // side-effect: W.clear()
	W.set_flush(false);

	bool german=false;
	color xbm_col(255,255,50);
	char *xbm1,*xbm2;

  if (EX.loc_text().head()=="Kiel") {
		german=true;
	  W.init(0,100,0);
	  xbm1 = W.create_bitmap(400,500,German_Int_bits);
	  W.put_bitmap(16,5,xbm1,xbm_col);
	  xbm2 = W.create_bitmap(400,500,German_Bd_bits);
	  W.put_bitmap(16,5,xbm2);
  }

	double minx = min_all_x - (0.1 * diff_all);
	double miny = min_all_y - (0.1 * diff_all);
	double maxx = max_all_x + (0.1 * diff_all);
	double maxy = max_all_y + (0.1 * diff_all);

	sqrteps = diff_all/100.0; 
	eps = sqrteps*sqrteps; // eps is a squared value, see above

	/************************************************************************/
	/* draw coordinate axis                                                 */
	/************************************************************************/
	double dx   = (maxx - minx)/scale;
	double dy   = (maxy - miny)/scale;

	stmp=segment(minx,miny,maxx,miny);
	W.draw_arrow(stmp,black);
	stmp=segment(minx,miny,minx,maxy);
	W.draw_arrow(stmp,black);
	xs = minx;
	ys = miny;

	for(i=0;i<scale;++i)
	{
		stmp=segment(xs,miny,xs,min_all_y-(0.13*diff_all));
		W.draw_segment(stmp,black);
		stmp=segment(minx,ys,min_all_x-(0.13*diff_all),ys);
		W.draw_segment(stmp,black);
		if ((i%5)==0)
		{
			tmp = string ("%3.1lf",xs);
			p = point(xs,min_all_y-(0.15*diff_all));
			W.draw_ctext(p,tmp,black);
			tmp = string ("%3.1lf",ys);
			p = point(winminx+0.1*fabs(winminx),ys);
			W.draw_text(p,tmp,black);
		}
		xs += dx;
		ys += dy;
	}

	/***********************************************************************
	* draw solution
	************************************************************************/
	// DrawSol is treated as a list of situations (i.e. polygons). The first
	// weight serves as situation index and consecutive items with identical
	// index make up one situation
	double  sit_index; // track current situation
	list<point> Current_Situation; // collects points of current situation
	int sit_nr = 0; // counts the situations
	color col(0,0,255);
	// put sentinel at the end of DrawSol (wont be recognized as situation)
	sit_index=DrawSol[DrawSol.last()](0);
	DrawSol.append(DrawSol[DrawSol.last()]);
	DrawSol[DrawSol.last()](0) = sit_index + 1;
	sit_index = DrawSol[DrawSol.first()](0);
	// make sure first point gets drawn
	p = DrawSol.head().loc2point().translate(0.0,5*sqrteps);
	for(pointer_to_location = DrawSol.first(); 
	pointer_to_location;
	pointer_to_location = DrawSol.succ(pointer_to_location))
	{
		location &cur_loc = DrawSol[pointer_to_location]; // avoid unecessary copy

		p_old = p;
		p = cur_loc.loc2point();
		if (sit_index != cur_loc(0))
		{ // situation has changed
			if (LineAlg >= 300 && LineAlg < 400) {
				if (Current_Situation.size() == 2) {
					list_item pnts = Current_Situation.first();
					point pnt1 = Current_Situation.contents(pnts);
					pnts = Current_Situation.succ(pnts);
					point pnt2 = Current_Situation.contents(pnts);
					W.draw_line(pnt1,pnt2,col);
				} else {
					location tmp = DrawSol.contents(pointer_to_location);
					point pnt1 = tmp.loc2point();
					pointer_to_location = DrawSol.succ(pointer_to_location);
					tmp = DrawSol.contents(pointer_to_location);
					point pnt2 = tmp.loc2point();
					W.draw_line(pnt1,pnt2,col);
					pointer_to_location = DrawSol.succ(pointer_to_location);
					p_old = pnt2; p = pnt2;                                 
				}
			} else {
				if (Current_Situation.size()>=4) {
					Current_Situation = CONVEX_HULL(Current_Situation);
					W.draw_filled_polygon(Current_Situation,col);
				} else {
					W.draw_polygon(Current_Situation,col);
				}
			}
			// label the situation
			++sit_nr;
			txtopt = "Opt" + (string ("%1d",sit_nr));
			if (Current_Situation.size() == 1) {
				// just a little left and under
				q = point(p.xcoord()-diff_loc/50.0,p.ycoord()-diff_loc/50.0);
			} else { 
				// exactly in the center of gravity
				// (well, actually 1/P/./l_2/max, but that's a little ineffective)
				q = point(0,0);
				for(it = Current_Situation.first();
				it; it = Current_Situation.succ(it))
					q = q.translate(Current_Situation[it].to_vector());
				q = point(q.xcoord()/Current_Situation.size(),
					q.ycoord()/Current_Situation.size());
			}
			if(!NO_TEXT)  W.draw_ctext(q,txtopt,col);
			Current_Situation.clear(); // let's start a new situation
			sit_index = cur_loc(0);
		} // if situation changed
		// draw only new nodes
		//W.set_node_width(2);
		if (p_old.sqr_dist(p) > eps) W.draw_filled_node(p,col);
		Current_Situation.append(p);
	} // loop over DrawSol
	// remove sentinel
	(void) DrawSol.Pop();

  /************************************************************************
   * draw locations
   ************************************************************************/
	for(pointer_to_location=EX.first(); 
	pointer_to_location;
	pointer_to_location=EX.succ(pointer_to_location))
	{
		temp=EX.contents(pointer_to_location);
		txt=EX.name(pointer_to_location);
		p=temp.loc2point();
		W.draw_point(p,black);
		W.draw_text(p,txt,black);
		if (WI) { // print list of weights centered above the location
			txt = string("%1.2lf",temp(0));
			for (i = 1; i < Q; i++) txt += string (",%1.2lf",temp(i));
			qx = p.xcoord()-diff_all/50.0;
			qy = p.ycoord()+diff_all/50.0;
			q = point(qx,qy);
			W.draw_ctext(q,txt,black);
		}
	}

	/*******************************************************************
	* draw restrictions
	********************************************************************/
	if (Restr.size() > 0) DrawRestr(W,Restr);

  /***********************************************************************
   * draw convex hull of locations (one green polygon)
   ***********************************************************************/
	if (Cp) W.draw_polygon(*Cp,green);

  /***********************************************************************
   * draw critical situations (list of blue polygons)
   ***********************************************************************/
	if (LPolyp) {
		polygon poltmp;

		forall(poltmp,*LPolyp) W.draw_polygon(poltmp,col);
	}

  /***********************************************************************
   * draw gauges
   ***********************************************************************/
	if (DrawGp->size()>0) {
		polygauge G,Gtmp;
		list<point> pl,pltmp;
		segment s;
		double scale=(fabs(W.xmax())-fabs(W.xmin()))/30.0;
		forall(G,*DrawGp) {
			pl.clear();
			pltmp.clear();
			pl=G.vertices(G.origin());
			forall(p,pl) { // draw the fundamental directions
				s = segment(G.origin(),p);
				s = segment(G.origin(),s.direction(),s.length()*scale);
				W.draw_segment(s,black);
				pltmp.append(s.end());
			}
			W.draw_polygon(pltmp,black); // draw the unit circle
		}
	}
	(void) W.set_mode(drawmode_save); // put drawing mode back to normal
	W.set_flush(true); // let's flush again
	W.flush(); // right now!
	if(german) { W.del_bitmap(xbm1); W.del_bitmap(xbm2); }
} // void ViewGraph(window& W, facs_util &EX, restrictions& Restr)

void facs_util::View(double objval, string normact, list<location>& AlgSol, restrictions& Restr)
{
	list<polygauge> LP;
	list<int> LI;
	list<polygon> LPoly;

	this->View(objval, normact, AlgSol, Restr, LP, LI, LPoly);
}

void facs_util::View(double objval, string normact, list<location>& AlgSol, restrictions& Restr, list<polygauge>& LP, list<int>& LI)
{
	list<polygon> LPoly;
	this->View(objval, normact, AlgSol, Restr, LP, LI, LPoly);
}

void facs_util::View(double objval, string normact, list<location>& AlgSol, restrictions& Restr, list<polygon>& LPoly) 
{
	list<polygauge> LP;
	list<int> LI;

	this->View(objval, normact, AlgSol, Restr, LP, LI, LPoly);  
}

void facs_util::View(double objval, string normact, list<location>& AlgSol, restrictions& Restr, list<polygauge>& LP, list<int>& LI, int choice) {
	LineAlg = choice;
	list<polygon> LPoly;
	this->View(objval, normact, AlgSol, Restr, LP, LI, LPoly);
}

void facs_util::View(double objval, string normact, list<location>& AlgSol, restrictions& Restr, list<polygauge>& LP, list<int>& LI, list<polygon>& LPoly)
/********************************************************************
* Purpose   : Show optimum and objective value with actual norm
* Arguments : objval : objective value at AlgSol
*             normact : ASCII-name of active distance measure 
*             AlgSol : list of polygons making up the solution
*             Restr : active restricitions
*             LP : list of gauges to choose from for local distance measures
*             LI : maps locations to gauges
*             LPoly: List of additional polygons painted in blue
* Returns   : %
* Influences: %
* Date      : Nov 1996
* Author    : Lothar Schnell
* Last Update: Nov 1998 (Holger Hennes)
*********************************************************************/
{
	string title=("LOLA VERSION 2.0   View Graphics:  ");
	title+=("%s",normact);
	window W(600,650,title);
	W.set_node_width(4);
	W.set_show_coordinates(true);
	W.set_redraw(redraw_func);

    facs_util EX = *this;
	if (EX.size()<1) lola_error("There is less than one location",1514,1);
	WI=false;
	NO_TEXT=true;
	int i;
	int button; // button pressed by user
	double objind;
	int Q=head().wht_dim();
	location temp(2,Q);
	point p,q;
	polygauge G,Gtmp;
	list<polygauge> DrawG; // list of gauges being drawn at locations
	array<list_item> Loc2DrawG(size()); /* maps locations into the drawn gauges
	NULL means: no gauge drawn there */
	for(i = 0; i < size(); i++) { 
		Loc2DrawG[i] = (list_item) NULL; // no gauges drawn so far
	}

    list<point> Opt;
	list<point> C; // the convex hull of the locations
	list<point> EmptyPolygon; // an empty Polygon
	string tmp,txt;
	string file_name="results";
	string normtxt;
	string norm="for  ";
	normtxt = norm + normact;
	string line;

    solution.clear();
	DrawSol.clear();
	forall(temp,AlgSol) solution.append(temp);
	forall(temp,AlgSol) DrawSol.append(temp);

    W.buttons_per_line(3);

    objind = solution.head()(0);
    i = 0;

    W.button("Save Results",1);
//	W.button("Load Results",2);
	W.button("View Results",4);
	W.button("Convex Hull",3);
	W.button("Show/Hide Weights",5);
	if (LP.size() > 1)  W.button("View Gauge",6);
	if (LPoly.size() > 1)  W.button("Crit Situation",7);
	W.button("Show/Hide Optima",10);
	W.button("Refresh",8);
	W.button("Close",9);

	EXp = &EX;
	Wp = &W;
	Restrp = &Restr;
	LPolyp = (LPoly.size())?&LPoly:(list<polygon> *) NULL;
	Cp = (list<point> *) NULL; // no convex hull so far
	DrawGp = &DrawG;

	W.display(650,150);
	redraw_func();

	for(;;) { 
    
		button = W.read_mouse(p);
    
		if (button == -1) { // mouse points into the drawing area
		// check, if the mouse points at a location
			int pos=0;
			double eps = (W.xmax()-W.xmin())/100.0; // 1/100th of window width is close enough
			list_item it=first();
      
			while (it) {
				temp=contents(it);
				q = temp.loc2point();
				if ((fabs(p.xcoord()-q.xcoord())<=eps) &&
					(fabs(p.ycoord()-q.ycoord())<=eps)) break;
				it = succ(it);
				++pos;
			}
      
      // depict local distance measure at the location being pointed at
			if (it) { // we found it
				if (Loc2DrawG[pos]) { // there is already a gauge there
					(void) DrawG.del_item(Loc2DrawG[pos]); // so delete it
					Loc2DrawG[pos] = (list_item) NULL;
				} else { // draw a new gauge at the location
					if (LP.size() > 0) { // if there are gauges
		// determine local gauge,
						G=polygauge(LP[LP[LI[LI[pos]]]]);
						G=G.unit(); // scale,
						Gtmp=G.translate(q); // move,
						Loc2DrawG[pos] = DrawG.append(Gtmp); // and schedule for display
					} else if (normact=="l1" || normact=="linf") {
	    // draw standard gauge
						G = (normact=="l1")?G.l1():G.linf();
						Gtmp=G.translate(q);
						Loc2DrawG[pos] = DrawG.append(Gtmp);
					}
				}
				redraw_func();
			}
		} // if (button == -1) 
    
		if (button == 1) // "Save Results"
		{
			objind = solution.head()(0);
			panel PSave("Save results"); 
			PSave.string_item("file:",file_name);
			PSave.text_item("");
			PSave.text_item("");
			PSave.buttons_per_line(2);
			int bScancel = PSave.button("cancel");
			int bSok = PSave.button("ok");
			int butS = PSave.open(780,150);
			if (butS == bScancel) { 
				file_name = "results";
				continue;
			} else if (butS == bSok) {
				ofstream F(file_name,ios::out);
				if (! F) { 
					tmp = string ("Can't open file \"%s\"!",file_name);
					W.message(tmp);
				} else SaveLoc_Res(F,objval,normact);
			}
		} // if (button == 1) // "Save Results"
    
//		if (button == 2) // "Load Results"
/*		{
			panel PLoad("Load results");
			PLoad.string_item("file:",file_name);
			PLoad.text_item("");
			PLoad.text_item("");
			PLoad.buttons_per_line(2);
			int bLcancel = PLoad.button("cancel");
			int bLok = PLoad.button("ok");
			int butL = PLoad.open(780,200);
			if (butL == bLcancel)
			{ 
				file_name = "results";
				continue;
			} else if (butL == bLok)
			{
				ifstream F(file_name,ios::in);
				if (! F)
				{
					tmp = string ("Can't open file \"%s\"!",file_name);
					W.acknowledge(tmp);
				} else {
					ReadLoc_Res(F);
				}
			}
		} // if (button == 2) //"Load Results"*/
		
		if (button == 4) // "View Results"
		{
			objind = solution.head()(0);
			int i=0;
			panel PResults("Results of computation"); 
			PResults.text_item(string("%-21s            %-13s","OPTIMUM:","OBJECTIVE VALUE:"));
			PResults.text_item("");

			forall(temp,solution) {
				if(objind != temp(0)) {
					PResults.text_item("------------------------------ ");
					PResults.text_item("");
					objind = temp(0);
				}
				if(i==0) {
					tmp = string ("(%8.3lf ,%8.3lf )            %13.3lf ",temp[0],temp[1],objval);
					++i;
				} else tmp = string ("(%8.3lf ,%8.3lf )",temp[0],temp[1]);
				PResults.text_item(tmp);
				PResults.text_item("");
			}
			PResults.text_item(normtxt);
			PResults.button("OK",10);
			PResults.display(window::max,window::min);

			int butP = 0;

			butP = PResults.read_mouse();
			if (butP == 10) PResults.close();
		} // if (button == 4) // "View Results"
    
		if (button == 3) // "Convex Hull"
		{ 
			if (length() >= 3) {
				if (Cp) { 
				Cp = (list<point> *) NULL;
				} else {
					if (C.size() == 0) { // compute convex hull of locations
						list<point> Locations;
						list_item it;

						for(it = first(); it; it = succ(it)) // convert to points
							Locations.append(contents(it).loc2point());
						C = CONVEX_HULL(Locations);
					}
					Cp = &C;
				}
				redraw_func();
			}
		} // if (button == 3) "Convex Hull"
    
		if (button == 5) // "Show Weights"
		{
			if(WI) WI = false;
			else WI = true;
			redraw_func();
		}
		
		if (button == 10) // "Show Optimums"
		{
			if(NO_TEXT) NO_TEXT = false;
			else NO_TEXT = true;
			redraw_func();
		}
    
	    if (button == 6) // "View Gauge"
		{
			int gauge_num=0,but;
			polygauge GV;
			panel PGNum("Number of gauge"); 
			PGNum.text_item("Please enter the index of the desired gauge");
			PGNum.text_item("");
			PGNum.text_item(string("(between 0 and %i):",LP.size()-1));
			PGNum.int_item("",gauge_num);
			PGNum.button("Ok",1);
			PGNum.button("Quit",2);
			PGNum.display(window::max,window::min);
			but = PGNum.read_mouse();
			if(but == 2) {
				gauge_num=-1; PGNum.close();
			}
			else {
				while((gauge_num<0) || (gauge_num>LP.size()-1)) {
					but = PGNum.read_mouse();
					if(but == 2) {
						gauge_num=-1; PGNum.close(); break;
					}
				}
				if(gauge_num > -1) {
					PGNum.close();
					GV=polygauge(LP.contents(LP.get_item(gauge_num)));
					ViewGauge(GV);
				}
			}
		} //   if (button == 6) // "View Gauge"
    
    
		if (button == 7)  {// "Critical Situations"
      		polygon poltmp;
	
			if (LPolyp) {
				LPolyp = (list<polygon> *) NULL;
			} else {
				LPolyp = &LPoly;
			}
			redraw_func();
		}
    
		if (button == 8) redraw_func(); // "Refresh"
		if (button == 9) break; // "Close"
    } // event-handling loop
} // void facs_util::View(double objval, string normact, list<location>& AlgSol, restrictions& Restr, list<polygauge>& LP, list<int>& LI)
