#include <LOLA/restr_util.h> 

double Restr_max_dir(int i, restrictions& Restr)
/********************************************************************
* Purpose   : Computes the maximal coordinate (i-th) for all restrictions
* Returns   : maximum coordinate
* Influences: %
* Date      : Feb 1996
* Author    : Lothar Schnell
*********************************************************************/
{
  restriction *restr_temp;
  double coord=0;
  double maximum=0;
  double max_temp=0;
  point p_temp;
  for(list_item it = Restr.first(); it; it = Restr.succ(it))
  {
    restr_temp=Restr.contents(it);
    if (restr_temp->subtype()==restr_p) {
       restr_poly *restrict = NULL;
       restrict=(restr_poly*)(restr_temp);
       list<point> pl(restrict->vertices());
       forall(p_temp,pl) {
          if (i==0) {
            coord = L_MAX(coord,p_temp.xcoord());
          } else {
            coord = L_MAX(coord,p_temp.ycoord());
          }
       }
       max_temp=coord;
    }
    else if (restr_temp->subtype()==restr_r) {
       restr_rect *restrict = NULL;
       restrict=(restr_rect*)(restr_temp);
       list<point> pl(restrict->vertices());
       forall(p_temp,pl) {
          if (i==0) {
            coord = L_MAX(coord,p_temp.xcoord());
          } else {
            coord = L_MAX(coord,p_temp.ycoord());
          }
       }
       max_temp=coord;
    }
    else if (restr_temp->subtype()==restr_c) {
       restr_circle *restrict = NULL;
       restrict=(restr_circle*)(restr_temp);
       if (i==0) max_temp=restrict->center().xcoord()+restrict->radius();
       else max_temp=restrict->center().ycoord()+restrict->radius();
    }
    maximum =  L_MAX(max_temp,maximum);
  }
  return maximum;
}


double Restr_min_dir(int i, restrictions& Restr)
/********************************************************************
* Purpose   : Computes the minimal coordinate (i-th) for all restrictions
* Returns   : minimum coordinate
* Influences: %
* Date      : Feb 1996
* Author    : Lothar Schnell
*********************************************************************/
{
  restriction *restr_temp;
  double coord=L_INF;
  double minimum=L_INF;
  double min_temp=0;
  point p_temp;
  for(list_item it = Restr.first(); it; it = Restr.succ(it))
  {
    restr_temp=Restr.contents(it);
    if (restr_temp->subtype()==restr_p) {
       restr_poly *restrict = NULL;
       restrict=(restr_poly*)(restr_temp);
       list<point> pl(restrict->vertices());
       forall(p_temp,pl) {
          if (i==0) {
            coord = L_MIN(coord,p_temp.xcoord());
          } else {
            coord = L_MIN(coord,p_temp.ycoord());
          }
       }
       min_temp=coord;
    }
    else if (restr_temp->subtype()==restr_r) {
       restr_rect *restrict = NULL;
       restrict=(restr_rect*)(restr_temp);
       list<point> pl(restrict->vertices());
       forall(p_temp,pl) {
          if (i==0) {
            coord = L_MIN(coord,p_temp.xcoord());
          } else {
            coord = L_MIN(coord,p_temp.ycoord());
          }
       }
       min_temp=coord;
    }
    else if (restr_temp->subtype()==restr_c) {
       restr_circle *restrict = NULL;
       restrict=(restr_circle*)(restr_temp);
       if (i==0) min_temp=restrict->center().xcoord()-restrict->radius();
       else min_temp=restrict->center().ycoord()-restrict->radius();
    }
    minimum = L_MIN(min_temp,minimum);
   }
   return minimum;
}

double Restr_diff_value(int i, restrictions& Restr)
/********************************************************************
* Purpose   : Computes the difference between minimum and maximum
* Returns   : difference
* Influences: %
* Date      : Feb 1996
* Author    : Lothar Schnell
*********************************************************************/
{
  return Restr_max_dir(i,Restr)-Restr_min_dir(i,Restr);
}

void DrawRestr(window& W, restrictions& Restr)
/********************************************************************
* Purpose    : Reads a restriction file
* Returns    : %
* Influences : %
* Date       : Nov 1995
* Author     : Lothar Schnell
* Last Update: Oct 1998 (Holger Hennes)
*********************************************************************/
{
	color c=red;
	string txt=" ";
	restriction *restr_temp;
	if (Restr.size()<=0) return;
	restr_temp=Restr.head();
	if (restr_temp->type()!=t_barrier) {
		if (Restr.head()->inout()) {
			c = red;
			txt="     forbidden region inside"; 
		}
		else {
			c = violet;
			txt="     forbidden region outside"; 
		}
		W.draw_text(1.1*W.xmin(),0.95*W.ymax(),txt,c);
		forall(restr_temp,Restr) {
			if (restr_temp->subtype()==restr_p) {
				restr_poly *restrict = NULL;
				restrict=(restr_poly*)(restr_temp);
				//polygon P(restrict->vertices());
				//W.draw_polygon(P,c); in LEDA 3.6 vergessen, deshalb:
				W.draw_polygon(restrict->vertices(),c);
			}
			else if (restr_temp->subtype()==restr_r) {
				restr_rect *restrict = NULL;
				restrict=(restr_rect*)(restr_temp);
				//polygon P(restrict->vertices());
				//W.draw_polygon(P,c);
				W.draw_polygon(restrict->vertices(),c);
			}
			else if (restr_temp->subtype()==restr_c) {
				restr_circle *restrict = NULL;
				restrict=(restr_circle*)(restr_temp);
				circle C(restrict->center(),restrict->radius());
				W.draw_circle(C,c);
			}
/*
			else if (restr_temp->subtype()==restr_e) {
				restr_ell *restrict = NULL;
				restrict=(restr_ell*)(restr_temp);
				list<line> ellaxis=restrict->axis();
				if (ellaxis.head().horizontal() && ellaxis.tail().vertical()) {
					list<double> ellr=restrict->radi();
					W.draw_ellipse(restrict->center(),ellr.head(),ellr.tail(),c);
				}
			}
*/
		}
	}
	else {
		c = red;
		txt = "     barrier";
		W.draw_text(1.1*W.xmin(),0.95*W.ymax(),txt,c);
		if (restr_temp->subtype()==t_polygon) {
			polygon_barrier *restrict = NULL;
			restrict=(polygon_barrier*)(restr_temp);
			W.draw_polygon(restrict->vertices(),c);
		}
		else if (restr_temp->subtype()==t_segment) {
			segment_barrier *restrict = NULL;
			restrict=(segment_barrier*)(restr_temp);
			segment S(restrict->point1(),restrict->point2());
			W.draw_segment(S,c);
		}
		else if (restr_temp->subtype()==t_circle) {
			circle_barrier *restrict = NULL;
			restrict=(circle_barrier*)(restr_temp);
			circle C(restrict->center(),restrict->radius());
			W.draw_circle(C,c);
		}
	}
}
  

restr_poly ReadPoly(ifstream& file)
/********************************************************************
* Purpose   : Reads a polyhedron
* Returns   : polyhedron
* Influences: %
* Date      : Nov 1995
* Author    : Lothar Schnell
*********************************************************************/
{
  long act_pos;
  string tmp;
  point p;
  list<point> Poly;
    
  act_pos = file.tellg();
  while (tmp != "end" && !file.eof())
  {
    file.seekg (act_pos);
    file >> p;
    act_pos = file.tellg();
    Poly.append(p);
    file >> tmp;
  }
  file.seekg (act_pos);
  return Poly;
}

restr_circle ReadCirc(ifstream& file)
/********************************************************************
* Purpose   : Reads a circle
* Returns   : circle
* Influences: %
* Date      : Nov 1995
* Author    : Lothar Schnell
*********************************************************************/
{
  point m;
  double a;
  double bar=0;
  char Z;
  long act_pos;
  string tmp;
    
  file >> m >> a;

  act_pos = file.tellg();
  file >> Z;
  tmp = Z;
  if (tmp == "[") file >> bar >> Z;
  else file.seekg (act_pos);
    
  restr_circle tempc(m,a);
  restr_circle result(tempc);
  return result;
}

restr_rect ReadRect(ifstream& file)
/********************************************************************
* Purpose   : Reads a rectangle
* Returns   : rectangle
* Influences: %
* Date      : Nov 1995
* Author    : Lothar Schnell
*********************************************************************/
{
  double rada,radb,alpha;
  point p;
  double bar=0;
  char Z;
  long act_pos;
  string tmp;
    
  file >> p >> rada >> radb >> alpha;

  act_pos = file.tellg();
  file >> Z;
  tmp = Z;
  if (tmp == "[") file >> bar >> Z;
  else file.seekg (act_pos);
  alpha *= LEDA_PI/180;

  restr_rect tempr(p,rada,radb,alpha);
  restr_rect result(tempr);
  return result;
}

