#include <LOLA/discrete.h>
#include <avexec32.h>

double gcdis(double x1, double y1, double x2, double y2)
{
	double a=(90-x1)/57.2958;
	double b=(90-x2)/57.2958;
	double c=(y2-y1)/57.2958;
	double bma=b-a;
	double cm=1-cos(c);
	double sd=acos(cos(bma)-sin(a)*sin(b)*cm);
	return sd*6367.83;
}

matrix plan2disc(facs_util EX, facs_util EX2)
{
	location loc1,loc2;
	int i=0,j=0;
	matrix Mat(EX.length(),EX2.length());
	forall(loc1, EX) {
		j=0;
		forall(loc2, EX2) {
			Mat(i,j)=gcdis(loc1[0],loc1[1],loc2[0],loc2[1]);
			Mat(i,j)*=loc1(0);
			j++;
		}
		i++;
	}
	return Mat;
}

lolagraph plan2net(facs_util EX)
{
	lolagraph LG;
	location loc1,loc2;
	matrix Mat(EX.length(),EX.length());
	int i=0,j=0;
	forall(loc1,EX) {
		node_weight NW;
		NW.append(loc1(0));
		LG.new_node(NW);
		j=0;
		forall(loc2,EX) {
			Mat(i,j)=gcdis(loc1[0],loc1[1],loc2[0],loc2[1]);
			j++;
		}
		i++;
	}
	node v,w;
	i=0;
	forall_nodes(v,LG) {
		j=0;
		forall_nodes(w,LG) {
			LG.new_edge(v,w,Mat(i,j));
			j++;
		}
		i++;
	}
	return LG;
}

void output(ostream& out, list<location> L, facilities EX)
{
	list_item it, next_it;
	int i,j=0;
	location curr,org;
	out << "begin{polygonlist}" << endl;
	for(it = L.first(); it; it = L.succ(it)) {
		curr = L.contents(it);
		for(i = 0; i < curr.dim(); i++) {
			out << curr[i] << " ";
		}
		j=0;
		for (next_it=EX.first(); next_it; next_it=EX.succ(next_it)) {
			org=EX.contents(next_it);
			if (org == curr) {
				out << EX.loc_text()[EX.loc_text()[j]];
				break;
			}
			else j++;
		}
		out << endl;
    }
	out << "end{polygonlist}" << endl;
} 


int main(int argc, char **argv)
{
	facs_util EX;
	string choice= string(argv[1]);
	string locfile = string(argv[2]);
	ifstream file (locfile,ios::in);
	EX.ReadLoc(file);
	string algo = string(argv[3]);
    const char *ptxt=argv[4];
	double fix=atoi(ptxt);

	locfile = string(getenv("tmp"));
	locfile += string("/results.txt");
	ofstream outfile (locfile,ios::out);


    if (choice=="dis") {
		location loc;
		facs_util EX2;
		if (fix) {
			EX2=EX;
			for(int k=0;k<EX2.length();k++) EX2[EX2[k]](0)=fix;
		}
		else {
			locfile = string(argv[5]);
			ifstream file (locfile,ios::in);
			EX2.ReadLoc(file);
		}
		matrix Cost=plan2disc(EX,EX2);
		discrete DIS(EX,EX2,Cost);
		DIS.del_sol();
		double objval;
		if (algo=="Greedy") objval=DIS.UFLP_greedy();
		else if (algo=="Stingy") objval=DIS.UFLP_stingy();
		else if (algo=="Interchange") objval=DIS.UFLP_Interchange();
		else if (algo=="Dualoc") objval=DIS.UFLP_dualoc();
		matrix belong=DIS.Ret_belongto();
		
		if (DIS.alg_solution().size()==0) lola_error("Solve a problem first.",1304,1);
		outfile << "begin {result}" << endl;
		outfile << "begin {objective value}" << endl << objval << endl << "end {objective value}" << endl;
		output(outfile, DIS.alg_solution(), EX2);
		outfile << "begin {belong}" << endl;
		for (int i=0;i<belong.dim1();i++)
			for (int j=0;j<belong.dim2();j++)
			{
				if (belong(i,j)) {
					location loc1=EX[EX[i]],loc2=EX2[EX2[j]];
					outfile << loc1[0] <<" "<<loc1[1]<<" "<<loc2[0]<<" "<<loc2[1]<<endl<<flush;
				}
			}
		outfile << "end {belong}" << endl;
		outfile << "end {result}" << endl;
    }
	else
	{
		lolagraph LG=plan2net(EX);
		string obj = string(argv[4]);
		list<int> belong;
		double objval;
		list<sol_typ> LS;
		if (algo=="Greedy") {
			if (obj=="Median") LS=LG.N_greedy(fix,objval,belong,0);
			else LS=LG.N_greedy(fix,objval,belong,1);
		}
		else if (algo=="Partitioning") {
			if (obj=="Median") LS=LG.N_partitioning(fix,objval,belong,0);
			else LS=LG.N_partitioning(fix,objval,belong,1);
		}
		else if (algo=="Exchange") LS=LG.Nmed_exchange(fix,objval,belong);
		else if (algo=="Exact") LS=LG.N_median(fix,objval,belong);
		if (LS.size()==0) lola_error("Solve a problem first.",1305,1);
		list<location> LL;
		for (list_item it=LS.first();it; it=LS.succ(it)) {
			LL.append(EX[EX[LS.contents(it).source]]);
		}
		outfile << "begin {result}" << endl;
		outfile << "begin {objective value}" << endl << objval << endl << "end {objective value}" << endl;
		output(outfile, LL, EX);
		outfile << "begin {belong}" << endl;
		for (int i=0;i<belong.size();i++) {
			location loc1=EX[EX[i]],loc2=LL[LL[belong[belong[i]]]];
			outfile << loc1[0] <<" "<<loc1[1]<<" "<<loc2[0]<<" "<<loc2[1]<<endl<<flush;
		}
		outfile << "end {belong}" << endl;
		outfile << "end{result}" << endl;

	}
	outfile.close();
	return 0;
}
