%{
#include <math.h>
#include <LOLA/discrete.h>
#include <LOLA/location.h>
%}

%option noyywrap
%option c++
%option yylineno


%x Fehlerzustand
%x Eingabe
%x WeightEingabe
%x Text
%x Fertig
%x ForceNewline
%x Comment
%x Costs
%x Kosteneingabe
%x Supply
%x Demand
%x Dimension

/* Variablen */
%{
  int dQ,dd,n,m,m_sup,dcounter=0; // Q = weightdimension, d = dimension
  double dnumber; // scratch space
  list<double> dcoords,dweights; // accumulators for content of one line
  list<location> EX_sup,EX_dem; // accumulator for content of one file
  list<string> LocTxt_dem,LocTxt_sup; // accumulator for name tags
  bool dtxtflag=false; // whether there was a name tag already found in line
  string derrormessage = "Syntaxerror in location file!";
  int dcalling_condition; // return-adress for <*>-rules
  bool dlinebreak_legal = false; // flag to allow the next linebreak
//  string dfilename = ""; // name of the file parsed, for error messages
  list<double> cur_costs; // fuer ReadDiscrete
  int nulin=0; // number of matrix-lines for ReadDiscrete
%}
ws [ \t]
DIGIT [0-9]
NODIGIT [^0-9\n\r]
INTEGER ("+"|"-")?{DIGIT}+
REAL {INTEGER}(\.{DIGIT}*)?([eE]{INTEGER})?
NOEOL [^\n\015]
ID [a-z][a-z0-9]*
%{
#undef yyFlexLexer
#define yyFlexLexer disFlexLexer
#include <FlexLexer.h>
/* Routine fuer Diskrete Probleme */
void discrete::ReadDiscrete(ifstream& file)
/*********************************************************************
* Purpose   : Reads a file for discrete problems 
* Returns   : % 
* Influences: %
* Date      : Aug 1998 
* Author    : Karen Schulze 
*********************************************************************/
{
EX_dem.clear();
EX_sup.clear();
LocTxt_dem.clear();
LocTxt_sup.clear();
dcounter=0;
nulin=0;
FlexLexer* dlexer = new yyFlexLexer();
if (!file) lola_error("Can't open file!",3106,1);
else { //jetzt flexen
                int remember=2;
               while(remember > 1) remember=dlexer->yylex(&file);
                if (remember==1) { lola_error(derrormessage,3107,1); }
        }
int i,j;
// Kostenmatrix erzeugen
matrix cmat(EX_dem.length(),m_sup);
for(i=0;i<EX_dem.length();i++)
	for(j=0;j<m_sup;j++)
		cmat(i,j)=cur_costs.pop();
this->DEMAND.clear();
this->SUPPLY.clear();
this->Loctxt.clear();
this->Loctxt_sup.clear();
location loctmp;
forall(loctmp,EX_dem) this->DEMAND.append(loctmp);
forall(loctmp,EX_sup) this->SUPPLY.append(loctmp);
this->COST=cmat;
this->Loctxt=LocTxt_dem;
this->Loctxt_sup=LocTxt_sup;
delete dlexer;
}

%}

%%
	/************* A N F A N G S Z U S T A N D ************************************/
[ \n]+  /*Vor Beginn Leerzeilen zulassen */ 

	/* allow arbitrary whitespace */
<*>[[:blank:]\r]
"begin"[ ]*"{"[ ]*"discrete"[ ]*"}"[ ]*\n               { //cout << "Typ gefunden " << endl;
                                                          BEGIN(Demand);}

<ForceNewline>\n {
  /* Wait for end-of-line but allow <*>-rules to apply. This is a dummy
  * start condition if you don't want to do anything at eol or are too
  * lazy to implement an own \n-rule for some start condition */
  BEGIN(dcalling_condition);
}

	/************* D E M A N D ************************************/
<Demand>"begin"[ ]*"{"[ ]*"demand"[ ]*"}"[ ]* { // the previous rule is definitely longer and thus takes precedence
  //cout << "no dimension" ;
  dd = 2; dQ = 1; // defaults
  dcalling_condition=Eingabe;
  BEGIN(ForceNewline);
}

	/************* C o m m e n t s ************************************/
	/* quick and ineffcient way to do comments - see man flex for
	   alternatives */
<*>"\x2F\x2F".*  { /* (\x2F = /) not matching newline is important, so
		   the end-of-line information is retained */
  dlinebreak_legal = true;
}

<*>"\x2F\*" { dcalling_condition = YY_START; BEGIN(Comment); } /* \x2F = / */
<Comment>. // this has to be just one character, so */ takes precedence
<Comment>\*\x2F { BEGIN(dcalling_condition); } /* \x2F = / */


	/************* E I N G A B E  ***********************************/
	

<Eingabe>{REAL}{ws}    { // matching whitespace here makes the scanner faster ;-)) and improves error message quality
  dnumber=atof(YYText());
  dcoords.append(dnumber);dcounter++;
   //cout << "Eingabe " << dnumber << " Counter" << dcounter << endl;
  if (dcounter>=dd) {dcounter=0;BEGIN(WeightEingabe);}
} 

<Eingabe>"end"[  ]*"{"[ ]*"demand"[ ]*"}"\n  { //cout << "Ende Demand erfolgreich gefunden" << endl;
		//cout << "nun optional die Supply-Standorte" << endl;
		EX_dem=EX_sup;
		EX_sup.clear();
		LocTxt_dem=LocTxt_sup;
		LocTxt_sup.clear();
		BEGIN(Supply);
		} 

<Eingabe>"end"[  ]*"{"[ ]*"supply"[ ]*"}"\n { //cout << "Ende Supply erfolgreich gefunden" << endl;
		//cout << "nun die Kostenmatrix" << endl;
		BEGIN(Costs);
		} 

<Eingabe>"]"\n	{//cout << "Ende der Supply-Dimension" << endl;
		}

<Eingabe>. {
	    derrormessage=string("Coordinate #%d expected in discrete file at line %d, found \"%s%%s\" instead.",dcounter+1,lineno(),YYText());
	    BEGIN(Fehlerzustand);
}
	/************* S U P P L Y  ***********************************/

<Supply>"begin"[ ]*"{"[ ]*"supply"[ ]*"}"[ ]*"[" { // the previous rule is definitely longer and thus takes precedence
  BEGIN(Dimension);
}

<Supply>"begin"[ ]*"{"[ ]*"costmatrix"[ ]*"}"[ ]*\n 	{
	// no supply 
	n=0;m=0;
	m_sup=EX_dem.length(); // quadratische Matrix
        BEGIN(Kosteneingabe);
	}

<Supply>. {
  derrormessage=string("'Begin {supply} [x]' or 'Begin {costmatrix}' expected in discrete file at line %d, found \"%s%%s\" instead",lineno(),YYText());
  BEGIN(Fehlerzustand);
  }
	/************* D I M E N S I O N  ***********************************/
<Dimension>{INTEGER} {
  dQ=atoi(YYText());
  if (dQ<1 || dQ>2) {
  	derrormessage=string("Dimension between 1 or 2 expected in discrete file at line %d, found \"%s%%s\" instead",lineno(),YYText());
  	BEGIN(Fehlerzustand);
	} else 
  	BEGIN(Eingabe);
}

<Dimension>. {
  derrormessage = string("Dimension (1 or 2 possible) expected in discrete file at line %d, found \"%s%%s\" instead.",lineno(),YYText());
  BEGIN(Fehlerzustand);
}

	/************* W E I G H T E I N G A B E  ***********************************/

<WeightEingabe>{REAL}/[ \]\t\n\r/]    {
  dnumber=atof(YYText());dcounter++;
  dweights.append(dnumber);
  if (dcounter>=dQ ) BEGIN(Text);
}

<WeightEingabe>"L_INF"/[ \]\t\n\r/]    {
  dnumber=L_INF;dcounter++;
  dweights.append(dnumber);
  if (dcounter>=dQ) BEGIN(Text);
}


<WeightEingabe>. {
  derrormessage=string("Weight #%d expected in discrete file at line %d, found \"%s%%s\" instead",dcounter+1,lineno(),YYText());
  BEGIN(Fehlerzustand);
  }

	/************* C O S T S   ***********************************/
<Costs>"begin"[ ]*"{"[ ]*"costmatrix"[ ]*"}"[ ]*\n               {
n=0; // Zeilen EX_dem
m=0; // Spalten EX_sup
m_sup=EX_sup.length();
BEGIN(Kosteneingabe);
}
<Costs>. {
        derrormessage=string("'begin {costmatrix}' expected in line %d",lineno(),YYText());
        BEGIN(Fehlerzustand);
        }
        /************** K O S T E N E I N G A B E ***********/

<Kosteneingabe>\n    {
  //cout << "information please: " << EX.length() << endl;
  if (m==m_sup) {n++;m=0; //cout << "Counter zero" << endl;
                        }
  if (m!=0 ) { derrormessage=string("Number of Entries in costmatrix line %d does not match dimension.",lineno()-1); BEGIN(Fehlerzustand); }
  if (n>EX_dem.length() ) { derrormessage=string("Number of lines in costmatrix  does not match number of demand-points.",lineno()-1); BEGIN(Fehlerzustand); }

}
<Kosteneingabe>{REAL}{ws}*   {
        double onecost=atof(YYText());
        //cout << "Eingabe " << onecost << " Counter" << dcounter+1 << endl;
        m++;
        cur_costs.append(onecost);
	}

<Kosteneingabe>"L_INF"{ws}*   {
        double onecost=L_INF;
        //cout << "Eingabe " << onecost << " Counter" << dcounter+1 << endl;
        m++;
        cur_costs.append(onecost);
	}

<Kosteneingabe>"end"  {
if (n<EX_dem.length() ) { derrormessage=string("Number of lines in costmatrix  does not match dimension.",lineno()-1); BEGIN(Fehlerzustand); }
	else {//cout << "tutti paletti" << endl;
		return(0);}
}

<Kosteneingabe>. {
                derrormessage=string("unexpected text  in line %d: %s",lineno(),YYText());
                BEGIN(Fehlerzustand);
                }


	/************* T E X T   ***********************************/
<Text>"["[^\r\n\]]*"]"          {
	if (!dtxtflag) {
		string txp=YYText();
		int textlength=txp.length();
		txp = txp.del(--textlength);
		txp = txp.del(0);
		LocTxt_sup.append(txp);
		dtxtflag=true;
		// cout << "Text angehaengt " << endl;
		} else {
		derrormessage=string("Second name (\"%s\") found in discrete file at line %d.",YYText(),lineno());
		return(1);
		}
	}

<Text>\n {
//cout << "Objekt facilities erzeugen..." << endl;
location temp(dd,dQ);
int i=0; 

forall(dnumber,dcoords) temp[i++]=dnumber;
i=0;
forall(dnumber,dweights) temp(i++)=dnumber; 
EX_sup.append(temp); 
dcoords.clear();
dweights.clear();
if (!dtxtflag) LocTxt_sup.append(""); // empty name
dtxtflag=false;
dcounter=0;
BEGIN(Eingabe);
}

<Text>. {
	if (dtxtflag) {
		derrormessage=string("End of line expected in discrete file at line %d, found \"%s%%s\" instead.",lineno(),YYText());              
		} else {
		derrormessage=string("Name of location expected in discrete file at line %d, found \"%s%%s\" instead.",lineno(),YYText());
		}
	BEGIN(Fehlerzustand);
	}

	/************* F E H L E R Z U S T A N D  ***********************************/
<Fehlerzustand>{NOEOL}* {
//cout << "Fehler im Location-File..." << endl;
// insert rest of line in location of a %s
derrormessage = string(derrormessage.cstring(),YYText());
return 1;
}

<Fehlerzustand>\n { // falls der Fehler am Zeilenende war
derrormessage = string(derrormessage.cstring(),"");
return 1;
}

	/************ L e e r z e i l e n ***********************************/
<*>\n { // eat up otherwise unwanted newline
if (!dlinebreak_legal) {
switch(YY_START) {
case Eingabe:
case WeightEingabe:
{
derrormessage=string("Illegal linebreak in discrete file at line %d.",lineno() - 1);
return 1;
}
}
} else { dlinebreak_legal = false; }
}

%% 

/* Hauptprogramm, liest ein file ein, nur zum Testen, wenn nicht in LOLA eingebunden.. */
#ifdef DEBUGLEX
main(int argc, char** argv) {
  int rc;

  if (argc != 2) {
    cerr << "Please exactly one file as input.\n";
    exit(1);
  }

  ifstream file (argv[1],ios::in);

//  dfilename = string(argv[1]);
  FlexLexer* dlexer = new yyFlexLexer();
  rc = dlexer->yylex(&file);
  if (rc) { 
    cerr << string("Error(%d): ",rc) << derrormessage << endl;
    exit(1);
  }
  return 0;
}
#endif

//Local Variables:
//mode: c++
//End:

