%{
#include <math.h>
#if defined(LOLA_LIBSRC_PU) || defined(DEBUGLEX)
   #include <LOLA/facs_util.h>
#endif
%}

%option noyywrap
%option c++
%option yylineno

/* Zustaende fuer ReadMat*/

%x Dimension
%x Fehlerzustand
%x Eingabe
%x Klammer 
%x ForceNewline
%x Comment
 
/* Variablen */
%{
  int D,Counter=0; //  D = dimension
  list<double> mat_value; // accumulators for content of one line
  string Errormessage = "Syntaxerror in matrix file!";
  int Errorcode = 1501;
  int Calling_condition; // return-adress for <*>-rules
  bool Linebreak_legal = false; // flag to allow the next linebreak
  string Filename = ""; // name of the file parsed, for error messages
%}
ws [ \t]
DIGIT [0-9]
NODIGIT [^0-9\n\r]
INTEGER ("+"|"-")?{DIGIT}+
REAL {INTEGER}(\.{DIGIT}*)?([eE]{INTEGER})?
NOEOL [^\n\015]
ID [a-z][a-z0-9]*
%{
#undef yyFlexLexer
#define yyFlexLexer matFlexLexer
#include <FlexLexer.h>
#ifdef LOLA_LIBSRC_PU
	#ifdef NO_FLEX
matrix facs_util::ReadMat(ifstream& file)
/********************************************************************
* Purpose    : Reads a matrix file with the independence matrix
* Returns    : matrix
* Influences : %
* Date       : Oct 1995
* Author     : Lothar Schnell
* Last Update: Nov 1998 (Holger Hennes)
*********************************************************************/
{
cout << "ungeflextes ReadMat" << endl;
 string tmp;
 int Q=head().wht_dim();
 register int l;
 register int k;
 char Z;
 matrix w(Q,Q);

 if (!file) lola_error("Can't open file !",1508,1);
 else {
  while ((tmp != "begin") && !file.eof()) file >> tmp;

  file >> tmp;
  if (tmp!="{matrix}") lola_error("This is not a matrix file !",1509,1);

  file >> Z >> Q >> Z;

  for(l=0;l<Q;l++)
   for(k=0;k<Q;k++) file >> w(l,k);

  file >> tmp;
 }
 return w;
}
	#else // geflext
/******** ReadMat aus facs_util.h **********/
matrix facs_util::ReadMat(ifstream& file)
{
string tmp;
int Q=head().wht_dim();
register int l;
register int k;
char Z;
matrix w(Q,Q);
Counter=0;
mat_value.clear();
if (!file) lola_error("Can't open file !",1510,1);
else { //jetzt flexen
               FlexLexer* lexmat = new matFlexLexer();
                int remember=2;
               while(remember > 1) remember=lexmat->yylex(&file);
                if (remember==1) { lola_error(Errormessage,1511,1); }
        }
for (l=0;l<Q;l++)
  for(k=0;k<Q;k++)
	w(l,k)=mat_value.pop();
return w;
}
		#endif
	#endif
%}

%%
	/************* A N F A N G S Z U S T A N D ************************************/
[ \n]+  /*Vor Beginn Leerzeilen zulassen */ 

	/* allow arbitrary whitespace */
<*>[[:blank:]\r]

"begin"[ ]*"{"[ ]*"matrix"[ ]*"}"[ ]*"["               {//cout << "Anfang mat" << endl;
							BEGIN(Dimension);}

"begin"[ ]*"{"[ ]*"location"[ ]*"}"[ ]* { // the previous rule is definitely longer and thus takes precedence
  //cout << "no dimension" ;
   D=2; // defaults
  Calling_condition=Eingabe;
  BEGIN(ForceNewline);
}


<ForceNewline>\n {
  /* Wait for end-of-line but allow <*>-rules to apply. This is a dummy
  * start condition if you don't want to do anything at eol or are too
  * lazy to implement an own \n-rule for some start condition */
  BEGIN(Calling_condition);
}

	/************* C o m m e n t s ************************************/
	/* quick and ineffcient way to do comments - see man flex for
	   alternatives */
<*>"\x2F\x2F".*  { /* (\x2F = /) not matching newline is important, so
		   the end-of-line information is retained */
  Linebreak_legal = true;
}

<*>"\x2F\*" { Calling_condition = YY_START; BEGIN(Comment); } /* \x2F = / */
<Comment>. // this has to be just one character, so */ takes precedence
<Comment>\*\x2F { BEGIN(Calling_condition); } /* \x2F = / */

	/************* D I M E N S I O N ************************************/
<Dimension>{INTEGER} {
  D=atoi(YYText());
  if (D < 1) {
    Errormessage=string("Dimension too small in matrix file at line %d.",lineno());
    return 1;
  } else {
    //cout << "Dimension: " << D << endl;
    BEGIN(Klammer);
  }
}

<Dimension>. {
  Errormessage = string("Dimension expected in matrix file at line %d, found \"%s%%s\" instead.",lineno(),YYText());
  BEGIN(Fehlerzustand);
}


	/************* K L A M M E R  ***********************************/

<Klammer>"]" /* ] is OK :-) */
<Klammer>\n { BEGIN(Eingabe); } /* \n is necessary */
<Klammer>. {
  Errormessage = string("Closing bracket expected in matrix file at line %d, found \"%s%%s\" instead.",lineno(),YYText());
  BEGIN(Fehlerzustand);
}

	/************* E I N G A B E  ***********************************/
	
<Eingabe>\n    { 
  if (Counter==D) {Counter=0; //cout << "Counter zero" << endl;
			}
  if (Counter!=0) { Errormessage=string("Number of Entries in matrix-file line %d does not match dimension.",lineno()-1); BEGIN(Fehlerzustand); }
}

<Eingabe>{REAL}{ws}*    { // matching whitespace here makes the scanner faster ;-)) and improves error message quality
  mat_value.append(atof(YYText()));Counter++;
   //cout << "Eingabe " << mat_value.tail() << " Counter" << Counter << endl;
} 

<Eingabe>"end"  { //cout << "Ende erfolgreich gefunden" << endl;
		       return 0; // hier Ende
		} 

<Eingabe>. {
	    Errormessage=string("Entry #%d expected in matrix file at line %d, found \"%s%%s\" instead.",Counter+1,lineno(),YYText());
	    BEGIN(Fehlerzustand);
}


	/************* F E H L E R Z U S T A N D  ***********************************/
<Fehlerzustand>{NOEOL}* {
//cout << "Fehler im Location-File..." << endl;
// insert rest of line in location of a %s
Errormessage = string(Errormessage.cstring(),YYText());
return 1;
}

<Fehlerzustand>\n { // falls der Fehler am Zeilenende war
Errormessage = string(Errormessage.cstring(),"");
return 1;
}

	/************ L e e r z e i l e n ***********************************/
<*>\n { // eat up otherwise unwanted newline
if (!Linebreak_legal) {
switch(YY_START) {
case Dimension:
case Eingabe:
{
Errormessage=string("Illegal linebreak in location file at line %d.",lineno() - 1);
return 1;
}
}
} else { Linebreak_legal = false; }
}

%% 

/* Hauptprogramm, liest ein file ein, nur zum Testen, wenn nicht in LOLA eingebunden.. */
#ifdef DEBUGLEX
main(int argc, char** argv) {
  int rc;

  if (argc != 2) {
    cerr << "Please exactly one file as input.\n";
    exit(1);
  }

  ifstream file (argv[1],ios::in);

  Filename = string(argv[1]);
  FlexLexer* lexer = new matFlexLexer();
  rc = lexer->yylex(&file);
  if (rc) { 
    cerr << string("Error(%d): ",rc) << Errormessage << endl;
    exit(1);
  } else {
   cout << " Eingelesenes Objekt:" << endl;
   mat_value.print();
  }
  return 0;
}
#endif

//Local Variables:
//mode: c++
//End:

