%{
#include <math.h>
#if defined(LOLA_LIBSRC_PU) || defined(DEBUGLEX)
   #include <LOLA/restr_util.h>
#endif
%}

%option noyywrap
%option c++
%option yylineno

/* Zustaende fuer ReadMat*/

%x Dimension
%x Fehlerzustand
%x Restr_Typ
%x Barrier
%x BarKlammer
%x Eingabe_poly
%x Eingabe_circ
%x Eingabe_rect
%x Eingabe_seg
%x Klammer 
%x ForceNewline
%x Comment
 
/* Variablen */
%{
  int restr_D,restr_Counter=0; //  D = dimension
  string restr_errmsg = "Syntaxerror in matrix file!";
  int restr_errcd = 1410;
  int restr_callcond; // return-adress for <*>-rules
  bool restr_lbreak = false; // flag to allow the next linebreak
  list<double> restr_coord; // Collecting the doubles
  restrictions all_restr; // All Restrictions together
  restriction *one_restr; // Just one restriction
  int bar_type=0; // Barrier-Value 0-infinity
  int restr_type=0;
  list<point> Poly; // for Polyhedrons
%}
ws [ \t]
DIGIT [0-9]
NODIGIT [^0-9\n\r]
INTEGER ("+"|"-")?{DIGIT}+
REAL {INTEGER}(\.{DIGIT}*)?([eE]{INTEGER})?
NOEOL [^\n\015]
ID [a-z][a-z0-9]*
%{
#undef yyFlexLexer
#define yyFlexLexer restrFlexLexer
#include <FlexLexer.h>
#ifdef LOLA_LIBSRC_PU
/******** ReadRestr aus restr_util.h/facs_util.h **********/
restrictions ReadRestr(ifstream& file)
/********************************************************************
* Purpose   : Reads a restriction file
* Returns   : %
* Influences: %
* Date      : July 1998 
* Author    : Karen Schulze 
*********************************************************************/
{
all_restr.clear();
Poly.clear();
restr_coord.clear();
restr_Counter=0;
if (!file) lola_error("Can't open file!",1409,1);
else { //jetzt flexen
               FlexLexer* lexrestr = new restrFlexLexer();
                int remember=2;
               while(remember > 1) remember=lexrestr->yylex(&file);
                if (remember==1) { lola_error(restr_errmsg,restr_errcd,1); }
        }
return all_restr;
}
	#endif
%}

%%
	/************* A N F A N G S Z U S T A N D ************************************/
[ \n]+  /*Vor Beginn Leerzeilen zulassen */ 

	/* allow arbitrary whitespace */
<*>[[:blank:]\r]

"begin"[ ]*"{"[ ]*"restriction"[ ]*"}"[ ]*"["               {//cout << "Anfang mat" << endl;
							BEGIN(Dimension);}

"begin"[ ]*"{"[ ]*"restriction"[ ]*"}"[ ]* { // the previous rule is definitely longer and thus takes precedence
  //cout << "no dimension" ;
   restr_D=2; // defaults
  restr_callcond=Restr_Typ;
  BEGIN(ForceNewline);
}


<ForceNewline>\n {
  /* Wait for end-of-line but allow <*>-rules to apply. This is a dummy
  * start condition if you don't want to do anything at eol or are too
  * lazy to implement an own \n-rule for some start condition */
  BEGIN(restr_callcond);
}

	/************* C o m m e n t s ************************************/
	/* quick and ineffcient way to do comments - see man flex for
	   alternatives */
<*>"\x2F\x2F".*  { /* (\x2F = /) not matching newline is important, so
		   the end-of-line information is retained */
  restr_lbreak = true;
}

<*>"\x2F\*" { restr_callcond = YY_START; BEGIN(Comment); } /* \x2F = / */
<Comment>. // this has to be just one character, so */ takes precedence
<Comment>\*\x2F { BEGIN(restr_callcond); } /* \x2F = / */

	/************* D I M E N S I O N ************************************/
<Dimension>{INTEGER} {
  restr_D=atoi(YYText());
  if (restr_D < 1) {
    restr_errmsg=string("Dimension too small in matrix file at line %d.",lineno());
    return 1;
  } else {
    //cout << "Dimension: " << restr_D << endl;
    BEGIN(Klammer);
  }
}

<Dimension>. {
  restr_errmsg = string("Dimension expected in matrix file at line %d, found \"%s%%s\" instead.",lineno(),YYText());
  BEGIN(Fehlerzustand);
}


	/************* K L A M M E R  ***********************************/

<Klammer>"]" /* ] is OK :-) */
<Klammer>\n { BEGIN(Restr_Typ); } /* \n is necessary */
<Klammer>. {
  restr_errmsg = string("Closing bracket expected in matrix file at line %d, found \"%s%%s\" instead.",lineno(),YYText());
  BEGIN(Fehlerzustand);
}

	/************* R E S T R _ T Y P ********************************/
<Restr_Typ>"begin"[ ]*"{"[ ]*"polyhedron"[ ]*"}"[ ]*"["               {//cout << "Anfang Polyhedron" << endl;
							restr_type=1; //polyhedron!
                                                        BEGIN(Barrier);}

<Restr_Typ>"begin"[ ]*"{"[ ]*"polyhedron"[ ]*"}"[ ]* { // the previous rule is definitely longer and thus takes precedence
  //cout << "no barrier_type" ;
   bar_type=0; // defaults
  restr_type=1; //"polyhedron";
  restr_callcond=Eingabe_poly;
  BEGIN(ForceNewline);
}

<Restr_Typ>"begin"[ ]*"{"[ ]*"conpolyhedron"[ ]*"}"[ ]*"["               {//cout << " Anfang ConPolyhedron" << endl;
                                                        restr_type=2; //conpolyhedron!
                                                        BEGIN(Barrier);}

<Restr_Typ>"begin"[ ]*"{"[ ]*"conpolyhedron"[ ]*"}"[ ]* { // the previous rule is definitely longer and thus takes precedence
  //cout << "no barrier_type" ;
   bar_type=0; // defaults
  restr_type=2; //"conpolyhedron";
  restr_callcond=Eingabe_poly;
  BEGIN(ForceNewline);
}

<Restr_Typ>"begin"[ ]*"{"[ ]*"circle"[ ]*"}"[ ]*"["               {//cout << " Anfang Circle" << endl;
                                                        restr_type=3; //circle
                                                        BEGIN(Barrier);}

<Restr_Typ>"begin"[ ]*"{"[ ]*"circle"[ ]*"}"[ ]* { // the previous rule is definitely longer and thus takes precedence
  //cout << "no barrier_type" ;
   bar_type=0; // defaults
  restr_type=3; //"circle";
  restr_callcond=Eingabe_circ;
  BEGIN(ForceNewline);
}

<Restr_Typ>"begin"[ ]*"{"[ ]*"rectangle"[ ]*"}"[ ]*"["               {//cout << " Anfang Rectangle" << endl;
                                                        restr_type=4; //rectangle
                                                        BEGIN(Barrier);}

<Restr_Typ>"begin"[ ]*"{"[ ]*"rectangle"[ ]*"}"[ ]* { // the previous rule is definitely longer and thus takes precedence
  //cout << "no barrier_type" ;
   bar_type=0; // defaults
  restr_type=4; //"rectangle";
  restr_callcond=Eingabe_rect;
  BEGIN(ForceNewline);
}

<Restr_Typ>"begin"[ ]*"{"[ ]*"segment"[ ]*"}"[ ]*"["               {
                                                        restr_type=5; //segmente
                                                        BEGIN(Barrier);}

<Restr_Typ>"begin"[ ]*"{"[ ]*"segment"[ ]*"}"[ ]* 	{ // the previous rule is definitely longer and thus takes precedence
  bar_type=0; // defaults
  restr_type=5; //"segment";
  restr_callcond=Eingabe_seg;
  BEGIN(ForceNewline);
}

<Restr_Typ>"end"[ ]*"{"[ ]*"restriction"[ ]*"}"[ ]*    {//cout << "end restriction" << endl;
                                                        return 0; // Zurueck
                                                        }

<Restr_Typ>. {
  restr_errmsg = string(" 'begin{restr_type}' or 'end {restriction} ' expected in restriction file, line %d.",lineno());
  BEGIN(Fehlerzustand);
}
	/************* B A R R I E R ********************************/
<Barrier>{INTEGER} {
  bar_type=atoi(YYText());
  if (bar_type < 0) {
    restr_errmsg=string("Value between 0 and infinity (inf) expected in line %d of the restriction-file.",lineno());
    return 1;
  } else {
    //cout << "Barrier: " << bar_type << endl; 
   BEGIN(BarKlammer);
  }
}

<Barrier>"inf" {bar_type=-1;
		//cout << endl << "Barriere!!!" << endl;
		BEGIN(BarKlammer);
		}

<Barrier>. {
  restr_errmsg = string("Barrier-value expected in restriction-file  at line %d, found \"%s%%s\" instead.",lineno(),YYText());
  BEGIN(Fehlerzustand);
}

        /************* B A R K L A M M E R  ***********************************/

<BarKlammer>"]" /* ] is OK :-) */
<BarKlammer>\n { restr_Counter=0;
		 Poly.clear();
		 restr_coord.clear();
                //cout << "restr_type" << restr_type << endl; 
		switch(restr_type) 
		 {
		  case 1: 
		  case 2: {BEGIN(Eingabe_poly);break;}
		  case 3: {BEGIN(Eingabe_circ);break;}
		  case 4: {BEGIN(Eingabe_rect);break;}
		  case 5: {BEGIN(Eingabe_seg);break;}
		 }
	}
<BarKlammer>. {
  restr_errmsg = string("Closing bracket expected in restriction file at line %d, found \"%s%%s\" instead.",lineno(),YYText());
  BEGIN(Fehlerzustand);
}

	/************* E I N G A B E _ P O L Y  ***********************************/
	
<Eingabe_poly>\n    { 
  if (restr_Counter==restr_D) {restr_Counter=0; //cout << "Counter zero" << endl;
				double c1=restr_coord.pop();
				double c2=restr_coord.pop();
				Poly.append(point(c1,c2)); // 3-D geht gar nicht, hihi
				restr_coord.clear();
			}
  if (restr_Counter!=0) { restr_errmsg=string("Number of Coordinates in restriction-file line %d does not match dimension.",lineno()-1); BEGIN(Fehlerzustand); }
}

<Eingabe_poly>{REAL}{ws}*    { // matching whitespace here makes the scanner faster ;-)) and improves error message quality
  restr_coord.append(atof(YYText()));restr_Counter++;
   //cout << "Eingabe " << restr_coord.tail() << " Restr_Counter" << restr_Counter << endl;
} 

<Eingabe_poly>"end"[ ]*"{"[ ]*"polyhedron"[ ]*"}"[ ]*\n  { //cout << "Ende erfolgreich gefunden" << endl;
//		 Poly.print();
		bool def=false;
		switch(bar_type) {
		  case 0: { one_restr=new restr_poly (Poly);
			   one_restr->in_forbid();
			   break;}
		  case -1: {one_restr=new polygon_barrier(Poly);
			    break;}
		  default: {def=true;break;}
		}
		if (def) { restr_errmsg=string("Only Restrictions=[0] and Barriers=[inf] are supported so far !!! ");
                           BEGIN(Fehlerzustand);
			} else {
				all_restr.append(one_restr);
	     			BEGIN(Restr_Typ);} 
		} 

<Eingabe_poly>"end"[ ]*"{"[ ]*"conpolyhedron"[ ]*"}"[ ]*\n  { //cout << "Ende erfolgreich gefunden" << endl;
// 		Poly.print();
		bool def=false;
                switch(bar_type) {
                  case 0: { one_restr=new restr_poly (Poly);
                           one_restr->in_forbid();
                           break;}
                  case -1: {one_restr=new polygon_barrier(Poly);
                            break;}
                  default: {def=true;break;}
                }
                if (def) { restr_errmsg=string("Only Restrictions=[0] and Barrie rs=[inf] are supported so far !!! ");
                           BEGIN(Fehlerzustand);
                        } else {
                                all_restr.append(one_restr);
                                BEGIN(Restr_Typ);}
                }
<Eingabe_poly>. {
	    restr_errmsg=string("Coordinate #%d expected in restriction-file at line %d, found \"%s%%s\" instead.",restr_Counter+1,lineno(),YYText());
	    BEGIN(Fehlerzustand);
}

	/************* E I N G A B E _ C I R C  ***********************************/
<Eingabe_circ>{REAL}{ws}*    { // matching whitespace here makes the scanner faster ;-)) and improves error message quality
 	 restr_coord.append(atof(YYText()));restr_Counter++;
	 //cout << "Eingabe " << restr_coord.tail() << " Restr_Counter" << restr_Counter << endl;
}

<Eingabe_circ>"end"[ ]*"{"[ ]*"circle"[ ]*"}"[ ]*\n  { //cout << "Ende erfolgreich gefunden" << endl;
			if (restr_Counter!=restr_D+1 ) {
				restr_errmsg=string("Number of Entries is wrong in line %d in restriction-file.",lineno());
				BEGIN(Fehlerzustand);}
			bool def=false;
			double c1=restr_coord.pop();
			double c2=restr_coord.pop();
			point m(c1,c2); // mehr als zwei geht nicht,hihi
		       switch(bar_type) {
			case 0: {one_restr=new restr_circle(m,restr_coord.Pop());
                        	one_restr->in_forbid();
				break;}
		        case -1: {one_restr=new circle_barrier(m,restr_coord.Pop());
				  break;
				 }
			default: {def=true;break;}
	      
		 	}
		      if (def) { restr_errmsg=string("Only Restrictions=[0] and Barriers=[inf] are supported so far !!! ");
                                 BEGIN(Fehlerzustand);
				} else {
                      		all_restr.append(one_restr);
                      		BEGIN(Restr_Typ);}
                }

<Eingabe_circ>. {
            restr_errmsg=string("Coordinate #%d expected in restriction-file at line %d, found \"%s%%s\" instead.",restr_Counter+1,lineno(),YYText());
            BEGIN(Fehlerzustand);
}

       /************* E I N G A B E _ R E C T   ********************************
***/
<Eingabe_rect>{REAL}{ws}*    { // matching whitespace here makes the scanner fas ter ;-)) and improves error message quality
         restr_coord.append(atof(YYText()));restr_Counter++;
         //cout << "Eingabe " << restr_coord.tail() << " Restr_Counter" << restr_Counter << endl;
}

<Eingabe_rect>"end"[ ]*"{"[ ]*"rectangle"[ ]*"}"[ ]*\n  { //cout << "Ende erfolgreich gefunden" << endl;
                        if (restr_Counter!=2*restr_D+1 ) {
                                restr_errmsg=string("Number of Entries is wrong in line %d in restriction-file.",lineno());
                                BEGIN(Fehlerzustand);}
			bool def=false;
			double c1=restr_coord.pop();
			double c2=restr_coord.pop();
                        point m1(c1,c2); // mehr al s zwei geht nicht,hihi
			c1=restr_coord.pop();
			c2=restr_coord.pop();
			switch(bar_type) {
				case 0: {one_restr=new restr_rect(m1,c1,c2,restr_coord.pop()*LEDA_PI/180);
					one_restr->in_forbid();
					break;
					}
				case -1: { // Rechteck in Polygon ueberfuehren
					Poly.append(m1); // Rot.punkt
					point m2(m1.xcoord(),m1.ycoord()+c1);
					point m3(m1.xcoord()+c2,m2.ycoord());
					point m4(m3.xcoord(),m1.ycoord());
					double alp=restr_coord.pop()*LEDA_PI/180;
					Poly.append(m2.rotate(m1,alp));
					Poly.append(m3.rotate(m1,alp));
					Poly.append(m4.rotate(m1,alp));	
					one_restr=new polygon_barrier(Poly);
					break;
					}
				default: {def=true;
					break;} 
			}
			if (def) { restr_errmsg=string("Only Restrictions=[0] and Barriers=[inf] are supported so far !!! ");
                                 BEGIN(Fehlerzustand);
                                } else {
                                all_restr.append(one_restr);
                                BEGIN(Restr_Typ);}
                }

<Eingabe_rect>. {
            restr_errmsg=string("Coordinate #%d expected in restriction-file at line %d, found \"%s%%s\" instead.",restr_Counter+1,lineno(),YYText());
            BEGIN(Fehlerzustand);
}

        /************* E I N G A B E _ S E G  ***********************************/
<Eingabe_seg>{REAL}{ws}*    { // matching whitespace here makes the scanner faster ;-)) and improves error message quality
         restr_coord.append(atof(YYText()));restr_Counter++;
         //cout << "Eingabe " << restr_coord.tail() << " Restr_Counter" << restr_Counter << endl;
}

<Eingabe_seg>"end"[ ]*"{"[ ]*"segment"[ ]*"}"[ ]*\n  { //cout << "Ende erfolgreich gefunden" << endl;
                        if (restr_Counter!=2*restr_D ) {
                                restr_errmsg=string("Number of Entries is wrong in line %d in res triction-file.",lineno());
                                BEGIN(Fehlerzustand);}
			double c1=restr_coord.pop();
			double c2=restr_coord.pop();
                        point m(c1,c2); // mehr als zwei geht nicht
			c1=restr_coord.pop();
			c2=restr_coord.pop();
                        point n(c1,c2); // mehr als zwei geht nicht ,hihi
                       switch(bar_type) {
                        /*case 0: {one_restr=new restr_circle(m,restr_coord.Pop());
                                one_restr->in_forbid();
                                all_restr.append(one_restr);
                                break;}*/
                        case -1: {one_restr=new segment_barrier(m,n);
                                  break;
                                 }
                        default: {restr_errmsg=string("Only Barriers=[inf] is supported so far !!! ");
                                        BEGIN(Fehlerzustand);
                                        break;}

                        }
                      all_restr.append(one_restr);
                      BEGIN(Restr_Typ);
                }

<Eingabe_seg>. {
            restr_errmsg=string("Coordinate #%d expected in restriction-file at line %d, found \"%s%%s\" instead.",restr_Counter+1,lineno(),YYText());
            BEGIN(Fehlerzustand);
}


	/************* F E H L E R Z U S T A N D  ***********************************/
<Fehlerzustand>{NOEOL}* {
//cout << "Fehler im Location-File..." << endl;
// insert rest of line in location of a %s
restr_errmsg = string(restr_errmsg.cstring(),YYText());
return 1;
}

<Fehlerzustand>\n { // falls der Fehler am Zeilenende war
restr_errmsg = string(restr_errmsg.cstring(),"");
return 1;
}

	/************ L e e r z e i l e n ***********************************/
<*>\n { // eat up otherwise unwanted newline
if (!restr_lbreak) {
switch(YY_START) {
case Dimension:
case Barrier:
{
restr_errmsg=string("Illegal linebreak in location file at line %d.",lineno() - 1);
return 1;
}
}
} else { restr_lbreak = false; }
}

%% 

/* Hauptprogramm, liest ein file ein, nur zum Testen, wenn nicht in LOLA eingebunden.. */
#ifdef DEBUGLEX
main(int argc, char** argv) {
  int rc;

  if (argc != 2) {
    cerr << "Please exactly one file as input.\n";
    exit(1);
  }

  ifstream file (argv[1],ios::in);

  // Filename = string(argv[1]);
  FlexLexer* lexer = new restrFlexLexer();
  rc = lexer->yylex(&file);
  if (rc) { 
    cerr << string("Error(%d): ",rc) << restr_errmsg << endl;
    exit(1);
  } else {
   //cout << " Eingelesenes Objekt:" << endl;
   //Poly.print(); 
 }
  return 0;
}
#endif

//Local Variables:
//mode: c++
//End:

