#! /usr/local/bin/perl5 -w
########################################################################
# Pupose: test out the flex scanner for location files (stand-alone)
# Features: all examples I could come up with
# Calling convention: none
# Influences: well, who cares? :-)
# Returns: always 0
# Date: 29.7.1997
# Author: MCM
# Bugs/Defiancies:
#       - no &usage
#       - quick hack
#       - tests invented by author ;-)
#       - input should be more randomized
#######################################################################

require 5.001;

use English;
use Getopt::Long;

my ($Head) = q{$Id: testloc.pl,v 1.1 1997/08/01 13:39:53 mcm Exp $ };
my ($progname) = $0;
$progname =~ s%^[^/]+/%%; # chop the path off

my($opt_version,$opt_help,$verbose);

&usage unless (GetOptions("--version"=>\$opt_version,
			  "-h|--help"=>\$opt_help,
			  "--verbose"=>\$verbose)
	       );

&usage if ($opt_help);
if ($opt_version) {
    print "This is $progname, Version $Head\n";
    exit(0);
}

my ($executable) = "scanner";
my ($testfilename) = "test.loc";

my (@input,@expect,@name); # hold the test inputs, expected outputs and names of the tests

my ($test) = 0; # counter

srand(time ^ $$);
my ($random_coord) = sprintf("%10.2f",rand(5000)-2500);

############################################################################
##  syntaktisch korrekte Beispiele
############################################################################

$name[$test] = "plain normal";
$input[$test] = <<INPUT
begin {location} [2,1]
15.01 16.00 955 [Koeln]
10.00 $random_coord 577 [Duesseldorf]
9.00 52.00 540 [Duisburg]
20.00 53.00 -621  [Essen]
42.00 58.00 610  [Dortmund]
48.00 90.00 261  [Muenster]
end {location}
INPUT
    ;

#prototypical answer
$expect[$test] = <<EXPECT
 Eingelesenes Objekt:
begin\\s*{location}\\s*\\[2,1\\]
15.01 16  955  \\[Koeln\\]
10 $random_coord\\d*  577  \\[Duesseldorf\\]
9 52  540  \\[Duisburg\\]
20 53  -621  \\[Essen\\]
42 58  610  \\[Dortmund\\]
48 90  261  \\[Muenster\\]
end\\s*{location}
EXPECT
    ;
$expect[$test] =~ s/ +/\\s+/gm; # allow arbitrary whistspace

$name[++$test] = "alternative formulation";
$input[$test] = <<INPUT
begin{location}  [ 2 , 1 ]   
0.1501e2 16.00 955
10. $random_coord 577 [Duesseldorf]
9. 52.00 540 [Duisburg]
20. 53.00 -621  []
42.00 5.800e1 610  [Dortmund]
48.00 90.00 261  [Muenster]
end		{location}
INPUT
    ;

$expect[$test] = $expect[0];
$expect[$test] =~ s/Koeln|Essen//mg;

$name[++$test] = "with C-M like from DOS, default weight dimension";
$input[$test] = <<INPUT
begin {location} [2]

15.01 16.00 955 [Koeln]

10.00 $random_coord 577 [Duesseldorf]

9.00 52.00 540 [Duisburg]

20.00 53.00 -621  [Essen]

42.00 58.00 610  [Dortmund]

48.00 90.00 261  [Muenster]

end {location}

INPUT
    ;

$expect[$test] = $expect[0];

$name[++$test] = "with comment and w/o dimensions";
$input[$test] = <<INPUT
begin{location} //[2,1]
15.01 16.00 955 [Koeln]
10.00 /* 20.00 53.00 -621  [Essen]
34.01 */ $random_coord 577 [Duesseldorf]
9.00 52.00 540 [Duisburg]
20.00 /* 67.8 */ 53.00 -621  [Essen]
42.00 58.00 610  [Dortmund]
48.00 90.00 261  [Muenster]
end{location}
INPUT
    ;

$expect[$test] = $expect[0];

$name[++$test] = "another comment";
$input[$test] = <<INPUT
begin {location}
15.01 16.00 955 [Koeln]//
//
10.00 $random_coord 577 [Duesseldorf]
//20.00 36.00 577 [Duesseldorf]
9.00 52.00 540// [Duisburg]
    20.00 53.00 -621/* some comment goes here */	[Essen]
42.00 58.00 610  [/*Dortmund*/]
// /* Hallo */
/* //
*/ //
48.00 90.00 261  [Muenster]//

end {location}

INPUT
    ;
$expect[$test] = $expect[0];
$expect[$test] =~ s/Duisburg//m;
$expect[$test] =~ s/Dortmund/\/\\*Dortmund\\*\//m; # no comments in names

############################################################################
##  syntaktisch falsche Beispiele
############################################################################

$name[++$test] = "wrong delimiter in dimensions";
$input[$test] = <<INPUT
begin {location} [2|1]
end {location}
INPUT
    ;
$expect[$test] = "Error.*\\|1\\](.|\\n)*"; # tests, if error is reported at correct place

$name[++$test] = "dimension forgotten";
$input[$test] = <<INPUT
begin {location} [,1]
end {location}
INPUT
    ;
$expect[$test] = "Error.*,1\\](.|\\n)*"; # tests, if error is reported at correct place

$name[++$test] = "weight forgotten";
$input[$test] = <<INPUT
begin {location} [2,1]
15.01 16.00 955 [Koeln]
10.00 577 [Duesseldorf]
9.00 52.00 540 [Duisburg]
20.00 53.00 -621  [Essen]
42.00 58.00 610  [Dortmund]
48.00 90.00 261  [Muenster]
end {location}
INPUT
    ;
$expect[$test] = "Error.*\\[Duesseldorf\\](.|\\n)*"; # tests, if error is reported at correct place

$name[++$test] = "coordinate forgotten";
$input[$test] = <<INPUT
begin {location} [2,1]
15.01 16.00 [Koeln]
10.00 $random_coord 577 [Duesseldorf]
9.00 52.00 540 [Duisburg]
20.00 53.00 -621  [Essen]
42.00 58.00 610  [Dortmund]
48.00 90.00 261  [Muenster]
end {location}
INPUT
    ;
$expect[$test] = "Error.*\\[Koeln\\](.|\\n)*"; # tests, if error is reported at correct place

$name[++$test] = "overlapping comments";
$input[$test] = <<INPUT
begin {location} [2,1]
// /*
15.01 16.00 955 [Koeln] */
10.00 $random_coord 577 [Duesseldorf]
9.00 52.00 540 [Duisburg]
20.00 53.00 -621  [Essen]
42.00 58.00 610  [Dortmund]
48.00 90.00 261  [Muenster]
end {location}
INPUT
    ;
$expect[$test] = "Error.*at line 3.*\\*/.*(.|\\n)*"; # tests, if error is reported at correct place

$name[++$test] = "dimension too small";
$input[$test] = <<INPUT
begin {location} [0,1]
15.01 16.00 955 [Koeln]
10.00 $random_coord 577 [Duesseldorf]
9.00 52.00 540 [Duisburg]
20.00 53.00 -621  [Essen]
42.00 58.00 610  [Dortmund]
48.00 90.00 261  [Muenster]
end {location}
INPUT
    ;
$expect[$test] = "Error.*[dD]imension.*at line 1(.|\\n)*";

$name[++$test] = "line concatenation via comment";
$input[$test] = <<INPUT
begin {location} [2,1]
15.01 16.00 955 [Koeln] /*
*/10.00 $random_coord 577 [Duesseldorf]
9.00 52.00 540 [Duisburg]
20.00 53.00 -621  [Essen]
42.00 58.00 610  [Dortmund]
48.00 90.00 261  [Muenster]
end {location}
INPUT
    ;
$expect[$test] = "Error.*[eE]nd of line.*at line 3(.|\\n)*";

$name[++$test] = "illegal linebreak in data";
$input[$test] = <<INPUT
begin {location} [2,1]
15.01 
 16.00 955 [Koeln]
INPUT
    ;
$expect[$test] = "Error.*linebreak.*at line 2(.|\\n)*";

$name[++$test] = "illegal linebreak in dimension";
$input[$test] = <<INPUT
begin {location} [2
,1]
15.01 16.00 955 [Koeln]
INPUT
    ;
$expect[$test] = "Error.*linebreak.*at line 1(.|\\n)*";

$name[++$test] =  "doubled name";
$input[$test] = <<INPUT
begin {location} [2,1]
    15.01 16.00 955 [Koeln] [Hamburg]
INPUT
    ;

$expect[$test] = "Error.*name.*\\[Hamburg\\].*at line 2(.|\\n)*";

$name[++$test] =  "illegal radix character";
$input[$test] = <<INPUT
begin {location} [2,1]
    15,01 16.00 955 [Koeln] [Hamburg]
INPUT
    ;

$expect[$test] = "Error.*at line 2.*15,01(.|\\n)*";

$name[++$test] =  "illegal exponentation character";
$input[$test] = <<INPUT
begin {location} [2,1]
    15.01f5 16.00 955 [Koeln]
INPUT
    ;

$expect[$test] = "Error.*at line 2.*15\\.01f5(.|\\n)*";

$name[++$test] =  "no pre-radix part";
$input[$test] = <<INPUT
begin {location} [2,1]
    .01 16.00 955 [Koeln]
INPUT
    ;

$expect[$test] = "Error.*at line 2.*\\.01(.|\\n)*";

##########################################################################
## main routine
##########################################################################

my ($output); # holds the combined stdout and stderr of the command run
my ($extrapre,$extrapost); # hold additional, unexpected output
my ($status,$signal); # hold exit code and cause of abnormal end

die("cannot execute $executable") unless (-x $executable);

for ($test = 0; $test <= $#input; $test++) {
    die("Cannot write to $testfilename") unless open(TOUT,">$testfilename");
    print TOUT $input[$test];
    close(TOUT);
    print "Test \x23$test"; # \x23 = #
    if ($verbose) {
	print ",".$name[$test];
    }
    print ": ";
    $output = qx{$executable $testfilename 2>&1}; # let it run!
    $status = $? >> 8; # exit code
    $signal = $? & 255; # signal that caused the command to exit
    $re = $expect[$test];
    if ($output =~ /$re/m ) {
      print "PASS";
      print " (rc=$status) " if ($status != 0);
      print " (SIG $signal) " unless ($signal == 0);
      $extrapre = $PREMATCH;
      $extrapost = $POSTMATCH;
      if ("$extrapre$extrapost" !~ /\s*/m) { # allow additional whitespace
	  if ($verbose) { # show what's exra
	      print "\nLeading extra output:\n$extrapre\n";
	      print "Trailing extra outptu:\n$extrapost\n";
	  } else {
	      print " (extra output)\n";
	  }
      } else {
	  print "\n";
      }
    } else { # output doesn't meet expectancy
	print "FAIL (rc=$status)";
	print " (SIG $signal) " unless ($signal == 0);
	print "\n";
	if ($verbose) {
	    print "Actual output:\n";
	    print $output;
	    print "End of actual output.\n";
	}
    }
}
unlink($testfilename);
exit(0);

sub usage {
    print "You call, I test :-)\n";
    exit(0);
}

# why am I doing this in perl instead of expect? Arrghhh.

#Local Variables:
#mode: perl
#End:
